/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client */

/**
 * @class An app banner
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the image
 */
Client.AppBanner = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("meta");
  this.domObj.setAttribute("name", "apple-itunes-app");
  this.metaObj = document.createElement("meta");
  this.metaObj.setAttribute("name", "google-play-app");
  this.updateElement(element);
  this.attachEvents(element.events);
  //
  var h = document.getElementsByTagName('head')[0];
  if (h) {
    h.appendChild(this.domObj);
    h.appendChild(this.metaObj);
  }
};


// Make Client.AppBanner extend Client.Element
Client.AppBanner.prototype = new Client.Element();


/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.AppBanner.prototype.updateElement = function (el)
{
  if (el.itunesAppId) {
    this.itunesAppId = el.itunesAppId;
    this.updateBanner();
    delete el.itunesAppId;
  }
  if (el.gplayAppId) {
    this.gplayAppId = el.gplayAppId;
    this.updateBanner();
    delete el.gplayAppId;
  }
  if (el.argument) {
    this.argument = el.argument;
    this.updateBanner();
    delete el.argument;
  }
  if (el.title) {
    this.title = el.title;
    this.updateBanner();
    delete el.title;
  }
  if (el.author) {
    this.author = el.author;
    this.updateBanner();
    delete el.author;
  }
  if (el.price) {
    this.price = el.price;
    this.updateBanner();
    delete el.price;
  }
  if (el.icon) {
    this.icon = el.icon;
    this.updateBanner();
    delete el.icon;
  }
  if (el.force) {
    this.force = el.force;
    this.updateBanner();
    delete el.force;
  }
  if (el.button) {
    this.button = el.button;
    this.updateBanner();
    delete el.button;
  }
  if (el.options) {
    this.options = el.options;
    if (typeof this.options === "string") {
      try {
        this.options = JSON.parse(this.options);
      } catch (ex) {
        this.options = {};
      }
    }
    this.updateBanner();
    delete el.options;
  }

  if (el.visible !== undefined) {
    this.visible = el.visible;
    if (this.smartBanner && this.openButton) {
      if (el.visible)
        this.smartBanner.show();
      else
        this.smartBanner.hide();
    }
    delete el.visible;
  }
};


/**
 * Update element properties
 */
Client.AppBanner.prototype.updateBanner = function ()
{
  if (!this.updateTimerId) {
    this.updateTimerId = setTimeout(function () {
      delete this.updateTimerId;
      this.domObj.setAttribute("content", "app-id=" + this.itunesAppId + ", app-argument=" + this.argument);
      this.metaObj.setAttribute("content", "app-id=" + this.gplayAppId + ", app-argument=" + this.argument);
      if (!this.smartBanner) {
        //
        // Setting up options
        var opt = this.options || {};
        if (this.argument && (opt.daysReminder === undefined || Client.mainFrame.isEditing()))
          opt.daysReminder = -1;
        if (this.argument && (opt.daysHidden === undefined || Client.mainFrame.isEditing()))
          opt.daysHidden = -1;
        if (this.force)
          opt.force = this.force;
        if (this.title)
          opt.title = this.title;
        if (this.author)
          opt.author = this.author;
        if (this.button)
          opt.button = this.button;
        if (this.icon)
          opt.icon = this.icon;
        if (this.price)
          opt.price = {ios: this.price, android: this.price, windows: this.price};
        //
        this.smartBanner = new SmartBanner(opt);
        this.smartBanner.hide();
        var sb = document.getElementsByClassName("smartbanner")[0];
        //
        // Change hide / show function
        if (sb) {
          this.smartBanner._hide = this.smartBanner.hide;
          this.smartBanner._show = this.smartBanner.show;
          this.smartBanner.show = function () {
            sb.style.display = "block";
            sb.offsetTop;
            this._show();
          }.bind(this.smartBanner);
          this.smartBanner.hide = function () {
            this._hide();
            setTimeout(function () {
              sb.style.display = "";
            }, 250);
          }.bind(this.smartBanner);
        }
        //
        this.openButton = document.getElementsByClassName("smartbanner-button")[0];
        if (this.openButton) {
          this.openButton.addEventListener('click', this.bannerClick.bind(this), false);
        }
        if (this.visible !== false) {
          setTimeout(function () {
            this.smartBanner.show();
          }.bind(this), 500);
        }
      }
    }.bind(this), 100);
  }
};


/**
 * Update element properties
 */
Client.AppBanner.prototype.bannerClick = function (ev)
{
  // Check if we are on a mobile browser
  var device = Client.mainFrame.device;
  var onmob = device.isMobile && device.shell && (device.shell instanceof Client.ShellEmulator);
  if (this.argument && onmob) {
    ev.preventDefault();
    //
    // IOS handling
    if (device.operatingSystem === "ios") {
      //
      window.location.href = this.argument;
      //
      setTimeout(function () {
        var msg = (this.button || "Open") + " " + this.title + " on the App Store?";
        var b = confirm(msg);
        if (b) {
          window.location.href = this.openButton.href;
        }
      }.bind(this), 500);
    }
    //
    // Android handling
    if (device.operatingSystem === "android") {
      this.openSesame();
    }
    //
    setTimeout(function () {
      this.smartBanner.show();
    }.bind(this), 3000);
  }
};


/**
 * Update element properties
 */
Client.AppBanner.prototype.openSesame = function ()
{
  var method = 'iframe';
  var fallbackFunction = function () {
    if (method == 'iframe') {
      window.location = "market://details?id=" + this.gplayAppId;
    }
  };
  if (navigator.userAgent.match(/Chrome/) && !navigator.userAgent.match("Version/")) {
    method = 'intent';
    var args = this.argument.split("://");
    var loc = "intent://" + args[1] + "#Intent;scheme=" + args[0] + ";package=" + this.gplayAppId + ";end";
    document.location = loc;
  }
  else if (navigator.userAgent.match(/Firefox/)) {
    window.location = this.argument;
  }
  else {
    var iframe = document.createElement("iframe");
    iframe.style.border = "none";
    iframe.style.width = "1px";
    iframe.style.height = "1px";
    iframe.src = this.argument;
    document.body.appendChild(iframe);
  }
  setTimeout(fallbackFunction, 750);
  return false;
};

