/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client */

/**
 * @class A bootstrap carousel element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSCarousel = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("div");
  this.domObj.setAttribute("data-ride", "carousel");
  //
  this.innerObj = document.createElement("div");
  this.innerObj.className = "carousel-inner";
  this.domObj.appendChild(this.innerObj);
  //
  if (Client.mainFrame.isEditing())
    this.domObj.id = btoa(this.id);
  else
    this.domObj.id = this.id;
  //
  this.updateElement(element);
  //
  this.attachEvents(element.events);
  //
  parent.appendChildObject(this, this.domObj);
  //
  // Create children after attaching element as external components, rely on dom structure
  this.createChildren(element);
};

// Make BSCarousel extend Client.Container
Client.BSCarousel.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSCarousel.prototype.updateElement = function (el)
{
  var update = this.domObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  var pthis = this;
  //
  // specific class name for this text
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  if (el.controls !== undefined) {
    this.controls = el.controls;
    delete el.controls;
    this.updateControls();
  }
  if (el.indicators !== undefined) {
    this.indicators = el.indicators;
    delete el.indicators;
    this.updateIndicators();
  }
  if (el.interval !== undefined) {
    this.domObj.setAttribute("data-interval", el.interval);
    delete el.interval;
  }
  if (el.wrap !== undefined) {
    this.domObj.setAttribute("data-wrap", el.wrap);
    delete el.wrap;
  }
  if (el.keyboard !== undefined) {
    this.domObj.setAttribute("data-keyboard", el.keyboard);
    delete el.keyboard;
  }
  if (el.autoStart === true) {
    setTimeout(function () {
      pthis.cycle();
    }, 10);
  }
  //
  if (update) {
    //
    var cs = "carousel slide";
    //
    if (this.className)
      cs += " " + this.className;
    //
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};


/**
 * Attach events handler
 * @param {Array} events - array of the events to handle
 */
Client.BSCarousel.prototype.attachEvents = function (events)
{
  if (!events)
    return;
  //
  var pthis = this;
  var pos = events.indexOf("onChangePage");
  if (pos >= 0) {
    events.splice(pos, 1);
    setTimeout(function () {
      $("#" + pthis.domObj.id).on('slid.bs.carousel', function (ev) {
        for (var page = 0; page < pthis.innerObj.childNodes.length; page++) {
          if (pthis.innerObj.childNodes[page] === ev.relatedTarget)
            break;
        }
        var e = [{obj: pthis.id, id: "onChangePage", content: {direction: ev.direction, page: page}}];
        Client.mainFrame.sendEvents(e);
      });
    }, 10);
  }
  //
  Client.Container.prototype.attachEvents.call(this, events);
};


/**
 * Append a child DOM Object to this element DOM Object
 * Usually the child element is not contained in this element array
 * it will be pushed there just after this function call.
 * @param {Element} child child element that requested the insertion
 * @param {HTMLElement} domObj child DOM object to add
 */
Client.BSCarousel.prototype.appendChildObject = function (child, domObj)
{
  if (domObj.tagName === "DIV") {
    domObj.classList.add("item");
    if (!this.elements || this.elements.length === 0)
      domObj.classList.add("active");
    this.innerObj.appendChild(domObj);
  }
  else {
    var d = document.createElement("div");
    d.className = "item" + (!this.elements || this.elements.length === 0 ? " active" : "");
    this.innerObj.appendChild(d);
    d.appendChild(domObj);
  }
  this.updateIndicators();
};


/**
 * Update controls
 */
Client.BSCarousel.prototype.updateControls = function ()
{
  if (this.controls) {
    if (!this.leftObj) {
      this.leftObj = document.createElement("a");
      this.leftObj.innerHTML = "<span class=\"glyphicon glyphicon-chevron-left\" aria-hidden=\"true\"></span><span class=\"sr-only\">Previous</span>";
      this.leftObj.className = "left carousel-control";
      this.leftObj.href = "#" + this.domObj.id;
      this.leftObj.setAttribute("data-slide", "prev");
      this.domObj.appendChild(this.leftObj);
    }
    if (!this.rightObj) {
      this.rightObj = document.createElement("a");
      this.rightObj.innerHTML = "<span class=\"glyphicon glyphicon-chevron-right\" aria-hidden=\"true\"></span><span class=\"sr-only\">Next</span>";
      this.rightObj.className = "right carousel-control";
      this.rightObj.href = "#" + this.domObj.id;
      this.rightObj.setAttribute("data-slide", "next");
      this.domObj.appendChild(this.rightObj);
    }
  }
  else {
    if (this.leftObj) {
      this.domObj.removeChild(this.leftObj);
      delete this.leftObj;
    }
    if (this.rightObj) {
      this.domObj.removeChild(this.rightObj);
      delete this.rightObj;
    }
  }
};


/**
 * Update indicators
 */
Client.BSCarousel.prototype.updateIndicators = function ()
{
  if (this.indicators) {
    if (!this.indObj) {
      this.indObj = document.createElement("ol");
      this.indObj.className = "carousel-indicators";
      this.domObj.appendChild(this.indObj);
    }
    var l = this.innerObj.childNodes.length;
    while (this.indObj.childNodes.length > l) {
      this.indObj.removeChild(this.indObj.lastChild);
    }
    while (this.indObj.childNodes.length < l) {
      var v = document.createElement("li");
      v.setAttribute("data-target", "#" + this.domObj.id);
      v.setAttribute("data-slide-to", this.indObj.childNodes.length);
      if (this.indObj.childNodes.length === 0)
        v.className = "active";
      else
        v.style.marginLeft = "4px";
      this.indObj.appendChild(v);
    }
  }
  else {
    if (this.indObj) {
      this.domObj.removeChild(this.indObj);
      delete this.indObj;
    }
  }
};

/**
 * Go to next slide
 */
Client.BSCarousel.prototype.next = function ()
{
  $("#" + this.domObj.id).carousel("next");
};

/**
 * Go to prev slide
 */
Client.BSCarousel.prototype.prev = function ()
{
  $("#" + this.domObj.id).carousel("prev");
};

/**
 * Pause
 */
Client.BSCarousel.prototype.pause = function ()
{
  $("#" + this.domObj.id).carousel("pause");
};

/**
 * Cycle
 */
Client.BSCarousel.prototype.cycle = function ()
{
  $("#" + this.domObj.id).carousel("cycle");
};

/**
 * Goto
 * @param {int} slide
 */
Client.BSCarousel.prototype.goto = function (slide)
{
  $("#" + this.domObj.id).carousel(slide);
};


/**
 * One of my children element has been removed
 * @param {Element} child
 */
Client.BSCarousel.prototype.onRemoveChildObject = function (child)
{
  var pthis = this;
  //
  var toRemove;
  if (child.domObj.tagName !== "DIV")
    toRemove = child.domObj.parentNode;
  //
  setTimeout(function () {
    if (toRemove)
      pthis.innerObj.removeChild(toRemove);
    pthis.updateIndicators();
  }, 10);
};
