/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client */

/**
 * @class A bootstrap dropdown element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSDropDown = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  // Creating dropdown container
  this.domObj = document.createElement("div");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  //
  parent.appendChildObject(this, this.domObj);
  //
  // Create children after attaching element as external components, rely on dom structure
  this.createChildren(element);
};

// Make BSDropDown extend Client.Element
Client.BSDropDown.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSDropDown.prototype.updateElement = function (el)
{
  // Force update for the first time
  var update = this.domObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  if (el.type !== undefined) {
    this.type = el.type;
    update = true;
    delete el.type;
  }
  //
  // specific class name for this layout
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  //
  if (update) {
    //
    var cs;
    var ingroup = Client.BSGroup && this.parent instanceof Client.BSGroup;
    //
    if (ingroup || true)
      cs = "btn-group" + (this.type === "du" ? " dropup" : "");
    else
      cs = (this.type === "du" ? "dropup" : "dropdown");
    //
    if (this.className)
      cs += " " + this.className;
    //
    this.domObj.className = cs;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};



/**
 * @class A bootstrap dropdown menu element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSDDMenu = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  // Creating dropdown menu container
  this.domObj = document.createElement("ul");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  //
  parent.appendChildObject(this, this.domObj);
  //
  // Create children after attaching element as external components, rely on dom structure
  this.createChildren(element);
};

// Make BSDDMenu extend Client.Element
Client.BSDDMenu.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSDDMenu.prototype.updateElement = function (el)
{
  // Force update for the first time
  var update = this.domObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  // specific class name for this layout
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  //
  if (el.rightAligned !== undefined) {
    this.rightAligned = el.rightAligned;
    update = true;
    delete el.rightAligned;
  }
  //
  if (update) {
    //
    var cs = "dropdown-menu";
    if (this.rightAligned)
      cs += " dropdown-menu-right";
    if (this.className)
      cs += " " + this.className;
    //
    this.domObj.className = cs;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};



/**
 * @class A bootstrap dropdown menu item
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSDDMenuItem = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  var pthis = this;
  //
  // Creating dropdown menu container
  this.liObj = document.createElement("li");
  this.domObj = document.createElement("a");
  this.domObj.href = "javascript:void(0)";
  this.liObj.appendChild(this.domObj);
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  //
  // Attach a click handler to close the menu if the object has also the click event
  if (element.events.indexOf("onClick") > -1) {
    this.domObj.addEventListener("click", function () {
      $('#' + pthis.parent.id).dropdown('toggle');
    });
  }
  //
  parent.appendChildObject(this, this.liObj);
  //
  // Create children after attaching element as external components, rely on dom structure
  this.createChildren(element);
};

// Make BSDDMenuItem extend Client.Element
Client.BSDDMenuItem.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSDDMenuItem.prototype.updateElement = function (el)
{
  // Force update for the first time
  var update = this.liObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  // specific class name for this layout
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  if (el.type !== undefined) {
    this.type = el.type;
    update = true;
    delete el.type;
  }
  if (el.disabled !== undefined) {
    this.disabled = el.disabled;
    update = true;
    delete el.disabled;
  }
  if (this.type === "header") {
    this.domObj.style.display = "none";
    if (el.innerText !== undefined) {
      this.liObj.innerText = el.innerText;
      delete el.innerText;
    }
    if (el.innerHTML !== undefined) {
      this.liObj.innerHTML = el.innerHTML;
      delete el.innerHTML;
    }
  }
  //
  if (update) {
    //
    var cs = "";
    if (this.type === "header")
      cs += " dropdown-header";
    if (this.type === "divider")
      cs += " divider";
    if (this.type === "divider")
      cs += " divider";
    if (this.disabled)
      cs += " disabled";
    if (this.className)
      cs += " " + this.className;
    //
    this.liObj.className = cs;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};

/**
 * Return the dom root object of this element
 * @returns {DomElement}
 */
Client.BSDDMenuItem.prototype.getRootObject = function ()
{
  return this.liObj;
};
