/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client */

/**
 * @class A bootstrap form element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSForm = function (element, parent, view)
{
  element.tag = "form";
  Client.Container.call(this, element, parent, view);
  var pthis = this;
  this.domObj.addEventListener("submit", function (ev) {
    if (pthis.sendSubmit) {
      var e = [{obj: pthis.id, id: "onSubmit", content: pthis.saveEvent(ev)}];
      Client.mainFrame.sendEvents(e);
    }
    ev.preventDefault();
    return false;
  }, true);
};

// Make BSForm extend Client.Container
Client.BSForm.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSForm.prototype.updateElement = function (el)
{
  var update = this.domObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  // specific class name for this text
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  if (el.inline !== this.inline) {
    this.inline = el.inline;
    update = true;
    delete el.inline;
  }
  if (el.horizontal !== this.horizontal) {
    this.horizontal = el.horizontal;
    update = true;
    delete el.horizontal;
  }
  //
  if (update) {
    //
    var cs = "";
    //
    if (Client.BSNavSection && this.parent instanceof Client.BSNavSection)
      cs += " navbar-form";
    //
    if (this.inline)
      cs += " form-inline";
    if (this.horizontal)
      cs += " form-horizontal";
    //
    if (this.className)
      cs += " " + this.className;
    //
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};


/**
 * Attach events handler
 * @param {Array} events - array of the events to handle
 */
Client.BSForm.prototype.attachEvents = function (events)
{
  if (!events)
    return;
  //
  var pos = events.indexOf("onSubmit");
  if (pos >= 0) {
    events.splice(pos, 1);
    this.sendSubmit = true;
  }
  //
  Client.Container.prototype.attachEvents.call(this, events);
};



/**
 * @class A bootstrap input element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSInput = function (element, parent, view)
{
  Client.Input.call(this, element, parent, view);
  //
  if (Client.mainFrame.isEditing())
    this.domObj.id = btoa(this.id);
};

// Make BSForm extend Client.Input
Client.BSInput.prototype = new Client.Input();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSInput.prototype.updateElement = function (el)
{
  var updateDom = !this.rootObj;
  var updateClasses = updateDom;
  //
  Client.BSHelper.updateElement(el, this);
  //
  // specific class name for this input
  if (el.className !== undefined) {
    this.className = el.className;
    updateClasses = true;
    delete el.className;
  }
  //
  // input size
  if (el.xs !== undefined) {
    this.xs = el.xs;
    updateDom = true;
    delete el.xs;
  }
  if (el.sm !== undefined) {
    this.sm = el.sm;
    updateDom = true;
    delete el.sm;
  }
  if (el.md !== undefined) {
    this.md = el.md;
    updateDom = true;
    delete el.md;
  }
  if (el.lg !== undefined) {
    this.lg = el.lg;
    updateDom = true;
    delete el.lg;
  }
  //
  // Change type
  if (el.type && this.type !== el.type) {
    this.type = el.type;
    updateDom = true;
  }
  //
  // Change group
  if (el.formGroup !== undefined && el.formGroup !== this.group) {
    this.group = el.formGroup;
    updateDom = true;
    delete el.formGroup;
  }
  //
  // Change group
  if (el.feedback !== undefined && el.feedback !== this.feedback) {
    this.feedback = el.feedback;
    updateDom = true;
    delete el.feedback;
  }
  //
  // Change label
  if (el.label !== undefined && el.label !== this.label) {
    this.label = el.label;
    delete el.label;
    //
    if (this.label) {
      if (this.labelText)
        this.labelText.textContent = this.label;
      else
        updateDom = true;
    }
  }
  //
  // Change help text
  if (el.helpText !== undefined && el.helpText !== this.helpText) {
    this.helpText = el.helpText;
    delete el.helpText;
    //
    if (this.helpText) {
      if (this.helpObj)
        this.helpObj.textContent = this.helpText;
      else
        updateDom = true;
    }
    else {
      if (this.helpObj)
        this.helpObj.textContent = "";
    }
  }
  //
  // Disabled / Enabled
  if (el.disabled !== undefined && this.needWrapper()) {
    this.disabled = el.disabled;
    updateClasses = true;
  }
  //
  // inline check / radio
  if (el.inline !== undefined && this.needWrapper()) {
    this.inline = el.inline;
    delete el.inline;
    updateClasses = true;
  }
  //
  // height
  if (el.height !== undefined) {
    this.height = el.height;
    delete el.height;
    updateClasses = true;
  }
  //
  // state
  if (el.state !== undefined) {
    this.state = el.state;
    delete el.state;
    //
    if (this.needWrapper() && !this.wrapperObj)
      updateDom = true;
    //
    updateClasses = true;
  }
  //
  // DateTimePicker
  if (el.datePicker) {
    this.orgType = el.type;
    var opt = {useCurrent: false};
    if (el.type === "date") {
      opt.format = "L";
      el.type = "text";
    }
    if (el.type === "time") {
      opt.format = "LT";
      el.type = "text";
    }
    if (el.type === "datetime") {
      el.type = "text";
    }
    //
    if (Client.mainFrame.isEditing())
      this.domObj.id = btoa(this.id);
    else
      this.domObj.id = this.id;
    var id = "#" + this.domObj.id;
    //
    $(id).datetimepicker(opt).on("dp.change", function (e) {
      if (this.skipChange)
        return;
      var l = [];
      var v = e.date;
      if (this.orgType === "date")
        v = v.format("YYYY-MM-DD");
      if (this.orgType === "time")
        v = v.format("HH:mm");
      if (this.orgType === "datetime")
        v = v.format("YYYY-MM-DD HH:mm");
      l.push({obj: this.id, id: "chgProp", content: {name: "value", value: v, clid: Client.id}});
      l.push({obj: this.id, id: "chgProp", content: {name: "errorText", value: this.domObj.validationMessage, clid: Client.id}});
      l.push({obj: this.id, id: "onChange", content: this.saveEvent(e)});
      Client.mainFrame.sendEvents(l);
    }.bind(this));
    this.datePicker = $(id).data("DateTimePicker");
    delete el.datePicker;
  }
  //
  if (el.value !== undefined && this.datePicker) {
    var m = moment(el.value, this.datePicker.format() || undefined);
    if (!m.isValid())
      m = moment(el.value);
    if (m.isValid()) {
      this.skipChange = true;
      this.datePicker.date(m);
      this.datePicker.viewDate(m);
      this.domObj.value = m.format(this.datePicker.format() || "L LT");
      this.skipChange = false;
    }
    else {
      this.skipChange = true;
      this.datePicker.date(moment());
      this.datePicker.viewDate(moment());
      this.domObj.value = "";
      this.skipChange = false;
    }
    delete el.value;
  }
  //
  // DateTimePicker options
  if (el.pickerOptions && this.datePicker) {
    var opt = el.pickerOptions;
    if (typeof opt === "string") {
      try {
        opt = JSON.parse(opt);
      }
      catch (ex) {
      }
    }
    //
    this.datePicker.options(opt);
    delete el.pickerOptions;
  }
  //
  if (updateDom)
    this.createDom();
  //
  if (updateDom || updateClasses)
    this.updateClasses();
  //
  Client.Input.prototype.updateElement.call(this, el);
};


/**
 * shows the picker
 */
Client.BSInput.prototype.showPicker = function ()
{
  this.datePicker.show();
};


/**
 * Returns true if this object require a wrapping label
 */
Client.BSInput.prototype.insideLabel = function ()
{
  return this.type === "radio" || this.type === "checkbox";
};


/**
 * Returns true if this object require a wrapping div
 */
Client.BSInput.prototype.needWrapper = function ()
{
  return  (this.insideLabel() && !this.inline) ||
          (this.state && !this.group && !(Client.BSGroup && this.parent instanceof Client.BSGroup));
};


/**
 * Returns true if this object require a sizing div
 */
Client.BSInput.prototype.needSize = function ()
{
  return this.lg || this.md || this.sm || this.xs;
};


/**
 * Returns true if this object is in an horizontal form
 */
Client.BSInput.prototype.hForm = function ()
{
  var p = this.parent;
  while (p) {
    if (p instanceof Client.BSForm)
      return p.horizontal;
    p = p.parent;
  }
};


/**
 * Create/update the DOM structure for this input
 */
Client.BSInput.prototype.createDom = function ()
{
  // Remove previous objects
  if (this.helpObj)
    this.helpObj.parentNode.removeChild(this.helpObj);
  if (this.labelObj && this.labelObj !== this.rootObj)
    this.labelObj.parentNode.removeChild(this.labelObj);
  if (this.feedObj)
    this.feedObj.parentNode.removeChild(this.feedObj);
  this.groupObj = undefined;
  this.labelObj = undefined;
  this.labelText = undefined;
  this.sizeObj = undefined;
  this.wrapperObj = undefined;
  this.helpObj = undefined;
  this.feedObj = undefined;
  //
  // Creating objects
  var newRoot;
  var installLabel;
  var installHelpText;
  var installRoot;
  var inputGroup = Client.BSGroup && this.parent instanceof Client.BSGroup && this.parent.type === "input";
  //
  // Group
  if (this.group) {
    this.groupObj = document.createElement("div");
    newRoot = this.groupObj;
  }
  //
  // Label
  if (this.label && !this.insideLabel()) {
    this.labelObj = document.createElement("label");
    this.labelObj.setAttribute("for", this.id);
    this.labelText = this.labelObj;
    if (newRoot && !inputGroup)
      newRoot.appendChild(this.labelObj);
    else
      installLabel = true;
  }
  //
  // Size object
  if (this.needSize()) {
    this.sizeObj = document.createElement("div");
    if (newRoot)
      newRoot.appendChild(this.sizeObj);
    else
      newRoot = this.sizeObj;
  }
  //
  var inputTg = this.sizeObj || newRoot;
  //
  // Special div
  if (this.needWrapper()) {
    this.wrapperObj = document.createElement("div");
    this.wrapperObj.className = this.type;
    if (inputTg)
      inputTg.appendChild(this.wrapperObj);
    else
      newRoot = this.wrapperObj;
  }
  //
  inputTg = this.wrapperObj || inputTg;
  //
  // Creating the inside label if needed
  if (this.insideLabel()) {
    // Create label now
    this.labelObj = document.createElement("label");
    if (inputTg)
      inputTg.appendChild(this.labelObj);
    else
      newRoot = this.labelObj;
    this.labelText = document.createTextNode("");
    this.labelObj.appendChild(this.labelText);
  }
  //
  // Installing root object now, before moving the domObj
  if (newRoot && this.rootObj !== newRoot) {
    var target = this.rootObj || this.domObj;
    target.parentNode.replaceChild(newRoot, target);
  }
  else {
    installRoot = this.rootObj !== undefined;
  }
  //
  // Recycle input
  if (this.insideLabel()) {
    this.labelObj.insertBefore(this.domObj, this.labelText);
  }
  else {
    if (inputTg && inputTg !== this.domObj)
      inputTg.appendChild(this.domObj);
    else
      newRoot = this.domObj;
  }
  //
  // Help text
  if (this.helpText) {
    this.helpObj = document.createElement("p");
    this.helpObj.className = "help-block";
    this.helpObj.textContent = this.helpText;
    if (newRoot && newRoot !== this.domObj && !inputGroup)
      newRoot.appendChild(this.helpObj);
    else
      installHelpText = true;
  }
  //
  if (this.labelText)
    this.labelText.textContent = this.label;
  //
  // Installing objects
  if (installRoot)
    this.rootObj.parentNode.replaceChild(newRoot, this.rootObj);
  if (installLabel) {
    if (inputGroup)
      this.parent.domObj.parentNode.insertBefore(this.labelObj, this.parent.domObj);
    else
      newRoot.parentNode.insertBefore(this.labelObj, newRoot);
  }
  if (installHelpText) {
    if (inputGroup)
      this.parent.domObj.parentNode.insertBefore(this.helpObj, this.parent.domObj.nextSibling);
    else
      newRoot.parentNode.insertBefore(this.helpObj, newRoot.nextSibling);
  }
  //
  // feedback (always right after input)
  if (this.feedback) {
    this.feedObj = document.createElement("span");
    this.domObj.parentNode.insertBefore(this.feedObj, this.domObj.nextSibling);
  }
  //
  if (this.insideLabel() && !this.customSpan) {
    this.customSpan = document.createElement("span");
    this.customSpan.className = "custom-" + this.type;
    this.domObj.parentNode.insertBefore(this.customSpan, this.domObj.nextSibling);
  }
  //
  this.rootObj = newRoot;
};


/**
 * Create/update the DOM structure for this input
 */
Client.BSInput.prototype.updateClasses = function ()
{
  // Input object
  var cs = this.insideLabel() ? "" : "form-control";
  if (this.type === "static")
    cs += "-static";
  //
  if (this.height === "sm")
    cs += " input-sm";
  //
  if (this.height === "lg")
    cs += " input-lg";
  //
  if (this.className)
    cs += " " + this.className;
  //
  this.domObj.className = cs;
  //
  // Label obj
  if (this.labelObj) {
    cs = "";
    //
    if (this.hForm() && !this.insideLabel()) {
      if (this.lg)
        cs = " col-lg-" + (12 - this.lg);
      if (this.md)
        cs = " col-md-" + (12 - this.md);
      if (this.sm)
        cs = " col-sm-" + (12 - this.sm);
      if (this.xs)
        cs = " col-xs-" + (12 - this.xs);
    }
    //
    if (this.insideLabel()) {
      if (this.inline)
        cs += " " + this.type + "-inline";
    }
    else
      cs += " control-label";
    //
    this.labelObj.className = cs;
  }
  //
  // Sizing obj
  if (this.sizeObj) {
    var cs = "";
    //
    if (this.lg)
      cs = " col-lg-" + this.lg;
    if (this.md)
      cs = " col-md-" + this.md;
    if (this.sm)
      cs = " col-sm-" + this.sm;
    if (this.xs)
      cs = " col-xs-" + this.xs;
    //
    if (this.hForm() && this.insideLabel()) {
      if (this.lg)
        cs += " col-lg-offset-" + (12 - this.lg);
      if (this.md)
        cs += " col-md-offset-" + (12 - this.md);
      if (this.sm)
        cs += " col-sm-offset-" + (12 - this.sm);
      if (this.xs)
        cs += " col-xs-offset-" + (12 - this.xs);
    }
    //
    this.sizeObj.className = cs;
  }
  //
  // wrapper
  if (this.wrapperObj) {
    if (this.disabled)
      this.wrapperObj.classList.add("disabled");
    else
      this.wrapperObj.classList.remove("disabled");
    //
    // Add state management without group
    if (!this.group && !(Client.BSGroup && this.parent instanceof Client.BSGroup)) {
      var cl = this.wrapperObj.classList;
      cl.remove("has-success");
      cl.remove("has-warning");
      cl.remove("has-error");
      cl.remove("has-feedback");
      if (this.state)
        cl.add("has-" + this.state);
      if (this.feedback)
        cl.add("has-feedback");
    }
  }
  //
  // group
  if (this.groupObj) {
    var cs = "form-group";
    //
    if (this.hForm() && !this.insideLabel()) {
      if (this.height === "sm")
        cs += " form-group-sm";
      if (this.height === "lg")
        cs += " form-group-lg";
    }
    //
    if (this.state)
      cs += " has-" + this.state;
    if (this.feedback)
      cs += " has-feedback";
    //
    this.groupObj.className = cs;
  }
  else {
    if (Client.BSGroup && this.parent instanceof Client.BSGroup) {
      //
      var cl = this.parent.domObj.parentNode.classList;
      cl.remove("has-success");
      cl.remove("has-warning");
      cl.remove("has-error");
      cl.remove("has-feedback");
      if (this.state)
        cl.add("has-" + this.state);
      if (this.feedback)
        cl.add("has-feedback");
    }
  }
  //
  // feedback
  if (this.feedObj) {
    var cs = "glyphicon glyphicon-";
    //
    if (this.state) {
      if (this.state === "success")
        cs += "ok";
      if (this.state === "warning")
        cs += "warning";
      if (this.state === "error")
        cs += "remove";
      cs += "-sign form-control-feedback";
    }
    else {
      cs = "sr-only";
    }
    //
    this.feedObj.className = cs;
  }
};


/**
 * Remove the element and its children from the element map
 * @param {boolean} firstLevel - if true remove the dom of the element too
 * @param {boolean} triggerAnimation - if true and on firstLevel trigger the animation of 'removing'
 */
Client.BSInput.prototype.close = function (firstLevel, triggerAnimation)
{
  if (this.helpObj)
    this.helpObj.parentNode.removeChild(this.helpObj);
  delete this.helpObj;
  if (this.labelObj && this.labelObj !== this.rootObj)
    this.labelObj.parentNode.removeChild(this.labelObj);
  delete this.labelObj;
  if (this.feedObj)
    this.feedObj.parentNode.removeChild(this.feedObj);
  delete this.feedObj;
  if (this.rootObj && this.rootObj !== this.domObj)
    this.rootObj.parentNode.removeChild(this.rootObj);
  delete this.rootObj;
  //
  Client.Input.prototype.close.call(this, firstLevel, triggerAnimation);
};


/**
 * Set custom error message
 * @param {string} message - the error message
 * @param {bool} srv
 */
Client.BSInput.prototype.setError = function (message, srv)
{
  if (message) {
    if (this.prevState !== "error") {
      this.prevState = this.state;
      this.prevHelpText = this.helpText;
    }
    this.updateElement({state: "error", helpText: message});
  }
  else {
    this.updateElement({state: this.prevState || "", helpText: this.prevHelpText || ""});
  }
  //
  if (message.length === 0 && this.errorPopup)
    this.errorPopup.close();
  //
  this.errorMessage = message;
  //
  if (srv) {
    if (message)
      this.srvError = {message: message, value: this.domObj.value};
    else
      delete this.srvError;
  }
  //
  this.domObj.setCustomValidity(message);
};
