/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client */

/**
 * @class A container for the entire page
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSContainer = function (element, parent, view)
{
  Client.Container.call(this, element, parent, view);
};

// Make Client.MDCard extend Client.Container
Client.BSContainer.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSContainer.prototype.updateElement = function (el)
{
  // Force update for the first time
  var update = this.domObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  // fluid container
  if (el.type !== undefined) {
    this.type = el.type;
    update = true;
    delete el.type;
  }
  //
  // specific class name for this layout
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  //
  if (update) {
    //
    var cs = this.type ? this.type : "container";
    //
    if (this.className)
      cs += " " + this.className;
    //
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};


/**
 * @class A container for the cell content
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSRow = function (element, parent, view)
{
  Client.Container.call(this, element, parent, view);
};

// Make Client.MDCard extend Client.Container
Client.BSRow.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSRow.prototype.updateElement = function (el)
{
  // Force update for the first time
  var update = this.domObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  // specific class name for this layout
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  //
  if (update) {
    //
    var cs = "row";
    //
    if (this.className)
      cs += " " + this.className;
    //
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};


/**
 * @class A container for a piece of content
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSCell = function (element, parent, view)
{
  Client.Container.call(this, element, parent, view);
};

// Make Client.MDCard extend Client.Container
Client.BSCell.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSCell.prototype.updateElement = function (el)
{
  // Force update for the first time
  var update = this.domObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  // fluid container
  if (el.xs !== undefined) {
    this.xs = el.xs;
    update = true;
    delete el.xs;
  }
  if (el.sm !== undefined) {
    this.sm = el.sm;
    update = true;
    delete el.sm;
  }
  if (el.md !== undefined) {
    this.md = el.md;
    update = true;
    delete el.md;
  }
  if (el.lg !== undefined) {
    this.lg = el.lg;
    update = true;
    delete el.lg;
  }
  if (el.offsetXs !== undefined) {
    this.offsetXs = el.offsetXs;
    update = true;
    delete el.offsetXs;
  }
  if (el.offsetSm !== undefined) {
    this.offsetSm = el.offsetSm;
    update = true;
    delete el.offsetSm;
  }
  if (el.offsetMd !== undefined) {
    this.offsetMd = el.offsetMd;
    update = true;
    delete el.offsetMd;
  }
  if (el.offsetLg !== undefined) {
    this.offsetLg = el.offsetLg;
    update = true;
    delete el.offsetLg;
  }
  //
  // specific class name for this layout
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  //
  if (update) {
    //
    var cs = "";
    if (this.lg)
      cs += " col-lg-" + this.lg;
    if (this.md)
      cs += " col-md-" + this.md;
    if (this.sm)
      cs += " col-sm-" + this.sm;
    if (this.xs)
      cs += " col-xs-" + this.xs;
    if (!cs)
      cs = "col-md-4";
    //
    if (this.offsetXs)
      cs += " col-xs-offset-" + this.offsetXs;
    if (this.offsetSm)
      cs += " col-sm-offset-" + this.offsetSm;
    if (this.offsetMd)
      cs += " col-md-offset-" + this.offsetMd;
    if (this.offsetLg)
      cs += " col-lg-offset-" + this.offsetLg;
    //
    if (this.className)
      cs += " " + this.className;
    //
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};
