/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client */

/**
 * @class A bootstrap group element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSGroup = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  // Creating dropdown container
  this.domObj = document.createElement("div");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  //
  if (Client.mainFrame.isEditing())
    this.domObj.id = btoa(this.id);
  //
  parent.appendChildObject(this, this.getRootObject());
  //
  // Create children after attaching element as external components, rely on dom structure
  this.createChildren(element);
};

// Make BSGroup extend Client.Element
Client.BSGroup.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSGroup.prototype.updateElement = function (el)
{
  // Force update for the first time
  var update = this.domObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  if (el.type !== undefined) {
    this.type = el.type;
    update = true;
    delete el.type;
  }
  //
  if (el.vertical !== undefined) {
    this.vertical = el.vertical;
    update = true;
    delete el.vertical;
  }
  //
  if (el.height !== undefined) {
    this.height = el.height;
    update = true;
    delete el.height;
  }
  //
  if (el.justified !== undefined) {
    this.justified = el.justified;
    update = true;
    delete el.justified;
  }
  //
  // specific class name for this layout
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  //
  if (update) {
    //
    var cs = "btn-group";
    if (this.type === "tb")
      cs = "btn-toolbar";
    if (this.type === "input") {
      cs = "input-group";
      if (!this.rootObj) {
        this.rootObj = document.createElement("div");
        if (this.domObj.parentNode)
          this.domObj.parentNode.replaceChild(this.rootObj, this.domObj);
        this.rootObj.appendChild(this.domObj);
      }
    }
    if (this.type === "list")
      cs = "list-group";
    if (this.type === "medialist")
      cs = "media-list";
    if (this.type === "mediaitem")
      cs = "media";
    if (this.type === "mediabody")
      cs = "media-body";
    if (this.type === "panel")
      cs = "panel-group";
    if (this.type === "modalheader")
      cs = "modal-header";
    if (this.type === "modalfooter")
      cs = "modal-footer";
    if (this.type === "modalbody")
      cs = "modal-body";
    //
    if (this.type === "btn-check" || this.type === "btn-radio")
      this.domObj.setAttribute("data-toggle", "buttons");
    else
      this.domObj.removeAttribute("data-toggle");
    //
    if (this.height)
      cs += " " + cs + "-" + this.height;
    //
    if (this.vertical)
      cs += "-vertical";
    //
    if (this.justified)
      cs += " btn-group-justified";
    //
    if (this.className)
      cs += " " + this.className;
    //
    this.domObj.className = cs;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};


/**
 * Return the dom root object of this element
 * @returns {DomElement}
 */
Client.BSGroup.prototype.getRootObject = function ()
{
  return this.rootObj ? this.rootObj : this.domObj;
};


/**
 * @class A bootstrap addon element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSAddon = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  // Creating dropdown container
  this.domObj = document.createElement("span");
  //
  // Search for buttons / dropdowns
  if (element.children) {
    for (var i = 0; i < element.children.length; i++) {
      var c = element.children[i].class || element.children[i].c;
      if (c === "BSButton" || c === "BSDropDown" || c === "BSGroup")
        this.hasButton = true;
      if (c === "BSDDMenu")
        this.hasDropDown = true;
    }
  }
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  //
  parent.appendChildObject(this, this.domObj);
  //
  // Create children after attaching element as external components, rely on dom structure
  this.createChildren(element);
};

// Make BSGroup extend Client.Element
Client.BSAddon.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSAddon.prototype.updateElement = function (el)
{
  // Force update for the first time
  var update = this.domObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  if (el.type !== undefined) {
    this.type = el.type;
    update = true;
    delete el.type;
  }
  //
  if (update) {
    //
    var cs = this.hasButton ? "input-group-btn" : "input-group-addon";
    //
    if (this.className)
      cs += " " + this.className;
    //
    this.domObj.className = cs;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};



/**
 * @class A bootstrap list item
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSListItem = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  // Creating dropdown menu container
  this.domObj = document.createElement("a");
  this.domObj.href = "javascript:void(0)";
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  //
  parent.appendChildObject(this, this.domObj);
  //
  // Create children after attaching element as external components, rely on dom structure
  this.createChildren(element);
};

// Make BSListItem extend Client.Element
Client.BSListItem.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSListItem.prototype.updateElement = function (el)
{
  // Force update for the first time
  var update = this.domObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  // specific class name for this layout
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  if (el.active !== undefined) {
    this.active = el.active;
    update = true;
    delete el.active;
  }
  if (el.disabled !== undefined) {
    this.disabled = el.disabled;
    update = true;
    delete el.disabled;
  }
  if (el.itemStyle !== undefined) {
    this.itemStyle = el.itemStyle;
    update = true;
    delete el.itemStyle;
  }
  //
  if (update) {
    //
    var cs = "list-group-item";
    //
    if (this.disabled)
      cs += " disabled";
    if (this.active)
      cs += " active";
    if (this.itemStyle)
      cs += " list-group-item-" + this.itemStyle;
    //
    if (this.className)
      cs += " " + this.className;
    //
    this.domObj.className = cs;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};



/**
 * @class A bootstrap media object
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSMediaObject = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  // Creating dropdown menu container
  this.domObj = document.createElement("div");
  this.linkObj = document.createElement("a");
  this.linkObj.href = "javascript:void(0)";
  this.imgObj = document.createElement("img");
  this.imgObj.className = "media-object";
  this.domObj.appendChild(this.linkObj);
  this.linkObj.appendChild(this.imgObj);
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  //
  parent.appendChildObject(this, this.domObj);
};

// Make BSMediaObject extend Client.Element
Client.BSMediaObject.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSMediaObject.prototype.updateElement = function (el)
{
  // Force update for the first time
  var update = this.domObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  // specific class name for this layout
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  if (el.halign !== undefined) {
    this.halign = el.halign;
    update = true;
    delete el.halign;
  }
  if (el.valign !== undefined) {
    this.valign = el.valign;
    update = true;
    delete el.valign;
  }
  if (el.src !== undefined) {
    this.imgObj.src = el.src;
    delete el.src;
  }
  //
  if (update) {
    //
    var cs = "media-" + (this.halign ? this.halign : "left");
    //
    if (this.valign)
      cs += " media-" + this.valign;
    //
    if (this.className)
      cs += " " + this.className;
    //
    this.domObj.className = cs;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};


/**
 * @class A bootstrap modal element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSModal = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("div");
  //
  this.rootObj = document.createElement("div");
  this.rootObj.className = "modal fade";
  this.dialogObj = document.createElement("div");
  this.dialogObj.className = "modal-dialog";
  this.rootObj.appendChild(this.dialogObj);
  this.domObj = document.createElement("div");
  this.dialogObj.appendChild(this.domObj);
  //
  if (Client.mainFrame.isEditing()) {
    this.rootObj.id = btoa(this.id);
    this.rootObj.style.display = "block";
    this.rootObj.classList.remove("fade");
  }
  else
    this.rootObj.id = this.id;
  //
  this.updateElement(element);
  //
  this.attachEvents(element.events);
  //
  parent.appendChildObject(this, this.rootObj);
  //
  // Create children after attaching element as external components, rely on dom structure
  this.createChildren(element);
  //
  this.updateCloseBtn();
  //
  var pthis = this;
  setTimeout(function () {
    $("#" + pthis.rootObj.id).on('hidden.bs.modal', function (ev) {
      //
      var e = [({obj: pthis.parent.id, id: "close", content: {}})];
      Client.mainFrame.sendEvents(e);
    });
  }, 10);
};

// Make BSModal extend Client.Element
Client.BSModal.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSModal.prototype.updateElement = function (el)
{
  var update = this.rootObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  // specific class name for this text
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  if (el.width !== undefined) {
    this.dialogObj.className = "modal-dialog modal-" + el.width;
    delete el.width;
  }
  if (el.closeButton !== undefined) {
    this.closeButton = el.closeButton;
    this.updateCloseBtn();
    delete el.closeButton;
  }
  //
  if (el.visible && !Client.mainFrame.isEditing()) {
    var pthis = this;
    setTimeout(function () {
      $("#" + pthis.rootObj.id).appendTo("body").modal("show");
    }, 10);
  }
  //
  if (update) {
    //
    var cs = "modal-content";
    //
    if (this.className)
      cs += " " + this.className;
    //
    this.domObj.className = cs;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};


/**
 * Update element properties
 */
Client.BSModal.prototype.updateCloseBtn = function ()
{
  if (this.closeButton && !this.closeObj) {
    for (var i = 0; i < this.ne(); i++) {
      var h = this.elements[i];
      if (h instanceof Client.BSGroup && h.type === "modalheader") {
        this.closeObj = document.createElement("button");
        this.closeObj.type = "button";
        this.closeObj.className = "close";
        this.closeObj.innerHTML = "<span aria-hidden=\"true\">&times;</span>";
        this.closeObj.setAttribute("data-dismiss", "modal");
        h.domObj.insertBefore(this.closeObj, h.domObj.firstChild);
      }
    }
  }
  if (!this.closeButton && this.closeObj) {
    this.closeObj.parentNode.removeChild(this.closeObj);
    delete this.closeObj;
  }
};


/**
 * Remove the element and its children from the element map
 * @param {boolean} firstLevel - if true remove the dom of the element too
 * @param {boolean} triggerAnimation - if true and on firstLevel trigger the animation of 'removing'
 */
Client.BSModal.prototype.close = function (firstLevel, triggerAnimation)
{
  var pthis = this;
  $("#" + pthis.rootObj.id).modal("hide");
  setTimeout(function () {
    Client.Element.prototype.close.call(pthis, firstLevel, triggerAnimation);
  }, 200);
};


/**
 * Return the dom root object of this element
 * @returns {DomElement}
 */
Client.BSModal.prototype.getRootObject = function ()
{
  return this.domObj.parentNode.parentNode;
};
