/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client */

/**
 * @class Boostrap Helper class
 */
Client.BSHelper = {};


/**
 * Recognize common BS properties
 * @param {Object} el - properties to update
 * @param {Object} bsobj - bs object to use
 */
Client.BSHelper.updateElement = function (el, bsobj)
{
  var b = false;
  //
  if (el.tooltip !== undefined && !Client.mainFrame.isEditing()) {
    //
    b = false;
    if (typeof el.tooltip === "object") {
      for (var j in el.tooltip) {
        bsobj.domObj.setAttribute("data-" + j, el.tooltip[j]);
        if (j === "title")
          bsobj.domObj.setAttribute("data-original-" + j, el.tooltip[j]);
        b = true;
      }
      delete bsobj.domObj.title;
    }
    else {
      bsobj.domObj.title = el.tooltip;
      bsobj.domObj.setAttribute("data-original-title", el.tooltip);
      if (el.tooltip)
        b = true;
    }
    //
    if (b) {
      setTimeout(function () {
        $('#' + bsobj.domObj.id).tooltip();
      }, 10);
    }
    //
    delete el.tooltip;
  }
  if (el.popover !== undefined && !Client.mainFrame.isEditing()) {
    //
    b = false;
    if (typeof el.popover === "object") {
      for (var j in el.popover) {
        bsobj.domObj.setAttribute("data-" + j, el.popover[j]);
        if (j === "title")
          bsobj.domObj.setAttribute("data-original-" + j, el.popover[j]);
        b = true;
      }
    }
    else {
      bsobj.domObj.setAttribute("data-content", el.popover);
      if (el.popover)
        b = true;
    }
    //
    if (b) {
      setTimeout(function () {
        $('#' + bsobj.domObj.id).popover();
      }, 10);
    }
    //
    delete el.popover;
  }
};


/**
 * Display a confirmation box
 * @param {String} opt - contains the text to display in the confirm box and the callback id
 */
Client.MainFrame.prototype.confirm = function (opt)
{
  var t1 = opt.txt.split("|");
  var text = t1[t1.length - 1];
  var title;
  var size;
  if (t1.length > 1)
    title = t1[0];
  if (t1.length > 2)
    size = t1[1];
  //
  bootbox.confirm({
    message: text,
    title: title,
    size: size,
    backdrop: true,
    callback: function (r) {
      var e = [{id: "popupBoxReturn", content: {res: r, cbId: opt.cbId}}];
      this.sendEvents(e);
    }.bind(this)
  });
};


/**
 * Display a prompt box
 * @param {Object} opt - contains the text to display in the prompt box and the default input text
 */
Client.MainFrame.prototype.prompt = function (opt)
{
  bootbox.prompt({
    title: opt.txt,
    value: opt.def,
    backdrop: true,
    callback: function (r) {
      var e = [{id: "popupBoxReturn", content: {res: r, cbId: opt.cbId}}];
      this.sendEvents(e);
    }.bind(this)
  });
};


/**
 * Execute alert command
 * @param {String} txt
 */
Client.MainFrame.prototype.alert = function (txt)
{
  var t1 = txt.split("|");
  var text = t1[t1.length - 1];
  var title;
  var size;
  if (t1.length > 1)
    title = t1[0];
  if (t1.length > 2)
    size = t1[1];
  //
  bootbox.alert({
    message: text,
    title: title,
    size: size,
    backdrop: true
  });
};


/**
 * Close a popup
 */
Client.MainFrame.prototype.closePopup = function ()
{
  var r = document.querySelector("button[data-bb-handler=cancel]");
  if (!r)
    r = document.querySelector("button[data-bb-handler=ok]");
  if (r)
    r.click();
};


/**
 * Open a standard popup
 * @param {object} cnt
 */
Client.MainFrame.prototype.popup = function (cnt)
{
  var o = {};
  o.message = cnt.message;
  o.title = cnt.title;
  o.onEscape = cnt.onEscape;
  o.backdrop = cnt.backdrop;
  o.closeButton = cnt.closeButton;
  o.animate = cnt.animate;
  o.className = cnt.className;
  o.size = cnt.size;
  o.buttons = cnt.buttons;
  o.callback = function (r) {
    var e = [{id: "popupBoxReturn", content: {res: r, cbId: cnt.cbId}}];
    this.sendEvents(e);
  }.bind(this);
  //
  bootbox[cnt.type || "alert"](o);
};
