/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client */

/**
 * @class A bootstrap text element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSText = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  // Creating text container
  this.domObj = document.createElement(element.tag || "span");
  this.tag = element.tag;
  delete element.tag;
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
};

// Make BSText extend Client.Element
Client.BSText.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSText.prototype.updateElement = function (el)
{
  var update = this.domObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  // specific class name for this text
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  //
  // text style
  if (el.txtStyle !== undefined) {
    this.txtStyle = el.txtStyle;
    update = true;
    delete el.txtStyle;
  }
  //
  if (el.tag && el.tag !== this.tag) {
    // recreating dom obj
    var nd = document.createElement(el.tag);
    for (var v in this.domObj) {
      try {
        nd[v] = this.domObj[v];
      }
      catch (ex) {
      }
    }
    this.domObj.parentNode.replaceChild(nd, this.domObj);
    this.domObj = nd;
    //
    this.tag = el.tag;
    delete el.tag;
  }
  //
  if (update) {
    //
    var cs = "";
    //
    if (Client.BSNavSection && this.parent instanceof Client.BSNavSection)
      cs += " navbar-text";
    if (Client.BSListItem && this.parent instanceof Client.BSListItem) {
      if (this.tag && this.tag.startsWith("h"))
        cs += " list-group-item-heading";
      else
        cs += " list-group-item-text";
    }
    if (Client.BSGroup && this.parent instanceof Client.BSGroup && this.parent.type === "mediabody") {
      if (this.tag.startsWith("h"))
        cs += " media-heading";
    }
    //
    if (this.txtStyle)
      cs += " " + this.txtStyle;
    //
    if (this.className)
      cs += " " + this.className;
    //
    this.domObj.className = cs;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};


/**
 * @class A bootstrap button element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSButton = function (element, parent, view)
{
  if (!element.type)
    element.type = "button";
  if (parent.justified)
    element.tag = "a";
  if (parent.type === "btn-check" || parent.type === "btn-radio")
    element.tag = "label";
  //
  Client.Button.call(this, element, parent, view);
  //
  if (parent.type === "btn-check") {
    this.inputObj = document.createElement("input");
    this.inputObj.type = "checkbox";
    this.inputObj.autocomplete = "off";
    this.domObj.appendChild(this.inputObj);
  }
  if (parent.type === "btn-radio") {
    this.inputObj = document.createElement("input");
    this.inputObj.type = "radio";
    this.inputObj.name = this.parent.name;
    this.inputObj.autocomplete = "off";
    this.domObj.appendChild(this.inputObj);
  }
};

// Make Client.MDCard extend Client.Container
Client.BSButton.prototype = new Client.Button();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSButton.prototype.updateElement = function (el)
{
  // Force update for the first time
  var update = this.domObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  if (el.btnstyle !== undefined) {
    this.btnstyle = el.btnstyle;
    update = true;
    delete el.btnstyle;
  }
  if (el.height !== undefined) {
    this.height = el.height;
    update = true;
    delete el.height;
  }
  if (el.block !== undefined) {
    this.block = el.block;
    update = true;
    delete el.block;
  }
  if (el.active !== undefined) {
    this.active = el.active;
    update = true;
    delete el.active;
  }
  if (el.split !== undefined) {
    this.split = el.split;
    update = true;
    delete el.split;
  }
  if (el.toggleable !== undefined) {
    this.toggleable = el.toggleable;
    update = true;
    delete el.toggleable;
  }
  //
  // specific class name for this layout
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  //
  if (update) {
    //
    var cs = "btn btn-" + (this.btnstyle ? this.btnstyle : "default");
    if (Client.BSNavSection && this.parent instanceof Client.BSNavSection)
      cs += " navbar-btn";
    if (this.height)
      cs += " btn-" + this.height;
    if (this.block)
      cs += " btn-block";
    if (this.active)
      cs += " active";
    //
    if ((Client.BSDropDown && this.parent instanceof Client.BSDropDown) ||
            (Client.BSAddon && this.parent instanceof Client.BSAddon && this.parent.hasDropDown)) {
      //
      if (!this.caretObj) {
        this.caretObj = document.createElement("span");
        this.caretObj.className = "caret";
      }
      //
      if (this.split) {
        this.domObj.removeAttribute("data-toggle");
        var pthis = this;
        if (!this.splitObj) {
          setTimeout(function () {
            pthis.splitObj = document.createElement("button");
            pthis.splitObj.className = pthis.domObj.className + " dropdown-toggle";
            pthis.splitObj.style.cssText = pthis.domObj.style.cssText;
            pthis.domObj.parentNode.insertBefore(pthis.splitObj, pthis.domObj.nextSibling);
            pthis.splitObj.appendChild(pthis.caretObj);
            pthis.splitObj.setAttribute("data-toggle", "dropdown");
          }, 0);
        }
      }
      else {
        // NO SPLIT
        cs += " dropdown-toggle";
        this.domObj.setAttribute("data-toggle", "dropdown");
        this.domObj.appendChild(this.caretObj);
        if (this.splitObj) {
          this.splitObj.parentNode.removeChild(this.splitObj);
          this.splitObj = undefined;
        }
      }
    }
    else {
      if (this.caretObj) {
        this.caretObj.parentNode.removeChild(this.caretObj);
        this.caretObj = undefined;
      }
      if (this.splitObj) {
        this.splitObj.parentNode.removeChild(this.splitObj);
        this.splitObj = undefined;
      }
      if (this.toggleable)
        this.domObj.setAttribute("data-toggle", "button");
      else
        this.domObj.removeAttribute("data-toggle");
    }
    //
    if (this.className)
      cs += " " + this.className;
    //
    this.domObj.className = cs;
  }
  //
  Client.Button.prototype.updateElement.call(this, el);
};


/**
 * @class A bootstrap icon element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSIcon = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("span");
  this.domObj.setAttribute("aria-hidden", true);
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
};

Client.BSIcon.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSIcon.prototype.updateElement = function (el)
{
  // Force update for the first time
  var update = this.domObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  if (el.icon !== undefined) {
    this.icon = el.icon;
    update = true;
    delete el.icon;
  }
  //
  // specific class name for this layout
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  //
  if (update) {
    //
    var cs = "glyphicon glyphicon-" + this.icon;
    //
    if (this.className)
      cs += " " + this.className;
    //
    this.domObj.className = cs;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};
