/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client */

/**
 * @class A bootstrap alert element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSAlert = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  // Creating dropdown menu container
  this.domObj = document.createElement("div");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  //
  parent.appendChildObject(this, this.domObj);
  //
  // Create children after attaching element as external components, rely on dom structure
  this.createChildren(element);
};

// Make BSAlert extend Client.Element
Client.BSAlert.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSAlert.prototype.updateElement = function (el)
{
  // Force update for the first time
  var update = this.domObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  // specific class name for this layout
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  //
  if (el.type !== undefined) {
    this.type = el.type;
    update = true;
    delete el.type;
  }
  //
  if (el.dismissible !== undefined) {
    this.dismissible = el.dismissible;
    update = true;
    delete el.dismissible;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
  //
  if (update) {
    //
    var cs = "alert alert-" + (this.type ? this.type : "info");
    if (this.dismissible) {
      cs += " alert-dismissible";
      if (!this.btnObj) {
        this.btnObj = document.createElement("button");
        this.domObj.appendChild(this.btnObj);
        this.btnObj.innerHTML = "<span aria-hidden=true>&times;</span>";
        this.btnObj.className = "close";
        this.btnObj.setAttribute("data-dismiss", "alert");
      }
    }
    else if (this.btnObj) {
      this.domObj.removeChild(this.btnObj);
      this.btnObj = undefined;
    }
    //
    if (this.className)
      cs += " " + this.className;
    //
    this.domObj.className = cs;
  }
};


/**
 * @class A bootstrap progress element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSProgress = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  // Creating dropdown menu container
  this.domObj = document.createElement("div");
  this.valueObj = document.createElement("div");
  this.labelObj = document.createElement("span");
  this.domObj.appendChild(this.valueObj);
  this.valueObj.appendChild(this.labelObj);
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  //
  parent.appendChildObject(this, this.domObj);
};

// Make BSAlert extend Client.Element
Client.BSProgress.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSProgress.prototype.updateElement = function (el)
{
  // Force update for the first time
  var update = this.domObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  // specific class name for this layout
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  //
  if (el.type !== undefined) {
    this.type = el.type;
    update = true;
    delete el.type;
  }
  //
  if (el.value !== undefined) {
    this.value = el.value;
    this.valueObj.style.width = this.value + "%";
    this.labelObj.textContent = this.value + "%";
    delete el.value;
  }
  //
  if (el.hasLabel !== undefined) {
    this.hasLabel = el.hasLabel;
    update = true;
    delete el.hasLabel;
  }
  //
  if (el.striped !== undefined) {
    this.striped = el.striped;
    update = true;
    delete el.striped;
  }
  //
  if (el.active !== undefined) {
    this.active = el.active;
    update = true;
    delete el.active;
  }
  //
  if (update) {
    //
    this.domObj.className = "progress " + (this.className ? this.className : "");
    //
    var cs = "progress-bar";
    if (this.type)
      cs += " progress-bar-" + this.type;
    if (this.striped)
      cs += " progress-bar-striped";
    if (this.active)
      cs += " active";
    this.valueObj.className = cs;
    //
    this.labelObj.className = this.hasLabel ? "" : "sr-only";
    if (this.hasLabel)
      this.valueObj.minWidth = "2em";
    else
      this.valueObj.minWidth = "";
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};


/**
 * @class A bootstrap panel element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSPanel = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  // Creating dropdown menu container
  this.domObj = document.createElement("div");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  //
  parent.appendChildObject(this, this.domObj);
  //
  // Create children after attaching element as external components, rely on dom structure
  this.createChildren(element);
  //
  if (this.collapsible)
    this.updateCollapsible();
};

// Make BSPanel extend Client.Element
Client.BSPanel.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSPanel.prototype.updateElement = function (el)
{
  // Force update for the first time
  var update = this.domObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  if (el.collapsed !== undefined) {
    this.collapsed = el.collapsed;
    delete el.collapsed;
    if (!update)
      this.updateCollapsed();
  }
  //
  // specific class name for this layout
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  //
  if (el.type !== undefined) {
    this.type = el.type;
    update = true;
    delete el.type;
  }
  //
  if (el.collapsible !== undefined) {
    this.collapsible = el.collapsible;
    delete el.collapsible;
    this.updateCollapsible();
  }
  //
  if (update) {
    var cs = "panel panel-" + (this.type ? this.type : "default");
    this.domObj.className = cs;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};

/**
 * Update element properties
 */
Client.BSPanel.prototype.updateCollapsible = function ()
{
  var ph, pb;
  for (var i = 0; i < this.ne(); i++) {
    var ps = this.elements[i];
    if (ps instanceof Client.BSPanelSection) {
      if (!ph && (ps.type === "heading" || ps.type === "footer"))
        ph = ps;
      if (!pb && ps.type === "body")
        pb = ps;
    }
  }
  //
  if (ph && pb) {
    if (this.collapsible) {
      ph.domObj.setAttribute("data-toggle", "collapse");
      if (this.collapsed)
        ph.domObj.classList.add("collapsed");
      //
      if (!this.pcObj) {
        this.pcObj = document.createElement("div");
        this.pcObj.className = "panel-collapse collapse " + (this.collapsed ? "" : "in");
        if (Client.mainFrame.isEditing())
          this.pcObj.id = btoa(this.id);
        else
          this.pcObj.id = "pc_" + this.id;
        pb.domObj.parentNode.replaceChild(this.pcObj, pb.domObj);
        this.pcObj.appendChild(pb.domObj);
      }
      //
      ph.domObj.setAttribute("data-target", "#" + this.pcObj.id);
      //
      var p = this.parent;
      while (p) {
        if (Client.BSGroup && p instanceof Client.BSGroup && p.type === "panel") {
          ph.domObj.setAttribute("data-parent", "#" + p.domObj.id);
          break;
        }
        p = p.parent;
      }
    }
    else {
      ph.domObj.removeAttribute("data-toggle");
      ph.domObj.removeAttribute("data-target");
      ph.domObj.removeAttribute("data-parent");
    }
  }
};

/**
 * Update element properties
 */
Client.BSPanel.prototype.updateCollapsed = function ()
{
  if (this.pcObj) {
    if (this.collapsed)
      $("#" + this.pcObj.id).collapse("hide");
    else
      $("#" + this.pcObj.id).collapse("show");
  }
};



/**
 * @class A bootstrap panel element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSPanelSection = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  // Creating dropdown menu container
  this.domObj = document.createElement("div");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  //
  parent.appendChildObject(this, this.domObj);
  //
  // Create children after attaching element as external components, rely on dom structure
  this.createChildren(element);
};

// Make BSPanel extend Client.Element
Client.BSPanelSection.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSPanelSection.prototype.updateElement = function (el)
{
  // Force update for the first time
  var update = this.domObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  // specific class name for this layout
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  //
  if (el.type !== undefined) {
    this.type = el.type;
    update = true;
    delete el.type;
  }
  //
  if (update) {
    var cs = "panel-" + (this.type ? this.type : "body");
    this.domObj.className = cs;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};


/**
 * @class A bootstrap well element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.BSWell = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  // Creating dropdown menu container
  this.domObj = document.createElement("div");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  //
  parent.appendChildObject(this, this.domObj);
  //
  // Create children after attaching element as external components, rely on dom structure
  this.createChildren(element);
};

// Make BSWell extend Client.Element
Client.BSWell.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.BSWell.prototype.updateElement = function (el)
{
  // Force update for the first time
  var update = this.domObj.parentNode === null;
  //
  Client.BSHelper.updateElement(el, this);
  //
  // specific class name for this layout
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  //
  if (el.height !== undefined) {
    this.height = el.height;
    update = true;
    delete el.height;
  }
  //
  if (update) {
    //
    var cs = "well";
    if (this.height)
      cs += " " + this.height;
    //
    if (this.className)
      cs += " " + this.className;
    //
    this.domObj.className = cs;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};
