/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client */

/**
 * @class A drag&drop file upload element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the progress bar
 */
Client.DropZone = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("form");
  this.domObj.enctype = "multipart/form-data";
  this.domObj.className = "dropzone";
  this.initDropZone(element.events);
  parent.appendChildObject(this, this.domObj);
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  //
  this.filesMap = [];
  this.count = 1;
};

// Make Client.ProgressBar extend Client.Element
Client.DropZone.prototype = new Client.Element();


/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.DropZone.prototype.updateElement = function (el)
{
  this.purgeMyProp(el);
  //
  if (el.parallelUploads !== undefined) {
    this.dropzone.options.parallelUploads = el.parallelUploads;
    delete el.parallelUploads;
  }
  //
  if (el.maxFilesize !== undefined) {
    this.dropzone.options.maxFilesize = el.maxFilesize;
    delete el.maxFilesize;
  }
  //
  if (el.paramName) {
    this.dropzone.options.paramName = el.paramName;
    delete el.paramName;
  }
  //
  if (el.uploadMultiple) {
    this.dropzone.options.uploadMultiple = el.uploadMultiple;
    delete el.uploadMultiple;
  }
  //
  if (el.addRemoveLinks !== undefined) {
    this.dropzone.options.addRemoveLinks = el.addRemoveLinks;
    delete el.addRemoveLinks;
  }
  //
  if (el.previewsContainer) {
    this.dropzone.options.previewsContainer = el.previewsContainer;
    delete el.previewsContainer;
  }
  //
  if (el.clickable !== undefined) {
    this.dropzone.options.clickable = el.clickable;
    delete el.clickable;
  }
  //
  if (el.createImageThumbnails !== undefined) {
    this.dropzone.options.createImageThumbnails = el.createImageThumbnails;
    delete el.createImageThumbnails;
  }
  //
  if (el.maxThumbnailFilesize !== undefined) {
    this.dropzone.options.maxThumbnailFilesize = el.maxThumbnailFilesize;
    delete el.maxThumbnailFilesize;
  }
  //
  if (el.thumbnailWidth !== undefined) {
    this.dropzone.options.thumbnailWidth = el.thumbnailWidth;
    delete el.thumbnailWidth;
  }
  //
  if (el.thumbnailHeight !== undefined) {
    this.dropzone.options.thumbnailHeight = el.thumbnailHeight;
    delete el.thumbnailHeight;
  }
  //
  if (el.maxFiles !== undefined) {
    this.dropzone.options.maxFiles = el.maxFiles;
    delete el.maxFiles;
  }
  //
  if (el.acceptedFiles) {
    this.dropzone.options.acceptedFiles = el.acceptedFiles;
    delete el.acceptedFiles;
  }
  //
  if (el.autoProcessQueue !== undefined) {
    this.dropzone.options.uploadMultiple = el.autoProcessQueue;
    delete el.autoProcessQueue;
  }
  //
  if (el.defaultMessage && el.defaultMessage !== this.dropzone.options.dictDefaultMessage) {
    this.dropzone.options.dictDefaultMessage = el.defaultMessage;
    //
    // Get the container of the message
    var divs = this.domObj.getElementsByClassName("dz-message");
    var defaultMsgDiv = divs ? divs[0] : undefined;
    //
    // Change the message, if no message use the default one
    if (defaultMsgDiv)
      defaultMsgDiv.innerHTML = el.defaultMessage ? el.defaultMessage : this.dropzone.defaultOptions.dictDefaultMessage;
    //
    delete el.defaultMessage;
  }
  //
  if (el.invalidFileTypeText) {
    this.dropzone.options.dictInvalidFileType = el.invalidFileTypeText;
    delete el.invalidFileTypeText;
  }
  //
  if (el.fileTooBigText) {
    this.dropzone.options.dictFileTooBig = el.fileTooBigText;
    delete el.fileTooBigText;
  }
  //
  if (el.responseErrorText) {
    this.dropzone.options.dictResponseError = el.responseErrorText;
    delete el.responseErrorText;
  }
  //
  if (el.cancelUploadText) {
    this.dropzone.options.dictCancelUpload = el.cancelUploadText;
    delete el.cancelUploadText
  }
  //
  if (el.cancelUploadConfirmationText) {
    this.dropzone.options.dictCancelUploadConfirmation = el.cancelUploadConfirmationText;
    delete el.cancelUploadConfirmationText;
  }
  //
  if (el.removeFileText) {
    this.dropzone.options.dictRemoveFile = el.removeFileText;
    delete el.removeFileText;
  }
  //
  if (el.maxFilesExceededText) {
    this.dropzone.options.dictMaxFilesExceeded = el.maxFilesExceededText;
    delete el.maxFilesExceededText;
  }
  //
  if (el.serverUrl) {
    this.serverUrl = el.serverUrl;
    delete el.serverUrl;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};


/**
 * Initialize the dropzone
 */
Client.DropZone.prototype.initDropZone = function ()
{
  var url = this.serverUrl ? this.serverUrl : Client.Utils.getRESTQueryString({msgType: "dropzone"});
  this.options = {url: url};
  this.dropzone = new Dropzone(this.domObj, this.options);
};


/**
 * Attach events handler
 * @param {Array} events - array of the events to handle
 */
Client.DropZone.prototype.attachEvents = function (events)
{
  // Add onDrop event
  if (events.indexOf("onDrop") >= 0) {
    this.dropzone.on("drop", function (event) {
      var e = [{obj: this.id, id: "onDrop", content: {event: event}}];
      Client.mainFrame.sendEvents(e);
    }.bind(this));
  }
  //
  // Add onDragStart event
  if (events.indexOf("onDragStart") >= 0) {
    this.dropzone.on("dragstart", function (event) {
      var e = [{obj: this.id, id: "onDragStart", content: {event: event}}];
      Client.mainFrame.sendEvents(e);
    }.bind(this));
  }
  //
  // Add onDragEnd event
  if (events.indexOf("onDragEnd") >= 0) {
    this.dropzone.on("dragend", function (event) {
      var e = [{obj: this.id, id: "onDragEnd", content: {event: event}}];
      Client.mainFrame.sendEvents(e);
    }.bind(this));
  }
  //
  // Add onDragEnter event
  if (events.indexOf("onDragEnter") >= 0) {
    this.dropzone.on("dragenter", function (event) {
      var e = [{obj: this.id, id: "onDragEnter", content: {event: event}}];
      Client.mainFrame.sendEvents(e);
    }.bind(this));
  }
  //
  // Add onDragOver event
  if (events.indexOf("onDragOver") >= 0) {
    this.dropzone.on("dragover", function (event) {
      var e = [{obj: this.id, id: "onDragOver", content: {event: event}}];
      Client.mainFrame.sendEvents(e);
    }.bind(this));
  }
  //
  // Add onDragLeave event
  if (events.indexOf("onDragLeave") >= 0) {
    this.dropzone.on("dragleave", function (event) {
      var e = [{obj: this.id, id: "onDragLeave", content: {event: event}}];
      Client.mainFrame.sendEvents(e);
    }.bind(this));
  }
  //
  // Add the addedfile event
  this.dropzone.on("addedfile", function (file) {
    // Save the added file in the map
    this.filesMap.push({id: this.count++, file: file});
    //
    // Add onAddFile event
    if (events.indexOf("onAddFile") >= 0) {
      var e = [{obj: this.id, id: "onAddFile", content: {file: this.getFileInfo(file)}}];
      Client.mainFrame.sendEvents(e);
    }
    //
    // Add onFileClick event
    if (events.indexOf("onFileClick") >= 0) {
      file.previewElement.addEventListener("click", function () {
        var e = [{obj: this.id, id: "onFileClick", content: {file: this.getFileInfo(file)}}];
        Client.mainFrame.sendEvents(e);
      }.bind(this));
    }
  }.bind(this));
  //
  // Add removedfile event
  this.dropzone.on("removedfile", function (file) {
    // Get file info
    var fileinfo = this.getFileInfo(file, true);
    //
    // Remove the file from the map
    this.filesMap.splice(fileinfo.index, 1);
    delete fileinfo.index;
    //
    // Add onRemoveFile event
    if (events.indexOf("onRemoveFile") >= 0) {
      var e = [{obj: this.id, id: "onRemoveFile", content: {file: fileinfo}}];
      Client.mainFrame.sendEvents(e);
    }
  }.bind(this));
  //
  // Add onProcessingFile event
  if (events.indexOf("onProcessingFile") >= 0) {
    this.dropzone.on("processing", function (file) {
      var e = [{obj: this.id, id: "onProcessingFile", content: {file: this.getFileInfo(file)}}];
      Client.mainFrame.sendEvents(e);
    }.bind(this));
  }
  //
  // Add onUploadProgress event
  if (events.indexOf("onUploadProgress") >= 0) {
    this.dropzone.on("uploadprogress", function (file, progress, bytesSent) {
      if (progress === 100 || !this.uploadProgressTime || (this.uploadProgressTime && this.uploadProgressTime - Date.now() > 500)) {
        var e = [{obj: this.id, id: "onUploadProgress", content: {file: this.getFileInfo(file), progress: progress, bytesSent: bytesSent}}];
        Client.mainFrame.sendEvents(e);
        //
        this.uploadProgressTime = Date.now();
      }
    }.bind(this));
  }
  //
  // Add onSuccess event
  if (events.indexOf("onSuccess") >= 0) {
    this.dropzone.on("success", function (file, serverResponse) {
      var info = this.getFileInfo(file);
      //
      // Add path and public url
      var sr = JSON.parse(serverResponse);
      info.path = sr[0].path;
      info.publicUrl = sr[0]._publicUrl;
      //
      var e = [{obj: this.id, id: "onSuccess", content: info}];
      Client.mainFrame.sendEvents(e);
    }.bind(this));
  }
  //
  // Add onCancel event
  this.dropzone.on("canceled", function (file) {
    // Remove the file from the map
    var keys = Object.keys(this.filesMap);
    for (var i = 0; i < keys.length; i++) {
      if (this.filesMap[keys[i]] === file)
      {
        delete this.filesMap[keys[i]];
        break;
      }
    }
    //
    if (events.indexOf("onCancel") >= 0) {
      var e = [{obj: this.id, id: "onCancel", content: {info: this.getFileInfo(file)}}];
      Client.mainFrame.sendEvents(e);
    }
  }.bind(this));

  //
  // Add onMaxFilesReached event
  if (events.indexOf("onMaxFilesReached") >= 0) {
    this.dropzone.on("maxfilesreached", function (file) {
      var e = [{obj: this.id, id: "onMaxFilesReached", content: {}}];
      Client.mainFrame.sendEvents(e);
    }.bind(this));
  }
  //
  // Add onMaxFilesExceeded event
  if (events.indexOf("onMaxFilesExceeded") >= 0) {
    this.dropzone.on("maxfilesexceeded", function (file) {
      var e = [{obj: this.id, id: "onMaxFilesExceeded", content: {file: this.getFileInfo(file)}}];
      Client.mainFrame.sendEvents(e);
    }.bind(this));
  }
  //
  // Add on QueueComplete event
  if (events.indexOf("onQueueComplete") >= 0) {
    this.dropzone.on("queuecomplete", function () {
      var e = [{obj: this.id, id: "onQueueComplete", content: {}}];
      Client.mainFrame.sendEvents(e);
    }.bind(this));
  }
  //
  Client.Element.prototype.attachEvents.call(this, events);
};


/**
 * Get all files in the dropzone
 * @param {string} cbId - callback id
 * @returns {Array} - array containing the files id
 */
Client.DropZone.prototype.getFiles = function (cbId)
{
  var files = [];
  for (var i = 0; i < this.filesMap.length; i++)
    files.push(this.filesMap[i].id);
  //
  var e = [{obj: this.id, id: "cb", content: {res: files, cbId: cbId}}];
  Client.mainFrame.sendEvents(e);
};


/**
 * Get all accepted files in the dropzone
 * @param {string} cbId - callback id
 * @returns {Array}
 */
Client.DropZone.prototype.getAcceptedFiles = function (cbId)
{
  var f = this.dropzone.getAcceptedFiles();
  var files = f.map(function (file) {
    var index = this.dropzone.files.indexOf(file);
    return this.filesMap[index].id;
  }.bind(this));
  //
  var e = [{obj: this.id, id: "cb", content: {res: files, cbId: cbId}}];
  Client.mainFrame.sendEvents(e);
};


/**
 * Get all rejected files in the dropzone
 * @param {string} cbId - callback id
 * @returns {Array}
 */
Client.DropZone.prototype.getRejectedFiles = function (cbId)
{
  var f = this.dropzone.getRejectedFiles();
  var files = f.map(function (file) {
    var index = this.dropzone.files.indexOf(file);
    return this.filesMap[index].id;
  }.bind(this));
  //
  var e = [{obj: this.id, id: "cb", content: {res: files, cbId: cbId}}];
  Client.mainFrame.sendEvents(e);
};


/**
 * Get all queued files in the dropzone
 * @param {string} cbId - callback id
 * @returns {Array}
 */
Client.DropZone.prototype.getQueuedFiles = function (cbId)
{
  var f = this.dropzone.getQueuedFiles();
  var files = f.map(function (file) {
    var index = this.dropzone.files.indexOf(file);
    return this.filesMap[index].id;
  }.bind(this));
  //
  var e = [{obj: this.id, id: "cb", content: {res: files, cbId: cbId}}];
  Client.mainFrame.sendEvents(e);
};


/**
 * Get all uploading files in the dropzone
 * @param {string} cbId - callback id
 * @returns {Array}
 */
Client.DropZone.prototype.getUploadingFiles = function (cbId)
{
  var f = this.dropzone.getUploadingFiles();
  var files = f.map(function (file) {
    var index = this.dropzone.files.indexOf(file);
    return this.filesMap[index].id;
  }.bind(this));
  //
  var e = [{obj: this.id, id: "cb", content: {res: files, cbId: cbId}}];
  Client.mainFrame.sendEvents(e);
};


/**
 * Remove a file from the dropzone. It does not remove it from the server.
 * @param {type} file
 */
Client.DropZone.prototype.removeFile = function (fileId)
{
  for (var i = 0; i < this.filesMap.length; i++) {
    if (this.filesMap[i].id === fileId) {
      this.dropzone.removeFile(this.dropzone.files[i]);
      break;
    }
  }
};


/**
 * Remove all files in the dropzone
 * @param {Boolean} uploading - if true removes files that are currenty uploading too
 */
Client.DropZone.prototype.removeAllFiles = function (uploading)
{
  this.dropzone.removeAllFiles(uploading);
};


/**
 * Uploads queued files
 */
Client.DropZone.prototype.processQueue = function ()
{
  this.dropzone.processQueue();
};


/**
 * Return an object containing info about a file
 * @param {File} file
 * @returns {Object}
 */
Client.DropZone.prototype.getFileInfo = function (file, fileRemoved)
{
  var fileInfo = {};
  fileInfo.accepted = file.accepted;
  fileInfo.height = file.height;
  fileInfo.lastModifiedDate = file.lastModifiedDate;
  fileInfo.name = file.name;
  fileInfo.processing = file.processing;
  fileInfo.size = file.size;
  fileInfo.status = file.status;
  fileInfo.type = file.type;
  fileInfo.width = file.width;
  //
  // If the file was removed it is not in the files list any more
  if (fileRemoved) {
    // Copy the filesMap array
    var files = this.filesMap.slice();
    //
    // Search the files in the dropzone
    var index = 0;
    var i = 0;
    while (i < files.length) {
      var f = files[i].file;
      var found = false;
      for (var j = 0; j < this.dropzone.files.length && !found; j++) {
        // If the file is in the dropzone, remove it from the files array
        if (this.dropzone.files[j] === f) {
          files.splice(i, 1);
          found = true;
        }
      }
      //
      // If the file is not in the dropzone, it's the file I was looking for
      if (!found) {
        fileInfo.id = files[0].id;
        fileInfo.index = index;
        break;
      }
      //
      index++;
    }
  }
  else
    fileInfo.id = this.filesMap[this.dropzone.files.indexOf(file)].id;
  //
  return fileInfo;
};


/**
 * Show a file in the dropzone
 * @param {String} filename - name of the file
 * @param {Integer} filesize - size of the file in bytes
 * @param {String} fileUrl - file url
 * @param {Object} options - optional parameters:
 *                            - type: the mime type of the added file
 */
Client.DropZone.prototype.showFile = function (filename, filesize, fileUrl, options)
{
  // Create the mock file
  var mockFile = {name: filename, size: filesize, lastModifiedDate: new Date(), accepted: true, status: Dropzone.SUCCESS};
  if (options && options.type)
    mockFile.type = options.type;
  //
  // Add the added file to the file list of the dropzone object
  this.dropzone.files.push(mockFile);
  //
  // Call the default addedfile event handler
  this.dropzone.emit("addedfile", mockFile);
  //
  // Show the thumbnail of the file
  this.dropzone.createThumbnailFromUrl(mockFile, fileUrl);
  //
  // Make sure that there is no progress bar, etc...
  this.dropzone.emit("complete", mockFile);
  //
  // If you use the maxFiles option, make sure you adjust it to the correct amount
  var existingFileCount = 1; // The number of files already uploaded
  this.dropzone.options.maxFiles = this.dropzone.options.maxFiles - existingFileCount;
};
