/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client */

/**
 * @class
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the dialog
 * @extends Client.Element
 */
Client.Interactable = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("div");
  parent.appendChildObject(this, this.domObj);
  //
  this.interactablesList = {};
  this.updateElement(element);
  this.attachEvents(element.events);
  //
  this.lastResizeEdge = -1;
};

// Make Client.Interactable extend Client.Element
Client.Interactable.prototype = new Client.Element();


/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.Interactable.prototype.updateElement = function (el)
{
  this.dragOptions = this.interactablesList[this.draggableElement] ? this.interactablesList[this.draggableElement].draggable() : (this.dragOptions || {});
  this.resizeOptions = this.interactablesList[this.resizableElement] ? this.interactablesList[this.resizableElement].resizable() : (this.resizeOptions || {});
  this.dropOptions = this.interactablesList[this.droppableElement] ? this.interactablesList[this.droppableElement].dropzone() : (this.dropOptions || {});
  //
  if (el.dragType) {
    this.dragType = el.dragType;
    //
    delete el.dragType;
  }
  //
  // Set the snapping of the draggable elements
  if (el.dragSnapTargets) {
    if (!this.dragOptions.snap)
      this.dragOptions.snap = {};
    //
    if (!this.dragOptions.snap.targets)
      this.dragOptions.snap.targets = [];
    //
    this.dragOptions.snap.targets = el.dragSnapTargets;
    //
    delete el.dragSnapTargets;
  }
  //
  // Set a snap grid for the draggable elements
  if (el.dragSnapGrid) {
    // Create the grid
    var gridTarget = interact.createSnapGrid(el.dragSnapGrid);
    //
    if (!this.dragOptions.snap)
      this.dragOptions.snap = {};
    //
    if (!this.dragOptions.snap.targets)
      this.dragOptions.snap.targets = [];
    //
    this.dragOptions.snap.enabled = true;
    this.dragOptions.snap.targets.push(gridTarget);
    //
    delete el.dragSnapGrid;
  }
  //
  if (el.dragRestriction) {
    var restrObj = document.getElementById(el.dragRestriction);
    if (!restrObj)
      restrObj = el.dragRestriction;
    //
    if (!this.dragOptions.restrict)
      this.dragOptions.restrict = {};
    //
    this.dragOptions.restrict.restriction = restrObj;
    //
    delete el.dragRestriction;
  }
  //
  if (el.dragInertia) {
    this.dragOptions.inertia = el.dragInertia;
    //
    delete el.dragInertia;
  }
  //
  if (el.dragAutoScroll) {
    var container = document.getElementById(el.dragAutoScroll.container);
    if (container)
      el.dragAutoScroll.container = container;
    this.dragOptions.autoScroll = el.dragAutoScroll;
    //
    delete el.dragAutoScroll;
  }
  //
  if (el.dragAxis) {
    this.dragOptions.axis = el.dragAxis;
    //
    delete el.dragAxis;
  }
  //
  if (el.dragIgnoreFrom) {
    if (el.dragIgnoreFrom.indexOf(".") !== 0)
      this.dragOptions.ignoreFrom = "." + el.dragIgnoreFrom;
    else
      this.dragOptions.ignoreFrom = el.dragIgnoreFrom;
    //
    delete el.dragIgnoreFrom;
  }
  //
  // Set the snapping of the resizable elements
  if (el.resizeSnapTargets) {
    if (!this.resizeOptions.snap)
      this.resizeOptions.snap = {};
    //
    if (!this.resizeOptions.snap.resizeSnapTargets)
      this.resizeOptions.snap.resizeSnapTargets = [];
    //
    this.resizeOptions.snap.enabled = true;
    this.resizeOptions.snap.targets = el.resizeSnapTargets;
    //
    delete el.resizeSnapTargets;
  }
  //
  // Set a snap grid for the draggable elements
  if (el.resizeSnapGrid) {
    // Create the grid
    var gridTarget = interact.createSnapGrid(el.resizeSnapGrid);
    //
    if (!this.resizeOptions.snap)
      this.resizeOptions.snap = {};
    //
    if (!this.resizeOptions.snap.targets)
      this.resizeOptions.snap.targets = [];
    //
    this.resizeOptions.snap.targets.push(gridTarget);
    //
    delete el.resizeSnapGrid;
  }
  //
  // Set the edges that fire the resize
  if (el.resizeEdges) {
    this.resizeOptions.edges = el.resizeEdges;
    //
    delete el.resizeEdges;
  }
  //
  if (el.resizeRestriction) {
    var restrObj = document.getElementById(el.resizeRestriction);
    if (!restrObj)
      restrObj = el.resizeRestriction;
    //
    if (!this.resizeOptions.restrict)
      this.resizeOptions.restrict = {};
    //
    this.resizeOptions.restrict.restriction = restrObj;
    //
    delete el.resizeRestriction;
  }
  //
  if (el.resizeInertia !== undefined) {
    this.resizeOptions.inertia = el.resizeInertia;
    //
    delete el.resizeInertia;
  }
  //
  if (el.resizeAutoScroll) {
    var container = document.getElementById(el.resizeAutoScroll.container);
    if (container)
      el.resizeAutoScroll.container = container;
    this.resizeOptions.autoScroll = el.resizeAutoScroll;
    //
    delete el.resizeAutoScroll;
  }
  //
  if (el.resizeInvert) {
    this.resizeOptions.invert = el.resizeInvert;
    //
    delete el.resizeInvert;
  }
  //
  if (el.resizePreserveAspectRatio !== undefined) {
    this.resizeOptions.preserveAspectRatio = el.resizePreserveAspectRatio;
    //
    delete el.resizePreserveAspectRatio;
  }
  //
  if (el.dropOverlap !== undefined) {
    this.dropOptions.overlap = el.dropOverlap;
    //
    delete el.dropOverlap;
  }
  //
  if (el.dragType) {
    this.dragType = el.dragType;
    delete el.dragType;
  }
  //
  if (el.dragAcceptedElements) {
    this.initDragInteract(el.dragAcceptedElements);
    //
    delete el.dragAcceptedElements;
  }
  //
  if (el.resizeAcceptedElements) {
    this.initResizeInteract(el.resizeAcceptedElements);
    //
    delete el.resizeAcceptedElements;
  }
  //
  if (el.dropAcceptedElements) {
    this.initDropInteract(el.dropAcceptedElements);
    //
    delete el.dropAcceptedElements;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};


/**
 * Attach events handler
 * @param {Array} events - array of the events to handle
 */
Client.Interactable.prototype.attachEvents = function (events)
{
  if (this.interactablesList[this.draggableElement]) {
    this.interactablesList[this.draggableElement].on("dragstart", function (event) {
      var targetRect = event.target.getBoundingClientRect();
      this.originalX = targetRect.left;
      this.originalY = targetRect.top;
      //
      // If I have to create a copy of the dragged element
      if (this.dragType === "copy") {
        this.targetX = parseFloat(event.target.getAttribute('data-x')) || 0;
        this.targetY = parseFloat(event.target.getAttribute('data-y')) || 0;
        //
        // Clone the target element
        var clone = event.target.cloneNode(true);
        clone.id = Math.floor(Math.random() * 100);
        event.target.parentNode.appendChild(clone);
        //
        // Get the current position of the target and the cloned element
        var targetRect = event.target.getBoundingClientRect();
        var cloneRect = clone.getBoundingClientRect();
        //
        // Position the cloned element
        var cloneX = (targetRect.left - cloneRect.left) + (parseFloat(event.target.getAttribute('data-x')) || 0);
        var cloneY = (targetRect.top - cloneRect.top) + (parseFloat(event.target.getAttribute('data-y')) || 0);
        clone.style.transform = "translate(" + cloneX + "px," + cloneY + "px)";
        clone.setAttribute('data-x', cloneX);
        clone.setAttribute('data-y', cloneY);
        //
        // Move the target element to its original position
        event.target.style.transform = "translate(" + this.targetX + "px, " + this.targetY + "px)";
        event.target.setAttribute('data-x', this.targetX);
        event.target.setAttribute('data-y', this.targetY);
      }
      //
      if (events.indexOf("onDragStart") >= 0) {
        var e = [{obj: this.id, id: "onDragStart", content: {draggedElementId: event.target.id}}];
        Client.mainFrame.sendEvents(e);
      }
    }.bind(this));
  }
  //
  if (this.interactablesList[this.draggableElement]) {
    this.interactablesList[this.draggableElement].on("dragmove", function (event) {
      if (events.indexOf("onDragMove") >= 0) {
        var e = [{obj: this.id, id: "onDragMove", content: {draggedElementId: event.target.id, elementEnterId: event.dragEnter ? event.dragEnter.id : undefined, elementLeaveId: event.dragLeave ? event.dragLeave.id : undefined}}];
        Client.mainFrame.sendEvents(e);
      }
    }.bind(this));
  }
  //
  if (events.indexOf("onDragEnd") >= 0) {
    if (this.interactablesList[this.draggableElement]) {
      this.interactablesList[this.draggableElement].on("dragend", function (event) {
        var e = [{obj: this.id, id: "onDragEnd", content: {draggedElementId: event.target.id}}];
        Client.mainFrame.sendEvents(e);
      }.bind(this));
    }
  }
  //
  if (events.indexOf("onDragEnter") >= 0) {
    if (this.interactablesList[this.droppableElement]) {
      this.interactablesList[this.droppableElement].on("dragenter", function (event) {
        var e = [{obj: this.id, id: "onDragEnter", content: {dropzoneId: event.target.id, draggedElementId: event.relatedTarget.id}}];
        Client.mainFrame.sendEvents(e);
      }.bind(this));
    }
  }
  //
  if (events.indexOf("onDragLeave") >= 0) {
    if (this.interactablesList[this.droppableElement]) {
      this.interactablesList[this.droppableElement].on("dragleave", function (event) {
        var e = [{obj: this.id, id: "onDragLeave", content: {dropzoneId: event.target.id, draggedElementId: event.relatedTarget.id}}];
        Client.mainFrame.sendEvents(e);
      }.bind(this));
    }
  }
  //
  if (events.indexOf("onDropActivate") >= 0) {
    if (this.interactablesList[this.droppableElement]) {
      this.interactablesList[this.droppableElement].on("dropactivate", function (event) {
        var e = [{obj: this.id, id: "onDropActivate", content: {dropzoneId: event.target.id, draggedElementId: event.relatedTarget.id}}];
        Client.mainFrame.sendEvents(e);
      }.bind(this));
    }
  }
  //
  if (events.indexOf("onDropDeactivate") >= 0) {
    if (this.interactablesList[this.droppableElement]) {
      this.interactablesList[this.droppableElement].on("dropdeactivate", function (event) {
        var e = [{obj: this.id, id: "onDropDeactivate", content: {dropzoneId: event.target.id, draggedElementId: event.relatedTarget.id}}];
        Client.mainFrame.sendEvents(e);
      }.bind(this));
    }
  }
  //
  if (events.indexOf("onDropMove") >= 0) {
    if (this.interactablesList[this.droppableElement]) {
      this.interactablesList[this.droppableElement].on("dropmove", function (event) {
        var e = [{obj: this.id, id: "onDropMove", content: {dropzoneId: event.target.id, draggedElementId: event.relatedTarget.id}}];
        Client.mainFrame.sendEvents(e);
      }.bind(this));
    }
  }
  //
  if (events.indexOf("onDrop") >= 0) {
    if (this.interactablesList[this.droppableElement]) {
      this.interactablesList[this.droppableElement].on("drop", function (event) {
        var e = [{obj: this.id, id: "onDrop", content: {dropzoneId: event.target.id, draggedElementId: event.relatedTarget.id, dx: event.dragEvent.dx, dy: event.dragEvent.dy}}];
        Client.mainFrame.sendEvents(e);
      }.bind(this));
    }
  }
  //
  if (events.indexOf("onResizeStart") >= 0) {
    if (this.interactablesList[this.resizableElement]) {
      this.interactablesList[this.resizableElement].on("resizestart", function (event) {
        var e = [{obj: this.id, id: "onResizeStart", content: {resizedElementId: event.target.id}}];
        Client.mainFrame.sendEvents(e);
      }.bind(this));
    }
  }
  //
  if (events.indexOf("onResizeEnd") >= 0) {
    if (this.interactablesList[this.resizableElement]) {
      this.interactablesList[this.resizableElement].on("resizeend", function (event) {
        var e = [{obj: this.id, id: "onResizeEnd", content: {resizedElementId: event.target.id, dx: event.dx, dy: event.dy, edge: this.lastResizeEdge}}];
        Client.mainFrame.sendEvents(e);
        //
        this.lastResizeEdge = -1;
      }.bind(this));
    }
  }
  //
  if (events.indexOf("onResizeMove") >= 0) {
    if (this.interactablesList[this.resizableElement]) {
      this.interactablesList[this.resizableElement].on("resizemove", function (event) {
        var e = [{obj: this.id, id: "onResizeMove", content: {resizedElementId: event.target.id, dx: event.dx, dy: event.dy, edge: this.lastResizeEdge}}];
        Client.mainFrame.sendEvents(e);
      }.bind(this));
    }
  }
  //
  Client.Element.prototype.attachEvents.call(this, events);
};


/**
 * Enable drag on elements
 * @param {Element|String} elements - the draggable elements
 */
Client.Interactable.prototype.initDragInteract = function (elements) {
  function dragMoveListener(event) {
    var target = event.target;
    //
    // Keep the dragged position in the data-x/data-y attributes
    var x = (parseFloat(target.getAttribute('data-x')) || 0) + event.dx;
    var y = (parseFloat(target.getAttribute('data-y')) || 0) + event.dy;
    //
    // Translate the element
    target.style.transform = 'translate(' + x + 'px, ' + y + 'px)';
    //
    // Update the position attributes
    target.setAttribute('data-x', x);
    target.setAttribute('data-y', y);
  }
  //
  var parameters = {};
  switch (elements) {
    // The parent of the interact element is the draggable element
    case "parent":
      parameters.draggableElement = this.domObj.parentElement;
      //
      break;

      // The siblings of the interact element are the draggable elements
    case "siblings":
      for (var i = 0; i < this.domObj.parentElement.children.length; i++) {
        var c = this.domObj.parentElement.children[i];
        if (c.id === this.domObj.id)
          continue;
        c.classList.add("interact-draggable-element");
      }
      parameters.draggableElement = ".interact-draggable-element";
      parameters.context = this.domObj.parentElement;
      //
      break;

    case "children":
      for (var i = 0; i < this.domObj.children.length; i++)
        this.domObj.children[i].classList.add("interact-draggable-element");
      parameters.draggableElement = ".interact-draggable-element";
      //
      break;

    case "view":
      for (var i = 0; i < this.view.elements.length; i++)
        this.view.elements[i].domObj.classList.add("interact-draggable-element");
      parameters.draggableElement = ".interact-draggable-element";
      //
      break;

    default:
      if (elements.indexOf(".") !== 0)
        elements = "." + elements;
      parameters.draggableElement = elements;
      //
      break;
  }
  //
  // Set up drag options
  this.dragOptions.onmove = dragMoveListener;
  this.snapToDropzone();
  //
  if (!this.interactablesList[parameters.draggableElement]) {
    var globalOptions = {context: parameters.context};
    if (this.dragOptions.ignoreFrom) {
      globalOptions.ignoreFrom = this.dragOptions.ignoreFrom;
      delete this.dragOptions.ignoreFrom;
    }
    //
    var interactable = interact(parameters.draggableElement, globalOptions).draggable(this.dragOptions);
    this.interactablesList[parameters.draggableElement] = interactable;
    this.draggableElement = parameters.draggableElement;
  }
  else {
    var interactable = this.interactablesList[parameters.draggableElement];
    if (interactable)
      interactable.draggable(this.dragOptions);
  }
};


/**
 * Enable resize on elements
 * @param {Element|String} elements - the resizable elements
 */
Client.Interactable.prototype.initResizeInteract = function (elements) {
  var pthis = this;
  function resizeListener(event) {
    var target = event.target;
    var x = (parseFloat(target.getAttribute('data-x')) || 0);
    var y = (parseFloat(target.getAttribute('data-y')) || 0);
    //
    // Update the element's style
    target.style.width = event.rect.width + 'px';
    target.style.height = event.rect.height + 'px';
    //
    // Translate when resizing from top or left edges
    x += event.deltaRect.left;
    y += event.deltaRect.top;
    //
    target.style.transform = 'translate(' + x + 'px,' + y + 'px)';
    target.setAttribute('data-x', x);
    target.setAttribute('data-y', y);
    //
    if (event.deltaRect.top !== 0)
      pthis.lastResizeEdge = 1;
    else if (event.deltaRect.left !== 0)
      pthis.lastResizeEdge = 2;
    else if (event.deltaRect.bottom !== 0)
      pthis.lastResizeEdge = 3;
    else if (event.deltaRect.right !== 0)
      pthis.lastResizeEdge = 4;
  }
  //
  var parameters = {};
  switch (elements) {
    case "parent":
      parameters.resizableElement = this.domObj.parentElement;
      //
      break;

    case "siblings":
      for (var i = 0; i < this.domObj.parentElement.children.length; i++) {
        var c = this.domObj.parentElement.children[i];
        if (c.id === this.domObj.id)
          continue;
        c.classList.add("interact-resizable-element");
      }
      parameters.resizableElement = ".interact-resizable-element";
      parameters.context = this.domObj.parentElement;
      //
      break;

    case "children":
      for (var i = 0; i < this.domObj.children.length; i++)
        this.domObj.children[i].classList.add("interact-resizable-element");
      parameters.resizableElement = ".interact-resizable-element";
      //
      break;

    case "view":
      for (var i = 0; i < this.view.elements.length; i++)
        this.view.elements[i].domObj.classList.add("interact-resizable-element");
      parameters.resizableElement = ".interact-resizable-element";
      //
      break;

    default:
      if (elements.indexOf(".") !== 0)
        elements = "." + elements;
      parameters.resizableElement = elements;
      //
      break;
  }
  //
  if (!this.resizeOptions.edges)
    this.resizeOptions.edges = {top: true, left: true, bottom: true, right: true};
  //
  if (!this.interactablesList[parameters.resizableElement]) {
    var interactable = interact(parameters.resizableElement, {context: parameters.context}).resizable(this.resizeOptions).on("resizemove", resizeListener);
    this.interactablesList[parameters.resizableElement] = interactable;
    this.resizableElement = parameters.resizableElement;
  }
  else {
    var interactable = this.interactablesList[parameters.resizableElement];
    if (interactable) {
      interactable.resizable(this.resizeOptions).on("resizemove", resizeListener);
      this.resizableElement = parameters.resizableElement;
    }
  }
};


/**
 * Enable drag on elements
 * @param {Element|String} elements - the draggable elements
 */
Client.Interactable.prototype.initDropInteract = function (elements) {
  var parameters = {};
  switch (elements) {
    case "parent":
      parameters.droppableElement = this.domObj.parentElement;
      //
      break;

    case "siblings":
      for (var i = 0; i < this.domObj.parentElement.children.length; i++) {
        var c = this.domObj.parentElement.children[i];
        if (c.id === this.domObj.id)
          continue;
        c.classList.add("interact-droppable-element");
      }
      parameters.droppableElement = ".interact-droppable-element";
      //
      break;

    case "children":
      for (var i = 0; i < this.domObj.children.length; i++)
        this.domObj.children[i].classList.add("interact-droppable-element");
      parameters.droppableElement = ".interact-droppable-element";
      //
      break;

    case "view":
      for (var i = 0; i < this.view.elements.length; i++)
        this.view.elements[i].domObj.classList.add("interact-droppable-element");
      parameters.droppableElement = ".interact-droppable-element";
      //
      break;

    default:
      if (elements.indexOf(".") !== 0)
        elements = "." + elements;
      parameters.droppableElement = elements;
      //
      break;
  }
  //
  if (!this.interactablesList[parameters.droppableElement]) {
    var interactable = interact(parameters.droppableElement).dropzone(this.dropOptions);
    this.interactablesList[parameters.droppableElement] = interactable;
    this.droppableElement = parameters.droppableElement;
  }
  else {
    var interactable = this.interactablesList[parameters.droppableElement];
    if (interactable)
      interactable.dropzone(this.dropOptions);
  }
  //
  this.snapToDropzone();
};


Client.Interactable.prototype.snapToDropzone = function ()
{
  if (this.dragType !== "drag")
    return;
  //
  if (!this.droppableElement)
    return;
  //
  if (!this.dragOptions.snap)
    this.dragOptions.snap = {};
  if (!this.dragOptions.snap.targets)
    this.dragOptions.snap.targets = [];
  //
  var elems = [];
  if (typeof this.droppableElement === "string") {
    var c = this.droppableElement;
    var idx = c.indexOf(".");
    if (idx === 0)
      c = c.substring(1);
    elems = document.getElementsByClassName(c);
  }
  else
    elems = [this.droppableElement];
  //
  for (var i = 0; i < elems.length; i++) {
    var e = elems[i];
    var style = window.getComputedStyle(e);
    this.dragOptions.snap.targets.push({x: parseInt(style.left), y: parseInt(style.top), range: 100});
  }
};


Client.Interactable.prototype.swapElements = function (draggedElementId, dropElementId)
{
  // Reposition the drop element
  var dropElem = document.getElementById(dropElementId);
  var dropElemRect = dropElem.getBoundingClientRect();
  //
  // 1. Calculate the distance
  var dx = this.originalX - dropElemRect.left;
  var dy = this.originalY - dropElemRect.top;
  //
  // 2. Calculate the relative distance to move
  var ox = (parseFloat(dropElem.getAttribute('data-x')) || 0) + dx;
  var oy = (parseFloat(dropElem.getAttribute('data-y')) || 0) + dy;
  //
  // 3. Move the element
  dropElem.style.transform = "translate(" + ox + "px," + oy + "px)";
  dropElem.setAttribute('data-x', ox);
  dropElem.setAttribute('data-y', oy);
  //
  this.originalX = dropElemRect.left;
  this.originalY = dropElemRect.top;
  //
  this.swapped = true;
};


/**
 * Return the distance between a point and an element
 * @param {Float} originalX - the x coordinate
 * @param {Float} originalY - the y coordinate
 * @param {Object} elem - an element
 * @returns {Object}
 */
Client.Interactable.prototype.calculateRelativeDistance = function (elem, originalX, originalY)
{
  var elemRect = elem.getBoundingClientRect();
  //
  // Calculate the distance
  var dx = originalX - elemRect.left;
  var dy = originalY - elemRect.top;
  //
  // Calculate the relative distance
  var ox = (parseFloat(elem.getAttribute('data-x')) || 0) + dx;
  var oy = (parseFloat(elem.getAttribute('data-y')) || 0) + dy;
  //
  return {x: ox, y: oy};
};


/**
 * Move an element to its last position
 * @param {String} id - the id of the element to move
 */
Client.Interactable.prototype.undoMovement = function (id)
{
  var elem = document.getElementById(id);
  if (!elem)
    return;
  //
  var dist = this.calculateRelativeDistance(elem, this.originalX, this.originalY);
  elem.style.transform = "translate(" + dist.x + "px," + dist.y + "px)";
  elem.setAttribute('data-x', dist.x);
  elem.setAttribute('data-y', dist.y);
};


/**
 * Move an element
 * @param {String} id - the id of the element to move
 * @param {Integer} dx - the x coordinate
 * @param {Integer} dy - the y coordinate
 */
Client.Interactable.prototype.moveElement = function (id, dx, dy)
{
  var target = document.getElementById(id);
  if (!target)
    return;
  //
  // Keep the dragged position in the data-x/data-y attributes
  var x = (parseFloat(target.getAttribute('data-x')) || 0) + dx;
  var y = (parseFloat(target.getAttribute('data-y')) || 0) + dy;
  //
  // Translate the element
  target.style.transform = 'translate(' + x + 'px, ' + y + 'px)';
  //
  // Update the position attributes
  target.setAttribute('data-x', x);
  target.setAttribute('data-y', y);
};
