/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client */


/**
 * @class A container for an input
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.IonAutoComplete = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.outerObj = document.createElement("ion-input");
  this.domObj = document.createElement("input");
  this.domObj.className = "text-input";
  this.backdrop = "combo";
  this.valueList = [];
  //
  this.outerObj.appendChild(this.domObj);
  //
  this.labelObj = document.createElement("ion-label");
  //
  this.clearObj = document.createElement("button");
  this.clearObj.className = "text-input-clear-icon disable-hover button button-clear want-click";
  //
  var pthis = this;
  this.clearObj.onmousedown = function () {
    pthis.domObj.value = "";
    pthis.onInputChange(!pthis.openOnFocus);
    pthis.domObj.focus();
    return false;
  };
  this.clearObj.ontouchstart = function (ev) {
    ev.preventDefault();
    pthis.domObj.value = "";
    pthis.onInputChange(!pthis.openOnFocus);
    pthis.domObj.focus();
    return false;
  };
  this.clearObj.style.display = "none";
  this.outerObj.appendChild(this.clearObj);
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.outerObj);
  this.outerObj.parentNode.insertBefore(this.labelObj, this.outerObj);
  //
  if (Client.IonItem && parent instanceof Client.IonItem)
    parent.domObj.classList.add("item-input");
  //
  this.domObj.addEventListener("focus", function (ev) {
    pthis.onInputFocus();
  });
  this.domObj.addEventListener("blur", function (ev) {
    pthis.onInputBlur(ev);
  });
  this.domObj.addEventListener("input", function (ev) {
    pthis.onInputChange();
  });
  this.domObj.onkeydown = function (ev) {
    pthis.onInputKeyDown(ev);
  };
};

Client.IonAutoComplete.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.IonAutoComplete.prototype.updateElement = function (el)
{
  this.purgeMyProp(el);
  //
  if (el.className !== undefined) {
    this.outerObj.className = el.className;
    delete el.className;
  }
  if (el.label !== undefined) {
    this.labelObj.textContent = el.label;
    if (this.parent instanceof Client.IonItem) {
      this.parent.domObj.classList.add("item-label-fixed");
    }
    delete el.label;
  }
  if (el.labelPosition !== undefined) {
    if (this.labelPosition) {
      this.labelObj.removeAttribute(this.labelPosition);
      if (this.parent instanceof Client.IonItem)
        this.parent.domObj.classList.remove("item-label-" + this.labelPosition);
    }
    //
    this.labelPosition = el.labelPosition;
    if (el.labelPosition) {
      this.labelObj.setAttribute(this.labelPosition, "");
      if (this.parent instanceof Client.IonItem)
        this.parent.domObj.classList.add("item-label-" + this.labelPosition);
    }
  }
  if (el.color !== undefined) {
    if (this.color)
      this.labelObj.removeAttribute(this.color);
    this.color = el.color;
    if (this.color)
      this.labelObj.setAttribute(this.color, "");
    delete el.color;
  }
  if (el.clear !== undefined) {
    this.clearObj.style.display = el.clear ? "" : "none";
  }
  if (el.disabled !== undefined) {
    this.disabled = el.disabled;
  }
  if (el.acceptNewValues !== undefined) {
    this.acceptNewValues = el.acceptNewValues;
    delete el.acceptNewValues;
  }
  if (el.backdrop !== undefined) {
    this.backdrop = el.backdrop;
    delete el.backdrop;
  }
  if (el.autoAccept !== undefined) {
    this.autoAccept = el.autoAccept;
    delete el.autoAccept;
  }
  if (el.openOnFocus !== undefined) {
    this.openOnFocus = el.openOnFocus;
    delete el.openOnFocus;
  }
  //
  if (el.list) {
    //
    // If el.list is a string make it a json object
    if (typeof el.list === "string")
      el.list = JSON.parse(el.list);
    //
    this.valueList = el.list;
    //
    // updates the combo
    if (!this.comboTimer) {
      this.comboTimer = window.setTimeout(function () {
        this.updateCombo();
      }.bind(this), 10);
    }
    //
    // Base class must not use the list property
    delete el.list;
  }
  //
  // If the select is bound to a datamap, the data property contains current options
  if (el.data) {
    // There are four cases
    // resetting options, or add/delete/update them
    if (el.data.columns)
      this.initOptions(el); // Reset case
    else if (el.data.pos !== undefined) {
      if (el.data.data) // Update case
        this.updateOption(el.data);
      else  // Remove case
        this.removeOption(el.data.pos);
    }
    else  // Add case
      this.addOption(el.data.data);
    //
    // updates the combo
    if (!this.comboTimer) {
      this.comboTimer = window.setTimeout(function () {
        this.updateCombo();
      }.bind(this), 10);
    }
    //
    // Base class must not use the data property
    delete el.data;
  }
  //
  if (el.value !== undefined) {
    this.setValue(el.value);
    if (this.dismissTimerId)
      this.deleteFilterAfterDismiss = true;
    else
      delete this.filter;
    delete el.value;
    //
    // If I'm in telecollaboration or testauto case, I want to be sure
    // that other clients close the autocomplete when a value is chosen
    if (el.clid && el.clid !== Client.id)
      this.closeCombo();
  }
  //
  if (el.filter !== undefined) {
    this.domObj.value = el.filter;
    this.filter = el.filter;
    delete el.filter;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};


/**
 * Attach events handler
 * @param {Array} events - array of the events to handle
 */
Client.IonAutoComplete.prototype.attachEvents = function (events)
{
  if (!events)
    return;
  //
  var pos = events.indexOf("onChange");
  if (pos >= 0) {
    events.splice(pos, 1);
    this.sendChanges = true;
  }
  var pos = events.indexOf("onFilter");
  if (pos >= 0) {
    events.splice(pos, 1);
    this.dataBound = true;
  }
  //
  Client.Element.prototype.attachEvents.call(this, events);
};


/**
 * Return the dom root object of this element
 * @returns {DomElement}
 */
Client.IonAutoComplete.prototype.getRootObject = function ()
{
  return this.outerObj;
};


/**
 * Remove this element
 */
Client.IonAutoComplete.prototype.close = function ()
{
  if (this.comboObj) {
    this.comboObj.remove();
    if (this.combo2)
      this.combo2.remove();
  }
  this.closed = true;
  //
  Client.Element.prototype.close.call(this);
};


/**
 * Initialize the input
 * @param {Object} el - element representation
 */
Client.IonAutoComplete.prototype.initOptions = function (el)
{
  this.colIndex = {v: -1, n: -1};
  this.valueList = [];
  this.dataBound = true;
  //
  // Search for pre-defined column names in the recordset
  for (var i = 0; i < el.data.columns.length; i++) {
    var c = el.data.columns[i].toLowerCase();
    if (c === "id" || c === "code" || c === "value" || c === "v")
      this.colIndex.v = i;
    if (c === "name" || (c === "description" && this.colIndex.n === -1) || c === "n")
      this.colIndex.n = i;
    if (c === "image" || c === "icon" || c === "src")
      this.colIndex.i = i;
    if ((c === "description" && this.colIndex.n !== i) || c === "detail" || c === "d")
      this.colIndex.d = i;
    if (c === "style" || c === "class" || c === "s")
      this.colIndex.s = i;
  }
  if (this.colIndex.v === -1)
    this.colIndex.v = 0;
  if (this.colIndex.n === -1)
    this.colIndex.n = el.data.columns.length > 0 ? 1 : 0;
  //
  var k = Object.keys(el.data.data);
  for (var j = 0; j < k.length; j++) {
    i = k[j];
    this.addOption(el.data.data[i]);
  }
};


/**
 * Remove an option from the select at the given position
 * @param {int} pos - the position of the option to remove
 */
Client.IonAutoComplete.prototype.removeOption = function (pos)
{
  this.valueList.splice(pos, 1);
};


/**
 * Add an option to the select
 * @param {Array} data - the option to add. The first element of the array represents the value,
 *                       the second element represents the text
 */
Client.IonAutoComplete.prototype.addOption = function (data)
{
  var x = {v: data[this.colIndex.v], n: data[this.colIndex.n]};
  if (this.colIndex.i !== undefined)
    x.src = data[this.colIndex.i];
  if (this.colIndex.s !== undefined)
    x.s = data[this.colIndex.s];
  if (this.colIndex.d !== undefined)
    x.d = data[this.colIndex.d];
  this.valueList.push(x);
};


/**
 * Update a select option
 * @param {Object} data - the data of the option to update. Example: {pos: 3, data:[3, "Andrea"]}
 */
Client.IonAutoComplete.prototype.updateOption = function (data)
{
  var v = this.valueList[data.pos];
  v.v = data.data[this.colIndex.v];
  v.n = data.data[this.colIndex.n];
  if (this.colIndex.i !== undefined)
    v.src = data.data[this.colIndex.i];
  if (this.colIndex.s !== undefined)
    v.s = data.data[this.colIndex.s];
  if (this.colIndex.d !== undefined)
    v.d = data.data[this.colIndex.d];
};


/**
 * The input got focus: open the combo
 */
Client.IonAutoComplete.prototype.onInputFocus = function ()
{
  if (this.dismissTimerId) {
    clearTimeout(this.dismissTimerId);
    delete this.dismissTimerId;
  }
  if (Client.mainFrame.isEditing())
    return;
  if (this.disabled)
    return;
  //
  if (Client.IonItem && this.parent instanceof Client.IonItem)
    this.parent.domObj.classList.add("input-has-focus");
  //
  setTimeout(function () {
    this.openCombo(this.openOnFocus ? "" : undefined);
    if (this.dataBound && this.openOnFocus) {
      this.sendFilter(" ");
      this.fullCombo = true;
    }
  }.bind(this), 200);
};


/**
 * Filter has changed
 */
Client.IonAutoComplete.prototype.onInputChange = function (nolist)
{
  if (this.disabled)
    return;
  if (Client.mainFrame.isEditing())
    return;
  if (this.lastChange === this.domObj.value)
    return;
  this.lastChange = this.domObj.value;
  //
  this.valueHasChanged = true;
  //
  if (Client.IonItem && this.parent instanceof Client.IonItem)
    this.parent.domObj.classList.toggle("input-has-value", this.domObj.value);
  //
  if (nolist) {
    this.openCombo();
  }
  else {
    if (this.dataBound)
      this.sendFilter(this.domObj.value);
    else
      this.openCombo(this.domObj.value);
  }
};


/**
 * clicked out of the input
 * @param {MouseEvent} ev
 */
Client.IonAutoComplete.prototype.onInputBlur = function (ev)
{
  var rt = ev.relatedTarget;
  var incombo = false;
  while (rt) {
    if (rt === this.comboObj) {
      incombo = true;
      break
    }
    rt = rt.parentNode;
  }
  if (!incombo) {
    if (Client.IonItem && this.parent instanceof Client.IonItem)
      this.parent.domObj.classList.remove("input-has-focus");
    if (this.isOpen()) {
      this.dismissTimerId = setTimeout(function () {
        if (this.autoAccept)
          this.accept();
        else
          this.dismiss();
      }.bind(this), 100);
    }
  }
};


/**
 * send filter value
 * @param {string} value
 * @param {event} ev
 */
Client.IonAutoComplete.prototype.sendFilter = function (value, ev)
{
  var e = [{obj: this.id, id: "chgProp", content: {name: "filter", value: value, clid: Client.id}}];
  e.push({obj: this.id, id: "onFilter", content: this.saveEvent(ev)});
  if (!Client.mainFrame.isEditing())
    Client.mainFrame.sendEvents(e);
};


/**
 * open the combo box with list values
 * @param {string} value
 */
Client.IonAutoComplete.prototype.openCombo = function (value)
{
  if (this.closed)
    return;
  //
  this.createCombo();
  this.comboOpen = true;
  //
  // Undefined means no list
  if (value === undefined) {
    this.showBackdrop(this.valueHasChanged);
    return;
  }
  //
  var up = this.wrapperObj.classList.contains("ion-autocomplete-up");
  //
  var exp = new RegExp(value.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&'), "i");
  var found = false;
  var pm = false;
  var count = 0;
  //
  for (var i = 0; i < this.valueList.length; i++) {
    var v = this.valueList[i];
    //
    var idx = v.n.search(exp);
    if (idx > -1) {
      count++;
      this.createComboItem(v, i, up);
      if (v.v === this.value)
        found = true;
      if (v.n === value)
        pm = true;
    }
  }
  if (this.acceptNewValues && !found && !pm && this.domObj.value) {
    this.createComboItem({v: this.domObj.value, n: "+" + this.domObj.value, new : true}, i, up);
    count++;
  }
  //
  if (up) {
    if (this.listObj.clientHeight > this.wrapperObj.clientHeight) {
      this.wrapperObj.style.justifyContent = "flex-start";
      this.comboObj.scrollTop = 9999;
    }
    else {
      this.wrapperObj.style.justifyContent = "";
      this.comboObj.scrollTop = 0;
    }
  }
  //
  this.showBackdrop(this.valueHasChanged || count);
  this.listObj.style.opacity = count ? "1" : "";
};


/**
 * shows or hides the backdrop
 * @param {bool} flag
 */
Client.IonAutoComplete.prototype.showBackdrop = function (flag)
{
  var op = flag ? ((Client.Ionic.platform === "md") ? "0.5" : "0.3") : "";
  //
  if (this.bdObj)
    this.bdObj.style.opacity = op;
  if (this.bd2)
    this.bd2.style.opacity = op;
};


/**
 * create the combo
 */
Client.IonAutoComplete.prototype.createCombo = function ()
{
  var appui = document.getElementById("app-ui");
  if (!this.comboObj) {
    this.comboObj = document.createElement("ion-modal");
    this.comboObj.className = "ion-modal-cmp show-page ion-autocomplete-cmp";
    //
    if (this.backdrop !== "none") {
      this.bdObj = document.createElement("ion-backdrop");
      this.bdObj.ontouchmove = function (ev) {
        ev.preventDefault();
      };
      this.comboObj.appendChild(this.bdObj);
    }
    //
    this.wrapperObj = document.createElement("div");
    this.wrapperObj.className = "modal-wrapper";
    this.comboObj.appendChild(this.wrapperObj);
    //
    this.listObj = document.createElement("ion-list");
    this.wrapperObj.appendChild(this.listObj);
    //
    appui.appendChild(this.comboObj);
    //
    if (this.backdrop === "all") {
      this.combo2 = document.createElement("ion-modal");
      this.combo2.className = "ion-modal-cmp show-page ion-autocomplete-cmp";
      this.bd2 = document.createElement("ion-backdrop");
      this.bd2.ontouchmove = function (ev) {
        ev.preventDefault();
      };
      this.combo2.appendChild(this.bd2);
      //
      appui.appendChild(this.combo2);
    }
  }
  //
  // empty the container
  while (this.listObj.firstChild) {
    this.listObj.firstChild.remove();
  }
  //
  // Locale container
  var obj = (Client.IonItem && this.parent instanceof Client.IonItem) ? this.parent.domObj : this.outerObj;
  //
  var pageobj = appui;
  var t = obj;
  while (t) {
    if (t.tagName === "ION-MODAL" || t.tagName === "ION-PAGE") {
      pageobj = t;
      break;
    }
    t = t.parentNode;
  }
  // Positioning is different inside a modal view
  var isApp = true;
  t = obj;
  while (t) {
    if (t.tagName === "ION-MODAL") {
      isApp = false;
      break;
    }
    t = t.parentNode;
  }
  //
  var rItem = obj.getBoundingClientRect();
  var rApp = pageobj.getBoundingClientRect();
  var pox = 0;
  var poy = 0;
  //
  // Calcolo offset left di pageobj rispetto ad appui
  if (pageobj.tagName === "ION-PAGE" && isApp) {
    var rui = appui.getBoundingClientRect();
    pox = rApp.left - rui.left;
    poy = rApp.top - rui.top;
  }
  //
  var h1 = (rApp.bottom - rItem.bottom);
  var h2 = (rItem.top - rApp.top);
  if (h2 > h1) {
    this.comboObj.style.top = "";
    this.comboObj.style.bottom = (rItem.top - 1) + "px";
    this.comboObj.style.height = h2 + "px";
    this.wrapperObj.classList.add("ion-autocomplete-up");
    //
    if (this.combo2) {
      this.combo2.style.bottom = "";
      this.combo2.style.top = (rItem.bottom + 1) + "px";
      this.combo2.style.height = h1 + "px";
    }
  }
  else {
    this.comboObj.style.bottom = "";
    this.comboObj.style.top = (rItem.bottom + 1) + "px";
    this.comboObj.style.height = h1 + "px";
    this.wrapperObj.classList.remove("ion-autocomplete-up");
    //
    if (this.combo2) {
      this.combo2.style.top = "";
      this.combo2.style.bottom = (rItem.top - 1) + "px";
      this.combo2.style.height = h2 + "px";
    }
  }
  //
  this.comboObj.style.left = (rItem.left - (isApp ? rApp.left - pox : 0)) + "px";
  this.comboObj.style.width = rItem.width + "px";
  if (this.combo2) {
    this.combo2.style.left = (rItem.left - (isApp ? rApp.left - pox : 0)) + "px";
    this.combo2.style.width = rItem.width + "px";
  }
  //
  this.comboObj.style.display = "block";
  if (this.combo2)
    this.combo2.style.display = "block";
};


/**
 * create the combo
 */
Client.IonAutoComplete.prototype.createComboItem = function (value, index, up)
{
  var item = document.createElement("ion-item");
  item.className = "item item-radio";
  var inner = document.createElement("div");
  inner.className = "item-inner";
  item.appendChild(inner);
  //
  var w = document.createElement("div");
  w.className = "input-wrapper";
  inner.appendChild(w);
  var l = document.createElement("ion-label");
  l.textContent = value.n;
  //
  // detail
  if (value.d) {
    var p = document.createElement("p");
    p.textContent = value.d;
    l.appendChild(p);
  }
  w.appendChild(l);
  //
  // style
  if (value.s) {
    if (value.s.indexOf(":") > -1) {
      item.style.cssText = value.s;
      l.style.cssText = value.s;
    }
    else
      item.classList.add(value.s);
  }
  //
  if (value.src) {
    if (value.src.substring(0, 4) === "ion:") {
      var img = document.createElement("ion-icon");
      img.setAttribute("item-left", "");
      img.className = "item-icon " + Client.IonHelper.getIconClass(value.src.substring(4));
      inner.insertBefore(img, w);
      if (value.v === this.value)
        img.setAttribute("primary", "");
    }
    else {
      var ava = document.createElement("ion-avatar");
      ava.setAttribute("item-left", "");
      var img = document.createElement("img");
      img.src = value.src;
      ava.appendChild(img);
      inner.insertBefore(ava, w);
    }
  }
  //
  var r = document.createElement("ion-radio");
  //
  var i = document.createElement("div");
  i.className = "radio-icon";
  r.appendChild(i);
  var ii = document.createElement("div");
  ii.className = "radio-inner";
  i.appendChild(ii);
  //
  var b = document.createElement("button");
  b.className = "item-cover disable-hover item-cover-default";
  b.setAttribute("value", value.v);
  if (value.new)
    b.setAttribute("new", true);
  r.appendChild(b);
  //
  if (value.v === this.value) {
    item.className += " item-radio-checked";
    i.className += " radio-checked";
    r.checked = true;
  }
  //
  w.appendChild(r);
  //
  if (up)
    this.listObj.insertBefore(item, this.listObj.firstChild);
  else
    this.listObj.appendChild(item);
  //
  Client.IonHelper.registerClickListener(this, b);
};


/**
 * create the combo
 * @param {event} ev
 */
Client.IonAutoComplete.prototype.myClick = function (ev)
{
  var v = ev.currentTarget.getAttribute("value");
  if (ev.currentTarget.getAttribute("new"))
    ev.new = true;
  this.setValue(v, true, ev);
  this.closeCombo();
};


/**
 * close the combo box
 */
Client.IonAutoComplete.prototype.closeCombo = function ()
{
  this.setValue(this.value);
  if (document.activeElement !== this.domObj) {
    if (Client.IonItem && this.parent instanceof Client.IonItem)
      this.parent.domObj.classList.remove("input-has-focus");
  }
  //
  this.valueHasChanged = false;
  this.comboOpen = false;
  //
  if (this.comboObj) {
    if (this.bd2)
      this.bd2.style.opacity = "";
    if (this.bdObj)
      this.bdObj.style.opacity = "";
    this.listObj.style.opacity = "";
    setTimeout(function () {
      this.comboObj.style.display = "none";
      if (this.combo2)
        this.combo2.style.display = "none";
    }.bind(this), 250);
  }
};


/**
 * commit a value
 * @param {event} ev
 */
Client.IonAutoComplete.prototype.setValue = function (value, emitChange, ev)
{
  var ris = false;
  //
  if (value === undefined)
    value = "";
  //
  this.value = value;
  //
  if (value && this.valueList) {
    for (var i = 0; i < this.valueList.length; i++) {
      if (this.valueList[i].v === value) {
        ris = true;
        this.domObj.value = this.valueList[i].n;
        if (this.parent instanceof Client.IonItem)
          this.parent.domObj.classList.add("input-has-value");
      }
    }
  }
  else {
    if (this.parent instanceof Client.IonItem)
      this.parent.domObj.classList.toggle("input-has-value", this.domObj.value);
  }
  if (emitChange) {
    // Save value as filter to prepare for dismission
    if (ev.new)
      this.filter = value;
    //
    var e = [{obj: this.id, id: "chgProp", content: {name: "value", value: value, clid: Client.id}}];
    if (this.sendChanges) {
      var er = this.saveEvent(ev);
      if (ev.new)
        er.newValue = true;
      e.push({obj: this.id, id: "onChange", content: er});
    }
    if (!Client.mainFrame.isEditing())
      Client.mainFrame.sendEvents(e);
  }
  return ris;
};


/**
 * Update the options in the combobox
 */
Client.IonAutoComplete.prototype.updateCombo = function ()
{
  if (this.isOpen())
    this.openCombo(this.fullCombo ? "" : this.domObj.value);
  else
    this.setValue(this.value);
  //
  this.comboTimer = 0;
  this.fullCombo = false;
};


/**
 * Returns true if the combo is open
 */
Client.IonAutoComplete.prototype.isOpen = function ()
{
  return this.comboOpen;
};



/**
 * Key Press of the Input (CanFilter Combo)
 * @param {Event} ev
 */
Client.IonAutoComplete.prototype.onInputKeyDown = function (ev)
{
  var k = ev.keyCode;
  //
  // Enter
  if (k === 13) {
    this.accept();
    this.domObj.blur();
  }
  // ESC
  if (k === 27) {
    this.dismiss();
  }
};


/**
 * Restore previuos value / filter
 */
Client.IonAutoComplete.prototype.dismiss = function ()
{
  if (this.filter) {
    this.domObj.value = this.filter;
    if (this.deleteFilterAfterDismiss)
      delete this.filter;
  }
  else {
    if (!this.setValue(this.value))
      this.domObj.value = "";
  }
  //
  delete this.deleteFilterAfterDismiss;
  this.closeCombo();
  this.domObj.blur();
};


/**
 * Accept first value if any
 */
Client.IonAutoComplete.prototype.accept = function ()
{
  if (this.isOpen()) {
    var b = this.comboObj.getElementsByTagName("BUTTON");
    if (b && b.length)
      this.myClick({currentTarget: b[0]});
  }
  this.closeCombo();
};


/**
 * This object has been hidden or shown
 * @param {Boolean} visible
 */
Client.IonAutoComplete.prototype.visibilityChanged = function (visible)
{
  // The label is outside the input: update label visibility
  if (this.labelObj)
    this.labelObj.style.display = this.getRootObject().style.display;
  Client.Element.prototype.visibilityChanged.call(this, visible);
};
