/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client */

/**
 * @class A container for a checkbox
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.IonCheckbox = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("ion-checkbox");
  //
  this.iconObj = document.createElement("div");
  this.iconObj.className = "checkbox-icon";
  this.innerObj = document.createElement("div");
  this.innerObj.className = "checkbox-inner";
  this.iconObj.appendChild(this.innerObj);
  this.domObj.appendChild(this.iconObj);
  //
  this.coverObj = document.createElement("button");
  this.coverObj.className = "item-cover disable-hover item-cover-default";
  Client.IonHelper.registerClickListener(this, this.coverObj);
  this.domObj.appendChild(this.coverObj);
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
};

Client.IonCheckbox.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.IonCheckbox.prototype.updateElement = function (el)
{
  if (el.color !== undefined) {
    if (this.color)
      this.domObj.removeAttribute(this.color);
    this.color = el.color;
    if (this.color)
      this.domObj.setAttribute(this.color, "");
    delete el.color;
  }
  if (el.checked !== undefined) {
    if (typeof el.checked === "string")
      el.checked = (el.checked === "true");
    if (el.checked)
      this.iconObj.classList.add("checkbox-checked");
    else
      this.iconObj.classList.remove("checkbox-checked");
  }
  if (el.disabled !== undefined) {
    if (el.disabled)
      this.domObj.classList.add("checkbox-disabled");
    else
      this.domObj.classList.remove("checkbox-disabled");
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};

/**
 * Attach events handler
 * @param {Array} events - array of the events to handle
 */
Client.IonCheckbox.prototype.attachEvents = function (events)
{
  if (!events)
    return;
  //
  var pos = events.indexOf("onChange");
  if (pos >= 0) {
    events.splice(pos, 1);
    this.sendOnChange = true;
  }
  //
  Client.Element.prototype.attachEvents.call(this, events);
};

/**
 * Checkbox clicked!
 * @param {Object} ev - event
 */
Client.IonCheckbox.prototype.myClick = function (ev)
{
  if (this.domObj.disabled)
    return;
  //
  Client.IonHelper.hapticFeedback();
  //
  var status = !this.domObj.checked;
  this.updateElement({checked: status});
  //
  var propName = "checked";
  var x = [{obj: this.id, id: "chgProp", content: {name: propName, value: this.domObj[propName], clid: Client.id}}];
  if (this.sendOnChange)
    x.push({obj: this.id, id: "onChange", content: this.saveEvent(ev)});
  Client.mainFrame.sendEvents(x);
};



/**
 * @class A container for a radio button
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.IonRadio = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("ion-radio");
  //
  this.iconObj = document.createElement("div");
  this.iconObj.className = "radio-icon";
  this.innerObj = document.createElement("div");
  this.innerObj.className = "radio-inner";
  this.iconObj.appendChild(this.innerObj);
  this.domObj.appendChild(this.iconObj);
  //
  this.coverObj = document.createElement("button");
  this.coverObj.className = "item-cover disable-hover item-cover-default";
  Client.IonHelper.registerClickListener(this, this.coverObj);
  this.domObj.appendChild(this.coverObj);
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
};

Client.IonRadio.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.IonRadio.prototype.updateElement = function (el)
{
  if (el.color !== undefined) {
    if (this.color)
      this.domObj.removeAttribute(this.color);
    this.color = el.color;
    if (this.color)
      this.domObj.setAttribute(this.color, "");
    delete el.color;
  }
  if (el.checked !== undefined) {
    var wc = this.domObj.getAttribute("checked");
    if (el.checked) {
      this.iconObj.classList.add("radio-checked");
      this.domObj.setAttribute("checked", true);
    }
    else {
      this.iconObj.classList.remove("radio-checked");
      this.domObj.removeAttribute("checked");
    }
    //
    if (this.parent instanceof Client.IonItem) {
      if (el.checked)
        this.parent.domObj.classList.add("item-radio-checked");
      else
        this.parent.domObj.classList.remove("item-radio-checked");
    }
    //
    // remove any other radio checked in the same group
    if (el.checked && !wc) {
      var p = this.parent;
      while (p) {
        if (p.domObj && p.domObj.getAttribute("radio-group"))
          break;
        p = p.parent;
      }
      if (p) {
        p.updateElement({value: this.domObj.value});
      }
    }
    //
    delete el.checked;
  }
  //
  if (el.disabled !== undefined) {
    if (el.disabled)
      this.domObj.classList.add("radio-disabled");
    else
      this.domObj.classList.remove("radio-disabled");
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};

/**
 * Attach events handler
 * @param {Array} events - array of the events to handle
 */
Client.IonRadio.prototype.attachEvents = function (events)
{
  if (!events)
    return;
  //
  var pos = events.indexOf("onChange");
  if (pos >= 0) {
    events.splice(pos, 1);
    this.sendOnChange = true;
  }
  //
  Client.Element.prototype.attachEvents.call(this, events);
};

/**
 * Checkbox clicked!
 * @param {Object} ev - event
 */
Client.IonRadio.prototype.myClick = function (ev)
{
  if (this.domObj.disabled)
    return;
  //
  this.updateElement({checked: true});
  //
  var propName = "checked";
  var x = [{obj: this.id, id: "chgProp", content: {name: propName, value: this.domObj[propName], clid: Client.id}}];
  if (this.sendOnChange)
    x.push({obj: this.id, id: "onChange", content: this.saveEvent(ev)});
  Client.mainFrame.sendEvents(x);
};


/**
 * @class A container for a checkbox
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.IonToggle = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("ion-toggle");
  //
  this.iconObj = document.createElement("div");
  this.iconObj.className = "toggle-icon";
  this.innerObj = document.createElement("div");
  this.innerObj.className = "toggle-inner";
  this.iconObj.appendChild(this.innerObj);
  this.domObj.appendChild(this.iconObj);
  //
  this.coverObj = document.createElement("button");
  this.coverObj.className = "item-cover disable-hover item-cover-default";
  Client.IonHelper.registerClickListener(this, this.coverObj);
  this.domObj.appendChild(this.coverObj);
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
};

Client.IonToggle.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.IonToggle.prototype.updateElement = function (el)
{
  if (el.color !== undefined) {
    if (this.color)
      this.domObj.removeAttribute(this.color);
    this.color = el.color;
    if (this.color)
      this.domObj.setAttribute(this.color, "");
    delete el.color;
  }
  if (el.checked !== undefined) {
    if (el.checked)
      this.iconObj.classList.add("toggle-checked");
    else
      this.iconObj.classList.remove("toggle-checked");
  }
  if (el.disabled !== undefined) {
    if (el.disabled)
      this.domObj.classList.add("toggle-disabled");
    else
      this.domObj.classList.remove("toggle-disabled");
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};

/**
 * Attach events handler
 * @param {Array} events - array of the events to handle
 */
Client.IonToggle.prototype.attachEvents = function (events)
{
  if (!events)
    return;
  //
  var pos = events.indexOf("onChange");
  if (pos >= 0) {
    events.splice(pos, 1);
    this.sendOnChange = true;
  }
  //
  Client.Element.prototype.attachEvents.call(this, events);
};

/**
 * Checkbox clicked!
 * @param {Object} ev - event
 */
Client.IonToggle.prototype.myClick = function (ev)
{
  if (this.domObj.disabled)
    return;
  //
  Client.IonHelper.hapticFeedback();
  //
  var status = !this.domObj.checked;
  this.updateElement({checked: status});
  //
  var propName = "checked";
  var x = [{obj: this.id, id: "chgProp", content: {name: propName, value: this.domObj[propName], clid: Client.id}}];
  if (this.sendOnChange)
    x.push({obj: this.id, id: "onChange", content: this.saveEvent(ev)});
  Client.mainFrame.sendEvents(x);
};


/**
 * @class A container for an input
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.IonInput = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  if (element.type === "textarea") {
    this.outerObj = document.createElement("ion-textarea");
    this.domObj = document.createElement("textarea");
  }
  else {
    this.outerObj = document.createElement("ion-input");
    this.domObj = document.createElement("input");
  }
  this.domObj.className = "text-input";
  //
  this.outerObj.appendChild(this.domObj);
  //
  this.outerObj.addEventListener("click", function (ev) {
    this.domObj.focus();
  }.bind(this));
  //
  this.labelObj = document.createElement("ion-label");
  //
  this.clearObj = document.createElement("button");
  this.clearObj.className = "text-input-clear-icon disable-hover button button-clear want-click";
  //
  var pthis = this;
  this.clearObj.onmousedown = function (ev) {
    pthis.updateElement({value: ""});
    if (pthis.domObj.oninput)
      pthis.domObj.oninput(ev);
    if (pthis.domObj.onchange)
      pthis.domObj.onchange(ev);
    pthis.domObj.focus();
    return false;
  };
  this.clearObj.ontouchstart = function (ev) {
    pthis.updateElement({value: ""});
    if (pthis.domObj.oninput)
      pthis.domObj.oninput(ev);
    if (pthis.domObj.onchange)
      pthis.domObj.onchange(ev);
    pthis.domObj.focus();
    return false;
  };
  this.clearObj.style.display = "none";
  this.outerObj.appendChild(this.clearObj);
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.outerObj);
  this.outerObj.parentNode.insertBefore(this.labelObj, this.outerObj);
  //
  if (Client.IonItem && parent instanceof Client.IonItem) {
    parent.domObj.classList.add("item-input");
    this.domObj.addEventListener("focus", function (ev) {
      parent.domObj.classList.add("input-has-focus");
    });
    this.domObj.addEventListener("blur", function (ev) {
      parent.domObj.classList.remove("input-has-focus");
    });
    this.domObj.addEventListener("input", function (ev) {
      parent.domObj.classList.toggle("input-has-value", this.value);
    });
  }
  //
  // ENTER -> BLUR for inputs
  this.domObj.onkeyup = function (ev) {
    if (ev.which === 13 && this.domObj.tagName !== "TEXTAREA") {
      this.domObj.blur();
    }
  }.bind(this);
};

Client.IonInput.prototype = new Client.Input();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.IonInput.prototype.updateElement = function (el)
{
  if (el.className !== undefined) {
    this.outerObj.className = el.className;
    delete el.className;
  }
  if (el.label !== undefined) {
    this.labelObj.textContent = el.label;
    if (Client.IonItem && this.parent instanceof Client.IonItem) {
      this.parent.domObj.classList.add("item-label-fixed");
    }
    delete el.label;
  }
  if (el.labelPosition !== undefined) {
    if (this.labelPosition) {
      this.labelObj.removeAttribute(this.labelPosition);
      if (Client.IonItem && this.parent instanceof Client.IonItem)
        this.parent.domObj.classList.remove("item-label-" + this.labelPosition);
    }
    //
    this.labelPosition = el.labelPosition;
    if (el.labelPosition) {
      this.labelObj.setAttribute(this.labelPosition, "");
      if (Client.IonItem && this.parent instanceof Client.IonItem)
        this.parent.domObj.classList.add("item-label-" + this.labelPosition);
    }
  }
  if (el.color !== undefined) {
    if (this.color)
      this.labelObj.removeAttribute(this.color);
    this.color = el.color;
    if (this.color)
      this.labelObj.setAttribute(this.color, "");
    delete el.color;
  }
  if (el.value !== undefined) {
    if (Client.IonItem && this.parent instanceof Client.IonItem) {
      this.parent.domObj.classList.toggle("input-has-value", el.value + "");
    }
  }
  if (el.clear !== undefined) {
    this.clearObj.style.display = el.clear ? "" : "none";
  }
  //
  Client.Input.prototype.updateElement.call(this, el);
};


/**
 * Return the dom root object of this element
 * @returns {DomElement}
 */
Client.IonInput.prototype.getRootObject = function ()
{
  return this.outerObj;
};


/**
 * This object has been hidden or shown
 * @param {Boolean} visible
 */
Client.IonInput.prototype.visibilityChanged = function (visible)
{
  // The label is outside the input: update label visibility
  this.labelObj.style.display = this.getRootObject().style.display;
  Client.Element.prototype.visibilityChanged.call(this, visible);
};


/**
 * Set custom error message
 * @param {String} message - the error message
 * @param {bool} srv
 */
Client.IonInput.prototype.setError = function (message, srv)
{
  Client.Element.prototype.setError.call(this, message, srv);
  //
  setTimeout(function () {
    if (this.outerObj && this.outerObj.parentNode) {
      if (message)
        this.outerObj.parentNode.classList.add("is-invalid");
      else
        this.outerObj.parentNode.classList.remove("is-invalid");
    }
  }.bind(this), 10);
};
