/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client,componentHandler */

/**
 * @class Material Design Helper class
 */
Client.IonHelper = {};

// Register ionmodal as modal dialog class
Client.View.dialogClass = "IonModal";

// disable webview scrolling on keyboard open
Client.mainFrame.processRequest([{obj: "device-keyboard", id: "disableScroll", cnt: {disable: true}}]);

// handle cursor on ios scroll
(function () {
  if (Client.mainFrame.device.operatingSystem !== "ios" || Client.mainFrame.theme.dontHoldCaret === "true")
    return;
  //
  var appui = document.getElementById("app-ui");
  var th = Client.IonHelper;
  //
  appui.addEventListener("touchmove", function (ev) {
    var x = document.activeElement;
    if (x && !th.fakeInput && (x.tagName === "INPUT" || x.tagName === "TEXTAREA")) {
      //
      // Create a clone of the input being focused
      th.lastInput = x;
      th.fakeInput = x.cloneNode();
      x.parentNode.appendChild(th.fakeInput);
      //
      // Hide the cursor, without changing focus
      x.style.display = "none";
    }
  }, {passive: true, capture: true});
  //
  // Restore previous input
  var removeFI = function () {
    th.lastInput.style.display = "";
    th.lastInput.focus();
    th.lastInput = undefined;
    th.fakeInput.remove();
    th.fakeInput = undefined;
    th.fakeTimeout = undefined;
  };
  //
  var endFI = function (ev) {
    // Set a timeout to restore input
    if (th.fakeInput && !th.fakeTimeout) {
      th.fakeTimeout = setTimeout(removeFI, 500);
    }
  };
  //
  var checkFI = function (ev) {
    // Another input gained focus, restore immediately the previous one
    if (th.fakeInput) {
      var x = document.activeElement;
      if (x !== th.lastInput)
        removeFI();
    }
  };
  //
  appui.addEventListener("touchend", endFI, {passive: true, capture: true});
  appui.addEventListener("touchcancel", endFI, {passive: true, capture: true});
  //
  appui.addEventListener("focus", checkFI, {passive: true, capture: true});
  appui.addEventListener("blur", checkFI, {passive: true, capture: true});
  //
  appui.addEventListener("scroll", function (ev) {
    //
    // Re-engage timer to wait for scrolling to end
    if (th.fakeTimeout) {
      clearTimeout(Client.IonHelper.fakeTimeout);
      th.fakeTimeout = setTimeout(removeFI, 300);
    }
  }, {passive: true, capture: true});
  //
}());

/**
 * Redefine close popup function
 */
Client.MainFrame.prototype.closePopup = function ()
{
  var appui = document.getElementById("app-ui");
  var c = appui.getElementsByTagName("ION-BACKDROP");
  for (var i = 0; i < c.length; i++) {
    var s = c[i].parentNode.tagName;
    if (s === "ION-ACTION-SHEET" || s === "ION-ALERT" || s === "ION-LOADING" || s === "ION-POPOVER")
      c[i].click();
  }
};


/**
 * returns true if a popup is open
 */
Client.IonHelper.hasPopup = function ()
{
  var appui = document.getElementById("app-ui");
  var c = appui.getElementsByTagName("ION-BACKDROP");
  for (var i = 0; i < c.length; i++) {
    var s = c[i].parentNode.tagName;
    if (s === "ION-ACTION-SHEET" || s === "ION-ALERT" || s === "ION-LOADING") {
      return true;
    }
  }
};


/**
 * Register click/touch listener to a clickable ionic element
 * @param {Object} element
 */
Client.IonHelper.registerClickListener = function (element, domObj)
{
  if (!domObj)
    domObj = element.domObj;
  //
  domObj.setAttribute("click-delay", 40);
  //
  var pthis = this;
  domObj.addEventListener("mousedown", function (ev) {
    pthis.activate(element, ev, true, domObj);
  }, {passive: true});
  domObj.addEventListener("touchstart", function (ev) {
    pthis.activate(element, ev, true, domObj);
  }, {passive: true});
  domObj.addEventListener("mouseup", function (ev) {
    pthis.activate(element, ev, false, domObj);
  }, {passive: true});
  domObj.addEventListener("mouseleave", function (ev) {
    pthis.activate(element, ev, false, domObj);
  }, {passive: true});
  domObj.addEventListener("touchend", function (ev) {
    pthis.activate(element, ev, false, domObj);
  }, {passive: true});
  domObj.addEventListener("touchcancel", function (ev) {
    pthis.activate(element, ev, false, domObj);
  }, {passive: true});
  domObj.addEventListener("click", function (ev) {
    pthis.click(element, ev, domObj);
  }, false);
};


/**
 * Add Listener to pointer events
 * @param {HTMLElement} domobj
 * @param {any} options
 * @param {Object} element
 */
Client.IonHelper.registerPointerEvents = function (domobj, options, element, capture, passive)
{
  passive = passive || "";
  //
  domobj.addEventListener("mousedown", function (ev) {
    if (this.pointerStart)
      return this.pointerStart(ev, options);
  }.bind(element), {capture: capture, passive: passive.indexOf("ps") === -1});
  domobj.addEventListener("touchstart", function (ev) {
    if (this.pointerStart)
      return this.pointerStart(ev, options);
  }.bind(element), {capture: capture, passive: passive.indexOf("ps") === -1});
  domobj.addEventListener("mousemove", function (ev) {
    if (this.pointerMove)
      return this.pointerMove(ev, options);
  }.bind(element), {capture: capture, passive: passive.indexOf("pm") === -1});
  domobj.addEventListener("touchmove", function (ev) {
    if (this.pointerMove)
      return this.pointerMove(ev, options);
  }.bind(element), {capture: capture, passive: passive.indexOf("pm") === -1});
  domobj.addEventListener("mouseup", function (ev) {
    if (this.pointerEnd)
      return this.pointerEnd(ev, options);
  }.bind(element), {capture: capture, passive: passive.indexOf("pe") === -1});
  domobj.addEventListener("touchend", function (ev) {
    if (this.pointerEnd)
      return this.pointerEnd(ev, options);
  }.bind(element), {capture: capture, passive: passive.indexOf("pe") === -1});
  domobj.addEventListener("mouseout", function (ev) {
    if (this.pointerOut)
      return this.pointerOut(ev, options);
  }.bind(element), {capture: capture, passive: passive.indexOf("po") === -1});
  domobj.addEventListener("touchcancel", function (ev) {
    if (this.pointerOut)
      return this.pointerOut(ev, options);
  }.bind(element), {capture: capture, passive: passive.indexOf("po") === -1});
};


/**
 * Activate / Deactivate object
 * @param {Object} element
 * @param {Object} ev - event
 * @param {Object} status - bool
 */
Client.IonHelper.activate = function (element, ev, status, _clickObj)
{
  // No activation near scroll events
  if (status && Client.mainFrame.isClickPrevented())
    return;
  //
  if (status) {
    _clickObj.classList.add("activated");
    Client.mainFrame.registerPreventListener(element.id, function () {
      Client.IonHelper.activate(element, ev, false, _clickObj);
    });
  }
  else {
    _clickObj.classList.remove("activated");
    Client.mainFrame.unregisterPreventListener(element.id);
  }
};


/**
 * run click effect on button
 * @param {Object} element
 * @param {Object} ev - event
 */
Client.IonHelper.click = function (element, ev, _clickObj)
{
  // Click are disabled? No action
  if (Client.mainFrame.isClickPrevented())
    return;
  //
  var eff = _clickObj.getElementsByTagName("ion-button-effect");
  var effObj = eff ? eff[0] : undefined;
  if (!effObj) {
    effObj = document.createElement("ion-button-effect");
    _clickObj.appendChild(effObj);
  }
  //
  if (element.myClick)
    element.myClick(ev, _clickObj);
  //
  // ripple effect on click
  if (Client.Ionic.platform === "md") {
    //
    var startCoord = {x: ev.x, y: ev.y};
    //
    // move ripple element out
    effObj.style.left = '-9999px';
    effObj.style.opacity = '';
    effObj.style.transform = 'scale(0.001) translateZ(0px)';
    effObj.style.transition = '';
    //
    // Read dimensions
    var clientRect = _clickObj.getBoundingClientRect();
    var top = clientRect.top;
    var left = clientRect.left;
    var width = clientRect.width;
    var height = clientRect.height;
    //
    // Prepare ripple effect
    var clientPointerX = (startCoord.x - left);
    var clientPointerY = (startCoord.y - top);
    var x = Math.max(Math.abs(width - clientPointerX), clientPointerX) * 2;
    var y = Math.max(Math.abs(height - clientPointerY), clientPointerY) * 2;
    var diameter = Math.min(Math.max(Math.sqrt(Math.pow(x, 2) + Math.pow(y, 2)), 64), 240);
    var radius = Math.sqrt(width + height);
    var scaleTransitionDuration = Math.max(1600 * Math.sqrt(radius / 300) + 0.5, 260);
    var opacityTransitionDuration = scaleTransitionDuration * 0.7;
    var opacityTransitionDelay = scaleTransitionDuration - opacityTransitionDuration;
    //
    // Start ripple
    effObj.style.width = effObj.style.height = diameter + 'px';
    effObj.style.marginTop = effObj.style.marginLeft = -(diameter / 2) + 'px';
    effObj.style.left = clientPointerX + 'px';
    effObj.style.top = clientPointerY + 'px';
    effObj.style.opacity = '0';
    effObj.style.transform = 'scale(1) translateZ(0px)';
    effObj.style.transition = 'transform ' + scaleTransitionDuration + 'ms,opacity ' +
            opacityTransitionDuration + 'ms ' + opacityTransitionDelay + 'ms';
  }
};


/**
 * Return the class class to apply for this icon
 * @param {string} icon
 */
Client.IonHelper.getIconClass = function (icon)
{
  if (!icon)
    return "";
  //
  // If in the icon name there is a space, the icon is not an ionic icon.
  // Don't add the prefix and use the icon name as is
  if (icon.indexOf(" ") > 0)
    return icon;
  //
  var prefix = "ion-";
  if (icon.substring(0, 3) !== "md-" && icon.substring(0, 4) !== "ios-") {
    prefix += Client.Ionic.platform + "-";
  }
  //
  return prefix + icon;
};


/**
 * Execute toast command
 * @param {object} options
 * @param {function} cb
 */
Client.IonHelper.createToast = function (options, cb)
{
  if (!options.position)
    options.position = "bottom";
  //
  var tw = document.createElement("div");
  tw.className = "toast-wrapper toast-" + options.position;
  //
  if (options.style) {
    if (options.style.indexOf(":") > -1)
      tw.style.cssText += options.style;
    else
      tw.classList.add(options.style);
  }
  //
  var tc = document.createElement("div");
  tc.className = "toast-container";
  tw.appendChild(tc);
  //
  var tm = document.createElement("div");
  tm.className = "toast-message";
  tm.textContent = options.message;
  tc.appendChild(tm);
  //
  if (options.showCloseButton) {
    var cbtn = document.createElement("button");
    cbtn.className = "toast-button disable-hover button button-clear";
    var sp = document.createElement("span");
    sp.className = "button-inner";
    sp.textContent = options.closeButtonText || "OK";
    cbtn.appendChild(sp);
    tc.appendChild(cbtn);
  }
  else if (!options.duration)
    options.duration = 3000;
  //
  var appui = document.getElementById("app-ui");
  if (options.position === "middle") {
    tw.opacity = "0.01";
    tw.style.top = Math.floor(appui.clientHeight / 2 - tw.clientHeight / 2) + "px";
  }
  appui.appendChild(tw);
  //
  // To enable immediate animation
  var r = tw.offsetTop;
  //
  // Enter transition
  var x = (Client.Ionic.platform === "md") ? 0 : 10;
  //
  tw.style.transition = "transform 400ms cubic-bezier(.36,.66,.04,1)";
  if (options.position === "bottom")
    tw.style.transform = "translateY(-" + x + "px)";
  if (options.position === "top")
    tw.style.transform = "translateY(" + x + "px)";
  if (options.position === "middle") {
    tw.style.transition = "opacity 400ms cubic-bezier(.36,.66,.04,1)";
    tw.style.opacity = "1";
  }
  //
  // Exit transition
  var f = function () {
    if (options.position === "middle")
      tw.style.opacity = "0.01";
    else
      tw.style.transform = "";
    setTimeout(function () {
      tw.remove();
      if (cb)
        cb();
    }, 500);
  };
  //
  if (options.showCloseButton) {
    cbtn.onclick = f;
  }
  else {
    setTimeout(f, options.duration);
  }
};


/**
 * Shows or hides loading status
 * @param {object} cnt
 */
Client.IonHelper.setLoading = function (options, cb)
{
  if (!options.id)
    options.id = "std-loading";
  //
  var lo = document.getElementById(options.id);
  if (lo) {
    var lc = lo.getElementsByClassName("loading-content")[0];
    var lw = lo.getElementsByClassName("loading-wrapper")[0];
    var bw = lo.getElementsByTagName("ION-BACKDROP")[0];
  }
  else if (!options.dismiss) {
    lo = document.createElement("ion-loading");
    lo.className = "loading-cmp";
    lo.id = options.id;
    //
    var bw = document.createElement("ion-backdrop");
    bw.tappable = "";
    bw.ontouchmove = function (ev) {
      ev.preventDefault();
    };
    lo.appendChild(bw);
    //
    var lw = document.createElement("div");
    lw.className = "loading-wrapper";
    lw.style.opacity = "0.01";
    lw.style.trasform = "scale(1.1)";
    lw.ontouchmove = function (ev) {
      ev.preventDefault();
    };
    lo.appendChild(lw);
    //
    if (options.showSpinner !== false) {
      var sp = document.createElement("div");
      sp.className = "loading-spinner";
      sp.innerHTML = Client.IonHelper.createSpinner();
      lw.appendChild(sp);
    }
    //
    var lc = document.createElement("div");
    lc.className = "loading-content";
    lw.appendChild(lc);
    //
    var appui = document.getElementById("app-ui");
    appui.appendChild(lo);
    //
    // To enable immediate animation
    var r = lo.offsetTop;
    //
    // Enter transition
    lw.style.transition = "transform 200ms ease-in-out, opacity 200ms ease-in-out";
    lw.style.opacity = "1";
    lw.style.trasform = "scale(1)";
    bw.style.transition = "opacity 200ms ease-in-out";
    if (options.showBackdrop !== false)
      bw.style.opacity = (Client.Ionic.platform === "md") ? "0.5" : "0.3";
  }
  //
  if (!lo)
    return;
  //
  if (options.style) {
    if (options.style.indexOf(":") > -1)
      lw.style.cssText += options.style;
    else
      lw.classList.add(options.style);
  }
  //
  // Message
  if (options.message)
    lc.innerHTML = options.message;
  //
  // Exit transition
  var f = function () {
    lw.style.opacity = "0";
    lw.style.transform = "scale(0.9)";
    bw.style.opacity = "0";
    setTimeout(function () {
      if (lo.parentNode)
        lo.remove();
      if (cb)
        cb();
    }, 250);
  };
  //
  if (options.duration) {
    clearTimeout(lo.tid);
    var tid = setTimeout(f, options.duration);
    lo.tid = tid;
  }
  else if (options.dismiss)
    f();
  else if (cb)
    cb();
};


/**
 * Execute alert command
 * @param {object} options
 * @param {function} cb
 */
Client.IonHelper.createAlert = function (options, cb)
{
  if (!options)
    options = {};
  //
  var bf = function (ev) {
    ev.stopImmediatePropagation();
    f();
  };
  window.addEventListener("popstate", bf, true);
  //
  var ao = document.createElement("ion-alert");
  ao.className = "alert-cmp" + (options.style ? " " + options.style : "");
  ao.tabindex = 0;
  //
  var bw = document.createElement("ion-backdrop");
  bw.tappable = "";
  bw.ontouchmove = function (ev) {
    ev.preventDefault();
  };
  ao.appendChild(bw);
  //
  var aw = document.createElement("div");
  aw.className = "alert-wrapper";
  aw.style.opacity = "0.01";
  aw.style.transform = "scale(1.1)";
  if (options.maxWidth)
    aw.style.maxWidth = options.maxWidth;
  ao.appendChild(aw);
  aw.ontouchmove = function (ev) {
    var obj = ev.srcElement;
    var canScroll = false;
    while (obj && obj !== aw && !canScroll) {
      canScroll = obj.scrollHeight > obj.offsetHeight;
      obj = obj.parentNode;
    }
    if (!canScroll)
      ev.preventDefault();
  };
  //
  var ah = document.createElement("div");
  ah.className = "alert-head";
  aw.appendChild(ah);
  //
  if (options.title) {
    var at = document.createElement("h2");
    at.className = "alert-title";
    at.innerHTML = options.title;
    ah.appendChild(at);
  }
  //
  if (options.subtitle) {
    var as = document.createElement("h3");
    as.className = "alert-sub-title";
    as.innerHTML = options.subtitle;
    ah.appendChild(as);
  }
  //
  if (options.message) {
    var am = document.createElement("div");
    am.className = "alert-message";
    am.innerHTML = options.message;
    aw.appendChild(am);
  }
  if (options.canFilter) {
    var filterToolbar = document.createElement("ion-toolbar");
    filterToolbar.className = "toolbar";
    var filterToolbarContent = document.createElement("div");
    filterToolbarContent.className = "toolbar-content";
    //
    filterToolbar.appendChild(filterToolbarContent);
    filterToolbarContent.appendChild(Client.IonHelper.createAlertSearchbar());
    aw.appendChild(filterToolbar);
  }
  //
  var inputToFocus;
  var buttonToFocus;
  if (options.inputs) {
    var ig, cg, rg, att;
    //
    var toScroll = undefined;
    for (var i = 0; i < options.inputs.length; i++) {
      var inp = options.inputs[i];
      //
      // Select group
      if (inp.type === "radio") {
        if (!rg) {
          rg = document.createElement("div");
          rg.className = "alert-radio-group";
          if (options.maxHeight)
            rg.style.maxHeight = options.maxHeight;
          aw.appendChild(rg);
        }
        att = rg;
      }
      else if (inp.type === "checkbox") {
        if (!cg) {
          cg = document.createElement("div");
          cg.className = "alert-checkbox-group";
          if (options.maxHeight)
            cg.style.maxHeight = options.maxHeight;
          aw.appendChild(cg);
        }
        att = cg;
      }
      else {
        if (!ig) {
          ig = document.createElement("div");
          ig.className = "alert-input-group";
          if (options.maxHeight)
            ig.style.maxHeight = options.maxHeight;
          aw.appendChild(ig);
        }
        att = ig;
      }
      //
      if (att === ig) {
        var wr = document.createElement("div");
        wr.className = "alert-input-wrapper" + (inp.label ? " label" : "") + (inp.class ? " " + inp.class : "");
        //
        if (inp.label) {
          var lb = document.createElement("ion-label");
          lb.className = "label-" + Client.Ionic.platform;
          lb.setAttribute("fixed", "");
          lb.textContent = inp.label;
          wr.appendChild(lb);
        }
        //
        var ie = document.createElement(inp.type === "textarea" ? "textarea" : "input");
        ie.className = "alert-input";
        ie.id = inp.id;
        if (inp.type) {
          if (inp.type === "textarea") {
            if (inp.rows)
              ie.rows = inp.rows;
          }
          else
            ie.type = inp.type;
        }
        if (inp.value)
          ie.value = inp.value;
        if (inp.placeholder)
          ie.placeholder = inp.placeholder;
        if (inp.focus)
          inputToFocus = ie;
        wr.appendChild(ie);
      }
      else {
        var wr = document.createElement("button");
        wr.className = "alert-tappable alert-" + inp.type + " disable-hover alert-" + inp.type + "-button alert-" + inp.type + "-button-default";
        wr.id = inp.id;
        if (inp.checked) {
          wr.setAttribute("aria-checked", true);
          if (!toScroll)
            toScroll = wr;
        }
        //
        wr.onclick = function (ev) {
          if (ev.currentTarget.className.indexOf("radio") > -1) {
            var el = ev.currentTarget.parentNode.getElementsByTagName("BUTTON");
            for (var i = 0; i < el.length; i++) {
              el[i].setAttribute("aria-checked", false);
            }
          }
          //
          var c = ev.currentTarget.getAttribute("aria-checked");
          ev.currentTarget.setAttribute("aria-checked", c !== "true");
          //
          Client.IonHelper.hapticFeedback();
        };
        if (inp.type === "radio") {
          wr.ondblclick = function (ev) {
            var c = bg.getElementsByTagName("BUTTON");
            if (c && c.length)
              c[c.length - 1].click();
          };
        }
        //
        var s = document.createElement("span");
        s.className = "button-inner";
        wr.appendChild(s);
        //
        var ic = document.createElement("div");
        ic.className = "alert-" + inp.type + "-icon";
        s.appendChild(ic);
        //
        var ii = document.createElement("div");
        ii.className = "alert-" + inp.type + "-inner";
        ic.appendChild(ii);
        //
        var l = document.createElement("div");
        l.className = "alert-" + inp.type + "-label";
        l.textContent = inp.label;
        s.appendChild(l);
        //
        if (options.canFilter)
          wr.setAttribute("label", inp.label);
      }
      //
      att.appendChild(wr);
      if (wr.tagName === "BUTTON")
        Client.IonHelper.registerClickListener(this, wr);
    }
  }
  //
  // Exit transition
  var f = function (ev) {
    window.removeEventListener("popstate", bf, true);
    var bn = ev ? parseInt(ev.currentTarget.id.substring(1)) : null;
    //
    var t1 = (Client.Ionic.platform === "md" && ev && (ev.x !== 0 || ev.y !== 0)) ? 250 : 0;
    var values;
    if (ig) {
      values = values || {};
      var c = ig.getElementsByTagName("INPUT");
      for (var i = 0; i < c.length; i++) {
        values[c[i].id] = c[i].value;
      }
      c = ig.getElementsByTagName("TEXTAREA");
      for (i = 0; i < c.length; i++) {
        values[c[i].id] = c[i].value;
      }
    }
    if (cg) {
      values = values || {};
      var c = cg.getElementsByTagName("BUTTON");
      for (var i = 0; i < c.length; i++) {
        values[c[i].id] = c[i].getAttribute("aria-checked") === "true";
      }
    }
    if (rg) {
      values = values || {};
      var c = rg.getElementsByTagName("BUTTON");
      for (var i = 0; i < c.length; i++) {
        if (c[i].getAttribute("aria-checked") === "true") {
          values.value = c[i].id;
        }
      }
    }
    setTimeout(function () {
      aw.style.opacity = "0";
      aw.style.transform = "scale(0.9)";
      bw.style.opacity = "0";
    }, t1);
    setTimeout(function () {
      if (ao.parentNode)
        ao.remove();
      if (cb)
        cb(bn, values);
    }, 250 + t1);
  };
  //
  bw.onclick = function () {
    f();
  };
  //
  if (options.buttons) {
    var bg = document.createElement("div");
    bg.className = "alert-button-group" + ((options.buttons.length > 2) ? " vertical" : "");
    //
    for (var i = 0; i < options.buttons.length; i++) {
      var btn = options.buttons[i];
      var b = document.createElement("button");
      b.className = "disable-hover alert-button " + ((btn.cssClass) ? "" + btn.cssClass : " alert-button-default" + (btn.destructive ? " alert-button-destructive " : ""));
      b.id = "b" + (btn.id || i);
      var s = document.createElement("span");
      s.className = "button-inner";
      s.textContent = btn.text || btn;
      b.appendChild(s);
      bg.appendChild(b);
      Client.IonHelper.registerClickListener(this, b);
      b.onclick = f;
      if (btn.focus)
        buttonToFocus = b;
    }
    aw.appendChild(bg);
  }
  //
  ao.onkeyup = function (ev) {
    if (ev.which === 27) {
      f();
    }
  };
  //
  var appui = document.getElementById("app-ui");
  appui.appendChild(ao);
  window.history.pushState("alert", "");
  //
  // To enable immediate transition
  var r = ao.offsetTop;
  //
  // Enter transition
  aw.style.transition = "transform 200ms ease-in-out, opacity 200ms ease-in-out";
  aw.style.opacity = "1";
  aw.style.transform = "scale(1)";
  bw.style.transition = "opacity 200ms ease-in-out";
  bw.style.opacity = (Client.Ionic.platform === "md") ? "0.5" : "0.3";
  //
  if (toScroll)
    toScroll.scrollIntoView(false);
  //
  setTimeout(function () {
    //
    if (inputToFocus)
      inputToFocus.focus();
    else if (buttonToFocus)
      buttonToFocus.focus();
    else if (bg) {
      var buttons = bg.getElementsByTagName("BUTTON");
      if (buttons.length)
        buttons[buttons.length - 1].focus();
    }
  }, 500);
};


/**
 * Execute alert command
 * @param {string} txt
 */
Client.MainFrame.prototype.alert = function (txt)
{
  var c = {title: "alert", message: txt, buttons: ["Ok"]};
  Client.IonHelper.createAlert(c, function () {
    //
    // This message makes sure that other client will close the popup accordingly
    var e = [{id: "popupBoxReturn", content: {}}];
    this.sendEvents(e);
  }.bind(this));
};


/**
 * Display a confirmation box
 * @param {String} opt - contains the text to display in the confirm box and the callback id
 */
Client.MainFrame.prototype.confirm = function (opt)
{
  var c = {title: "confirm", message: opt.txt, buttons: ["Cancel", "Ok"]};
  Client.IonHelper.createAlert(c, function (r) {
    var e = [{id: "popupBoxReturn", content: {res: r > 0, cbId: opt.cbId}}];
    this.sendEvents(e);
  }.bind(this));
};


/**
 * Display a prompt box
 * @param {Object} opt - contains the text to display in the prompt box and the default input text
 */
Client.MainFrame.prototype.prompt = function (opt)
{
  var c = {title: "prompt", message: opt.txt, buttons: ["Cancel", "Ok"], inputs: [{id: "prompt", value: opt.def, type: "text", focus: true}]};
  Client.IonHelper.createAlert(c, function (r, values) {
    var ris = (r > 0) ? values.prompt : "";
    var e = [{id: "popupBoxReturn", content: {res: ris, cbId: opt.cbId}}];
    this.sendEvents(e);
  }.bind(this));
};


/**
 * Execute alert command
 * @param {object} options
 * @param {function} cb
 */
Client.IonHelper.createActionSheet = function (options, cb)
{
  if (!options)
    options = {};
  //
  var bf = function (ev) {
    ev.stopImmediatePropagation();
    f();
  };
  window.addEventListener("popstate", bf, true);
  //
  var ao = document.createElement("ion-action-sheet");
  ao.className = "action-sheet-cmp";
  //
  var bw = document.createElement("ion-backdrop");
  bw.tappable = "";
  bw.ontouchmove = function (ev) {
    ev.preventDefault();
  };
  ao.appendChild(bw);
  //
  var aw = document.createElement("div");
  aw.className = "action-sheet-wrapper";
  ao.appendChild(aw);
  aw.ontouchmove = function (ev) {
    ev.preventDefault();
  };
  //
  var ac = document.createElement("div");
  ac.className = "action-sheet-container";
  aw.appendChild(ac);
  //
  var ag = document.createElement("div");
  ag.className = "action-sheet-group";
  ac.appendChild(ag);
  //
  if (options.title) {
    var at = document.createElement("div");
    at.className = "action-sheet-title";
    at.innerHTML = options.title;
    ag.appendChild(at);
  }
  //
  // Is subtitle supported in ionic?
  //if (options.subtitle) {
  //  var as = document.createElement("div");
  //  as.className = "action-sheet-sub-title";
  //  as.innerHTML = options.subtitle;
  //  ag.appendChild(as);
  //}
  //
  // Exit transition
  var f = function (ev) {
    window.removeEventListener("popstate", bf, true);
    var bn = ev ? parseInt(ev.currentTarget.id.substring(1)) : undefined;
    var t1 = (Client.Ionic.platform === "md" && ev && (ev.x !== 0 || ev.y !== 0)) ? 250 : 0;
    setTimeout(function () {
      aw.style.transform = "";
      bw.style.opacity = "";
    }, t1);
    setTimeout(function () {
      if (ao.parentNode)
        ao.remove();
      if (cb)
        cb(bn);
    }, 500 + t1);
  };
  //
  // Buttons
  if (options.buttons) {
    for (var i = 0; i < options.buttons.length; i++) {
      var btn = options.buttons[i];
      var b = document.createElement("button");
      b.className = "disable-hover " + (btn.destructive ? "action-sheet-destructive " : "") + "action-sheet-button " + ((btn.cssClass) ? "" + btn.cssClass : " action-sheet-button-default");
      b.id = "b" + (btn.id || i);
      Client.IonHelper.registerClickListener(this, b);
      b.onclick = f;
      //
      var s = document.createElement("span");
      s.className = "button-inner";
      //
      // Action sheet icons are supported only in android
      if (btn.icon && Client.Ionic.platform === "md") {
        var ic = document.createElement("ion-icon");
        ic.className = "action-sheet-icon " + Client.IonHelper.getIconClass(btn.icon);
        s.appendChild(ic);
      }
      //
      var t = document.createTextNode(btn.text || btn);
      s.appendChild(t);
      b.appendChild(s);
      //
      if (btn.cancel) {
        if (!cg) {
          var cg = document.createElement("div");
          cg.className = "action-sheet-group";
          ac.appendChild(cg);
        }
        cg.appendChild(b);
      }
      else {
        ag.appendChild(b);
      }
    }
  }
  //
  bw.onclick = function () {
    if (cg)
      cg.getElementsByTagName("BUTTON")[0].click();
    else
      f();
  };
  //
  var appui = document.getElementById("app-ui");
  appui.appendChild(ao);
  window.history.pushState("alert", "");
  //
  // To enable immediate transition
  var r = ao.offsetTop;
  //
  // Enter transition
  aw.style.transition = "transform 400ms cubic-bezier(.36,.66,.04,1)";
  aw.style.transform = "translateY(0%)";
  bw.style.transition = "opacity 400ms cubic-bezier(.36,.66,.04,1)";
  bw.style.opacity = (Client.Ionic.platform === "md") ? "0.26" : "0.4";
};


/**
 * Show a menu
 * @param {object} options
 * @param {function} cb
 */
Client.IonHelper.createMenu = function (options, cb)
{
  if (!options)
    options = {};
  //
  var bf = function (ev) {
    ev.stopImmediatePropagation();
    f();
  };
  window.addEventListener("popstate", bf, true);
  //
  // Use the view passed in the options, if any, or the last view opened
  var view = options.view ? Client.eleMap[options.view] : Client.mainFrame.views[Client.mainFrame.views.length - 1];
  //
  // Create the dialog element
  var dialog = new Client.IonModal({options: {autoclose: true, ionmenu: true, extcls: "ion-menu",
      ref: {id: options.refObj, whisker: true, whiskerSize: 14, position: options.position, offset: options.offset}}}, view, view);
  //
  // Detach dialog from view to prevent closing it on exit
  dialog.view = null;
  //
  // Create dialog content
  var inner = dialog.domObj;
  //
  var innerContent = document.createElement("ion-content");
  if (!Client.mainFrame.device.isMobile)
    innerContent.className = "has-scrollbar";
  inner.appendChild(innerContent);
  //
  var scroll = document.createElement("scroll-content");
  scroll.style.marginTop = "0px";
  scroll.style.marginBottom = "0px";
  innerContent.appendChild(scroll);
  //
  var list = document.createElement("ion-list");
  list.style.marginBottom = "0px";
  scroll.appendChild(list);
  //
  // Exit transition
  var f = function (ev, _clickObj) {
    //
    var obj = ev.srcElement;
    while (obj && obj.tagName !== "ION-LIST") {
      obj = obj.parentNode;
    }
    //
    ev.stopImmediatePropagation();
    //
    // Do not click on a disabled object
    if (!_clickObj && obj)
      return;
    //
    window.removeEventListener("popstate", bf, true);
    //
    setTimeout(function () {
      // Pass callback and elemId as parameters because menu popup is closed
      // after a transition and I want callback to be called at the end of that transition
      dialog.close(cb, _clickObj ? _clickObj.elemId : undefined);
    }, (Client.Ionic.platform === "md") ? 200 : 0);
  };
  //
  // Create the menu items
  if (options.items)
  {
    var myElement = {};
    myElement.myClick = f;
    //
    for (var i = 0; i < options.items.length; i++)
    {
      var item = options.items[i];
      //
      var button = document.createElement("button");
      button.className = "item";
      button.setAttribute("detail-none", true);
      button.elemId = item.id;
      if (item.disabled)
        button.setAttribute("disabled", true);
      //
      if (item.style) {
        if (item.style.indexOf(":") > -1)
          button.style.cssText += item.style;
        else
          button.classList.add(item.style);
      }
      //
      list.appendChild(button);
      Client.IonHelper.registerClickListener(myElement, button);
      //
      // If there is an icon, show it
      if (item.icon) {
        var icon = document.createElement("ion-icon");
        icon.className = Client.IonHelper.getIconClass(item.icon);
        icon.setAttribute("item-left", true);
        button.appendChild(icon);
      }
      //
      var itemInner = document.createElement("div");
      itemInner.className = "item-inner";
      button.appendChild(itemInner);
      //
      var itemWrapper = document.createElement("div");
      itemWrapper.className = "input-wrapper";
      itemInner.appendChild(itemWrapper);
      //
      var title = document.createElement("ion-label");
      title.setAttribute("item-left", true);
      itemWrapper.appendChild(title);
      //
      var titleText = document.createTextNode(item.title);
      title.appendChild(titleText);
      //
      if (item.subtitle)
      {
        var subtitle = document.createElement("p");
        subtitle.innerHtml = item.subtitle;
        subtitle.style.fontSize = "9pt";
        title.appendChild(subtitle);
      }
    }
  }
  //
  // Click outside the menu close it
  dialog.rootObj.onclick = f;
  //
  // Put the menu in the browser history
  window.history.pushState("alert", "");
  //
  // Position the menu
  dialog.positionElement();
};


/**
 * Open a standard popup
 * @param {object} cnt
 */
Client.MainFrame.prototype.popup = function (cnt)
{
  var options = cnt.options || {};
  //
  switch (options.type) {
    case "alert":
      Client.IonHelper.createAlert(options, function (r, values) {
        var ris = r;
        if (values) {
          ris = values;
          ris.button = r;
        }
        var e = [{id: "popupBoxReturn", content: {res: ris, cbId: cnt.cbId}}];
        this.sendEvents(e);
      }.bind(this));
      //
      break;

    case "actionsheet":
      Client.IonHelper.createActionSheet(options, function (r) {
        var e = [{id: "popupBoxReturn", content: {res: r, cbId: cnt.cbId}}];
        this.sendEvents(e);
      }.bind(this));
      //
      break;

    case "loading":
      Client.IonHelper.setLoading(options, function () {
        var e = [{id: "popupBoxReturn", content: {res: undefined, cbId: cnt.cbId}}];
        this.sendEvents(e);
      }.bind(this));
      //
      break;

    case "toast":
      Client.IonHelper.createToast(options, function () {
        var e = [{id: "popupBoxReturn", content: {res: undefined, cbId: cnt.cbId}}];
        this.sendEvents(e);
      }.bind(this));
      //
      break;
    case "menu":
      Client.IonHelper.createMenu(options, function (r) {
        var e = [{id: "popupBoxReturn", content: {res: r, cbId: cnt.cbId}}];
        this.sendEvents(e);
      }.bind(this));
      //
      break;
    default:
      // The standard implementation does nothing and returns undefined
      if (cnt.cbId) {
        var e = [{id: "popupBoxReturn", content: {res: undefined, cbId: cnt.cbId}}];
        this.sendEvents(e);
      }
      break;
  }
};


/**
 * Returns true if the object is an ionic input object
 * @param {Element} obj
 */
Client.IonHelper.isInput = function (obj)
{
  if (Client.IonInput && obj instanceof Client.IonInput)
    return true;
  if (Client.IonAutoComplete && obj instanceof Client.IonAutoComplete)
    return true;
  if (Client.IonSelect && obj instanceof Client.IonSelect)
    return true;
  if (Client.IonDateTime && obj instanceof Client.IonDateTime)
    return true;
  if (Client.IonToggle && obj instanceof Client.IonToggle)
    return true;
  if (Client.IonRadio && obj instanceof Client.IonRadio)
    return true;
  if (Client.IonCheckbox && obj instanceof Client.IonCheckbox)
    return true;
};


/**
 * Get the layer to resize
 * @param {Object} inputele
 */
Client.IonHelper.findLayer = function (inputele)
{
  var obj = inputele;
  var ctx;
  while (obj) {
    if (obj.tagName === "ION-ALERT" || obj.tagName === "ION-MODAL") {
      ctx = obj;
      break;
    }
    if (obj.tagName === "ION-PAGE") {
      ctx = obj;
      break;
    }
    obj = obj.parentNode;
  }
  return ctx;
};


/**
 * Get the layer to resize
 * @param {Object} inputele
 */
Client.IonHelper.findContent = function (inputele)
{
  var obj = inputele;
  var ctx;
  while (obj) {
    if (obj.tagName === "SCROLL-CONTENT") {
      ctx = obj;
      break;
    }
    obj = obj.parentNode;
  }
  return ctx;
};


/**
 * Trigger content recalculation
 * @param {Object} tag
 */
Client.IonHelper.triggerPositionContent = function (tag)
{
  if (!tag || !tag.id)
    return;
  //
  var ele = Client.eleMap[tag.id];
  //
  if (!ele || !ele.elements)
    return;
  //
  var go = true;
  while (go) {
    go = false;
    for (var i = 0; i < ele.ne(); i++) {
      var c = ele.elements[i];
      if (c instanceof Client.IonContent)
        c.positionContent();
      if (c instanceof Client.IonPage) {
        ele = c;
        go = true;
      }
    }
  }
};


/**
 * Keyboard changed... reset scroll list?
 * @param {bool} flag
 */
Client.IonHelper.onChangeKeyboardVisibility = function (flag)
{
  // Set or remove class
  var appui = document.getElementById("app-ui");
  appui.classList.toggle("keyboard-open", flag);
  //
  var th = Client.IonHelper;
  var kh = Client.mainFrame.device.keyboardHeight;
  //
  // Keyboard handling shoould occur only on iOS
  if (Client.mainFrame.device.operatingSystem !== "ios") {
    //
    // Android has a problem with blurring the active element on keyboard close
    if (!flag && document.activeElement)
      document.activeElement.blur();
    //
    // No full screen? android will resize by itself
    if (window.innerHeight !== screen.height && window.innerHeight !== screen.width) {
      //
      // In any case we need to recalc bottom margin because of a bottom tabbar or a fixed footer
      if (flag) {
        th.scrollLayer = th.findLayer(document.activeElement);
        th.triggerPositionContent(th.scrollLayer);
      }
      else {
        var sc = th.scrollLayer;
        th.scrollLayer = undefined;
        setTimeout(function () {
          th.triggerPositionContent(sc);
        }, 200);
      }
      return;
    }
  }
  //
  if (flag) {
    if (th.scrollLayer) {
      // keyboard was already open, maybe it has changed its height
      var delta = kh - th.keyboardHeight;
      if (delta !== 0) {
        var b = parseInt(th.scrollLayer.style.bottom);
        th.scrollLayer.style.transition = "none";
        th.scrollLayer.style.bottom = (b + delta) + "px";
        setTimeout(function () {
          th.scrollLayer.style.transition = "";
        }, 20);
        th.keyboardHeight = kh;
      }
      return;
    }
    //
    // Keyboard opened
    var re = document.activeElement.getBoundingClientRect().bottom;
    var rb = document.body.getBoundingClientRect().bottom;
    var amount = re - (rb - kh - 10);
    if (amount > 0) {
      var sc = th.findContent(document.activeElement);
      //
      if (sc) {
        var r = new Tweenable();
        var config = {
          from: {height: sc.scrollTop},
          to: {height: sc.scrollTop + kh},
          duration: 400,
          easing: "easeOutQuart",
          step: function (state) {
            sc.scrollTop = state.height;
          }
        };
        r.tween(config);
      }
    }
    //
    th.scrollLayer = th.findLayer(document.activeElement);
    if (th.scrollLayer) {
      //
      // Keyboard opened: recalc bottom margin to handle bottom tabbars and fixed footers
      th.triggerPositionContent(th.scrollLayer);
      //
      // Shrink the page up to make room for the keyboard
      var rc = th.scrollLayer.getBoundingClientRect().bottom;
      th.scrollLayer.style.bottom = (kh - (rb - rc)) + "px";
      th.keyboardHeight = kh;
    }
  }
  else {
    // Keyboard closed: recalc bottom margin to handle bottom tabbars and fixed footers
    var sc = th.scrollLayer;
    th.scrollLayer = undefined;
    th.keyboardHeight = undefined;
    if (sc) {
      // Un-shrink the page
      sc.style.bottom = "";
      //
      setTimeout(function () {
        th.triggerPositionContent(sc);
      }, 200);
    }
  }
};

Client.mainFrame.device.onChangeKeyboardVisibility = Client.IonHelper.onChangeKeyboardVisibility;


/**
 * Returns true if the toolbar needs a padding class
 * @param {element} obj
 */
Client.IonHelper.needsPadding = function (obj)
{
  var ris = ((window.innerHeight === screen.height || window.innerHeight === screen.width));
  if (ris) {
    var el = obj.domObj, offsetTop = 0;
    do {
      offsetTop += el.offsetTop;
      el = el.offsetParent;
    }
    while (el);
    if (offsetTop > 20)
      ris = false;
  }
  return ris;
};


/**
 * Update the ionic colors with the ones defined in the theme
 */
Client.IonHelper.updateCssRules = function ()
{
  if (this.cssUpdated)
    return;
  //
  if (!Client.mainFrame.theme) {
    console.error("Client theme not found");
    return;
  }
  //
  this.cssUpdated = true;
  //
  var colorsMap = {};
  if (Client.mainFrame.theme.primary && Client.mainFrame.theme.primary !== "#387EF5" && Client.mainFrame.theme.primary !== "rgb(56, 126, 245)")
  {
    colorsMap["rgb(56, 126, 245)"] = Client.mainFrame.theme.primary;
    colorsMap["rgb(53, 120, 233)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.primary, -0.05);
    colorsMap["rgb(52, 116, 225)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.primary, -0.1);
    colorsMap["rgb(12, 96, 238)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.primary, -0.2);
  }
  if (Client.mainFrame.theme.secondary && Client.mainFrame.theme.secondary !== "#32DB64" && Client.mainFrame.theme.secondary !== "rgb(50, 219, 100)")
  {
    colorsMap["rgb(50, 219, 100)"] = Client.mainFrame.theme.secondary;
    colorsMap["rgb(48, 208, 95)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.secondary, -0.05);
    colorsMap["rgb(46, 201, 92)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.secondary, -0.1);
    colorsMap["rgb(33, 185, 78)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.secondary, -0.2);
  }
  if (Client.mainFrame.theme.danger && Client.mainFrame.theme.danger !== "#F53D3D" && Client.mainFrame.theme.danger !== "rgb(245, 61, 61)")
  {
    colorsMap["rgb(245, 61, 61)"] = Client.mainFrame.theme.danger;
    colorsMap["rgb(233, 58, 58)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.danger, -0.05);
    colorsMap["rgb(225, 56, 56)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.danger, -0.1);
    colorsMap["rgb(243, 13, 13)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.danger, -0.2);
  }
  if (Client.mainFrame.theme.light && Client.mainFrame.theme.light !== "#F4F4F4" && Client.mainFrame.theme.light !== "rgb(244, 244, 244)")
  {
    colorsMap["rgb(244, 244, 244)"] = Client.mainFrame.theme.light;
    colorsMap["rgb(232, 232, 232)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.light, -0.05);
    colorsMap["rgb(224, 224, 224)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.light, -0.1);
    colorsMap["rgb(219, 219, 219)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.light, -0.2);
  }
  if (Client.mainFrame.theme.dark && Client.mainFrame.theme.dark !== "#222222" && Client.mainFrame.theme.dark !== "rgb(34, 34, 34)")
  {
    colorsMap["rgb(34, 34, 34)"] = Client.mainFrame.theme.dark;
    colorsMap["rgb(45, 45, 45)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.dark, 0.05);
    colorsMap["rgb(52, 52, 52)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.dark, 0.1);
    colorsMap["rgb(9, 9, 9)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.dark, 0.2);
  }
  if (Client.mainFrame.theme.bright && Client.mainFrame.theme.bright !== "#FFC125" && Client.mainFrame.theme.bright !== "rgb(255, 193, 37)")
  {
    colorsMap["rgb(255, 193, 37)"] = Client.mainFrame.theme.bright;
    colorsMap["rgb(242, 183, 35)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.bright, -0.05);
    colorsMap["rgb(235, 178, 34)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.bright, -0.1);
    colorsMap["rgb(241, 172, 0)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.bright, -0.2);
  }
  if (Client.mainFrame.theme.vibrant && Client.mainFrame.theme.vibrant !== "#663399" && Client.mainFrame.theme.vibrant !== "rgb(102, 51, 153)" && Client.mainFrame.theme.primary !== "rebeccapurple")
  {
    colorsMap["rgb(102, 51, 153)"] = Client.mainFrame.theme.vibrant;
    colorsMap["rebeccapurple"] = Client.mainFrame.theme.vibrant;
    colorsMap["rgb(110, 61, 158)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.vibrant, 0.05);
    colorsMap["rgb(114, 67, 161)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.vibrant, 0.1);
    colorsMap["rgb(77, 38, 115)"] = Client.IonHelper.shadeRGBColor(Client.mainFrame.theme.vibrant, 0.2);
  }
  //
  // If no color has changed there is nothing to do
  if (Object.keys(colorsMap).length === 0)
    return;
  //
  // Update css rules
  for (var i = 0; i < document.styleSheets.length; i++)
  {
    var cs = "";
    if (document.styleSheets[i]["rules"])
      cs = "rules";
    if (document.styleSheets[i]["cssRules"])
      cs = "cssRules";
    var ar = document.styleSheets[i][cs];
    if (!ar)
      continue;
    //
    for (var j = 0; j < ar.length; j++)
    {
      // Just the rules containing STYLE
      var ru = ar[j];
      if (!ru.style)
        continue;
      //
      var keys = Object.keys(colorsMap);
      var s = ru.style.cssText;
      var ns = s;
      for (var k = 0; k < keys.length; k++)
      {
        var oldValue = keys[k].replace("(", "\\(").replace(")", "\\)");
        var re = new RegExp(oldValue, 'g');
        ns = ns.replace(re, colorsMap[keys[k]]);
      }
      //
      if (s !== ns)
        ru.style.cssText = ns;
    }
  }
};


/**
 * Make a color lighter or darker
 * @param {type} color
 * @param {type} percent
 * @returns {String}
 */
Client.IonHelper.shadeRGBColor = function (color, percent)
{
  if (color.indexOf("#") === 0) {
    var f = parseInt(color.slice(1), 16);
    var t = percent < 0 ? 0 : 255;
    var p = percent < 0 ? percent * -1 : percent;
    var r = f >> 16;
    var g = f >> 8 & 0x00FF;
    var b = f & 0x0000FF;
    return "#" + (0x1000000 + (Math.round((t - r) * p) + r) * 0x10000 + (Math.round((t - g) * p) + g) * 0x100 + (Math.round((t - b) * p) + b)).toString(16).slice(1);
  }
  else {
    var f = color.split(",");
    var t = percent < 0 ? 0 : 255;
    var p = percent < 0 ? percent * -1 : percent;
    var r = parseInt(f[0].slice(4));
    var g = parseInt(f[1]);
    var b = parseInt(f[2]);
    return "rgb(" + (Math.round((t - r) * p) + r) + "," + (Math.round((t - g) * p) + g) + "," + (Math.round((t - b) * p) + b) + ")";
  }
};


/**
 * Return the ionic spinner html
 * @returns {String}
 */
Client.IonHelper.createSpinner = function ()
{
  var spinnerHtml = "";
  if (Client.Ionic.platform === "md")
    spinnerHtml = '<ion-spinner class="spinner-crescent"><svg viewBox="0 0 64 64" style="animation-duration: 750ms;"><circle transform="translate(32,32)" r="26"></circle></svg></ion-spinner>';
  else
    spinnerHtml = '<ion-spinner class="spinner-ios"><svg viewBox="0 0 64 64" style="transform: rotate(180deg); animation-delay: -1000ms; animation-duration: 1000ms;">' +
            '<line transform="translate(32,32)" y1="17" y2="29"></line>' +
            '</svg><svg viewBox="0 0 64 64" style="transform: rotate(210deg); animation-delay: -916.667ms; animation-duration: 1000ms;">' +
            '<line transform="translate(32,32)" y1="17" y2="29"></line>' +
            '</svg><svg viewBox="0 0 64 64" style="transform: rotate(240deg); animation-delay: -833.333ms; animation-duration: 1000ms;">' +
            '<line transform="translate(32,32)" y1="17" y2="29"></line>' +
            '</svg><svg viewBox="0 0 64 64" style="transform: rotate(270deg); animation-delay: -750ms; animation-duration: 1000ms;">' +
            '<line transform="translate(32,32)" y1="17" y2="29"></line>' +
            '</svg><svg viewBox="0 0 64 64" style="transform: rotate(300deg); animation-delay: -666.667ms; animation-duration: 1000ms;">' +
            '<line transform="translate(32,32)" y1="17" y2="29"></line>' +
            '</svg><svg viewBox="0 0 64 64" style="transform: rotate(330deg); animation-delay: -583.333ms; animation-duration: 1000ms;">' +
            '<line transform="translate(32,32)" y1="17" y2="29"></line>' +
            '</svg><svg viewBox="0 0 64 64" style="transform: rotate(0deg); animation-delay: -500ms; animation-duration: 1000ms;">' +
            '<line transform="translate(32,32)" y1="17" y2="29"></line>' +
            '</svg><svg viewBox="0 0 64 64" style="transform: rotate(30deg); animation-delay: -416.667ms; animation-duration: 1000ms;">' +
            '<line transform="translate(32,32)" y1="17" y2="29"></line>' +
            '</svg><svg viewBox="0 0 64 64" style="transform: rotate(60deg); animation-delay: -333.333ms; animation-duration: 1000ms;">' +
            '<line transform="translate(32,32)" y1="17" y2="29"></line>' +
            '</svg><svg viewBox="0 0 64 64" style="transform: rotate(90deg); animation-delay: -250ms; animation-duration: 1000ms;">' +
            '<line transform="translate(32,32)" y1="17" y2="29"></line>' +
            '</svg><svg viewBox="0 0 64 64" style="transform: rotate(120deg); animation-delay: -166.667ms; animation-duration: 1000ms;">' +
            '<line transform="translate(32,32)" y1="17" y2="29"></line>' +
            '</svg><svg viewBox="0 0 64 64" style="transform: rotate(150deg); animation-delay: -83.3333ms; animation-duration: 1000ms;">' +
            '<line transform="translate(32,32)" y1="17" y2="29"></line>' +
            '</svg></ion-spinner>';
  return spinnerHtml;
};


/**
 * Send an haptic feedback
 * @param {object} options
 */
Client.IonHelper.hapticFeedback = function (options)
{
  if (!Client.mainFrame.theme || !Client.mainFrame.theme.haptic)
    return;
  //
  if (!options)
    options = {};
  //
  var type = options.type || "";
  var style = options.style;
  //
  switch (type) {
    case ">m":
      type = (Client.Ionic.platform === "ios" ? "impact" : "virtualkey");
      style = "light";
      break;

    case "":
      type = (Client.Ionic.platform === "ios" ? "selection" : "acoustic");
      break;

    case "gestureSelection":
      if (style === "changed" && Client.Ionic.platform !== "ios") {
        type = "acoustic";
      }
      break;
  }
  //
  Client.mainFrame.device.processRequest({obj: "device-haptic", id: "feedback", cnt: {type: type, style: style}});
};


/**
 * Return an ionic searchbar to be embedded into an Alert
 * @returns {String}
 */
Client.IonHelper.createAlertSearchbar = function ()
{
  var searchObj = document.createElement("ion-searchbar");
  //
  var searchContainer = document.createElement("div");
  searchContainer.className = "searchbar-input-container";
  searchObj.appendChild(searchContainer);
  //
  var searchIcon = document.createElement("div");
  searchIcon.className = "searchbar-search-icon";
  searchIcon.style.marginLeft = "0px";
  searchContainer.appendChild(searchIcon);
  //
  var inputObj = document.createElement("input");
  inputObj.type = "text";
  inputObj.className = "searchbar-input";
  inputObj.placeholder = "Search";
  inputObj.onkeyup = function (ev) {
    if (ev.which === 13) {
      inputObj.blur();
    }
  };
  //
  inputObj.addEventListener("focus", function (ev) {
    searchObj.classList.add("searchbar-has-focus");
    //
    // When focusing the alert fix the height of the resultlist, so when filtering the list doesn't move
    var items = document.getElementsByClassName("alert-radio-group");
    if (!items || items.length === 0)
      items = document.getElementsByClassName("alert-checkbox-group");
    if (!items || items.length === 0)
      return;
    //
    items[0].style.height = items[0].offsetHeight + "px";
    items[0].style.width = items[0].offsetWidth + "px";
  });
  //
  inputObj.addEventListener("blur", function (ev) {
    searchObj.classList.remove("searchbar-has-focus");
  });
  //
  var onInputFunction = function (ev) {
    if (inputObj.value)
      searchObj.classList.add("searchbar-has-value");
    else
      searchObj.classList.remove("searchbar-has-value");
    //
    // Filter items
    var items = document.getElementsByClassName("alert-radio");
    if (!items || items.length === 0)
      items = document.getElementsByClassName("alert-checkbox");
    if (!items || items.length === 0)
      return;
    //
    var search = inputObj.value;
    var nvis = 0;
    var searchMode = 0;   // 0-StartWith, 1-WordBegin, 2-FullContent
    while (searchMode < 3) {
      for (var it = 0; it < items.length; it++) {
        var item = items[it];
        var matches = Client.IonHelper.itemMatches(item.getAttribute("label"), search, searchMode);
        item.style.display = matches ? "" : "none";
        if (matches)
          nvis++;
      }
      //
      // If i found something stop trying
      if (nvis > 0)
        break;
      //
      // Next mode
      searchMode++;
    }
  };
  inputObj.addEventListener("input", onInputFunction);
  searchContainer.appendChild(inputObj);
  //
  var clearObj = document.createElement("button");
  clearObj.className = "searchbar-clear-icon disable-hover button button-clear";
  clearObj.addEventListener("mousedown", (function (ev) {
    inputObj.value = "";
    onInputFunction();
    ev.stopPropagation();
    return false;
  }).bind(this), true);
  clearObj.addEventListener("touchstart", (function (ev) {
    inputObj.value = "";
    onInputFunction();
    ev.stopPropagation();
    return false;
  }).bind(this), true);
  searchContainer.appendChild(clearObj);
  //
  return searchObj;
};


/**
 * Check if a text matches a label string using a specified search mode
 * @param {String} label label to check
 * @param {String} text text that can match
 * @param {Int} searchMode 0-StartWith, 1-WordBegin, 2-FullContent
 */
Client.IonHelper.itemMatches = function (label, text, searchMode)
{
  if (!label)
    label = "";
  if (!text)
    text = "";
  //
  // No text, it matches!
  if (text === "")
    return true;
  //
  var match = false;
  switch (searchMode) {
    case 0: // Start With
      match = (label.toLowerCase().indexOf(text.toLowerCase()) === 0);
      break;

    case 1: // Word match
      var pos = label.toLowerCase().indexOf(' ' + text.toLowerCase());
      match = (pos !== -1);
      break;

    case 2: // Full content search
      var pos = label.toLowerCase().indexOf(text.toLowerCase());
      match = (pos !== -1);
      break;
  }
  //
  return match;
};
