/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client, moment */

/**
 * @class A container for a datetime picker
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.IonSelect = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.cancelText = "Cancel";
  this.okText = "Ok";
  this.options = [];
  this.value = undefined;
  this.valueList = [];
  //
  this.domObj = document.createElement("ion-select");
  //
  this.textObj = document.createElement("div");
  this.textObj.className = "select-text";
  this.domObj.appendChild(this.textObj);
  //
  this.iconObj = document.createElement("div");
  this.iconObj.className = "select-icon";
  this.innerObj = document.createElement("div");
  this.innerObj.className = "select-icon-inner";
  this.iconObj.appendChild(this.innerObj);
  this.domObj.appendChild(this.iconObj);
  //
  this.coverObj = document.createElement("button");
  this.coverObj.className = "item-cover disable-hover item-cover-default";
  Client.IonHelper.registerClickListener(this, this.coverObj);
  this.domObj.appendChild(this.coverObj);
  //
  this.clearObj = document.createElement("button");
  this.clearObj.className = "text-input-clear-icon disable-hover button button-clear want-click select-clear-icon";
  //
  this.clearObj.onmousedown = function (ev) {
    this.setValue("", true);
    return false;
  }.bind(this);
  this.clearObj.ontouchstart = function (ev) {
    this.setValue("", true);
    return false;
  }.bind(this);
  this.clearObj.style.display = "none";
  this.domObj.appendChild(this.clearObj);
  //
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  this.updateElement(element);
  //
  // let's see if we have a label on our left
  if (!this.labelObj && Client.IonItem && Client.IonLabel && this.parent instanceof Client.IonItem) {
    var b = false;
    for (var i = 0; i < this.parent.elements.length && !b; i++) {
      if (this.parent.elements[i] instanceof Client.IonLabel)
        b = true;
    }
    if (!b)
      this.domObj.classList.add("select-nolabel");
  }
};

Client.IonSelect.prototype = new Client.Element();


/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.IonSelect.prototype.updateElement = function (el)
{
  this.purgeMyProp(el);
  //
  if (el.cancelText !== undefined) {
    this.cancelText = el.cancelText;
    delete el.cancelText;
  }
  if (el.okText !== undefined) {
    this.okText = el.okText;
    delete el.okText;
  }
  if (el.disabled !== undefined) {
    this.disabled = el.disabled;
    this.domObj.classList.toggle("select-disabled", el.disabled);
    if (Client.IonItem && this.parent instanceof Client.IonItem)
      this.parent.domObj.classList.toggle("item-select-disabled", el.disabled);
  }
  if (el.placeHolder !== undefined) {
    this.placeHolder = el.placeHolder;
    this.showValue();
    delete el.placeHolder;
  }
  if (el.multiple !== undefined) {
    this.multiple = el.multiple;
    delete el.mutiple;
  }
  if (el.selectedText !== undefined) {
    this.selectedText = el.selectedText;
    this.showValue();
    delete el.selectedText;
  }
  if (el.title !== undefined) {
    this.title = el.title;
    delete el.title;
  }
  if (el.subtitle !== undefined) {
    this.subtitle = el.subtitle;
    delete el.subtitle;
  }
  if (el.canFilter !== undefined) {
    this.canFilter = el.canFilter;
    delete el.canFilter;
  }
  //
  if (el.value !== undefined) {
    this.setValue(el.value === null ? "" : el.value + "");
    delete el.value;
  }
  if (el.clear !== undefined) {
    this.clearObj.style.display = el.clear ? "" : "none";
    delete el.clear;
  }
  if (el.canFilter !== undefined) {
    this.canFilter = el.canFilter;
    delete el.canFilter;
  }
  if (el.maxWidth !== undefined) {
    this.maxWidth = el.maxWidth;
    delete el.maxWidth;
  }
  if (el.maxHeight !== undefined) {
    this.maxHeight = el.maxHeight;
    delete el.maxHeight;
  }
  //
  //
  // If there is a list of options, add them to the select
  // List example: [{"n":"Andrew","v":1},{"n":"Mary","v":2},{"n":"Luke","v":3}]
  if (el.list) {
    //
    // If el.list is a string make it a json object
    if (typeof el.list === "string")
      el.list = JSON.parse(el.list);
    //
    // Remove element children
    this.options.length = 0;
    //
    // Create an option for each item in the list
    for (var i = 0; i < el.list.length; i++) {
      var o = el.list[i];
      this.options.push({text: o.n, value: o.v});
    }
    //
    // restore value if present
    this.setValue(this.value);
    //
    // Base class must not use the list property
    delete el.list;
  }
  //
  if (el.data) {
    if (el.data.columns) {
      this.initInput(el);
      this.setValue(this.value);
    }
    else if (el.data.pos !== undefined) {
      if (el.data.data) // Update option
        this.updateOption(el.data);
      else  // Remove option
        this.removeOptionAtPos(el.data.pos);
    }
    else { // Add option
      this.addOption(el.data.data);
      if (!this.selectedText)
        this.setValue(this.value);
    }
    //
    // Base class must not use the data property
    delete el.data;
  }
  //
  // specific class name
  if (el.className !== undefined) {
    var cl = "";
    var ca = ["select-nolabel", "select-has-value", "select-disabled"];
    for (var i = 0; i < ca.length; i++) {
      if (this.domObj.classList.contains(ca[i]))
        cl += " " + ca[i];
    }
    cl += " " + el.className;
    this.domObj.className = cl;
    delete el.className;
  }
  //
  // Label
  if (el.label !== undefined) {
    if (!this.labelObj) {
      this.labelObj = document.createElement("ion-label");
      this.domObj.parentNode.insertBefore(this.labelObj, this.domObj);
      this.domObj.classList.remove("select-nolabel");
    }
    this.labelObj.textContent = el.label;
    if (this.parent instanceof Client.IonItem) {
      this.parent.domObj.classList.add("item-label-fixed");
    }
    delete el.label;
  }
  if (el.labelPosition !== undefined) {
    if (this.labelPosition && this.labelObj) {
      this.labelObj.removeAttribute(this.labelPosition);
      if (this.parent instanceof Client.IonItem)
        this.parent.domObj.classList.remove("item-label-" + this.labelPosition);
    }
    //
    this.labelPosition = el.labelPosition;
    if (el.labelPosition && this.labelObj) {
      this.labelObj.setAttribute(this.labelPosition, "");
      if (this.parent instanceof Client.IonItem)
        this.parent.domObj.classList.add("item-label-" + this.labelPosition);
    }
    //
    this.domObj.classList.toggle("select-nolabel", this.labelPosition === "hidden");
    //
    delete el.labelPosition;
  }
  if (el.color !== undefined) {
    if (this.color && this.labelObj)
      this.labelObj.removeAttribute(this.color);
    this.color = el.color;
    if (this.color && this.labelObj)
      this.labelObj.setAttribute(this.color, "");
    delete el.color;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};


/**
 * Show actual value
 * @param {Object} el - properties to update
 */
Client.IonSelect.prototype.showValue = function ()
{
  this.textObj.textContent = this.selectedText || this.placeHolder;
  this.textObj.classList.toggle("select-placeholder", !this.selectedText);
  this.domObj.classList.toggle("select-has-value", !!this.value);
  if (this.parent instanceof Client.IonItem)
    this.parent.domObj.classList.toggle("input-has-value", !!this.value);
};


/**
 * This object has been hidden or shown
 * @param {Boolean} visible
 */
Client.IonSelect.prototype.visibilityChanged = function (visible)
{
  // The label is outside the input: update label visibility
  if (this.labelObj)
    this.labelObj.style.display = this.getRootObject().style.display;
  Client.Element.prototype.visibilityChanged.call(this, visible);
};


/**
 * Set actual value
 * @param {string} value
 */
Client.IonSelect.prototype.setValue = function (value, emitChange)
{
  if (value === undefined || value === null)
    value = "";
  //
  this.value = value;
  this.selectedText = "";
  //
  // multiple values are separated by comma
  if (this.multiple && value.split)
    this.valueList = value.split(",");
  else
    this.valueList = [value];
  //
  for (var i = 0; i < this.options.length; i++) {
    var o = this.options[i];
    var isPresent = false;
    for (var j = 0; j < this.valueList.length; j++) {
      // == is right, === is not
      if (o.value == this.valueList[j]) {
        isPresent = true;
        break;
      }
    }
    if (isPresent) {
      if (this.selectedText)
        this.selectedText += ", ";
      this.selectedText += o.text;
    }
  }
  //
  this.showValue();
  //
  if (emitChange) {
    var x = [{obj: this.id, id: "chgProp", content: {name: "value", value: value, clid: Client.id}}];
    x.push({obj: this.id, id: "chgProp", content: {name: "selectedText", value: this.selectedText, clid: Client.id}});
    if (this.sendOnChange)
      x.push({obj: this.id, id: "onChange"});
    Client.mainFrame.sendEvents(x);
  }
};


/**
 * Attach events handler
 * @param {Array} events - array of the events to handle
 */
Client.IonSelect.prototype.attachEvents = function (events)
{
  if (!events)
    return;
  //
  var pos = events.indexOf("onChange");
  if (pos >= 0) {
    events.splice(pos, 1);
    this.sendOnChange = true;
  }
  //
  Client.Element.prototype.attachEvents.call(this, events);
};


/**
 * Datetime clicked!
 * @param {Object} ev - event
 */
Client.IonSelect.prototype.myClick = function (ev, domObj)
{
  // No click when editing in the view editor
  if (Client.mainFrame.isEditing())
    return;
  //
  if (this.domObj.disabled)
    return;
  //
  // Let's if there is a label
  if (this.title === undefined) {
    var ll = this.domObj.parentNode.getElementsByTagName("ION-LABEL");
    if (ll && ll.length) {
      this.title = ll[0].textContent;
    }
    else {
      this.title = "";
    }
  }
  //
  var opt = {};
  if (this.title)
    opt.title = this.title;
  if (this.subtitle)
    opt.subtitle = this.subtitle;
  if (this.canFilter)
    opt.canFilter = this.canFilter;
  if (this.maxWidth)
    opt.maxWidth = this.maxWidth;
  if (this.maxHeight)
    opt.maxHeight = this.maxHeight;
  for (var i = 0; i < this.options.length; i++) {
    var o = this.options[i];
    if (!opt.inputs)
      opt.inputs = [];
    opt.inputs.push({type: this.multiple ? "checkbox" : "radio", label: o.text, id: o.value, checked: this.valueList.indexOf(o.value) > -1});
  }
  //
  opt.buttons = [this.cancelText, this.okText];
  //
  Client.IonHelper.createAlert(opt, function (r, values) {
    if (r) {
      if (values) {
        delete values.button;
        if (this.multiple) {
          var v = [];
          var k = Object.keys(values);
          for (var i = 0; i < k.length; i++)
            if (values[k[i]])
              v.push(k[i]);
          this.setValue(v.join(","), true);
        }
        else
          this.setValue(values.value, true);
      }
    }
  }.bind(this));
  //
};


/**
 * Initialize the input
 * @param {Object} el - element representation
 */
Client.IonSelect.prototype.initInput = function (el)
{
  // First remove old options
  this.options.length = 0;
  //
  for (var i in el.data.data)
    this.addOption(el.data.data[i]);
};


/**
 * Remove an option from the select at the given position
 * @param {int} pos - the position of the option to remove
 */
Client.IonSelect.prototype.removeOptionAtPos = function (pos)
{
  this.options.splice(pos, 1);
};


/**
 * Add an option to the select
 * @param {Array} data - the option to add. The first element of the array represents the value,
 *                       the second element represents the text
 */
Client.IonSelect.prototype.addOption = function (data)
{
  this.options.push({text: data[1], value: data[0]});
};


/**
 * Update a select option
 * @param {Object} data - the data of the option to update. Example: {pos: 3, data:[3, "Andrea"]}
 */
Client.IonSelect.prototype.updateOption = function (data)
{
  var opt = this.options[data.pos];
  opt.value = data.data[0];
  opt.text = data.data[1];
};
