/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client */

/**
 * @class A container for a tab bar
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.IonTabs = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("ion-tabs");
  //
  this.tabbar = document.createElement("ion-tabbar");
  this.tabbar.className = "show-tabbar";
  //
  // iOS needs a kick to show the tabbar
  setTimeout(function () {
    var d = document.createElement("div");
    this.tabbar.appendChild(d);
    setTimeout(function () {
      d.remove();
    }, 0);
  }.bind(this), 500);
  this.domObj.appendChild(this.tabbar);
  //
  this.tabhl = document.createElement("tab-highlight");
  this.tabbar.appendChild(this.tabhl);
  //
  var sp = element.selectedPage || 0;
  if (!element.placement)
    element.placement = Client.Ionic.platform === "ios" ? "bottom" : "top";
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  //
  this.createChildren(element);
  //
  if (sp !== undefined)
    this.selectPage(sp);
};

Client.IonTabs.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.IonTabs.prototype.updateElement = function (el)
{
  this.purgeMyProp(el);
  //
  var update = this.domObj.parentNode === null;
  //
  // specific class name
  if (el.className !== undefined) {
    this.className = el.className;
    update = true;
    delete el.className;
  }
  if (el.placement !== undefined) {
    this.placement = el.placement;
    if (el.placement)
      this.domObj.setAttribute("tabsPlacement", el.placement);
    else
      this.domObj.removeAttribute("tabsPlacement");
    delete el.placement;
  }
  if (el.layout !== undefined) {
    this.layout = el.layout;
    if (el.layout)
      this.domObj.setAttribute("tabsLayout", el.layout);
    else
      this.domObj.removeAttribute("tabsLayout");
    for (var i = 0; i < this.ne(); i++) {
      var e = this.elements[i];
      if (e.updateLinkClass)
        e.updateLinkClass();
    }
    delete el.layout;
  }
  if (el.color !== undefined) {
    if (this.color)
      this.domObj.removeAttribute(this.color);
    this.color = el.color;
    if (this.color)
      this.domObj.setAttribute(this.color, "");
    delete el.color;
  }
  if (el.selectedPage !== undefined) {
    this.selectPage(el.selectedPage);
    delete el.selectedPage;
  }
  Client.Element.prototype.updateElement.call(this, el);
  //
  if (update) {
    var cs = "tabs-basic";
    if (this.className)
      cs += " " + this.className;
    this.domObj.className = cs;
  }
};

/**
 * Append a child DOM Object to this element DOM Object
 * Usually the child element is not contained in this element array
 * it will be pushed there just after this function call.
 * @param {Element} child child element that requested the insertion
 * @param {HTMLElement} domObj child DOM object to add
 */
Client.IonTabs.prototype.appendChildObject = function (child, domObj)
{
  if (child instanceof Client.IonTab) {
    var a = child.linkObj = document.createElement("a");
    a.setAttribute("for", child.id);
    a.style.display = child.visible !== false ? "" : "none";
    a.addEventListener("click", function (ev) {
      this.selectPage(child);
      child.sendClick(ev);
      this.sendPage(ev);
    }.bind(this));
    Client.IonHelper.registerClickListener(child, a);
    //
    var s = child.titleObj = document.createElement("span");
    s.className = "tab-button-text";
    s.textContent = child.title;
    a.appendChild(s);
    //
    var i = child.iconObj = document.createElement("ion-icon");
    i.className = "tab-button-icon " + Client.IonHelper.getIconClass(child.icon);
    if (this.layout === "iconBottom" || this.layout === "iconRight")
      a.appendChild(i);
    else
      a.insertBefore(i, s);
    //
    var b = child.badgeObj = document.createElement("ion-badge");
    b.className = "tab-badge badge-" + (child.badgecolor || "danger");
    if (child.badge)
      b.textContent = child.badge;
    a.appendChild(b);
    //
    this.tabbar.appendChild(a);
    //
    child.updateLinkClass();
  }
  //
  Client.Element.prototype.appendChildObject.call(this, child, domObj);
};

/**
 * select a different page
 * @param {Object} page - page index
 */
Client.IonTabs.prototype.selectPage = function (page)
{
  this.oldSelectedPage = this.selectedPage;
  this.selectedPage = undefined;
  //
  var pageEmpty = true;
  var allEmpty = true;
  var c = 0;
  for (var i = 0; i < this.ne(); i++) {
    var e = this.elements[i];
    if (e instanceof Client.IonTab) {
      if (e.ne())
        allEmpty = false;
      if (c === page || e === page) {
        if (e.ne())
          pageEmpty = false;
      }
      c++;
    }
  }
  //
  c = 0;
  for (var i = 0; i < this.ne(); i++) {
    var e = this.elements[i];
    if (e instanceof Client.IonTab) {
      //
      e.selected = (c === page || e === page);
      //
      if (!pageEmpty || allEmpty)
        e.updateElement({icon: "*"});
      //
      if (e.selected) {
        this.selectedPage = c;
        this.tabhl.style.transform = "translate3d(" + e.linkObj.offsetLeft + "px,0,0) scaleX(" + e.linkObj.clientWidth + ")";
        this.tabhl.className = "animate";
        //
        if (this.placement === "top") {
          var t = e.domObj.getElementsByTagName("ION-HEADER");
          var tot = 0;
          for (var j = 0; j < t.length; j++) {
            tot += t[j].clientHeight;
          }
          this.tabbar.style.marginTop = tot + "px";
        }
      }
      //
      c++;
    }
  }
};

/**
 * Attach events handler
 * @param {Array} events - array of the events to handle
 */
Client.IonTabs.prototype.attachEvents = function (events)
{
  if (!events)
    return;
  //
  var pos = events.indexOf("onChangePage");
  if (pos >= 0) {
    events.splice(pos, 1);
    this.sendChange = true;
  }
  //
  Client.Element.prototype.attachEvents.call(this, events);
};

/**
 * Attach events handler
 * @param {object} ev
 */
Client.IonTabs.prototype.sendPage = function (ev)
{
  var x = [{obj: this.id, id: "chgProp", content: {name: "selectedPage", value: this.selectedPage, clid: Client.id}}];
  if (this.sendChange)
    x.push({obj: this.id, id: "onChangePage", content: {oldPage: this.oldSelectedPage, newPage: this.selectedPage}});
  Client.mainFrame.sendEvents(x);
};




/**
 * @class A container for a tab page
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.IonTab = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("ion-tab");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  //
  this.createChildren(element);
};

Client.IonTab.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.IonTab.prototype.updateElement = function (el)
{
  if (el.title !== undefined) {
    this.title = el.title;
    if (this.titleObj)
      this.titleObj.textContent = el.title;
    delete el.title;
  }
  if (el.icon !== undefined) {
    this.icon = el.icon === "*" ? this.icon : el.icon;
    if (this.iconObj)
      this.iconObj.className = "tab-button-icon " + Client.IonHelper.getIconClass(this.icon + (this.selected || Client.Ionic.platform === "md" ? "" : "-outline"));
    if (el.icon === "*") {
      var old = this.domObj.style.display;
      this.domObj.style.display = this.visible !== false && this.selected ? "" : "none";
      this.domObj.classList.toggle("show-tab", this.selected);
      if (this.linkObj)
        this.linkObj.setAttribute("aria-selected", this.selected);
      if (this.selected) {
        // Recalc tab content position
        var x = this.getElements(Client.IonContent);
        for (var i = 0; i < x.length; i++)
          x[i].positionContent();
      }
      //
      if (this.domObj.style.display !== old) {
        this.visibilityChanged(this.domObj.style.display === "");
      }
    }
    delete el.icon;
  }
  if (el.badge !== undefined) {
    this.badge = el.badge;
    if (this.badgeObj)
      this.badgeObj.textContent = el.badge;
    delete el.badge;
    this.updateLinkClass();
  }
  if (el.disabled !== undefined) {
    this.disabled = el.disabled;
    delete el.disabled;
    this.updateLinkClass();
  }
  if (el.visible !== undefined) {
    if (this.linkObj)
      this.linkObj.style.display = el.visible ? "" : "none";
  }
  if (el.badgecolor !== undefined) {
    this.badgecolor = el.badgecolor;
    if (this.badgeObj)
      this.badgeObj.className = "tab-badge badge-" + el.badgecolor;
    delete el.badgecolor;
  }
  Client.Element.prototype.updateElement.call(this, el);
};


/**
 * Remove the element and its children from the element map
 * @param {boolean} firstLevel - if true remove the dom of the element too
 * @param {boolean} triggerAnimation - if true and on firstLevel trigger the animation of 'removing'
 */
Client.IonTab.prototype.close = function (firstLevel, triggerAnimation)
{
  if (this.linkObj)
    this.linkObj.remove();
  Client.Container.prototype.close.call(this, firstLevel, triggerAnimation);
};


/**
 * Remove the element and its children from the element map
 */
Client.IonTab.prototype.updateLinkClass = function ()
{
  if (this.linkObj) {
    var cs = "tab-button";
    if (this.parent.layout !== "icon-hide")
      cs += " has-icon";
    else
      cs += " has-title-only";
    if (this.parent.layout !== "title-hide")
      cs += " has-title";
    else
      cs += " has-icon-only";
    if (this.badge)
      cs += " has-badge";
    if (this.disabled)
      cs += " tab-disabled";
    //
    cs += " disable-hover";
    this.linkObj.className = cs;
  }
};

/**
 * Attach events handler
 * @param {Array} events - array of the events to handle
 */
Client.IonTab.prototype.attachEvents = function (events)
{
  if (!events)
    return;
  //
  var pos = events.indexOf("onClick");
  if (pos >= 0) {
    events.splice(pos, 1);
    this.sendOnClick = true;
  }
  //
  Client.Element.prototype.attachEvents.call(this, events);
};

/**
 * Attach events handler
 * @param {object} ev
 */
Client.IonTab.prototype.sendClick = function (ev)
{
  if (this.sendOnClick)
    Client.mainFrame.sendEvents([{obj: this.id, id: "onClick", content: this.saveEvent(ev)}]);
};
