/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client,componentHandler */

/**
 * @class Material Design: button element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDButton = function (element, parent, view)
{
  Client.Button.call(this, element, parent, view);
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDButton extend Client.Button
Client.MDButton.prototype = new Client.Button();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDButton.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  // button type: fab, mini-fab, icon, raised
  if (el.btnType !== undefined) {
    this.btnType = el.btnType;
    update = true;
    delete el.btnType;
  }
  // TO DELETE:
  if (el.color !== undefined) {
    this.color = el.color;
    update = true;
    delete el.color;
  }
  //
  if (update) {
    //
    // Composing class list: required classes
    var cs = "mdl-button mdl-js-button ";
    //
    if (this.btnType) {
      if (this.btnType === "mini-fab")
        cs += "mdl-button--fab ";
      cs += "mdl-button--" + this.btnType + " ";
    }
    if (this.color) {
      cs += "mdl-button--colored ";
      if (this.color === "primary")
        cs += "mdl-button--primary ";
      else if (this.color === "accent")
        cs += "mdl-button--accent ";
      else
        cs += "mdl-color--" + this.color + " ";
    }
    //
    // If button inside a footer, add a new class
    if (Client.MDFooterSection && this.parent instanceof Client.MDFooterSection)
      cs += "mdl-" + this.parent.parent.type + "-footer__social-btn ";
    //
    cs += Client.MDHelper.updateClass(this);
    //
    this.domObj.className = cs;
    //
    // Creating/Removing icon
    if (this.icon) {
      if (!this.iconObj) {
        this.iconObj = document.createElement("i");
        this.iconObj.className = "material-icons";
        this.domObj.appendChild(this.iconObj);
      }
      this.iconObj.innerText = this.icon;
    }
    else {
      if (this.iconObj) {
        if (this.iconObj.parentNode)
          this.iconObj.parentNode.removeChild(this.iconObj);
        this.iconObj = undefined;
      }
    }
  }
  //
  Client.Button.prototype.updateElement.call(this, el);
};
