/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client,componentHandler */

/**
 * @class A container for a card
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDCard = function (element, parent, view)
{
  Client.Container.call(this, element, parent, view);
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDCard extend Client.Container
Client.MDCard.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDCard.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  if (update) {
    //
    // Card required classes
    var cs = "mdl-card ";
    //
    cs += Client.MDHelper.updateClass(this);
    //
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};



/**
 * @class A container for a card section
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDCardSection = function (element, parent, view)
{
  Client.Container.call(this, element, parent, view);
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDCard extend Client.Container
Client.MDCardSection.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDCardSection.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  // section type
  if (el.secType !== undefined) {
    this.secType = el.secType;
    update = true;
    delete el.secType;
  }
  //
  // section border
  if (el.border !== undefined) {
    this.border = el.border;
    update = true;
    delete el.border;
  }
  //
  // section expandable property
  if (el.expand !== undefined) {
    this.expand = el.expand;
    update = true;
    delete el.expand;
  }
  //
  // inner text has a special handling for cards
  if (el.innerText !== undefined) {
    this.innerText = el.innerText;
    update = true;
    delete el.innerText;
  }
  //
  if (update) {
    //
    var cs = "";
    //
    if (this.secType)
      cs += "mdl-card__" + this.secType + " ";
    //
    if (this.border)
      cs += "mdl-card--border ";
    //
    if (this.expand)
      cs += "mdl-card--expand ";
    //
    cs += Client.MDHelper.updateClass(this);
    //
    this.domObj.className = cs;
    //
    if (this.innerText !== undefined) {
      //
      // Inner text becames a title for title card sections
      if (this.secType === "title") {
        if (!this.textObj) {
          this.textObj = document.createElement("span");
          this.textObj.className = "mdl-card__title-text";
          this.domObj.appendChild(this.textObj);
        }
        this.domObj.removeAttribute("innerText");
        this.textObj.innerText = this.innerText;
      }
      else {
        // Inner text remains as is for other card sections
        if (this.textObj) {
          this.textObj.parentNode.removeChild(this.textObj);
          this.textObj = undefined;
        }
        this.domObj.innerText = this.innerText;
      }
    }
    else {
      if (this.textObj) {
        this.textObj.parentNode.removeChild(this.textObj);
        this.textObj = undefined;
      }
    }
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};
