/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client,componentHandler */
/* global Client,dialogPolyfill */

/**
 * @class A MD Dialog
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDDialog = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("dialog");
  //
  // Use the polyfill for browser that does not support dialog element
  if (!this.domObj.showModal)
    dialogPolyfill.registerDialog(this.domObj);
  //
  var showOnCreation = false;
  if (element.visible === undefined || element.visible === true)
    showOnCreation = true;
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  document.body.appendChild(this.domObj);
  this.createChildren(element);
  //
  componentHandler.upgradeElement(this.domObj);
  //
  if (showOnCreation)
    this.updateElement({visible: true});
};

// Make Client.MDDialog extend Client.Container
Client.MDDialog.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDDialog.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  if (update) {
    var cs = "mdl-dialog ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  if (el.visible !== undefined) {
    this.visible = el.visible;
    //
    if (this.domObj.parentNode) {
      if (el.visible)
        this.domObj.showModal();
      else if (this.domObj.getAttribute("open") !== null)
        this.domObj.close();
    }
    delete el.visible;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};



/**
 * @class A MD Dialog Title
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDDialogTitle = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("div");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  this.createChildren(element);
  //
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDDialogTitle extend Client.Container
Client.MDDialogTitle.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDDialogTitle.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  if (update) {
    var cs = "mdl-dialog__title ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};



/**
 * @class Defines MD Dialog Content
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDDialogContent = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("div");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  this.createChildren(element);
  //
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDDialogContent extend Client.Container
Client.MDDialogContent.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDDialogContent.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  if (update) {
    var cs = "mdl-dialog__content ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};



/**
 * @class Defines MD Dialog Actions
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDDialogActions = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("div");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  this.createChildren(element);
  //
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDDialogActions extend Client.Container
Client.MDDialogActions.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDDialogActions.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  if (update) {
    var cs = "mdl-dialog__actions ";
    if (this.fullWidth)
      cs += "mdl-dialog__actions--full-width ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};
