/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client,componentHandler */

/**
 * @class A container for a material design footer element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDFooter = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("footer");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  this.createChildren(element);
  //
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDFooter extend Client.Container
Client.MDFooter.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDFooter.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  // mini or mega footer
  if (el.type !== undefined) {
    this.type = el.type;
    update = true;
    delete el.type;
  }
  //
  if (update) {
    var cs = "mdl-" + this.type + "-footer ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};



/**
 * @class A container for a footer section
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDFooterSection = function (element, parent, view)
{
  Client.Container.call(this, element, parent, view);
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDFooterSection extend Client.Container
Client.MDFooterSection.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDFooterSection.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  // left, right, top, middle or bottom
  if (el.type !== undefined) {
    this.type = el.type;
    update = true;
    delete el.type;
  }
  //
  if (update) {
    var cs = "mdl-" + Client.MDHelper.getFooterType(this) + "-footer__" + this.type + "-section ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};



/**
 * @class A container for a footer link list
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDFooterLinkList = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("ul");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  this.createChildren(element);
  //
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDFooterLinkList extend Client.Container
Client.MDFooterLinkList.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDFooterLinkList.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  if (update) {
    var cs = "mdl-" + Client.MDHelper.getFooterType(this) + "-footer__link-list ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};



/**
 * @class A footer link
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDFooterLink = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.liObj = document.createElement("li");
  this.domObj = document.createElement("a");
  this.liObj.appendChild(this.domObj);
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.liObj);
  //
  componentHandler.upgradeElement(this.liObj);
};

// Make Client.MDFooterLink extend Client.Element
Client.MDFooterLink.prototype = new Client.Element();

/**
 * Return the dom root object of this element
 * @returns {DomElement}
 */
Client.MDFooterLink.prototype.getRootObject = function ()
{
  return this.liObj;
};


/**
 * @class A footer button
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDFooterButton = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("button");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  //
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDFooterButton extend Client.Element
Client.MDFooterButton.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDFooterButton.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  if (update) {
    var cs = "mdl-" + Client.MDHelper.getFooterType(this) + "-footer__social-btn material-icons ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
    this.domObj.textContent = this.icon;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};
