/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client,componentHandler */

/**
 * @class Material Design Helper class
 */
Client.MDHelper = {};


/**
 * Recognize common MD properties
 * @param {Object} el - properties to update
 * @param {Object} mdobj - md object to use
 */
Client.MDHelper.updateElement = function (el, mdobj)
{
  var ris = mdobj.domObj && mdobj.domObj.parentNode === null;
  //
  // ripple: true / false
  if (el.ripple !== undefined) {
    mdobj.ripple = el.ripple;
    ris = true;
    delete el.ripple;
  }
  //
  // shadow: 2,3,4,6,8,16
  if (el.shadow !== undefined) {
    mdobj.shadow = el.shadow;
    ris = true;
    delete el.shadow;
  }
  //
  // back color
  if (el.backColor !== undefined) {
    mdobj.backColor = el.backColor;
    ris = true;
    delete el.backColor;
  }
  //
  // text color
  if (el.textColor !== undefined) {
    mdobj.textColor = el.textColor;
    ris = true;
    delete el.textColor;
  }
  //
  // specific class name
  if (el.className !== undefined) {
    mdobj.className = el.className;
    ris = true;
    delete el.className;
  }
  // icon to show
  if (el.icon !== undefined) {
    mdobj.icon = el.icon;
    ris = true;
    delete el.icon;
  }
  //
  // full width object
  if (el.fullWidth !== undefined) {
    mdobj.fullWidth = el.fullWidth;
    ris = true;
    delete el.fullWidth;
  }
  //
  // where to show (every, large, small)
  if (el.screenSize !== undefined) {
    mdobj.screenSize = el.screenSize;
    ris = true;
    delete el.screenSize;
  }
  //
  return ris;
};


/**
 * Returns common classes to apply
 * @param {Object} mdobj - md object to use
 */
Client.MDHelper.updateClass = function (mdobj)
{
  var cs = "";
  //
  if (mdobj.shadow)
    cs += "mdl-shadow--" + mdobj.shadow + "dp ";
  if (mdobj.ripple)
    cs += "mdl-js-ripple-effect ";
  if (mdobj.backColor)
    cs += "mdl-color--" + mdobj.backColor + " ";
  if (mdobj.textColor)
    cs += "mdl-color-text--" + mdobj.textColor + " ";
  if (mdobj.screenSize === "large")
    cs += "mdl-layout--large-screen-only ";
  if (mdobj.screenSize === "small")
    cs += "mdl-layout--small-screen-only ";
  if (mdobj.className)
    cs += mdobj.className;
  //
  return cs;
};


/**
 * Returns footer element type
 * @param {Object} mdobj - md object to use
 */
Client.MDHelper.getFooterType = function (mdobj)
{
  var p = mdobj.parent;
  while (p) {
    if (p instanceof Client.MDFooter)
      return p.type;
    p = p.parent;
  }
  return "";
};
