/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client,componentHandler */

/**
 * @class A container for the entire page
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDLayout = function (element, parent, view)
{
  Client.Container.call(this, element, parent, view);
  componentHandler.upgradeElement(this.domObj);
  //
  // If the element starts hidden it had set the display none on the domObj and not the parent, so after the upgrade
  // we must change that behaviour
  if (this.startHidden) {
    this.domObj.style.display = this.oldDisplay;
    this.oldDisplay = this.domObj.parentNode.style.display;
    this.domObj.parentNode.style.display = "none";
  }
};

// Make Client.MDCard extend Client.Container
Client.MDLayout.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDLayout.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  // Fixed Header
  if (el.fixedHeader !== undefined) {
    this.fixedHeader = el.fixedHeader;
    update = true;
    delete el.fixedHeader;
  }
  //
  // Fixed Drawer
  if (el.fixedDrawer !== undefined) {
    this.fixedDrawer = el.fixedDrawer;
    update = true;
    delete el.fixedDrawer;
  }
  //
  // Fixed Tabs
  if (el.fixedTabs !== undefined) {
    this.fixedTabs = el.fixedTabs;
    update = true;
    delete el.fixedTabs;
  }
  //
  if (update) {
    var cs = "mdl-layout mdl-js-layout ";
    if (this.fixedDrawer)
      cs += "mdl-layout--fixed-drawer ";
    if (this.fixedHeader)
      cs += "mdl-layout--fixed-header ";
    if (this.fixedTabs)
      cs += "mdl-layout--fixed-tabs ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};


/**
 * Return the dom root object of this element
 * @returns {DomElement}
 */
Client.MDLayout.prototype.getRootObject = function ()
{
  if (this.domObj.parentNode)
    return this.domObj.parentNode;
  else
    return this.domObj;
};



/**
 * @class A container the header of the layout
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDHeader = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("header");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  this.createChildren(element);
  //
  componentHandler.upgradeElement(this.domObj);
  //
  if (Client.mainFrame.isEditing())
    this.domObj.style.display = "block";
};

// Make Client.MDCard extend Client.Container
Client.MDHeader.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDHeader.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  // menu icon present?
  if (el.menuIcon !== undefined) {
    if (el.menuIcon && !this.iconObj) {
      this.iconObj = document.createElement("div");
      this.iconObj.className = "mdl-layout-icon";
      this.domObj.insertBefore(this.iconObj, this.domObj.firstChild);
    }
    if (!el.menuIcon && this.iconObj) {
      this.iconObj.parentNode.removeChild(this.iconObj);
      this.iconObj = undefined;
    }
    delete el.menuIcon;
  }
  //
  // Scrolling Header
  if (el.scroll !== undefined) {
    this.scroll = el.scroll;
    update = true;
    delete el.scroll;
  }
  //
  // Waterfall Header
  if (el.waterfall !== undefined) {
    this.waterfall = el.waterfall;
    update = true;
    delete el.waterfall;
  }
  //
  // Trasparent Header
  if (el.trasparent !== undefined) {
    this.trasparent = el.trasparent;
    update = true;
    delete el.trasparent;
  }
  //
  // Seamed Header
  if (el.seamed !== undefined) {
    this.seamed = el.seamed;
    update = true;
    delete el.seamed;
  }
  //
  if (update) {
    var cs = "mdl-layout__header ";
    if (this.scroll)
      cs += "mdl-layout__header--scroll ";
    if (this.trasparent)
      cs += "mdl-layout__header--trasparent ";
    if (this.seamed)
      cs += "mdl-layout__header--seamed ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};



/**
 * @class A container the header of the layout
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDHeaderRow = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("div");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  //
  parent.appendChildObject(this, this.domObj);
  //
  componentHandler.upgradeElement(this.domObj);
  //
  // Create children after attaching element as external components, rely on dom structure
  this.createChildren(element);
};

// Make Client.MDCard extend Client.Container
Client.MDHeaderRow.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDHeaderRow.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  var rebuild;
  //
  if (el.tabBar !== undefined) {
    rebuild = (el.tabBar !== this.tagBar && this.domObj.parentNode !== undefined);
    this.tabBar = el.tabBar;
    update = true;
    delete el.tabBar;
  }
  //
  if (update) {
    if (rebuild)
      componentHandler.downgradeElements(this.domObj);
    var cs = "mdl-layout__header-row ";
    if (this.tabBar)
      cs = "mdl-layout__tab-bar ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
    if (rebuild)
      componentHandler.upgradeElements(this.domObj);
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};



/**
 * @class A container the header of the layout
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDDrawer = function (element, parent, view)
{
  Client.Container.call(this, element, parent, view);
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDCard extend Client.Container
Client.MDDrawer.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDDrawer.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  if (update) {
    var cs = "mdl-layout__drawer ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};


/**
 * open or close the drawer
 * @param {bool} flag
 */
Client.MDDrawer.prototype.show = function (flag)
{
  if (flag)
    this.domObj.classList.add("is-visible");
  else
    this.domObj.classList.remove("is-visible");
};



/**
 * @class A container the header of the layout
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDNavigation = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("nav");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  this.createChildren(element);
  //
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDCard extend Client.Container
Client.MDNavigation.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDNavigation.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  if (update) {
    var cs = "mdl-navigation ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};



/**
 * @class A container for a content section
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDContent = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  var ct = "div";
  switch (element.type) {
    case "main":
    case "section":
      ct = element.type;
  }
  this.domObj = document.createElement(ct);
  //
  this.updateElement(element);
  //
  // Need to change the ID to allow querySelector calls
  if (!Client.mainFrame.isEditing())
    this.domObj.id = "o" + this.id;
  //
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  this.createChildren(element);
  //
  componentHandler.upgradeElement(this.domObj);
  //
  if (this.type === "main") {
    for (var i = 0; i < this.ne(); i++) {
      if (this.elements[i].type === "section")
        this.elements[i].connect();
    }
  }
};

// Make Client.MDCard extend Client.Container
Client.MDContent.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDContent.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  // specific class name for this layout
  if (el.type !== undefined) {
    this.type = el.type;
    update = true;
    delete el.type;
  }
  //
  // Nospacign for grids
  if (el.nospacing !== undefined) {
    this.nospacing = el.nospacing;
    update = true;
    delete el.nospacing;
  }
  //
  // is this the default page?
  if (el.default !== undefined) {
    this.default = el.default;
    update = true;
    delete el.default;
  }
  //
  if (update) {
    var cs = "page-content ";
    if (this.type === "main")
      cs += "mdl-layout__content ";
    if (this.type === "section")
      cs += "mdl-layout__tab-panel ";
    if (this.type === "grid") {
      cs += "mdl-grid ";
      if (this.nospacing)
        cs += "mdl-grid--no-spacing ";
    }
    if (this.default)
      cs += "is-active ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};


/**
 * Connect this "TAB" to a MDLink
 */
Client.MDContent.prototype.connect = function ()
{
  // Search for a MDLink to link
  // 1. search to ordinal position of "this" section
  var c = 0;
  for (var i = 0; i < this.parent.ne(); i++) {
    var p = this.parent.elements[i];
    if (p.type === "section")
      c++;
    if (p === this)
      break;
  }
  //
  // 2. search a tabbed header row
  var p = this.parent;
  while (p) {
    if (p instanceof Client.MDLayout)
      break;
    p = p.parent;
  }
  if (p) {
    var hdr;
    for (var i = 0; i < p.ne(); i++) {
      if (p.elements[i] instanceof Client.MDHeader) {
        hdr = p.elements[i];
        break;
      }
    }
    if (hdr) {
      var hdrrow;
      for (var i = 0; i < hdr.ne(); i++) {
        if (hdr.elements[i] instanceof Client.MDHeaderRow && hdr.elements[i].tabBar) {
          hdrrow = hdr.elements[i];
          break;
        }
      }
      if (hdrrow) {
        var l = 0;
        for (var i = 0; i < hdrrow.ne(); i++) {
          var h = hdrrow.elements[i];
          if (h instanceof Client.MDLink) {
            l++;
            if (l === c) {
              if (!h.domObj.href) {
                h.domObj.href = "#" + this.domObj.id;
                if (this.domObj.classList.contains("is-active"))
                  h.domObj.classList.add("is-active");
              }
              break;
            }
          }
        }
      }
    }
  }
};



/**
 * @class Defines MD Grid content
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDCell = function (element, parent, view)
{
  Client.Container.call(this, element, parent, view);
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDCell extend Client.Container
Client.MDCell.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDCell.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  // # of columns
  if (el.cols !== undefined) {
    this.cols = el.cols;
    update = true;
    delete el.cols;
  }
  if (el.desktopCols !== undefined) {
    this.desktopCols = el.desktopCols;
    update = true;
    delete el.desktopCols;
  }
  if (el.tabletCols !== undefined) {
    this.tabletCols = el.tabletCols;
    update = true;
    delete el.tabletCols;
  }
  if (el.phoneCols !== undefined) {
    this.phoneCols = el.phoneCols;
    update = true;
    delete el.phoneCols;
  }
  //
  // Hidden cols?
  if (el.hideDesktop !== undefined) {
    this.hideDesktop = el.hideDesktop;
    update = true;
    delete el.hideDesktop;
  }
  if (el.hideTablet !== undefined) {
    this.hideTablet = el.hideTablet;
    update = true;
    delete el.hideTablet;
  }
  if (el.hidePhone !== undefined) {
    this.hidePhone = el.hidePhone;
    update = true;
    delete el.hidePhone;
  }
  if (el.verticalAlign !== undefined) {
    this.verticalAlign = el.verticalAlign;
    update = true;
    delete el.verticalAlign;
  }
  //
  if (update) {
    var cs = "mdl-cell ";
    if (this.cols)
      cs += "mdl-cell--" + this.cols + "-col ";
    if (this.desktopCols)
      cs += "mdl-cell--" + this.desktopCols + "-col-desktop ";
    if (this.tabletCols)
      cs += "mdl-cell--" + this.tabletCols + "-col-tablet ";
    if (this.phoneCols)
      cs += "mdl-cell--" + this.phoneCols + "-col-phone ";
    //
    if (this.hideDesktop)
      cs += "mdl-cell--hide-desktop ";
    if (this.hideTablet)
      cs += "mdl-cell--hide-tablet ";
    if (this.hidePhone)
      cs += "mdl-cell--hide-phone ";
    //
    if (this.verticalAlign)
      cs += "mdl-cell--" + this.verticalAlign + " ";
    //
    cs += Client.MDHelper.updateClass(this);
    //
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};
