/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client,componentHandler */

/**
 * @class A container for a MD List Element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDList = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("ul");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  this.createChildren(element);
  //
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDList extend Client.Container
Client.MDList.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDList.prototype.updateElement = function (el)
{
  // Force update for the first time
  var update = Client.MDHelper.updateElement(el, this);
  //
  if (update) {
    var cs = "mdl-list ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};


/**
 * @class A container for MD List Item Element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDListItem = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("li");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  this.createChildren(element);
  //
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDListItem extend Client.Container
Client.MDListItem.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDListItem.prototype.updateElement = function (el)
{
  // Force update for the first time
  var update = Client.MDHelper.updateElement(el, this);
  //
  // how many rows?
  if (el.rows !== undefined) {
    this.rows = el.rows;
    update = true;
    delete el.rows;
  }
  //
  if (update) {
    var cs = "mdl-list__item ";
    if (this.rows === 2)
      cs += "mdl-list__item--two-line ";
    else if (this.rows === 3)
      cs += "mdl-list__item--three-line ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};



/**
 * @class A container for MD List Content Element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDListContent = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.type = "primary";
  this.domObj = document.createElement("span");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  this.createChildren(element);
  //
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDListContent extend Client.Container
Client.MDListContent.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDListContent.prototype.updateElement = function (el)
{
  // Force update for the first time
  var update = Client.MDHelper.updateElement(el, this);
  //
  // primary or secondary content?
  if (el.type !== undefined) {
    this.type = el.type;
    update = true;
    delete el.type;
  }
  //
  if (update) {
    var cs = "mdl-list__item-";
    switch (this.type) {
      case "primary" :
      case "secondary" :
        cs += this.type + "-content ";
        break
      case "info":
        cs += "secondary-info ";
        break;
      case "text":
        cs += "text-body ";
        break;
    }
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};



/**
 * @class A container for a MD List Action element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDListAction = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("a");
  this.domObj.href = "#";
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  this.createChildren(element);
  //
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDListAction extend Client.Container
Client.MDListAction.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDListAction.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  if (update) {
    var cs = "mdl-list__item-secondary-action ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
    //
    // Creating/Removing icon
    if (this.icon) {
      if (!this.iconObj) {
        this.iconObj = document.createElement("i");
        this.iconObj.className = "material-icons";
        this.domObj.appendChild(this.iconObj);
      }
      this.iconObj.innerText = this.icon;
    }
    else if (this.iconObj) {
      if (this.iconObj.parentNode)
        this.iconObj.parentNode.removeChild(this.iconObj);
      this.iconObj = undefined;
    }
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};


/**
 * @class A container for MD List Icon
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDListIcon = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("i");
  this.type = "icon";
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  //
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDListIcon extend Client.Element
Client.MDListIcon.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDListIcon.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  // avatar or icon?
  if (el.type !== undefined) {
    this.type = el.type;
    update = true;
    delete el.type;
  }
  //
  if (update) {
    var cs = "material-icons mdl-list__item-" + (this.type === "icon" ? "icon" : "avatar") + " ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
    this.domObj.innerText = this.icon;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};
