/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client,componentHandler */

/**
 * @class A material design menu
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDMenu = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  // Create menu container
  this.domObj = document.createElement("ul");
  //
  // attaching menu to previous element
  if (parent.ne()) {
    var v = parent.elements[parent.ne() - 1];
    if (v.domObj)
      this.domObj.setAttribute("for", v.domObj.id);
  }
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  //
  // Create menu items
  this.createChildren(element);
  //
  // Upgrading menu and items
  componentHandler.upgradeElement(this.domObj);
  for (var i = 0; i < this.ne(); i++) {
    componentHandler.upgradeElement(this.elements[i].domObj);
  }
};

// Make Client.MDMenu extend Client.Element
Client.MDMenu.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDMenu.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  // menu position and alignment
  if (el.position !== undefined) {
    this.position = el.position;
    update = true;
    delete el.position;
  }
  //
  if (update) {
    var cs = "mdl-menu mdl-js-menu ";
    if (this.position)
      cs += "mdl-menu--" + this.position + " ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};

/**
 * Show the menu
 */
Client.MDMenu.prototype.show = function (evt)
{
  if (this.domObj.MaterialMenu) {
    if (!this.domObj.classList.contains("is-visible"))
      this.domObj.MaterialMenu.handleForClick_(evt);
  }
};

/**
 * Hide the menu
 */
Client.MDMenu.prototype.hide = function ()
{
  if (this.domObj.MaterialMenu) {
    this.domObj.MaterialMenu.hide();
  }
};



/**
 * @class A material design menu item
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDMenuItem = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("li");
  this.updateElement(element, true);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  //
  this.createChildren(element);
  //
  // Note: the menu item will be upgraded by the menu container itself...
};

// Make Client.MDMenu extend Client.Element
Client.MDMenuItem.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDMenuItem.prototype.updateElement = function (el, firstTime)
{
  var update = Client.MDHelper.updateElement(el, this) || firstTime;
  //
  // menu items can be disasabled
  if (el.disabled !== undefined) {
    if (el.disabled)
      this.domObj.setAttribute("disabled", true);
    else
      this.domObj.removeAttribute("disabled");
    delete el.disabled;
  }
  //
  if (update) {
    var cs = "mdl-menu__item ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};
