/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client,componentHandler */

/**
 * @class A Material Design Progress Bar
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDProgress = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("div");
  this.updateElement(element, true);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  //
  // The initial update cannot set progress and buffer initial value,
  // so we need to update them manually after upgrading the component
  var pthis = this;
  this.domObj.addEventListener('mdl-componentupgraded', function () {
    if (pthis.progress)
      this.MaterialProgress.setProgress(pthis.progress);
    if (pthis.buffer)
      this.MaterialProgress.setBuffer(pthis.buffer);
  });
  componentHandler.upgradeElement(this.domObj);
};

Client.MDProgress.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 * @param {Bool} firstTime - true if called during initialization
 */
Client.MDProgress.prototype.updateElement = function (el, firstTime)
{
  var update = Client.MDHelper.updateElement(el, this) || firstTime;
  //
  // Progress value
  if (el.progress !== undefined) {
    this.progress = el.progress;
    if (this.domObj.MaterialProgress)
      this.domObj.MaterialProgress.setProgress(el.progress);
    delete el.progress;
  }
  //
  // Buffer value
  if (el.buffer !== undefined) {
    this.buffer = el.buffer;
    if (this.domObj.MaterialProgress)
      this.domObj.MaterialProgress.setBuffer(el.buffer);
    delete el.buffer;
  }
  //
  // indeterminate status
  if (el.indeterminate !== undefined) {
    this.indeterminate = el.indeterminate;
    update = true;
    delete el.indeterminate;
  }
  //
  if (update) {
    var cs = "mdl-progress mdl-js-progress ";
    if (this.indeterminate)
      cs += "mdl-progress__indeterminate ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};



/**
 * @class A Material Design Spinner
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDSpinner = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("div");
  this.updateElement(element, true);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  //
  componentHandler.upgradeElement(this.domObj);
};

Client.MDSpinner.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 * @param {Bool} firstTime - true if called during initialization
 */
Client.MDSpinner.prototype.updateElement = function (el, firstTime)
{
  var update = Client.MDHelper.updateElement(el, this) || firstTime;
  //
  // Spinner is active?
  if (el.active !== undefined) {
    this.active = el.active;
    update = true;
    delete el.active;
  }
  //
  // Single or multicolor
  if (el.singleColor !== undefined) {
    this.singleColor = el.singleColor;
    update = true;
    delete el.singleColor;
  }
  //
  if (update) {
    var cs = "mdl-spinner mdl-js-spinner ";
    if (this.singleColor)
      cs += "mdl-spinner--single-color ";
    if (this.active)
      cs += "is-active ";
    if (!firstTime)
      cs += "is-upgraded ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};
