/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client,componentHandler */

/**
 * @class A container for a material design snackbar (toast) element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDSnackBar = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("div");
  this.textObj = document.createElement("div");
  this.actionObj = document.createElement("button");
  this.domObj.appendChild(this.textObj);
  this.domObj.appendChild(this.actionObj);
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  this.createChildren(element);
  //
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDSnackBar extend Client.Element
Client.MDSnackBar.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDSnackBar.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  if (update) {
    var cs = "mdl-snackbar mdl-js-snackbar";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
    this.textObj.className = "mdl-snackbar__text";
    this.actionObj.className = "mdl-snackbar__action";
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};

/**
 * Attach events handler
 * @param {Array} events - array of the events to handle
 */
Client.MDSnackBar.prototype.attachEvents = function (events)
{
  if (!events)
    return;
  //
  var idx = events.indexOf("onClick");
  if (idx >= 0) {
    this.sendClick = true;
    var pthis = this;
    this.clickHandler = function (ev) {
      var e = [{obj: pthis.id, id: "onClick", content: pthis.saveEvent(ev)}];
      if (e[0].content)
        e[0].content.actionText = pthis.actionObj.textContent;
      else
        e[0].content = {actionText: pthis.actionObj.textContent};
      Client.mainFrame.sendEvents(e);
    };
    events.splice(idx);
  }
  //
  Client.Element.prototype.attachEvents.call(this, events);
};

/**
 * Shows the snackbar
 * @param {Object} data
 */
Client.MDSnackBar.prototype.show = function (data)
{
  if (this.sendClick && data.actionText)
    data.actionHandler = this.clickHandler;
  //
  this.domObj.MaterialSnackbar.showSnackbar(data);
};

