/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client,componentHandler */

/**
 * @class A container for a tabbed view
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDTab = function (element, parent, view)
{
  Client.Container.call(this, element, parent, view);
  componentHandler.upgradeElement(this.domObj);
  if (this.ripple)
    componentHandler.upgradeElements(this.domObj.children);
  //
  for (var i = 0; i < this.ne(); i++) {
    if (this.elements[i] instanceof Client.MDTabPanel)
      this.elements[i].connect();
  }
};

// Make Client.MDTab extend Client.Container
Client.MDTab.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDTab.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  if (update) {
    var cs = "mdl-tabs mdl-js-tabs ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};



/**
 * @class tabbed view tab bar
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDTabBar = function (element, parent, view)
{
  Client.Container.call(this, element, parent, view);
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDTabBar extend Client.Container
Client.MDTabBar.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDTabBar.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  if (update) {
    var cs = "mdl-tabs__tab-bar ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};



/**
 * @class tabbed view pages
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDTabPanel = function (element, parent, view)
{
  Client.Container.call(this, element, parent, view);
  //
  // Need to change the ID to allow querySelector calls
  if (!Client.mainFrame.isEditing())
    this.domObj.id = "o" + this.id;
  //
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDTabPanel extend Client.Container
Client.MDTabPanel.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDTabPanel.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  // is this the default page?
  if (el.default !== undefined) {
    this.default = el.default;
    update = true;
    delete el.default;
  }
  //
  if (update) {
    var cs = "mdl-tabs__panel ";
    if (this.default)
      cs += "is-active ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};

/**
 * Connect this "TAB" to a MDLink
 */
Client.MDTabPanel.prototype.connect = function ()
{
  // Search for a MDLink to link
  // 1. search to ordinal position of "this" panel
  var c = 0;
  for (var i = 0; i < this.parent.ne(); i++) {
    var p = this.parent.elements[i];
    if (p instanceof Client.MDTabPanel)
      c++;
    if (p === this)
      break;
  }
  //
  // 2. search a tabbed header row
  var p = this.parent;
  while (p) {
    if (p instanceof Client.MDTab)
      break;
    p = p.parent;
  }
  if (p) {
    var tabbar;
    for (var i = 0; i < p.ne(); i++) {
      if (p.elements[i] instanceof Client.MDTabBar) {
        tabbar = p.elements[i];
        break;
      }
    }
    if (tabbar) {
      var l = 0;
      for (var i = 0; i < tabbar.ne(); i++) {
        var t = tabbar.elements[i];
        if (t instanceof Client.MDLink) {
          l++;
          if (l === c) {
            if (!t.domObj.href) {
              t.domObj.href = "#" + this.domObj.id;
              if (this.domObj.classList.contains("is-active"))
                t.domObj.classList.add("is-active");
            }
            break;
          }
        }
      }
    }
  }
};
