/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client,componentHandler */

/**
 * @class A container for a data table element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDDataTable = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("table");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  this.createChildren(element);
  //
  componentHandler.upgradeElement(this.domObj);
  if (this.selectable)
    componentHandler.upgradeElements(this.domObj.children);
};

// Make Client.MDDataTable extend Client.Container
Client.MDDataTable.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDDataTable.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  // selectable rows?
  if (el.selectable !== undefined) {
    this.selectable = el.selectable;
    update = true;
    delete el.selectable;
  }
  //
  if (update) {
    var cs = "mdl-data-table mdl-js-data-table ";
    if (this.selectable)
      cs += "mdl-data-table--selectable ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};



/**
 * @class A container for a data section element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDTableSection = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("t" + element.type);
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  this.createChildren(element);
  //
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDTableSection extend Client.Container
Client.MDTableSection.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDTableSection.prototype.updateElement = function (el)
{
  if (el.type) {
    this.changeTag("t" + el.type);
    this.type = el.type;
    delete el.type;
    componentHandler.upgradeElement(this.domObj);
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};



/**
 * @class A container for a data row
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDTableRow = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("tr");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  this.createChildren(element);
  //
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDTableRow extend Client.Container
Client.MDTableRow.prototype = new Client.Container();



/**
 * @class A container for a data cell element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDTableCell = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  var ct = (parent.parent && parent.parent.type === "head") ? "th" : "td";
  this.domObj = document.createElement(ct);
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  this.createChildren(element);
  //
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDTableCell extend Client.Container
Client.MDTableCell.prototype = new Client.Container();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDTableCell.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  // non numeric data?
  if (el.nonNumeric !== undefined) {
    this.nonNumeric = el.nonNumeric;
    update = true;
    delete el.nonNumeric;
  }
  //
  if (update) {
    var cs = "";
    if (this.nonNumeric)
      cs += "mdl-data-table__cell--non-numeric ";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Container.prototype.updateElement.call(this, el);
};
