/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client,componentHandler */

/**
 * @class A material text element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDText = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement(this.getTag(element.type));
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  //
  componentHandler.upgradeElement(this.domObj);
};

// Make Client.MDText extend Client.Element
Client.MDText.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDText.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  // element type
  if (el.type !== undefined) {
    this.changeTag(this.getTag(el.type));
    componentHandler.upgradeElement(this.domObj);
    this.type = el.type;
    update = true;
    delete el.type;
  }
  //
  // badge
  if (el.badge !== undefined) {
    this.badge = el.badge;
    update = true;
    delete el.badge;
  }
  //
  if (update) {
    var cs = "";
    if (this.type === "title")
      cs += "mdl-layout-title ";
    if (this.type === "spacer")
      cs += "mdl-layout-spacer ";
    if (this.type === "icon")
      cs += "material-icons ";
    if (this.type === "logo")
      cs += "mdl-logo ";
    if (this.type === "heading")
      cs += "mdl-mega-footer__heading ";
    if (this.badge) {
      cs += "mdl-badge ";
      this.domObj.setAttribute("data-badge", this.badge);
    }
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};

/**
 * Returns tag name for this mdtext
 * @param {string} type - MDText type
 */
Client.MDText.prototype.getTag = function (type)
{
  var ct = type;
  switch (ct) {
    case "title":
    case "logo":
      ct = "span";
      break;
    case "spacer":
      ct = "div";
      break;
    case "icon":
      ct = "i";
      break;
    case "heading":
      ct = "h1";
      break;
  }
  if (!ct)
    ct = "span";
  return ct;
};




/**
 * @class A material navigation link
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.MDLink = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("a");
  //
  this.updateElement(element);
  this.attachEvents(element.events);
  parent.appendChildObject(this, this.domObj);
  //
  // Create children after attaching element as external components, rely on dom structure
  this.createChildren(element);
  //
  componentHandler.upgradeElement(this.domObj);
  //
  // After a click on a tab in editing mode, we must dispatch the onresize
  // event to the relevant section
  var pthis = this;
  if (Client.mainFrame.isEditing()) {
    this.domObj.addEventListener("click", function (ev) {
      if (pthis.parent.tabBar || (Client.MDTabBar && pthis.parent instanceof Client.MDTabBar)) {
        if (pthis.domObj.href) {
          var idx = pthis.domObj.href.indexOf("#");
          var ele = Client.eleMap[pthis.domObj.href.substring(idx + 5)];
          if (ele)
            ele.onResize(ev);
        }
      }
    });
  }
};

// Make Client.MDMenu extend Client.Element
Client.MDLink.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.MDLink.prototype.updateElement = function (el)
{
  var update = Client.MDHelper.updateElement(el, this);
  //
  if (update) {
    var cs = "mdl-navigation__link ";
    if (Client.MDTabBar && this.parent instanceof Client.MDTabBar)
      cs = "mdl-tabs__tab ";
    if (this.parent.tabBar)
      cs = "mdl-layout__tab";
    cs += Client.MDHelper.updateClass(this);
    this.domObj.className = cs;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};
