/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client */

/**
 * @class A round progressbar element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the progress bar
 */
Client.NoUiSlider = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("div");
  //
  parent.appendChildObject(this, this.domObj);
  //
  this.attachEvents(element.events);
  this.updateElement(element);
};

// Make Client.ProgressBar extend Client.Element
Client.NoUiSlider.prototype = new Client.Element();


/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.NoUiSlider.prototype.updateElement = function (el)
{
  this.purgeMyProp(el);
  //
  if (el.options) {
    if (!this.slider)
      this.createSlider(el.options);
    else
      this.slider.updateOptions(el.options);
    delete el.options;
  }
  if (el.value !== undefined) {
    this.set(el.value);
    delete el.value;
  }
  if (el.disabled !== undefined) {
    if (el.disabled)
      this.domObj.setAttribute('disabled', true);
    else
      this.domObj.removeAttribute('disabled');
    delete el.disabled;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};


/**
 * set value immediate
 * @param {Object} value
 */
Client.NoUiSlider.prototype.set = function (value)
{
  if (this.slider) {
    this.slider.set(value);
  }
};


/**
 * get value immediate
 * @param {string} cbId - callback id
 */
Client.NoUiSlider.prototype.get = function (cbId)
{
  var e = [{obj: this.id, id: "cb", content: {res: this.slider ? this.slider.get() : undefined, cbId: cbId}}];
  Client.mainFrame.sendEvents(e);
};


/**
 * Attach events handler
 * @param {Array} events - array of the events to handle
 */
Client.NoUiSlider.prototype.attachEvents = function (events)
{
  if (!events)
    return;
  //
  var pos = events.indexOf("onUpdate");
  if (pos >= 0) {
    events.splice(pos, 1);
    this.sendUpdate = true;
  }
  pos = events.indexOf("onChange");
  if (pos >= 0) {
    events.splice(pos, 1);
    this.sendChange = true;
  }
  //
  Client.Element.prototype.attachEvents.call(this, events);
};


/**
 * Attach events handler to real slider
 * @param {object} options
 */
Client.NoUiSlider.prototype.createSlider = function (options)
{
  try {
    if (typeof options === "string")
      options = JSON.parse(options);
    //
    this.slider = noUiSlider.create(this.domObj, options);
  } catch (ex) {
    return;
  }
  //
  var pthis = this;
  this.slider.on("update", function (values, handle, unencoded, tap, positions) {
    if (pthis.sendUpdate) {
      if (!pthis.updateTimeout) {
        pthis.updateTimeout = setTimeout(function () {
          var e = [{obj: pthis.id, id: "chgProp", content: {name: "value", value: pthis.slider.get(), clid: Client.id}}];
          e.push({obj: pthis.id, id: "onUpdate", content: pthis.slider.get()});
          Client.mainFrame.sendEvents(e);
          delete pthis.updateTimeout;
        }, 100);
      }
    }
  });
  this.slider.on("change", function (values, handle, unencoded, tap, positions) {
    var e = [{obj: pthis.id, id: "chgProp", content: {name: "value", value: pthis.slider.get(), clid: Client.id}}];
    if (pthis.sendChange)
      e.push({obj: pthis.id, id: "onChange", content: pthis.slider.get()});
    Client.mainFrame.sendEvents(e);
  });
};
