/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2015
 * All rights reserved
 */

/* global Client, QRCode */

/**
 * @class An QRCode generator element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the input
 */
Client.QRCode = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  this.domObj = document.createElement("div");
  parent.appendChildObject(this, this.domObj);
  this.options = {text: " "};
  this.updateElement(element);
  this.attachEvents(element.events);
};

// Make Client.QRCode extend Client.Element
Client.QRCode.prototype = new Client.Element();


/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.QRCode.prototype.updateElement = function (el)
{
  this.purgeMyProp(el);
  //
  if (el.width !== undefined) {
    this.options.width = el.width;
    delete el.width;
  }
  //
  if (el.height !== undefined) {
    this.options.height = el.height;
    delete el.height;
  }
  //
  if (el.colorDark !== undefined) {
    this.options.colorDark = el.colorDark;
    delete el.colorDark;
  }
  //
  if (el.colorLight !== undefined) {
    this.options.colorLight = el.colorLight;
    delete el.colorLight;
  }
  //
  if (el.correctLevel !== undefined) {
    // var QRErrorCorrectLevel={L:1,M:0,Q:3,H:2};
    // see https://github.com/davidshimjs/qrcodejs source code
    this.options.correctLevel = el.correctLevel;
    delete el.correctLevel;
  }
  //
  if (el.text !== undefined) {
    this.options.text = el.text;
    delete el.text;
  }
  //
  this.drawCode();
  Client.Element.prototype.updateElement.call(this, el);
};


/*
 * Draw the canvas containing the QRCode
 */
Client.QRCode.prototype.drawCode = function ()
{
  // Empty div
  while (this.domObj.firstChild) {
    this.domObj.removeChild(this.domObj.firstChild);
  }
  // Populate div
  new QRCode(this.domObj, this.options);
};
