/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Client, StripeCheckout */

/**
 * @class A button to pay
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the button
 */
Client.StripeButton = function (element, parent, view)
{
  Client.Button.call(this, element, parent, view);
  //
  // Stripe Handler
  var pthis = this;
  this.handler = configure(this);
  //
  // Exit handler
  var pthis = this;
  this.popHandler = function () {
    pthis.handlerClose();
  };
  //
  window.addEventListener("popstate", this.popHandler);
  //
  this.domObj.addEventListener(Client.mainFrame.isEditing() ? "dblclick" : "click", function (e) {
    // Open Checkout with further options
    var x = {};
    if (pthis.logo !== undefined)
      x.image = pthis.logo;
    if (pthis.title !== undefined)
      x.name = pthis.title;
    if (pthis.text !== undefined)
      x.description = pthis.text;
    if (pthis.amount !== undefined)
      x.amount = pthis.amount;
    if (pthis.email !== undefined)
      x.email = pthis.email;
    if (pthis.currency !== undefined)
      x.currency = pthis.currency;
    if (pthis.panelLabel !== undefined)
      x.panelLabel = pthis.panelLabel;
    if (pthis.allowRememberMe !== undefined)
      x.allowRememberMe = pthis.allowRememberMe;
    pthis.handlerOpen(x);
    e.preventDefault();
  });
};


/**
 * configure handler
 * @param {Object} obj - "this" object
 */
function configure(obj) {
  //
  // if we are using indeshell
  if (Client && Client.mainFrame
          && Client.mainFrame.device
          && Client.mainFrame.device.shell
          && Client.mainFrame.device.shell === parent) {
    //
    var cmd = {obj: "device-stripe",
      id: "configure",
      cnt: {
        key: obj.key,
        obj: obj.id,
        id: "onPay"
      }
    };
    Client.mainFrame.device.shell.postMessage(cmd, "*");
    //
  } else {
    return  StripeCheckout.configure({
      key: obj.key,
      token: function (token) {
        var e = [{obj: obj.id, id: "onPay", content: token}];
        Client.mainFrame.sendEvents(e);
      }
    });
  }
}

// Make Client.StripeButton extend Client.Button
Client.StripeButton.prototype = new Client.Button();


Client.StripeButton.prototype.handlerClose = function ()
{
  var pthis = this;
  //
  // if we are using indeshell
  if (Client && Client.mainFrame
          && Client.mainFrame.device
          && Client.mainFrame.device.shell
          && Client.mainFrame.device.shell === parent) {
    var cmd = {obj: "device-stripe",
      id: "handlerClose",
      cnt: {}
    };
    Client.mainFrame.device.shell.postMessage(cmd, "*");
  }
  else {
    pthis.handler.close();
  }
};


Client.StripeButton.prototype.handlerOpen = function (params)
{
  var pthis = this;
  //
  // if we are using indeshell
  if (Client && Client.mainFrame
          && Client.mainFrame.device
          && Client.mainFrame.device.shell
          && Client.mainFrame.device.shell === parent) {
    var cmd = {obj: "device-stripe",
      id: "handlerOpen",
      cnt: params
    };
    Client.mainFrame.device.shell.postMessage(cmd, "*");
  }
  else {
    pthis.handler.open(params);
  }
};


/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.StripeButton.prototype.updateElement = function (el)
{
  if (el.key !== undefined) {
    this.key = el.key;
    delete el.key;
    //
    // if the key was modify re-configure the handler
    this.handler = configure(this);
  }
  if (el.logo !== undefined) {
    this.logo = el.logo;
    delete el.logo;
  }
  if (el.title !== undefined) {
    this.title = el.title;
    delete el.title;
  }
  if (el.text !== undefined) {
    this.text = el.text;
    delete el.text;
  }
  if (el.amount !== undefined) {
    this.amount = el.amount;
    delete el.amount;
  }
  if (el.currency !== undefined) {
    this.currency = el.currency;
    delete el.currency;
  }
  if (el.email !== undefined) {
    this.email = el.email;
    delete el.email;
  }
  if (el.panelLabel !== undefined) {
    this.panelLabel = el.panelLabel;
    delete el.panelLabel;
  }
  if (el.allowRememberMe !== undefined) {
    this.allowRememberMe = el.allowRememberMe;
    delete el.allowRememberMe;
  }
  //
  Client.Button.prototype.updateElement.call(this, el);
};


/**
 * Remove the element and its children from the element map
 * @param {boolean} firstLevel - if true remove the dom of the element too
 */
Client.StripeButton.prototype.close = function (firstLevel)
{
  window.removeEventListener("popstate", this.popHandler);
  Client.Button.prototype.close.call(this, firstLevel);
};
