/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */


/* global Client, Switchery */

/**
 * @class An IOS Switch element
 * @param {Object} element - the element description
 * @param {View|Element} parent - the parent element
 * @param {View} view - the view containing the input
 */
Client.Switch = function (element, parent, view)
{
  Client.Element.call(this, element, parent, view);
  //
  if (!element.options)
    element.options = {};
  //
  this.domObj = document.createElement("input");
  this.domObj.type = "checkbox";
  parent.appendChildObject(this, this.domObj);
  this.attachEvents(element.events);
  //
  this.switch = new Switchery(this.domObj, element.options);
  this.inputObj = this.domObj;
  this.domObj = this.switch.switcher;
  //
  this.updateElement(element);
};

// Make Client.Switch extend Client.Element
Client.Switch.prototype = new Client.Element();

/**
 * Update element properties
 * @param {Object} el - properties to update
 */
Client.Switch.prototype.updateElement = function (el)
{
  this.purgeMyProp(el);
  //
  if (el.options !== undefined) {
    this.switch.setSpeed();
    this.switch.setSize();
    this.switch.colorize();
    this.switch.setPosition(false);
    delete el.options;
  }
  //
  if (el.disabled !== undefined) {
    //
    if (this.switch) {
      if (el.disabled && !this.switch.isDisabled())
        this.switch.disable();
      if (!el.disabled && this.switch.isDisabled())
        this.switch.enable();
    }
    delete el.disabled;
  }
  //
  if (el.checked !== undefined) {
    this.inputObj.checked = el.checked;
    this.switch.setPosition(false);
    delete el.checked;
  }
  //
  Client.Element.prototype.updateElement.call(this, el);
};


/**
 * Attach events handler
 * @param {Array} events - array of the events to handle
 */
Client.Switch.prototype.attachEvents = function (events)
{
  if (!events)
    return;
  //
  var pthis = this;
  var idx = events.indexOf("onChange");
  if (idx >= 0) {
    events.splice(idx, 1);
    this.domObj.onchange = function (ev) {
      var e = [{obj: pthis.id, id: "chgProp", content: {name: "checked", value: pthis.switch.isChecked(), clid: Client.id}},
        {obj: pthis.id, id: "onChange", content: pthis.saveEvent(ev)}];
      Client.mainFrame.sendEvents(e);
    };
  }
  //
  Client.Element.prototype.attachEvents.call(this, events);
};
