/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global App, Client */

var Proxy = Proxy || {};


/**
 * Create the offline proxy object
 */
Proxy.createProxy = function ()
{
  // OfflineBuild is a global inside offlineBuild.js
  if (typeof OfflineBuild === "undefined" || !OfflineBuild) {
    //
    // Get the SID/CID parameters from query string, session storage or cookie
    var sid = Client.Utils.getQueryVariable("sid");
    sid = sid || sessionStorage.getItem("sid");
    sid = sid || Client.Utils.getCookie("sid");
    //
    var cid = Client.Utils.getQueryVariable("cid");
    cid = cid || sessionStorage.getItem("cid");
    cid = cid || Client.Utils.getCookie("cid");
    //
    // Move from cookie store to session store
    sessionStorage.setItem("sid", sid);
    Client.Utils.deleteCookie("sid");
    sessionStorage.setItem("cid", cid);
    Client.Utils.deleteCookie("cid");
    //
    // acid/appid are specific to ideapp
    var acid = Client.Utils.getQueryVariable("acid");
    var appid = Client.Utils.getQueryVariable("appid");
    //
    if (!sid) {
      alert("Cannot connect to IDE server: session ID not specified");
      return;
    }
    //
    this.socket = io(Client.resourceOrigin);
    //
    // send a message with the sid, acid (IDE case), and cid (master case)
    this.socket.emit("asid", {sid: sid, acid: acid, cid: cid});
    //
    // define method send of socket
    this.socket.send = function (content) {
      this.emit("offmsg", {appid: appid, sid: sid, content: content});
    };
    //
    // listener for messages from IDE
    this.socket.on("offmsg", function (m) {
      //
      // A new code to evaluate has arrived from the IDE server
      if (m.type === "eval") {
        try {
          eval(m.content);      // jshint ignore:line
        }
        catch (ex) {
          console.error("Syntax Error:", ex);
          console.warn(m.content);
        }
      }
      //
      // A new code to evaluate has arrived from the IDE server
      if (m.type === "events")
        Proxy.send(null, m.content);
      //
      // The code is completed, so I can start the application
      if (m.type === "codecomplete")
        Proxy.loadClasses(undefined, undefined, Proxy.runApplication);
    });
    //
    // if the offline proxy disconnects, let's terminate the session
    this.socket.on("disconnect", function () {
      if (Client && Client.mainFrame)
        Client.mainFrame.terminate();
    });
  }
  else {
    this.socket = {send: function () {
        // console.log("this is an offline build, there is no socket");
      }};
    //
    // Load the server side code (if it isn't loaded yet)
    // and run the application
    if (!Client.appVersion)
      Client.appVersion = location.search.slice(1);
    Proxy.loadClasses(Client.resourceName, Client.appVersion, Proxy.runApplication);
  }
  //
  // Check if generators are available
  var genAvailable = true;
  try {
    eval("(function *(){})"); // jshint ignore:line
  }
  catch (err) {
    genAvailable = false;
  }
  //
  // Now I need to request offline code
  this.socket.send({request: "offcode", ga: genAvailable});
  //
  // add postmessage listener for datamodel
  window.addEventListener("message", function (ev) {
    if (ev.data.destination === "dm")
      App.CordovaSQLiteAdaptor.processMessage(ev.data); // ev.data is cmd
  });
};


/*
 * A rolling rock gathers no moss.
 */
Proxy.runApplication = function ()
{
  console.log("Create App");
  Proxy.session = App.createSession();
  Proxy.session.proxy = Proxy;
  App.fs = App.FS.createFS({app: Proxy.session});
  Client.setProxy(Proxy);
  Client.createApp();
};


/*
 * send the stop app signal to the ide
 */
Proxy.stopApplication = function ()
{
  this.socket.send({request: "stop"});
};


/**
 * Exchange data between client and server part of the application
 * @param {object} sender - who sent the message
 * @param {array} events
 * @param {string} cid - client identifier
 */
Proxy.send = function (sender, events, cid)
{
  if (sender === Proxy.session)
    Client.mainFrame.processRequest(events);
  else
    App.processAppMessage({sid: "S1", cid: cid, content: events});
};


/**
 * Load a list of classes (JS files) then call the callback when finisched
 * @param {string} appName
 * @param {string} appVersion
 * @param {function} callback - function to be called at the end
 */
Proxy.loadClasses = function (appName, appVersion, callback)
{
  // If there are no classes to load, report to callee
  if (!App.classesToLoad || App.classesToLoad.length === 0)
    return callback();
  //
  var head = document.getElementsByTagName('head')[0];
  //
  // Need to override standard app.js contained in the minified version with the
  // actual generated one, only on packaged apps, not ide ones
  if (typeof OfflineBuild !== "undefined" && OfflineBuild) {
    var cs = document.getElementsByTagName("SCRIPT");
    for (var idx = 0; idx < cs.length; idx++) {
      if (cs[idx].src.indexOf("appserver.min.js") > 0) {
        App.classesToLoad.push("../server/app.js");
        break;
      }
    }
  }
  //
  var appendScript = function (i) {
    // If all classes have been loaded, call the callback
    if (i >= App.classesToLoad.length)
      return callback();
    //
    var script = document.createElement('script');
    script.type = 'text/javascript';
    if (appName)
      script.src = (Client && Client.resourceHive ? Client.resourceHive : '') + 'apps/' + appName + App.classesToLoad[i].substring(2) + (appVersion ? '?' + appVersion : '');
    else
      script.src = (Client && Client.resourceOrigin ? Client.resourceOrigin : '') + App.classesToLoad[i] + (appVersion ? '?' + appVersion : '');
    //
    script.onload = function () {
      appendScript(++i);
    };
    //
    script.onerror = function (err) {
      console.error("Error while loading script " + this.src);
    };
    //
    head.appendChild(script);
  };
  //
  // Load the first file
  appendScript(0);
};


/**
 * Send DTT data to the IDE
 * @param {object} data - dtt data
 */
Proxy.sendDTT = function (data)
{
  if (this.socket)
    this.socket.send({request: "dtt", content: data});
};
