/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global Element */

/**
 * Redefine the querySelector function because is used by MD but doesn't escape the special char
 * of the ID, so we must do the escaping
 */
Element.prototype.__querySelector = Element.prototype.querySelector;
Element.prototype.querySelector = function (sele) {
  var escapedSelector = "";
  for (var i = 0; i < sele.length; i++) {
    var char = sele.charAt(i);
    //
    // Special characters must be escaped with \\
    if (":.#".indexOf(char) !== -1 && i > 0)
      escapedSelector += "\\" + char;
    else if (('!"' + "$%&'()*+,/;<=>?@[\]^`{|}~").indexOf(char) !== -1)
      escapedSelector += "\\" + char;
    else
      escapedSelector += char;
  }
  //
  return this.__querySelector(escapedSelector);
};



var Client = Client || {};

/**
 * @class Class for managing editing
 */
Client.ViewEdit = function ()
{
  this.id = "editm";
  Client.eleMap[this.id] = this;
  //
  this.appDiv = document.getElementById("app-ui");
  this.appDiv.style.visibility = "";
  this.msButton = -1;
  this.planMode = false;
  this.editMode = false;
  //
  window.onerror = function (errorMsg, url, lineNumber) {
    alert(url + ":" + lineNumber + "\n" + errorMsg);
    // Just let default handler run.
    return false;
  };
};

/**
 * Activates or deactivates the edit mode
 * @param {boolean} md - true or false to activate/deactivate editing
 * @param {Object} proxy - proxy, the client events are sent to this object
 * @param {boolean} readonly - if the editing is true the editmode can be readonly, to disable D&D and resize
 */
Client.MainFrame.prototype.setEditMode = function (md, proxy, readonly)
{
  // Check the existence of the viewEditor, if present and in plan Mode remove the plan
  var edObj = Client.eleMap.editm;
  if (!Client.eleMap.editm)
    edObj = new Client.ViewEdit();
  else if (edObj.planMode && edObj.setPlanMode)
    edObj.setPlanMode(false);
  //
  // Create proxy and set EditMode
  edObj.editMode = md;
  edObj.editProxy = proxy;
  if (readonly !== undefined)
    edObj.readOnly = readonly;
  var pthis = edObj;
  //
  // Add or remove generic handlers
  if (md) {
    // Create handlers function
    this.mdf = function (ev) {
      pthis.onMouseDown(ev);
    };
    this.muf = function (ev) {
      pthis.onMouseUp(ev);
    };
    this.movrf = function (ev) {
      pthis.onMouseOver(ev);
    };
    this.mmf = function (ev) {
      pthis.onMouseMove(ev);
    };
    this.moutf = function (ev) {
      pthis.onMouseOut(ev);
    };
    this.odsf = function (ev) {
      pthis.onDragStart(ev);
    };
    this.odef = function (ev) {
      pthis.onDragEnter(ev);
    };
    this.odlf = function (ev) {
      pthis.onDragLeave(ev);
    };
    this.odrf = function (ev) {
      pthis.onDrop(ev);
    };
    this.octxm = function (ev) {
      pthis.onContextMenu(ev);
    };
    this.okdf = function (ev) {
      pthis.keyHandler(ev);
    };
    this.clk = function (ev) {
      pthis.onClick(ev);
    };
    this.dblck = function (ev) {
      pthis.onDoubleClick(ev);
    };
    //
    document.body.classList.add("editing");
    //
    // Handle selection & resize operation
    edObj.appDiv.onmousedown = this.mdf;
    document.body.onmouseup = this.muf;
    edObj.appDiv.onmouseover = this.movrf;
    edObj.appDiv.onmousemove = this.mmf;
    edObj.appDiv.onmouseout = this.moutf;
    edObj.appDiv.ondblclick = this.dblck;
    document.onclick = this.clk;
    //
    // Handle D&D operations
    document.body.ondragstart = this.odsf;
    document.body.ondragenter = this.odef;
    document.body.ondragover = this.odef;
    document.body.ondragleave = this.odlf;
    document.body.ondrop = this.odrf;
    document.body.oncontextmenu = this.octxm;
    document.body.onkeydown = this.okdf;
  }
  else {
    // Remove all "edit-mode" handlers
    delete edObj.appDiv.onmousedown;
    delete this.mdf;
    delete document.body.onmouseup;
    delete this.muf;
    delete edObj.appDiv.onmouseover;
    delete this.movrf;
    delete edObj.appDiv.onmousemove;
    delete this.mmf;
    delete edObj.appDiv.onmouseout;
    delete this.moutf;
    delete edObj.appDiv.ondblclick;
    delete this.dblck;
    delete document.onclick;
    delete this.clk;
    //
    delete document.body.ondragstart;
    delete this.odsf;
    delete document.body.ondragenter;
    delete document.body.ondragover;
    delete this.odef;
    delete document.body.ondragleave;
    delete this.odlf;
    delete document.body.ondrop;
    delete this.odrf;
    delete document.body.oncontextmenu;
    delete this.octxm;
    delete document.body.onkeydown;
    delete this.okdf;
    //
    this.mdf = function (ev) {
      pthis.onVisualHint(ev, "mouseDown");
      pthis.onMouseDown(ev);
    };
    this.mmf = function (ev) {
      pthis.onVisualHint(ev, "mouseMove");
    };
    //
    document.body.classList.remove("editing");
    //
    // Even if read only, I want to send visual hints
    edObj.appDiv.onmousedown = this.mdf;
    edObj.appDiv.onmousemove = this.mmf;
  }
  //
  // Change objects ids to reflect edit mode change
  var el = Object.keys(Client.eleMap);
  for (var i = 0; i < el.length; i++) {
    if (Client.eleMap[el[i]].domObj && Client.eleMap[el[i]].domObj.id && Client.eleMap[el[i]].domObj.id !== "app-ui")
      Client.eleMap[el[i]].domObj.id = (md ? "dmo_" : "") + Client.eleMap[el[i]].id;
  }
};


/**
 * returns true if the application is in EditMode, false otherwise
 */
Client.MainFrame.prototype.isEditing = function ()
{
  if (!Client.eleMap.editm || (Client.eleMap.editm.editMode === false && Client.eleMap.editm.planMode === false))
    return false;
  return true;
};


/**
 * returns true if the application is in EditMode, false otherwise
 */
Client.MainFrame.prototype.isPlanning = function ()
{
  if (!Client.eleMap.editm || Client.eleMap.editm.planMode === false)
    return false;
  return true;
};


/**
 * clear all children hilight
 */
Client.MainFrame.prototype.clearHilight = function ()
{
  // Search all objects and remove them (using class)
  var hobjs = document.getElementsByClassName("hilightClass");
  while (hobjs.length > 0) {
    // Get element from hilight object id, set not hilighted
    var ele = Client.eleMap[hobjs.item(0).id];
    if (ele)
      ele.hilighted = false;
    //
    // Remove hilight class
    hobjs.item(0).classList.remove("hilightClass");
  }
  hobjs = document.getElementsByClassName("element-hidden-highlighted");
  while (hobjs.length > 0) {
    // Get element from hilight object id, set not hilighted
    var eleId = hobjs.item(0).id;
    if (eleId.indexOf("dmo_") === 0)
      eleId = eleId.substring(4);
    var ele = Client.eleMap[eleId];
    if (ele && ele.visible === true)
      ele.updateElement({visible: false});
    hobjs.item(0).classList.remove("element-hidden-highlighted");
  }
};


/**
 * Update the hilight objects
 */
Client.MainFrame.prototype.updateHilight = function ()
{

};


/**
 * Get the computed style and the style object of an Element domObj
 * @param {String} id - ID of the element
 */
Client.MainFrame.prototype.getStyle = function (id)
{
  // Create the Object
  var ret = {cs: {}, st: {}};
  //
  // Get the object from the map
  var ele = Client.eleMap[id];
  //
  // if found set the computed style and the style
  if (ele && ele.domObj) {
    ret.cs = window.getComputedStyle(ele.domObj, null);
    ret.st = ele.domObj.style;
  }
  //
  return ret;
};


/**
 * Hilight the element (or hide the hilight)
 * @param {boolean} hl
 */
Client.Element.prototype.hilightObj = function (hl)
{
  // Object has not any dom element?
  if (!this.domObj)
    return;
  //
  if (hl) {
    //
    var nl = document.getElementsByClassName("hilightClass");
    if (nl.length < 2) {
      //
      // Let's see if there are some altcontainers that prevent me to be seen
      // BUT don't do it if there is more than one object selected because it is possible
      // to enter in a loop if they are on different pages
      var alt = this.parent;
      var obj = this;
      while (alt) {
        if (alt instanceof Client.AltContainer) {
          var idx = alt.elements ? alt.elements.indexOf(obj) : -1;
          if (idx > -1 && idx !== alt.selectedPage)
            alt.updateElement({selectedPage: idx});
        }
        obj = alt;
        alt = alt.parent;
      }
    }
    //
    this.getRootObject().classList.add("hilightClass");
    //
    // If the element or one of its parent are hidden show them
    var hid = this;
    while (hid) {
      if (hid.visible === false && hid.updateElement) {
        hid.updateElement({visible: true, forhighlight: true});
        //
        // NOTE : Don't use getRootObject since we use the class to find the object, so the class must be set on the element
        // that has the ID
        hid.domObj.classList.add("element-hidden-highlighted");
      }
      hid = hid.parent;
    }
    //
    this.hilighted = true;
  }
  else {
    this.domObj.classList.remove("hilightClass");
    //
    // Hide the parent element when deselcting this element
    var hid = this;
    while (hid) {
      if (hid.visible === true && hid.getRootObject && hid.getRootObject().classList.contains("element-hidden-highlighted")) {
        hid.updateElement({visible: false});
        hid.domObj.classList.remove("element-hidden-highlighted");
      }
      hid = hid.parent;
    }
    //
    this.hilighted = false;
  }
};


/**
 * Start drag operation
 * @param {DragEvent} ev
 */
Client.ViewEdit.prototype.onDragStart = function (ev)
{
  // Drag only if editing and not in readOnly
  if (!this.editMode || !this.editProxy || this.readOnly) {
    ev.preventDefault();
    return;
  }
  //
  // Get the dragged object
  var realObj = this.getElementObj(ev.target);
  if (realObj) {
    // The real drop operation is done on all the selected objects (if absolute drop only the absolute objects, else only the NOT absolute)
    var drid = realObj.id.indexOf("dmo_") !== -1 ? realObj.id.substring(4, realObj.id.length) : realObj.id;
    this.drtobj = [Client.eleMap[drid]];
    //
    // know the position of the 'real' dragged object..
    var abs = window.getComputedStyle(realObj, null).position === "absolute";
    var hl = document.getElementsByClassName("hilightClass");
    //
    // For all hilighted objects: add them to the dragged objects list (only if the position is the same of the 'real' dragged object)
    for (var hli = 0; hli < hl.length; hli++) {
      var selId = hl[hli].id;
      if (!selId) {
        // For example the IonTextarea has an external object that is highlighted but the real domObj is internal..
        // the external object hasn't the id so we check if there is a direct child of the object that has the ID, if not skip the drag
        var ch = hl[hli].firstChild;
        while (ch && !selId) {
          if (ch && ch.id)
            selId = ch.id;
          //
          ch = ch.nextSibling;
        }
      }
      if (selId) {
        var hliid = selId.indexOf("dmo_") !== -1 ? selId.substring(4, selId.length) : selId;
        var so = Client.eleMap[hliid];
        var abs2 = window.getComputedStyle(so.domObj, null).position === "absolute";
        //
        // add the object to the list
        if (so && so.domObj !== realObj && abs === abs2) {
          this.drtobj.push(so);
          drid += " " + so.id;
        }
      }
    }
    if (this.drtobj && this.drtobj.length > 0)
    {
      ev.dataTransfer.setData("objid", drid.trim());
      //
      // Change position of the dragged object to 0,0.. so when repositioning the coordinates are ok!
      ev.dataTransfer.setDragImage(realObj, 0, 0);
    }
    else
      ev.preventDefault(); // No Drag
  }
};


/**
 * HTML5 Drag over an object
 * @param {DragEvent} ev
 */
Client.ViewEdit.prototype.onDragEnter = function (ev)
{
  if (!this.editMode || !this.editProxy || this.readOnly)
    return;
  //
  if (ev.dataTransfer.types.indexOf("Files") >= 0)
    return;
  //
  // Search if drop operation is allowed (drop on a valid object)
  var obd = this.getElementObj(ev.target);
  //
  // The VIEW is a valid drop target for the tool operations
  if (ev.dataTransfer.types.indexOf("tooldrop") !== -1 && ev.target.id === "app-ui")
    obd = ev.target;
  //
  if (!obd)
    return;
  //
  // Get the Element
  var ele = null;
  for (var mp in Client.eleMap) {
    if (Client.eleMap[mp] && Client.eleMap[mp].domObj === obd) {
      ele = Client.eleMap[mp];
      break;
    }
  }
  //
  // if the element is found the drag operation is ok
  if (ele) {
    ev.preventDefault();
    //
    // If dragging an absolute object the operation is ever 0 (on Object), see ONDROP
    // -> only if the drag operation is an OBJ operation
    if (ev.dataTransfer.types.indexOf("objid") !== -1) {
      var ok = true;
      if (this.drtobj) {
        var dragDom = this.drtobj[0].domObj;
        ok = window.getComputedStyle(dragDom, null).position === "absolute";
      }
      if (ok) {
        // Get the drop container: hilight it!
        var dropCnt = ele;
        while (dropCnt) {
          if (dropCnt instanceof Client.Container || dropCnt instanceof Client.AltContainer || dropCnt instanceof Client.View)
            break;
          dropCnt = dropCnt.parent;
        }
        dropCnt.domObj.classList.add("drop-over");
        //
        this.drtype = 0;
        return;
      }
    }
    //
    // Clear old effects
    obd.classList.remove("drop-left");
    obd.classList.remove("drop-right");
    obd.classList.remove("drop-top");
    obd.classList.remove("drop-bottom");
    obd.classList.remove("drop-over");
    //
    // detect drop type using positions
    // Calculate the top/left of the object
    var l = obd.scrollLeft;
    var t = obd.scrollTop;
    var p = document.getElementById("app-ui");
    var obj = obd;
    //
    // Sum parent top/left
    while (obj && obj !== p) {
      t += obj.offsetTop - obj.scrollTop + obj.clientTop;
      l += obj.offsetLeft - obj.scrollLeft + obj.clientLeft;
      obj = obj.offsetParent;
    }
    //
    // Last: consider scrollbar..
    t += document.body.scrollTop;
    l += document.body.scrollLeft;
    //
    // Now calc delta
    var dy = ev.clientY - t;
    var dx = ev.clientX - l;
    //
    // Calculate drop operation (if the drag is left/right/top/bottom/over)
    // - calculate the limits
    var ld = (obd.offsetWidth / 3 > 10 ? 10 : obd.offsetWidth / 3);
    var rd = ((obd.offsetWidth / 3) * 2 < obd.offsetWidth - 10 ? obd.offsetWidth - 10 : (obd.offsetWidth / 3) * 2);
    var td = (obd.offsetHeight / 3 > 10 ? 10 : obd.offsetHeight / 3);
    var bd = ((obd.offsetHeight / 3) * 2 < obd.offsetHeight - 10 ? obd.offsetHeight - 10 : (obd.offsetHeight / 3) * 2);
    var dropOp = 0;
    //
    // Now : decide drop operation!
    if (dx < ld) {
      dropOp = -2; // Left
      obd.classList.add("drop-left");
    }
    else if (dx > rd) {
      dropOp = 2; // Right
      obd.classList.add("drop-right");
    }
    else if (dy < td) {
      dropOp = -1; // Top
      obd.classList.add("drop-top");
    }
    else if (dy > bd) {
      dropOp = 1; // Bottom
      obd.classList.add("drop-bottom");
    }
    //
    this.drtype = dropOp;
    //
    // The view can accept only a 'drop into' operation and only if is a tooldrop operation
    if (ele instanceof Client.View) {
      this.drtype = 0;
      //
      obd.classList.remove("drop-left");
      obd.classList.remove("drop-right");
      obd.classList.remove("drop-top");
      obd.classList.remove("drop-bottom");
    }
    if (this.drtype === 0)
      obd.classList.add("drop-over");
    //
    // Try to check if the D&D operation is permitted
    var drOp = ev.dataTransfer.types.indexOf("tooldrop") !== -1 ? 1 : 2;
    var sourceObj = (this.drtobj && this.drtobj.length > 0 ? this.drtobj[0].id : null);
    var ctrl = ev.ctrlKey || ev.metaKey;
    ev.dataTransfer.dropEffect = this.editProxy.isObjectDraggable(drOp, ele.id, this.drtype, sourceObj, ctrl, ev.altKey, ev.shiftKey);
  }
};


/**
 * HTML5 Drag leaves an object
 * @param {DragEvent} ev
 */
Client.ViewEdit.prototype.onDragLeave = function (ev)
{
  // Clear old effects
  var realTarget = this.getElementObj(ev.target);
  if (realTarget) {
    realTarget.classList.remove("drop-left");
    realTarget.classList.remove("drop-right");
    realTarget.classList.remove("drop-top");
    realTarget.classList.remove("drop-bottom");
  }
  //
  // Clear drag operation
  this.drtype = undefined;
  //
  // Remove drop-over
  var dol = document.getElementsByClassName("drop-over");
  while (dol.length > 0)
    dol[0].classList.remove("drop-over");
};


/**
 * HTML5 Drop done
 * @param {DragEvent} ev
 */
Client.ViewEdit.prototype.onDrop = function (ev)
{
  // if not editing or the operation is not set.. for security reason exit
  if (!this.editMode || !this.editProxy || this.drtype === undefined || this.readOnly)
    return;
  //
  // Prevent to tell the browser that the drop is done by us
  ev.preventDefault();
  //
  // Search object.. if not found.. exit..
  var obd = this.getElementObj(ev.target);
  //
  // The VIEW is a valid drop target for the tool operations
  if (ev.dataTransfer.types.indexOf("tooldrop") !== -1 && ev.target.id === "app-ui")
    obd = ev.target;
  //
  if (!obd)
    return;
  //
  // Get the Element from the map
  var ele = null;
  for (var mp in Client.eleMap) {
    if (Client.eleMap[mp] && Client.eleMap[mp].domObj === obd) {
      ele = Client.eleMap[mp];
      break;
    }
  }
  //
  // Detect the drag operation: can be an object d&d (same as tree d&d operation) or can be a toolbox drop operation
  if (ev.dataTransfer.types.indexOf("objid") !== -1) {
    // Get the data of the dragged objects
    var dragid = ev.dataTransfer.getData("objid");
    //
    // if found the object and the data do the drop operation
    if (ele && dragid !== "") {
      // find the container of the dropped object (Container, AltContainer, View)
      var dropCnt = ele;
      while (dropCnt) {
        if (dropCnt instanceof Client.Container || dropCnt instanceof Client.AltContainer || dropCnt instanceof Client.View)
          break;
        dropCnt = dropCnt.parent;
      }
      //
      var r = [];
      //
      // Get the position of the dropped object
      if (this.drtobj) {
        var abs = dropCnt.layout === "absolute" || window.getComputedStyle(this.drtobj[0].domObj, null).position === "absolute";
        //
        // For each dropped object
        for (var i = 0; i < this.drtobj.length; i++)
        {
          // Check D&D on an absolute positioned Object
          if (abs) {
            // if D&D operation on absolute object:
            // get the container of the dropped object and get the container of the dragged object
            // if they are the same is a repos operation, else is a drop (on the new container..)
            var dragCnt = this.drtobj[i];
            while (dragCnt) {
              if (dragCnt instanceof Client.Container || dragCnt instanceof Client.AltContainer || dragCnt instanceof Client.View)
                break;
              dragCnt = dragCnt.parent;
            }
            //
            // Drop of the object on the destination container
            if (dragCnt !== dropCnt)
              r.push({dragid: this.drtobj[i].id, dropid: dropCnt.id, op: this.drtype, c: "ondrop", alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey});
            //
            // Now repos the object (remove from the mouse position the OFFSET of the destination container)
            var rect = dropCnt.domObj.getBoundingClientRect();
            var dx = ev.clientX - rect.left;
            var dy = ev.clientY - rect.top;
            r.push({rsid: this.drtobj[i].id, l: dx + "px", t: dy + "px", c: "resize", alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey});
          }
          else {
            // Drop operation
            r.push({dragid: this.drtobj[i].id, dropid: ele.id, op: this.drtype, c: "ondrop", alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey});
          }
        }
        this.editProxy.appCmd(r);
      }
      else { // No dragged object in view editor, we are dragging from the treeview
        this.editProxy.appCmd([{dragid: dragid, dropid: dropCnt.id, op: this.drtype, c: "ondrop", alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey}]);
      }
    }
  }
  else if (ev.dataTransfer.types.indexOf("tooldrop") !== -1) {
    // D&D operation of the toolview:
    if (ele)
      this.editProxy.appCmd([{toolid: ev.dataTransfer.getData("tooldrop"), dropid: ele.id, op: this.drtype, c: "ontooldrop", alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey}]);
  }
  //
  // Clear operation
  this.drtobj = undefined;
  this.msButton = -1;
  this.onDragLeave(ev);
};


/**
 * Handles the mouse down on a object: if the object is resizable start resizing and send the click to the proxy
 * if the click is not on a resize handle or a draggable object start rectangular selection
 * @param {MouseEvent} ev
 */
Client.ViewEdit.prototype.onMouseDown = function (ev)
{
  if (!this.editProxy)
    return;
  //
  // Detect selected object
  var obd = this.getElementObj(ev.target);
  var ele = null;
  //
  // memorize Data
  var startRectSel = true;
  this.msButton = ev.button;
  this.msCoordX = ev.clientX;
  this.msCoordY = ev.clientY;
  this.msAlreadySel = false;
  //
  // Click on an object : check if is element.. if not continue (maybe we can start the rectangular selection)
  if (obd) {
    // an id is needed..
    var oid = obd.id.indexOf("dmo_") !== -1 ? obd.id.substring(4, obd.id.length) : obd.id;
    ele = Client.eleMap[oid];
    //
    // Sometimes the ID of the object is not compatible with jquery so we use btoa(). If the element is not in the map try to search by using reverse atob() of the ID
    if (!ele && oid) {
      try {
        ele = Client.eleMap[atob(oid)];
      }
      catch (ex) {
      }
    }
    //
    if (ele) {
      var stopeve = true;
      var sendeve = true;
      //
      // Check if the object is already selected
      var hl = document.getElementById("hlo-" + oid);
      this.msAlreadySel = (hl !== null);
      //
      // Don't stop propagation if the object is draggable..
      // DRAG HANDLING
      if (obd.draggable) {
        // prevent default && not start rectangular selection
        stopeve = false;
        startRectSel = false;
        //
        // if the object is already hilighted must not inform the server: the D&D is done on all selection
        if (this.msAlreadySel)
          sendeve = false;
      }
      else if (obd.style.cursor !== "" && obd.style.cursor !== "move" && this.msButton === 0 && !this.readOnly) {
        // RESIZE HANDLING
        // If the object is not draggable the mouse over has decided that is a resize operation
        this.mousePosX = ev.clientX;
        this.mousePosY = ev.clientY;
        this.deltaMovX = 0;
        this.deltaMovY = 0;
        this.resizing = true;
        startRectSel = false;
        this.resizeObj = obd;
        //
        // Consider zooming when calculating resize operation
        var z = document.body.style.zoom;
        var zm = z ? parseInt(z) / 100 : 1;
        var dy = this.adaptForZoom(ev.offsetY, zm);
        var dx = this.adaptForZoom(ev.offsetX, zm);
        //
        // Calc limits
        var to = (obd.offsetHeight * 1 / 3 > 10 ? 10 : obd.offsetHeight * 1 / 3);
        var bo = (obd.offsetHeight * 1 / 3 > 10 ? obd.offsetHeight - 10 : obd.offsetHeight * 2 / 3);
        var lo = (obd.offsetWidth * 1 / 3 > 10 ? 10 : obd.offsetWidth * 1 / 3);
        var ro = (obd.offsetWidth * 1 / 3 > 10 ? obd.offsetWidth - 10 : obd.offsetWidth * 2 / 3);
        //
        // Calc position and resize operation
        this.trX = 0, this.trY = 0;
        if (dx <= lo)
          this.trX = -1;
        else if (dx >= ro)
          this.trX = 1;
        else
          this.trX = 0;
        if (dy <= to)
          this.trY = -1;
        else if (dy >= bo)
          this.trY = 1;
        else
          this.trY = 0;
        //
        // Memorize original values
        // Resize on selected element AND other elements hilighted
        var resList = [this.resizeObj];
        var hls = document.getElementsByClassName("hilightClass");
        var hli;
        for (hli = 0; hli < hls.length; hli++) {
          var so = document.getElementById("dmo_" + hls[hli].id);
          if (so && so !== this.resizeObj)
            resList.push(so);
        }
        //
        // Memorize original dimensions (to handle % in style)
        for (hli = 0; hli < resList.length; hli++) {
          var robj = resList[hli];
          var objh = robj.offsetHeight;
          var objw = robj.offsetWidth;
          if (window.getComputedStyle) {
            var cStyle = window.getComputedStyle(robj, null);
            objh = cStyle.height;
            objh = cStyle.width;
          }
          robj.setAttribute("origH", objh);
          robj.setAttribute("origW", objw);
          robj.setAttribute("origL", robj.offsetLeft);
          robj.setAttribute("origT", robj.offsetTop);
          //
          // Set the 'real' style values
          robj.setAttribute("styleH", robj.style.height);
          robj.setAttribute("styleW", robj.style.width);
          robj.setAttribute("styleL", robj.style.left);
          robj.setAttribute("styleT", robj.style.top);
        }
      }
      //
      if (stopeve) {
        ev.preventDefault();
        ev.stopPropagation();
      }
      //
      // The container is selected in MouseUp : we need it deselected here for rectangular/D&D (only for selection, not resizing.. if resizing we must select here)
      if ((ele instanceof Client.Container || ele instanceof Client.AltContainer) && !(obd.style.cursor !== "" && obd.style.cursor !== "move"))
        sendeve = false;
      //
      if (sendeve) {
        var r = [{id: ele.id, c: "mouseDown", alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey, button: this.msButton, target: ev.target, offsetX: ev.offsetX, offsetY: ev.offsetY}];
        this.editProxy.appCmd(r);
      }
    }
  }
  //
  // Click on an object that is not resizable nor draggable: we can start the rectangular selection
  // RECTANGULAR SELECTION HANDLING
  if (startRectSel && this.msButton === 0 && !this.readOnly) {
    // Prevent default
    ev.preventDefault();
    ev.stopPropagation();
    ev.target.focus();
    //
    // Set position info
    this.inSelection = true;
    this.elasticX = ev.clientX;
    this.elasticY = ev.clientY;
    //
    // Calculating object positions
    this.objRectMap = [];
    for (var k in Client.eleMap) {
      var objUI = Client.eleMap[k];
      //
      // for each element (no container and altcontainer) memorize position.. used for hilighting
      if (objUI instanceof Client.Element) {
        if (objUI.domObj) {
          var objRect = objUI.domObj.getBoundingClientRect();
          this.objRectMap.push({rectX: objRect.left, rectY: objRect.top, rectW: objUI.domObj.offsetWidth, rectH: objUI.domObj.offsetHeight, rectObj: objUI, hil: objUI.hilighted});
        }
      }
    }
    //
    // Clear Hilight - selection must remain if CTRL is pressed
    if (!(ev.ctrlKey || ev.metaKey)) {
      var ol = document.getElementsByClassName("hilightClass");
      while (ol.length > 0)
        ol.item(0).classList.remove("hilightClass");
    }
  }
};


/**
 * Mouse up : clear resizing
 * @param {MouseEvent} ev
 */
Client.ViewEdit.prototype.onMouseUp = function (ev)
{
  // Check if the mouse is moved: if not and button = 0 is a click
  if (ev && ev.clientX > this.msCoordX - 2 && ev.clientX < this.msCoordX + 2 && ev.clientY > this.msCoordY - 2 && ev.clientY < this.msCoordY + 2 && !this.resizing && this.msButton === 0) {
    var obd = this.getElementObj(ev.target);
    //
    // if found the object handle the click
    if (obd) {
      var oid = obd.id.indexOf("dmo_") !== -1 ? obd.id.substring(4, obd.id.length) : obd.id;
      var ele = Client.eleMap[oid];
      if (ele) {
        if (!this.msAlreadySel) {  // Click on an container: select it
          this.editProxy.appCmd([{id: ele.id, c: "mouseDown", alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey, button: this.msButton}]);
        }
      }
    }
    else {
      if (ev.target && ev.target.id === "app-ui")  // Click on the app-ui : select the View
        this.editProxy.appCmd([{id: "", c: "viewSelect", alt: ev.altKey, ctr: false, sht: ev.shiftKey}]);
      else  // Click on an object that is not an element nor a child: deselect all
        this.editProxy.appCmd([{id: "", c: "deselect", alt: ev.altKey, ctr: false, sht: ev.shiftKey}]);
    }
    //
    this.resizing = false;
    this.resizeObj = null;
    delete this.deltaMovX;
    delete this.deltaMovY;
  }
  //
  // End of resizing
  if (this.resizing) {
    // Resize on selected element AND other elements hilighted
    var resList = [this.resizeObj];
    var hl = document.getElementsByClassName("hilightClass");
    for (var hli = 0; hli < hl.length; hli++) {
      // Create the list of the resized elements
      var so = document.getElementById("dmo_" + hl[hli].id);
      if (so && so !== this.resizeObj)
        resList.push(so);
    }
    //
    var r = [];
    for (var hli = 0; hli < resList.length; hli++) {
      var robj = resList[hli];
      //
      // If the movement is over the threshold send the resize message to the server
      if (this.deltaMovX >= 5 || this.deltaMovY >= 5) {
        var s = robj.style;
        //
        // Send message with new dimensions
        var rsid = robj.id.indexOf("dmo_") !== -1 ? robj.id.substring(4, robj.id.length) : robj.id;
        r.push({rsid: rsid, l: s.left, t: s.top, w: s.width, h: s.height, c: "resize", alt: ev ? ev.altKey : false, ctr: ev ? ev.ctrlKey || ev.metaKey : false, sht: ev ? ev.shiftKey : false, origh: robj.getAttribute("origH"), origw: robj.getAttribute("origW"), origl: robj.getAttribute("origL"), origt: robj.getAttribute("origT")});
      }
      else {
        // Restore the original values
        robj.style.height = robj.getAttribute("styleH");
        robj.style.width = robj.getAttribute("styleW");
        robj.style.left = robj.getAttribute("styleL");
        robj.style.top = robj.getAttribute("styleT");
      }
      //
      robj.setAttribute("origH", "");
      robj.setAttribute("origW", "");
      robj.setAttribute("origL", "");
      robj.setAttribute("origT", "");
      //
      robj.setAttribute("styleH", "");
      robj.setAttribute("styleW", "");
      robj.setAttribute("styleL", "");
      robj.setAttribute("styleT", "");
    }
    this.editProxy.appCmd(r);
    //
    // Clear UI
    this.resizing = false;
    this.resizeObj = null;
    delete this.deltaMovX;
    delete this.deltaMovY;
  }
  //
  // End of Rectangular selection
  if (this.inSelection) {
    // Clear data
    this.inSelection = false;
    this.elasticX = undefined;
    this.elasticY = undefined;
    this.objRectMap = null;
    //
    // Remove selection elements
    if (this.rectSel && this.rectSel.parentNode) {
      this.rectSel.parentNode.removeChild(this.rectSel);
      this.rectSel = null;
    }
    //
    // Now.. we must tell the proxy the selected objects!
    var r = [];
    var ol = document.getElementsByClassName("hilightClass");
    for (var i = 0; i < ol.length; i++) {
      r.push({id: ol.item(i).id, c: "mouseDown", alt: ev && ev.altKey, ctr: (i !== 0), sht: ev && ev.shiftKey, button: this.msButton});
    }
    this.editProxy.appCmd(r);
    //
    // If the selection ended on the device-ui the device rotates.. block that rotation
    if (ev && ev.target && ev.target.id === "device-ui")
      Client.mainFrame.device.stopRotation = true;
  }
  //
  // Clear button pressed (used to detect resize/drag operation)
  // Clear position (for detecting delta)
  this.msButton = -1;
  this.mousePosX = null;
  this.mousePosY = null;
  //
  // Send the mouse up to the main UI (protection: the MouseUp outside the FW is sent to the app..
  // so if we send the mouseUp there is a loop.. so we must send only a real mouse operation - with a real event)
  if (ev)
    this.editProxy.appCmd([{c: "mouseUp", alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey, btn: ev.button}]);
};


/**
 * Mouse over: handles hilight of the selected object (with mouse out)
 * @param {MouseEvent} ev
 */
Client.ViewEdit.prototype.onMouseOver = function (ev)
{
  // Add the overlay to the element
  var obd = this.getElementObj(ev.target);
  if (!obd)
    return;
  obd.classList.add("overHilight");
};


/**
 * Mouse move: detect resizing type or resize object
 * @param {MouseEvent} ev
 */
Client.ViewEdit.prototype.onMouseMove = function (ev)
{
  // Send the mouse move message to the editor (for handling split divider resize action)
  var r = [{c: "mouseMove", alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey, pageX: ev.pageX, target: ev.target, offsetX: ev.offsetX, offsetY: ev.offsetY}];
  this.editProxy.appCmd(r);
  //
  // Handle Rectangular Selection
  if (this.inSelection) {
    var z = document.body.style.zoom;
    var zm = z ? parseInt(z) / 100 : 1;
    //
    // Create the rectObject
    if (!this.rectSel) {
      this.rectSel = document.createElement("DIV");
      this.rectSel.className = "rectSelection";
      document.body.appendChild(this.rectSel);
      this.rectSel.style.top = this.adaptForZoom(this.elasticY, zm) + "px";
      this.rectSel.style.left = this.adaptForZoom(this.elasticX, zm) + "px";
      this.rectSel.style.height = "0px";
      this.rectSel.style.width = "0px";
    }
    //
    // rectangular object positioning
    var rect = {};
    if (ev.clientX > this.elasticX) {
      this.rectSel.style.width = this.adaptForZoom(ev.clientX - this.elasticX, zm) + "px";
      this.rectSel.style.left = this.adaptForZoom(this.elasticX + document.body.scrollLeft, zm) + "px";
      rect.rectX = this.adaptForZoom(this.elasticX, zm);
      rect.rectW = this.adaptForZoom(ev.clientX - this.elasticX, zm);
    }
    else {
      this.rectSel.style.left = this.adaptForZoom(ev.clientX + document.body.scrollLeft, zm) + "px";
      this.rectSel.style.width = this.adaptForZoom(this.elasticX - ev.clientX, zm) + "px";
      rect.rectX = this.adaptForZoom(ev.clientX, zm);
      rect.rectW = this.adaptForZoom(this.elasticX - ev.clientX, zm);
    }
    if (ev.clientY > this.elasticY) {
      this.rectSel.style.top = this.adaptForZoom(this.elasticY + document.body.scrollTop, zm) + "px";
      this.rectSel.style.height = this.adaptForZoom(ev.clientY - this.elasticY, zm) + "px";
      rect.rectY = this.adaptForZoom(this.elasticY, zm);
      rect.rectH = this.adaptForZoom(ev.clientY - this.elasticY, zm);
    }
    else {
      this.rectSel.style.top = this.adaptForZoom(ev.clientY + document.body.scrollTop, zm) + "px";
      this.rectSel.style.height = this.adaptForZoom(this.elasticY - ev.clientY, zm) + "px";
      rect.rectY = this.adaptForZoom(ev.clientY, zm);
      rect.rectH = this.adaptForZoom(this.elasticY - ev.clientY, zm);
    }
    //
    // Select or deselect nodes
    for (var i = 0; i < this.objRectMap.length; i++) {
      var objUI = this.objRectMap[i];
      var mustSelect = true;
      var hilighted = objUI.hil;
      var ox = objUI.rectX - document.body.scrollLeft;
      var oy = objUI.rectY - document.body.scrollTop;
      //
      // detect if the object must be selected or not :
      // using object rect AND selection rect
      // Select the object only if completely contained in the selection rect
      /*
       if ((oy < rect.rectY && oy + objUI.rectH < rect.rectY) ||
       (oy > rect.rectY + rect.rectH && oy + objUI.rectH > rect.rectY + rect.rectH))
       mustSelect = false;
       if ((ox < rect.rectX && ox + objUI.rectW < rect.rectX) ||
       (ox > rect.rectX + rect.rectW && ox + objUI.rectW > rect.rectX + rect.rectW))
       mustSelect = false;
       */
      mustSelect = false;
      if (oy >= rect.rectY && ox >= rect.rectX &&
              oy + objUI.rectH <= rect.rectY + rect.rectH && ox + objUI.rectW <= rect.rectX + rect.rectW)
        mustSelect = true;
      //
      // If the object was already hilighted && we are pressing CTRL the rectangular selection must not deselct it
      if (mustSelect)
        objUI.rectObj.hilightObj(true);
      else if (!(hilighted && (ev.ctrlKey || ev.metaKey)))
        objUI.rectObj.hilightObj(false);
    }
  }
  else if (!this.resizing && !this.readOnly) {
    // Handle resize decision
    var obd = this.getElementObj(ev.target);
    //
    // Mouse move with a button pressed and not resizing... sounds like a D&D operation..
    if (this.msButton !== -1)
      return;
    //
    // MouseOver on an object : check if is element.. if not exit (TODO: decide what to do..)
    if (!obd)
      return;
    //
    var cn = "";
    //
    // If an object has "display=inline" it is not resizable
    var compDisplay = window.getComputedStyle(obd, null)["display"];
    if (compDisplay === "inline") {
      cn = "move";
    }
    else {
      // Calculate zoom : we must consider it when handling position of the mouse
      var z = document.body.style.zoom;
      var zm = z ? parseInt(z) / 100 : 1;
      //
      // Get element & delta
      var eleid = obd.id.indexOf("dmo_") !== -1 ? obd.id.substring(4, obd.id.length) : obd.id;
      var ele = Client.eleMap[eleid];
      var dy = this.adaptForZoom(ev.offsetY, zm);
      var dx = this.adaptForZoom(ev.offsetX, zm);
      var pos = window.getComputedStyle(obd, null)["position"];
      //
      // Calc limits
      var to = (obd.offsetHeight * 1 / 3 > 10 ? 10 : obd.offsetHeight * 1 / 3);
      var bo = (obd.offsetHeight * 1 / 3 > 10 ? obd.offsetHeight - 10 : obd.offsetHeight * 2 / 3);
      var lo = (obd.offsetWidth * 1 / 3 > 10 ? 10 : obd.offsetWidth * 1 / 3);
      var ro = (obd.offsetWidth * 1 / 3 > 10 ? obd.offsetWidth - 10 : obd.offsetWidth * 2 / 3);
      //
      // Calc positions
      var trasfX = 0, trasfY = 0;
      if (dx <= lo)
        trasfX = -1;
      else if (dx >= ro)
        trasfX = 1;
      else
        trasfX = 0;
      if (dy <= to)
        trasfY = -1;
      else if (dy >= bo)
        trasfY = 1;
      else
        trasfY = 0;
      //
      switch (trasfX)
      {
        case - 1:
          switch (trasfY)
          {
            case - 1:
              cn = pos !== "absolute" ? "move" : "nw-resize";
              break;
            case 0:
              cn = pos !== "absolute" ? "move" : "e-resize";
              break;
            case 1:
              cn = pos !== "absolute" ? "move" : "sw-resize";
              break;
          }
          break;

        case 0:
          switch (trasfY)
          {
            case - 1:
              cn = pos !== "absolute" ? "move" : "n-resize";
              break;
            case 0:
              cn = "move";
              break;
            case 1:
              cn = "n-resize";
              break;
          }
          break;

        case 1:
          switch (trasfY)
          {
            case - 1:
              cn = pos !== "absolute" ? "move" : "sw-resize";
              break;
            case 0:
              cn = "e-resize";
              break;
            case 1:
              cn = "nw-resize";
              break;
          }
          break;
      }
    }
    //
    // A container cannot be dragged/moved
    if ((ele instanceof Client.Container || ele instanceof Client.AltContainer) && cn === "move")
      cn = "";
    //
    // Set cursor && draggability
    obd.style.cursor = cn;
    if (cn === "move")
      obd.draggable = true;
    else
      obd.draggable = false;
  }
  else if (this.resizing) {
    // HANDLE RESIZE
    // Set position (for security reason, set also in mouseDown)
    if (!this.mousePosX)
      this.mousePosX = ev.clientX;
    if (!this.mousePosY)
      this.mousePosY = ev.clientY;
    //
    // Calc delta (name is the same as the MouseEvent property of chrome..)
    var movementX = ev.clientX - this.mousePosX;
    var movementY = ev.clientY - this.mousePosY;
    this.deltaMovX += Math.abs(movementX);
    this.deltaMovY += Math.abs(movementY);
    //
    // No click && no text selection
    ev.preventDefault();
    //
    // Resize on selected element AND other elements hilighted
    var resList = [this.resizeObj];
    var hl = document.getElementsByClassName("hilightClass");
    for (var hli = 0; hli < hl.length; hli++) {
      var so = document.getElementById("dmo_" + hl[hli].id);
      if (so && so !== this.resizeObj)
        resList.push(so);
    }
    //
    for (var hli = 0; hli < resList.length; hli++) {
      var robj = resList[hli];
      //
      // Detect object actual dimension (or use offsetWidth if not set..)
      var objh = (robj.style.height && robj.style.height.indexOf("px") !== -1) ? parseInt(robj.style.height, 10) : robj.offsetHeight;
      var objw = (robj.style.width && robj.style.width.indexOf("px") !== -1) ? parseInt(robj.style.width, 10) : robj.offsetWidth;
      //
      // Apply resize to the object, using delta && the type of the resize (calculated in MouseDown)
      switch (this.trX) {
        case - 1: // LEFT
          switch (this.trY) {
            case - 1: // TOP
              robj.style.left = (robj.offsetLeft + movementX) + "px";
              robj.style.width = (objw - movementX) + "px";
              robj.style.top = (robj.offsetTop + movementY) + "px";
              robj.style.height = (objh - movementY) + "px";
              break;

            case 0: // CENTER
              robj.style.left = (robj.offsetLeft + movementX) + "px";
              robj.style.width = (objw - movementX) + "px";
              break;

            case 1: // BOTTOM
              robj.style.left = (robj.offsetLeft + movementX) + "px";
              robj.style.width = (objw - movementX) + "px";
              robj.style.height = (objh + movementY) + "px";
              break;
          }
          break;

        case 0: // CENTER
          switch (this.trY) {
            case - 1: // TOP
              robj.style.top = (robj.offsetTop + movementY) + "px";
              robj.style.height = (objh - movementY) + "px";
              break;

            case 0: // CENTER
              break;

            case 1: // BOTTOM
              robj.style.height = (objh + movementY) + "px";
              break;
          }
          break;

        case 1: // RIGTH
          switch (this.trY) {
            case - 1: // TOP
              robj.style.width = (objw + movementX) + "px";
              robj.style.top = (robj.offsetTop - movementY) + "px";
              robj.style.height = (objh - movementY) + "px";
              break;

            case 0: // CENTER
              robj.style.width = (objw + movementX) + "px";
              break;

            case 1: // BOTTOM
              robj.style.width = (objw + movementX) + "px";
              robj.style.height = (objh + movementY) + "px";
              break;
          }
          break;
      }
      //
      // Set new position for delta calc
      this.mousePosX = ev.clientX;
      this.mousePosY = ev.clientY;
      //
      // Repos hilight object (if present)
      var robjid = robj.id.indexOf("dmo_") !== -1 ? robj.id.substring(4, robj.id.length) : robj.id;
      var hlobj = document.getElementById("hlo-" + robjid);
      if (hlobj) {
        // Repos hilight
        hlobj.style.width = robj.offsetWidth + "px";
        hlobj.style.height = robj.offsetHeight + "px";
        var rect = robj.getBoundingClientRect();
        hlobj.style.top = (rect.top + window.scrollY) + "px";
        hlobj.style.left = (rect.left + window.scrollX) + "px";
      }
    }
  }
};


/**
 * removes the hilight on the object
 * @param {MouseEvent} ev
 */
Client.ViewEdit.prototype.onMouseOut = function (ev)
{
  var obd = this.getElementObj(ev.target);
  if (!obd)
    return;
  //
  obd.removeAttribute("draggable");
  obd.style.cursor = "";
  obd.classList.remove("overHilight");
};


/**
 * click on an object, sent to the proxy
 * @param {MouseEvent} ev
 */
Client.ViewEdit.prototype.onClick = function (ev)
{
  // Send message to editor
  var r = [{c: "onclick", alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey}];
  this.editProxy.appCmd(r);
};


/**
 * Double click on an object, used to show the editor
 * @param {MouseEvent} ev
 */
Client.ViewEdit.prototype.onDoubleClick = function (ev)
{
  if (!this.editMode || !this.editProxy || this.readOnly)
    return;
  //
  // Detect selected object
  var obd = this.getElementObj(ev.target);
  var ele = null;
  //
  while (obd) {
    if (obd.id && obd.id.indexOf("dmo_") !== -1)
      break;
    obd = obd.parentElement;
  }
  //
  // if found search the element && open the editor (if present)
  if (obd) {
    var oid = obd.id.indexOf("dmo_") !== -1 ? obd.id.substring(4, obd.id.length) : obd.id;
    ele = Client.eleMap[oid];
    //
    if (ele !== null) {
      if (ele.showEditor)
        ele.showEditor();
      else {
        // If the object hasn't a client editor send the message to the server,
        // in the IDE we could open its editor
        var r = [{id: oid, c: "doubleClick", alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey, button: ev.button}];
        this.editProxy.appCmd(r);
      }
    }
  }
};


/**
 * Context menu opening on the editor
 * @param {MouseEvent} ev
 */
Client.ViewEdit.prototype.onContextMenu = function (ev)
{
  // Stop Browser Context Menu
  ev.preventDefault();
  ev.stopPropagation();
  //
  // Send message to editor if context on an object
  var obd = this.getElementObj(ev.target);
  if (obd) {
    var obdid = obd.id.indexOf("dmo_") !== -1 ? obd.id.substring(4, obd.id.length) : obd.id;
    var r = [{id: obdid, c: "oncontext", alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey, x: ev.clientX, y: ev.clientY}];
    this.editProxy.appCmd(r);
  }
};


/**
 * Key press : send message to the ViewView
 * @param {MouseEvent} ev
 */
Client.ViewEdit.prototype.keyHandler = function (ev)
{
  if (this.readOnly) {
    ev.preventDefault();
    return;
  }
  //
  // Skipping default
  var handled = false;
  if ((ev.ctrlKey || ev.metaKey) && (ev.keyCode === 90 || ev.keyCode === 89)) // CTRL-Z or CTRL-Y
    handled = true;
  if ((ev.ctrlKey || ev.metaKey) && ev.keyCode === 83) // CTRL-S
    handled = true;
  if (ev.keyCode === 116) // F5
    handled = true;
  if (ev.keyCode === 112) // F1
    handled = true;
  if ((ev.ctrlKey || ev.metaKey) && ev.keyCode === 70) // CTRL-F
    handled = true;
  if (ev.keyCode === 65 && (ev.ctrlKey || ev.metaKey)) // CTRL-A for selecting all children
    handled = true;
  if (handled)
    ev.preventDefault();
  //
  // Send key to the proxy
  this.editProxy.appCmd([{key: ev.keyCode, c: "keydown", alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey}]);
};


/*
 * From a DOM object return the first parent with "dmo_" id (element), if not found returns null
 */
Client.ViewEdit.prototype.getElementObj = function (obj)
{
  var myObj = obj;
  while (myObj) {
    if (myObj.id)
      return myObj;
    //
    myObj = myObj.parentNode;
  }
  //
  return null;
};


/*
 * Adapt the dimension to the zoom factor
 * @param Number dim - number to adapt
 * @param Number zm - zoom (float - 0.5=50%), optional, if not passed is calculated
 */
Client.ViewEdit.prototype.adaptForZoom = function (dim, zm)
{
  // detect zoom factor if not set
  if (!zm)
    zm = (document.body.style.zoom ? parseInt(document.body.style.zoom) : 100) / 100;
  return dim / zm;
};


/**
 * Mouse move: detect resizing type or resize object
 * @param {MouseEvent} ev
 * @param {String} type - event type
 */
Client.ViewEdit.prototype.onVisualHint = function (ev, type)
{
  // Send the mouse move message to the editor (for handling split divider resize action)
  var r = [{c: type, target: ev.target, offsetX: ev.offsetX, offsetY: ev.offsetY, readOnly: true}];
  this.editProxy.appCmd(r);
};


/**
 * Open the editor of an object
 * @param {string} id - object to open the editor
 */
Client.ViewEdit.prototype.openWidgetEditor = function (id)
{
  var ele = Client.eleMap[id];
  if (ele && ele.showEditor)
    ele.showEditor();
};


/**
 * Activates or deactivates the edit mode
 * @param {boolean} plan - true or false to activate/deactivate editing
 * @param {Object} proxy - proxy, the client events are sent to this object
 * @param {boolean} readonly - if the editing is true the editmode can be readonly, to disable D&D and resize
 */
Client.MainFrame.prototype.setPlanMode = function (plan, proxy, readonly)
{
  var edObj = Client.eleMap.editm;
  if (!Client.eleMap.editm)
    edObj = new Client.ViewEdit();
  else if (edObj.editMode)
    edObj.setEditMode(false);
  //
  // Create proxy and set EditMode
  edObj.planMode = plan;
  edObj.editProxy = proxy;
  if (readonly !== undefined)
    edObj.readOnly = readonly;
  //
  //
  // Add or remove generic handlers
  if (plan) {
    document.body.classList.add("editing");
    //
    // Handle D&D operations
    document.body.ondragenter = edObj.onPlanDragEnter.bind(edObj);
    document.body.ondragover = edObj.onPlanDragOver.bind(edObj);
    document.body.ondragleave = edObj.onPlanDragLeave.bind(edObj);
    document.body.ondrop = edObj.onPlanDrop.bind(edObj);
    //
    document.body.ondragstart = edObj.onPlanDragStart.bind(edObj);
    document.body.ondragend = edObj.onPlanDragEnd.bind(edObj);
    //
    document.body.onclick = edObj.onPlanClick.bind(edObj);
  }
  else {
    document.body.classList.remove("editing");
    //
    // Remove all "edit-mode" handlers
    delete document.body.ondragenter;
    delete document.body.ondragover;
    delete document.body.ondragleave;
    delete document.body.ondrop;
    delete document.body.ondragstart;
    delete document.body.ondragend;
    delete document.body.onclick;
  }
  //
  // Change objects ids to reflect edit mode change
  var el = Object.keys(Client.eleMap);
  for (var i = 0; i < el.length; i++) {
    if (Client.eleMap[el[i]].domObj && Client.eleMap[el[i]].domObj.id && Client.eleMap[el[i]].domObj.id !== "app-ui")
      Client.eleMap[el[i]].domObj.id = (plan ? "dmo_" : "") + Client.eleMap[el[i]].id;
  }
};


Client.ViewEdit.prototype.startMainPlanDrag = function (opt)
{
  this.appDiv.classList.add("plan-drag");
  //
  if (opt && opt.showMainDropTarget)
    document.getElementById("main_plan_drop_target").classList.add("plan_accept_drop");
  //
  if (opt && opt.dropTarget) {
    for (var i = 0; i < opt.dropTarget.length; i++) {
      var top = document.getElementById("pl_d_u_" + opt.dropTarget[i].id);
      var left = document.getElementById("pl_d_l_" + opt.dropTarget[i].id);
      var right = document.getElementById("pl_d_r_" + opt.dropTarget[i].id);
      var bottom = document.getElementById("pl_d_d_" + opt.dropTarget[i].id);
      var main = document.getElementById("dmo_" + opt.dropTarget[i].id);
      //
      if (top && opt.dropTarget[i].top)
        top.classList.add("plan_accept_drop");
      if (left && opt.dropTarget[i].left)
        left.classList.add("plan_accept_drop");
      if (right && opt.dropTarget[i].right)
        right.classList.add("plan_accept_drop");
      if (bottom && opt.dropTarget[i].bottom)
        bottom.classList.add("plan_accept_drop");
      if (main && opt.dropTarget[i].self)
        main.classList.add("plan_accept_drop");
    }
  }
};


Client.ViewEdit.prototype.startSubPlanDrag = function (opt)
{
  this.appDiv.classList.add("sub-plan-drag");
};


Client.ViewEdit.prototype.endPlanDrag = function ()
{
  this.appDiv.classList.remove("plan-drag");
  this.appDiv.classList.remove("sub-plan-drag");
  //
  var list = document.getElementsByClassName("plan_accept_drop");
  while (list.length > 0)
    list[0].classList.remove("plan_accept_drop");
  list = document.getElementsByClassName("drop_target_active");
  while (list.length > 0)
    list[0].classList.remove("drop_target_active");
  //
  delete this.planSubDragContext;
};


/**
 * Handles a drag over an object in Plan Mode
 * @param {DragEven} ev
 */
Client.ViewEdit.prototype.onPlanDragStart = function (ev)
{
  if (!this.planMode || !this.editProxy || this.readOnly || !ev.target)
    return;
  //
  // Check if is a main widget or a SubWidget drag
  if (ev.target && ev.target.classList && ev.target.classList.contains("plan_drop_target_main")) {
    // this is the main body of a main widget
    ev.dataTransfer.setData("rootwidgetdrag", ev.target.id);
    //
    // Must use a timer: showing the divs here blocks the drag operation on Chrome 56
    window.setTimeout(function () {
      this.startMainPlanDrag();
    }.bind(this), 15);
    return;
  }
  //
  // Drag of an already present subwidget
  if (ev.target && ev.target.getAttribute("data-subwidget") !== null) {
    ev.dataTransfer.setData("secondarywidgetdrag", ev.target.id);
    ev.dataTransfer.setData("alreadypresentsubwidget", "true");
    //
    // Detect the 'context' of the operation: is the first parent that has 'data-subwidget' or 'data-widget' attribute
    var obj = ev.target.parentNode;
    while (obj) {
      if (obj.getAttribute("data-subwidget") || obj.getAttribute("data-widget"))
        break;
      //
      obj = obj.parentNode;
    }
    //
    // To permit the drag of the subwidget over another subwidget the context must accept the main drag
    if (obj && obj.getAttribute("plan_drop_target_sub_main") === "zone_main")
      this.planSubDragContext = obj.id;
  }
};


/**
 * Handles a drag over an object in Plan Mode
 * @param {DragEven} ev
 */
Client.ViewEdit.prototype.onPlanDragEnd = function (ev)
{
  this.endPlanDrag();
};


/**
 * Handles a drag over an object in Plan Mode
 * @param {DragEven} ev
 */
Client.ViewEdit.prototype.onPlanDragEnter = function (ev)
{
  if (!this.planMode || !this.editProxy || this.readOnly || !ev.target)
    return;
  //
  // Drag of a Main Widget
  if (ev.dataTransfer.types.indexOf("rootwidgetdrag") !== -1) {
    // Drag over the main target or a drop zone
    if (ev.target && ev.target.id && (ev.target.id === "main_plan_drop_target" || ev.target.id.substring(0, 5) === "pl_d_" || (ev.target.classList && ev.target.classList.contains("plan_drop_target_main") && ev.target.classList.contains("plan_accept_drop")))) {
      ev.preventDefault();
      ev.dataTransfer.dropEffect = "move";
    }
  }
  //
  // Drag of a secondary widget, accepetd on a secondary drop zone or a selected main widget
  if (ev.dataTransfer.types.indexOf("secondarywidgetdrag") !== -1) {
    // A subwidget from the WidgetBox can be dropped only on
    // - > DropZone
    // - > Main widget (se lo accetta)
    if (ev.target && ev.target.id && ((ev.target.classList && ev.target.classList.contains("plan_drop_target_sub")) ||
            (ev.target.classList && ev.target.classList.contains("plan_drop_target_sub_main") && ev.target.classList.contains("widget-selected")) ||
            (ev.target.getAttribute("plan_drop_target_sub_main") !== null))) {
      ev.preventDefault();
      ev.dataTransfer.dropEffect = "move";
    }
    else if (ev.dataTransfer.types.indexOf("alreadypresentsubwidget") !== -1 && this.planSubDragContext && ev.target.getAttribute("data-subwidget")) {
      // An already present subwidget can also be dropped on an already present subwidget of the same parent
      // (to swap their positions)
      // - > only if their parent accepts the main drag
      var drObj = ev.target.parentNode;
      var okDrop = false;
      while (drObj && !okDrop) {
        if (drObj.id === this.planSubDragContext)
          okDrop = true;
        //
        drObj = drObj.parentNode;
      }
      //
      if (okDrop) {
        ev.preventDefault();
        ev.dataTransfer.dropEffect = "move";
      }
    }
  }
};


/**
 * Handles a drag over an object in Plan Mode
 * @param {DragEven} ev
 */
Client.ViewEdit.prototype.onPlanDragOver = function (ev)
{
  this.onPlanDragEnter(ev);
  if (ev.target && ev.dataTransfer.dropEffect === "move" && ev.target.classList) {
    ev.target.classList.add("drop_target_active");
  }
};


/**
 * Handles a drag leave an object in Plan Mode
 * @param {DragEven} ev
 */
Client.ViewEdit.prototype.onPlanDragLeave = function (ev)
{
  if (ev.target && ev.target.classList)
    ev.target.classList.remove("drop_target_active");
};


/**
 * Handles a drag over an object in Plan Mode
 * @param {DragEven} ev
 */
Client.ViewEdit.prototype.onPlanDrop = function (ev)
{
  if (!this.planMode || !this.editProxy || this.readOnly || !ev.target)
    return;
  //
  // Prevent to tell the browser that the drop is done by us
  ev.preventDefault();
  //
  // Drag of a new Main Widget (from the widget box)
  if (ev.dataTransfer.types.indexOf("rootwidgetdrag") !== -1) {
    var draggedWidgetId = ev.dataTransfer.getData("rootwidgetdrag");
    //
    if (ev.target && ev.target.id === "main_plan_drop_target") {
      // Drag over the main target (possible only when there are no children)
      this.editProxy.appCmd([{c: "dropMainWidget", drag: draggedWidgetId, drop: "view", alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey, ev: ev}]);
    }
    else if (ev.target && ev.target.id.substring(0, 5) === "pl_d_") {
      // Drop af a new widget into the side box af an existing widget
      var dropTarget = ev.target.id.substring(7);
      var pos = ev.target.id.substring(5, 6); // r, l, d, u
      //
      // I could drop an already present widget onto himself, so we must skip this operation
      if (dropTarget === draggedWidgetId.substring(4))
        return;
      //
      this.editProxy.appCmd([{c: "dropMainWidget", drag: draggedWidgetId, drop: dropTarget, pos: pos, alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey, ev: ev}]);
    }
    else if (ev.target.classList && ev.target.classList.contains("plan_drop_target_main")) {
      // I could drop an already present widget onto himself, so we must skip this operation
      if (ev.target.id === draggedWidgetId)
        return;
      //
      // Drop of a new widget onto an existing widget
      this.editProxy.appCmd([{c: "dropMainWidget", drag: draggedWidgetId, drop: ev.target.id.substring(4), pos: "c", alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey, ev: ev}]);
    }
  }
  //
  if (ev.dataTransfer.types.indexOf("secondarywidgetdrag") !== -1) {
    var draggedWidgetId = ev.dataTransfer.getData("secondarywidgetdrag");
    //
    if (ev.target && ev.target.id.substring(0, 4) === "sdr_" && ev.target.id.indexOf("___") > 0) {
      // Drop on a subwidget zone, detect the zone and the target
      var sep = ev.target.id.indexOf("___");
      var zoneId = ev.target.id.substring(4, sep);
      var targetId = ev.target.id.substring(sep + 3);
      this.editProxy.appCmd([{c: "dropSecondaryWidget", drag: draggedWidgetId, drop: targetId, zone: zoneId, alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey, ev: ev}]);
    }
    else if (this.planSubDragContext && ev.dataTransfer.types.indexOf("alreadypresentsubwidget") !== -1 && ev.target.getAttribute("data-subwidget")) {
      // Drop of an already present subwidget on another subwidget of the same parent
      this.editProxy.appCmd([{c: "swapSecondaryWidget", drag: draggedWidgetId, drop: ev.target.id, alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey, ev: ev}]);
    }
    else if (ev.target.classList && ev.target.classList.contains("plan_drop_target_main") || (ev.target.getAttribute("plan_drop_target_sub_main") !== null)) {
      // Drop of a SubWidget onto a main widget body
      var zoneId = ev.target.getAttribute("plan_drop_target_sub_main") !== null ? ev.target.getAttribute("plan_drop_target_sub_main") : "c";
      this.editProxy.appCmd([{c: "dropSecondaryWidget", drag: draggedWidgetId, drop: ev.target.id.substring(4), zone: zoneId, alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey, ev: ev}]);
    }

  }
  //
  this.onPlanDragEnd();
};


/**
 * Realized the drop targets for Plan Mode
 */
Client.Element.prototype.createDropTargets = function ()
{
  // If the drop targets are already created or we are not in planning mode do nothing
  if (!Client.mainFrame.isPlanning() || this.planExtObj)
    return;
  //
  var acceptExtDrag = this.acceptedDropTargets ? (this.acceptedDropTargets.top === true ||
          this.acceptedDropTargets.right === true ||
          this.acceptedDropTargets.left === true ||
          this.acceptedDropTargets.bottom === true) : true;
  var acceptMainDrag = this.acceptedDropTargets ? this.acceptedDropTargets.self : true;
  //
  if (acceptExtDrag) {
    this.planExtObj = document.createElement("div");
    this.planExtObj.className = "plan_drop_target_parent";
    //
    // Replace the old element with the container
    if (this.domObj.parentNode)
      this.domObj.parentNode.replaceChild(this.planExtObj, this.domObj);
    //
    if ((this.acceptedDropTargets && this.acceptedDropTargets.top) || !this.acceptedDropTargets) {
      this.dropOverTarget = document.createElement("div");
      this.dropOverTarget.className = "plan_drop_target_up";
      this.dropOverTarget.id = "pl_d_u_" + this.id;
      this.planExtObj.appendChild(this.dropOverTarget);
    }
    //
    if ((this.acceptedDropTargets && this.acceptedDropTargets.left) || !this.acceptedDropTargets) {
      this.dropLeftTarget = document.createElement("div");
      this.dropLeftTarget.className = "plan_drop_target_left";
      this.dropLeftTarget.id = "pl_d_l_" + this.id;
      this.planExtObj.appendChild(this.dropLeftTarget);
    }
    //
    this.planExtObj.appendChild(this.domObj);
    //
    if ((this.acceptedDropTargets && this.acceptedDropTargets.right) || !this.acceptedDropTargets) {
      this.dropRightTarget = document.createElement("div");
      this.dropRightTarget.className = "plan_drop_target_right";
      this.dropRightTarget.id = "pl_d_r_" + this.id;
      this.planExtObj.appendChild(this.dropRightTarget);
    }
    //
    if ((this.acceptedDropTargets && this.acceptedDropTargets.bottom) || !this.acceptedDropTargets) {
      this.dropUnderTarget = document.createElement("div");
      this.dropUnderTarget.className = "plan_drop_target_down";
      this.dropUnderTarget.id = "pl_d_d_" + this.id;
      this.planExtObj.appendChild(this.dropUnderTarget);
    }
    //
    if (this.layout === "horizontal" || this.layout === "vertical")
      this.domObj.classList.add("plan_layout_container");
  }
  //
  if (acceptMainDrag)
    if (this.widgetLevel === 1) {
      this.domObj.classList.add("plan_drop_target_main");
      this.domObj.draggable = true;
    }
    else
      this.domObj.classList.add("plan_drop_target_main_layout");
  //
  if (this.widgetLevel === 1) {
    var style = window.getComputedStyle(this.domObj);
    if (style.position === "static")
      this.domObj.style.position = "relative";
  }
};


/**
 * Realize the drop zones for Plan Mode SubWidgets
 */
Client.Element.prototype.createDropZones = function ()
{
  // Create the Drop Targets for the SubWidgets
  if (this.planDropZones) {
    // Delete old drop zones
    var els = this.domObj.getElementsByClassName("plan_drop_target_sub");
    while (els.length > 0)
      els[0].parentNode.removeChild(els[0]);
    this.domObj.setAttribute("plan_drop_target_sub_main", "");
    //
    // Recreate them
    for (var i = 0; i < this.planDropZones.length; i++) {
      var d = this.planDropZones[i];
      if (d.self) {
        this.domObj.setAttribute("plan_drop_target_sub_main", d.id ? d.id : "zone_main");
      }
      else {
        var dropBox = document.createElement(d.tag);
        dropBox.id = "sdr_" + d.id + "___" + this.id;
        dropBox.className = "plan_drop_target_sub" + (d.class ? " " + d.class : "");
        if (d.style) {
          var kk = Object.keys(d.style);
          for (var j = 0; j < kk.length; j++)
            dropBox.style[kk[j]] = d.style[kk[j]];
        }
        //
        this.domObj.appendChild(dropBox);
      }
    }
  }
};


/**
 * Handles a click over an object in plan mode
 * @param {DragEven} ev
 */
Client.ViewEdit.prototype.onPlanClick = function (ev)
{
  if (!this.planMode || !this.editProxy || this.readOnly || !ev.target)
    return;
  var obj, p;
  var target = ev.target;
  //
  // Click on a drop zone, the target is the parent container
  if (target && target.id && target.id.substring(0, 4) === "sdr_")
    target = target.parentNode;
  //
  var selectScreen = false;
  if (target && !(target.getAttribute("data-widget") === "1" || target.getAttribute("data-subwidget") !== null)) {
    // Get the parent or the screen
    while (target) {
      if (target.getAttribute("data-widget") === "1" || target.getAttribute("data-subwidget") !== null)
        break;
      if (target.id && target.id === "app-ui") {
        selectScreen = true;
        break;
      }
      //
      target = target.parentNode;
    }
  }
  //
  // Click on a master widget or a subwidget
  if (target && target.id && (target.getAttribute("data-widget") === "1" || target.getAttribute("data-subwidget") !== null)) {
    // If clicking on an already selected subwidget we must select the main widget or the subwidget that is the parent
    if (target.getAttribute("data-subwidget") !== null && target.classList.contains("widget-selected")) {
      obj = target.parentNode;
      p = document.getElementById("app-ui");
      while (obj && obj !== p) {
        if (obj.getAttribute("data-widget") === "1" || obj.getAttribute("data-subwidget") !== null) {
          target = obj;
          break;
        }
        obj = obj.parentNode;
      }
    }
    //
    // Clear the old selection
    var sel = document.getElementsByClassName("widget-selected");
    while (sel.length > 0)
      sel[0].classList.remove("widget-selected");
    sel = document.getElementsByClassName("subwidget-selected");
    while (sel.length > 0)
      sel[0].classList.remove("subwidget-selected");
    //
    // Select the widget
    target.classList.add("widget-selected");
    if (target.getAttribute("data-subwidget")) {
      // This is a subwidget, in that case we must set on the main widget the "subwidget-selected" class
      obj = target;
      p = document.getElementById("app-ui");
      while (obj && obj !== p) {
        if (obj.getAttribute("data-widget") === "1") {
          obj.classList.add("subwidget-selected");
          break;
        }
        obj = obj.parentNode;
      }
    }
    //
    this.editProxy.appCmd([{c: "selectWidget", id: target.id, alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey}]);
  }
  else if (selectScreen) {
    this.editProxy.appCmd([{c: "selectScreen", alt: ev.altKey, ctr: ev.ctrlKey || ev.metaKey, sht: ev.shiftKey}]);
    //
    // Clear the old selection
    var sel = document.getElementsByClassName("widget-selected");
    while (sel.length > 0)
      sel[0].classList.remove("widget-selected");
    sel = document.getElementsByClassName("subwidget-selected");
    while (sel.length > 0)
      sel[0].classList.remove("subwidget-selected");
  }
};
