/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */
/* global module, require */

var App = App || {};

/**
 * @class A SyncFoundation Object
 * @param {Session} app that owns this sync instance
 * */
App.SyncFoundation = function (app)
{
  if (module && !App.View)
    App = require("../requireAll");
  //
  this.app = app;
  this.dataStore = null;
  this.tablesAlreadyChecked = false;
  //
  this.serverUrl = "";
  this.username = "";
  this.password = "";
  this.isSynchronizing = false;
  this.responseFormat = 1; // ex XMLDoc.JSON
  this.maxAge = 30;
  this.checkServer = 5;
  this.connectionTimeout = 0;     // Timeout in seconds to connect to the server sync
  //
  // This is the list of classes to be downloaded in the case of full synchronization
  this.docToSync = App.SyncDO.docToSync;
  //
  // This is the list of classes that support the optimistic lock
  this.optLockList = [];
  //
  // The following is a list of classes to be sent also in case of partial synchro
  // it can be used by the client if it wants to get the re-synchronization of a class,
  // or if you want to force the server resynchronization
  this.classToResync = [];
  //
  // This is a list of documents that have not been saved during synchronization
  this.errList = [];
  this.serverErrList = null;
  //
  // This is a list of documents that have had problems optimistic lock
  this.conflictList = [];
  this.conflictPropList = [];
  //
  // Disables saving of changes
  // Need when reupdating db client side to prevent insertions into ZZ_SYNC
  // So that then would then deleted
  this.dontAddVariations = false;
  //
  // Parameters for running the sessions
  this.sessionFlags = 0;
  //
  // Partial synchronization
  this.maxDocuments = 0;
  this.totalDocuments = -1;
  this.actualDocument = 0;
  this.aborted = false;
  this.completed = false;
  this.preCount = false;
  this.processedDocs = -1;
  this.iCanAbort = true;
  //
  this.lastDocument = ""; // Last document that the client has had
  //
  this.iNativeSync = true; // Native synchronization side shell
  //
  // Debug informations
  this.startSyncTS;
  this.serverElapsedTime = -1;
  this.networkElapsedTime = -1;
  this.clientElapsedTime = -1;
  this.totalChanges = -1;
  //
  this.resetSessionFlags();
};

App.SyncFoundation.FL_SYNCME = 1;
App.SyncFoundation.FL_SYNCPARTNER = 2;
App.SyncFoundation.FL_RESYNCPARTNER = 4;
App.SyncFoundation.FL_USEOPTLOCK = 8;
App.SyncFoundation.NULL_Marker = "~~NULL~~";


/*
 Object.defineProperty(App.SyncFoundation.prototype, "nativeSync", {
 get: function () {
 return (this.iNativeSync && this.app.ParamsObj().UseRD4 && !IDL.IsNull(this.app.ShellObject.IsInsideCaravel()));
 },
 set: function (newVal) {
 this.iNativeSync = newVal;
 }
 });


 /**
 * Torna lo stato stato di avanzamento della sincronizzazione
 * indicando alla shell se l'utente ha richiesto di interrompere il tutto
 * 0-99: in corso, 100: terminata con successo, -1: terminata con errori
 * @param {Boolean} aborted
 */
/*
 App.SyncFoundation.prototype.getProgress = function (aborted)
 {
 if (aborted === undefined)
 aborted = false;
 //
 return (new IDVariant(parseFloat(this.app.ShellObject.SendCmd("SYNC_PROGRESS", {ABORTED: (aborted ? "1" : "0")}))));
 };
 */

Object.defineProperty(App.SyncFoundation.prototype, "canAbort", {
  get: function () {
    return this.iCanAbort && (this.maxDocuments > 0);
  },
  set: function (newVal) {
    this.iCanAbort = newVal;
  }
});


Object.defineProperty(App.SyncFoundation.prototype, "dataStore", {
  get: function () {
    return this._dataStore;
  },
  set: function (newValue) {
    if (typeof newValue === "string")
      this._dataStore = new App[newValue](this.app);
    else
      this._dataStore = newValue;
  }
});


/**
 * Add a document type
 * @param {String} docClass
 */
App.SyncFoundation.prototype.addDocToSync = function (docClass)
{
  if (this.docToSync.indexOf(docClass) === -1)
    this.docToSync.push(docClass);
};


/**
 * Beg to enable optimistic locking for a document type
 * @param {String} docClass
 */
App.SyncFoundation.prototype.enableOptimisticLock = function (docClass)
{
  if (this.optLockList.indexOf(docClass) === -1)
    this.optLockList.push(docClass);
};


/**
 * Management flag session
 */
App.SyncFoundation.prototype.resetSessionFlags = function ()
{
  this.sessionFlags = 255;
};


/**
 * Test flag session
 * @param {Integer} flag to test
 */
App.SyncFoundation.prototype.testSessionFlag = function (flag)
{
  return (this.sessionFlags & flag) !== 0;
};


/**
 * Reset flag session
 * @param {Integer} flag to reset
 */
App.SyncFoundation.prototype.resetSessionFlag = function (flag)
{
  this.sessionFlags = this.sessionFlags & (~flag);
};


/**
 * Set flag session
 * @param {Integer} flag to set
 */
App.SyncFoundation.prototype.setSessionFlag = function (flag)
{
  this.sessionFlags = this.sessionFlags | flag;
};


/**
 * I add a class to resynchronize
 * @param {String} docClass
 */
App.SyncFoundation.prototype.resyncClass = function (docClass)
{
  if (this.classToResync.indexOf(docClass) === -1)
    this.classToResync.push(docClass);
};


/**
 * Returns the last date of synchronization
 * @param {Function} callback - function to be called at the end
 */
App.SyncFoundation.prototype.lastSynchronziation = function (callback)
{
  this.checkTables(function (result, error) {
    if (error || !result)
      return callback(null);
    //
    // I extract the last date synchro
    var sql = "select TS from ZZ_SYNC where ID=1 and STATUS='.'";
    this.dataStore.executeSQL(sql, true, function (rs, error) {
      if (error || this.dataStore.getRowCount(rs) === 0)
        callback(null);
      else
        callback(this.dataStore.convertValue(rs.rows[0].ts, App.Database.DataTypes.DATETIME));
    }.bind(this));
  }.bind(this));
};


/**
 * Returns the number of variatons in the list
 * @param {Function} callback - function to be called at the end
 */
App.SyncFoundation.prototype.variationNumber = function (callback)
{
  this.checkTables(function (result, error) {
    if (error || !result)
      return callback(null);
    //
    // Extract the number of variations
    var sql = "select count(*) as count from ZZ_SYNC where ID>1";
    this.dataStore.executeSQL(sql, true, function (rs, error) {
      if (error || this.dataStore.getRowCount(rs) === 0)
        callback(null);
      else
        callback(this.dataStore.convertValue(rs.rows[0].count, App.Database.DataTypes.INT));
    }.bind(this));
  }.bind(this));
};


/**
 * Reset variables for the partial sync
 */
App.SyncFoundation.prototype.clearPartialCounters = function ()
{
  this.totalDocuments = -1;
  this.actualDocument = 0;
  this.aborted = false;
  this.completed = false;
  this.processedDocs = 0;
  this.serverElapsedTime = 0;
  this.networkElapsedTime = 0;
  this.clientElapsedTime = 0;
  this.totalChanges = 0;
};


/**
 * Check that the tables for supporting the synchronization are existing
 * @param {Function} callback - function to be called at the end
 */
App.SyncFoundation.prototype.checkTables = function (callback)
{
  if (this.tablesAlreadyChecked)
    return callback(true);
  //
  // There is the datastore?
  if (!this.dataStore) {
    // Do un messaggio di errore e poi esco
    var dttit = this.app.dttobj.addItem("", App.DTT.ItemTypeMap.EXCEPTION, "SyncFoundation.CheckTables");
    this.app.dttobj.addItemParam("Error message", "No database can contain sync data; verify your license", dttit);
    return callback(false);
  }
  //
  this.tablesAlreadyChecked = true;
  //
  this.dataStore.executeSQL("select sql from sqlite_master where type = 'table' and name = 'ZZ_SYNC'", true, function (rs, error) {
    if (error)
      return callback(false);
    //
    if (this.dataStore.getRowCount(rs) > 0)
      return callback(true);
    //
    // Since instruction has had trouble with, probably the table
    // Does not exist yet, I try to create it
    // Create table ZZ_SYNC
    var sql = "create table ZZ_SYNC( ID integer primary key autoincrement not null, DNA varchar(1000) not null, STATUS varchar(1) not null, DOCDOM varchar(200), CONTENT varchar(10000), TS datetime default (datetime(CURRENT_TIMESTAMP,'localtime')))";
    this.dataStore.executeSQL(sql, false, function (result, error) {
      if (error)
        return callback(false);
      //
      // Insert first row
      sql = "insert into ZZ_SYNC(DNA, STATUS, TS) values ('0', '.', NULL)";
      this.dataStore.executeSQL(sql, false, function (result, error) {
        if (error) {
          this.app.dttobj.addWarning("", "SyncFoundation.CheckTables: ZZ_SYNC is not creatable " + error.message);
          callback(false);
        }
        else
          callback(true);
      }.bind(this));
    }.bind(this));
  }.bind(this));
};


/**
 * Records changes to a document. Normally
 * called after after save if the service is synchronization is active.
 * @param {Document} doc changed
 * @param {Function} callback - function to be called at the end
 */
App.SyncFoundation.prototype.addVariation = function (doc, callback)
{
  if (this.dontAddVariations)
    return callback();
  //
  // The object is really changed?
  if (!doc.inserted && !doc.updated && !doc.deleted)
    return callback();
  //
  // The object is deleted before being added ... nothing to do
  if (doc.inserted && doc.deleted)
    return callback();
  //
  this.checkTables(function (result, error) {
    if (!result)
      return callback();
    //
    var dna = this.getCompleteDNA(doc);
    var dom = this.getDomain(doc);
    var cnt = "";
    //
    // Calculating the current status
    var stt = "I";
    if (!doc.inserted) {
      if (doc.updated)
        stt = "U";
      if (doc.deleted)
        stt = "D";
    }
    if (stt !== "D") {
      cnt = this.getContent(doc);
      if (cnt === "")
        // No properties to be synchronized has changed !
        return callback();
    }
    //
    // If the document has suspended change synchronization status
    if (doc.suspendSync) {
      switch (stt)
      {
        case "I":
          stt = "0";
          break;
        case "U":
          stt = "1";
          break;
        case "D":
          stt = "2";
          break;
      }
    }
    //
    // Register the change of the object
    var sql = "insert into ZZ_SYNC(DNA, STATUS, DOCDOM, CONTENT) values (";
    sql += this.dataStore.toSql(dna, App.Database.DataTypes.VARCHAR) + ", ";
    sql += this.dataStore.toSql(stt, App.Database.DataTypes.VARCHAR) + ", ";
    sql += this.dataStore.toSql(dom, App.Database.DataTypes.VARCHAR) + ", ";
    sql += this.dataStore.toSql(cnt, App.Database.DataTypes.VARCHAR) + ")";
    this.dataStore.executeSQL(sql, false, callback);
  }.bind(this));
};


/**
 * Picks up the complete DNA of the document, including parent
 * @param {Document} doc
 */
App.SyncFoundation.prototype.getCompleteDNA = function (doc)
{
  var ris = "";
  while (doc) {
    ris = "{" + doc.getDNA() + "}" + ris;
    doc = doc.parent;
  }
  return ris;
};


/**
 * Picks up the document domain
 * @param {Document} doc
 */
App.SyncFoundation.prototype.getDomain = function (doc)
{
  return doc.do_domain;
};


/**
 * Returns a string representation in XML
 * changes made to the document
 * @param {Document} doc
 */
App.SyncFoundation.prototype.getContent = function (doc)
{
  var xn = {};
  var xd = {do: {$: xn}};
  var almostOne = false;
  var optlock = this.optLockList.indexOf(doc.schema.className) > -1;
  var md = doc.schema;
  //
  for (var i = 0; i < md.properties.length; i++) {
    var mdp = md.properties[i];
    if (doc.do_dontSync && doc.do_dontSync.indexOf(mdp.name) !== -1)
      continue;
    //
    var vatt = doc[mdp.name];
    var vold = doc.getOriginalValue(mdp.name);
    if (doc.inserted || JSON.stringify(vatt) !== JSON.stringify(vold)) {
      xn[mdp.tag] = this.valueToString(vatt);
      if (optlock && mdp.datatype !== App.Database.DataTypes.BLOB)
        xn[mdp.tag + "_old"] = this.valueToString(vold);
      almostOne = true;
      //
      // If I changed a primary key, even throwing an exception
      if (mdp.pk && !doc.inserted)
        throw new Error("Cannot sync a primary key change");
    }
  }
  //
  if (almostOne)
    return App.Xml.stringify(this.app, xd, {headless: true});
  else
    return "";
};


/**
 * Synchronize with the remote
 * @param {Function} callback - function to be called at the end
 */
App.SyncFoundation.prototype.synchronize = function (callback)
{
  this.startSyncTS = new Date();
  //
  this.checkSyncServer(function (result, error) {
    // The remote can not be reached, I go out
    if (!result) {
      if (this.nativeSync)
        this.endSynchronization(false, false, []);
      return callback(false);
    }
    //
    // If there is no class to synchronize add a warning to debug
    if (this.docToSync.length === 0) {
      var dttit = this.app.dttobj.addItem("", App.DTT.ItemTypeMap.EXCEPTION, "SyncFoundation.synchronize");
      this.app.dttobj.addItemParam("Error Message", "There is no class to synchronize", dttit);
    }
    //
    this.checkTables(function (result, error) {
      if (!result) {
        if (this.nativeSync)
          this.endSynchronization(false, false, []);
        return callback(false);
      }
      //
      this.clearPartialCounters();
      //
      if (!this.nativeSync) {
        this.isSynchronizing = true;
        this.serverErrList = [];
        var rok = true;
        var allok = true;
        //
        this.progress = 0;
        //
        var loopFinally = function () {
          this.isSynchronizing = false;
          this.dontAddVariations = false;
          this.classToResync = [];
          this.clientElapsedTime = (new Date()).getTime() - this.startSyncTS.getTime() - this.serverElapsedTime - this.networkElapsedTime;
        }.bind(this);
        //
        var doLoop = function (result, error) {
          if (error) {
            loopFinally();
            return callback(null, error);
          }
          //
          if (!this.completed && !this.aborted)
            loopBody();
          else {
            loopFinally();
            //
            // Returns OK if there were errors
            callback(allok);
          }
        }.bind(this);
        //
        var loopBody = function (result, error) {
          // I extract the event counter client
          var sql = "select DNA,DOCDOM,TS,CONTENT from ZZ_SYNC where ID=1 and STATUS='.'";
          this.dataStore.executeSQL(sql, true, function (rs, error) {
            if (error) {
              doLoop(null, error);
              return;
            }
            //
            var from = 0;
            var dom = "";
            var tsDT = null;
            var ts = "";
            var lastdoc = "";
            if (this.dataStore.getRowCount(rs) > 0) {
              var row = this.dataStore.getRow(rs, 0);
              from = row.DNA;
              dom = row.DOCDOM;
              if (row.TS) {
                ts = row.TS;
                tsDT = new Date(ts);
                ts = this.valueToString(tsDT);
              }
              lastdoc = row.CONTENT;
            }
            //
            var dttit = this.app.dttobj.addItem("", App.DTT.ItemTypeMap.INFO, "SyncFoundation.synchronize");
            this.app.dttobj.addItemParam("Message", "Start synchronization from ID=" + from, dttit);
            this.app.dttobj.addItemParam("Server", this.serverUrl, dttit);
            this.app.dttobj.addItemParam("UserName", this.username, dttit);
            this.app.dttobj.addItemParam("Password", this.password, dttit);
            this.app.dttobj.addItemParam("Last Dom", dom, dttit);
            this.app.dttobj.addItemParam("Last Doc", lastdoc, dttit);
            this.app.dttobj.addItemParam("Time Stamp", ts, dttit);
            this.app.dttobj.addItemParam("Pre Count", this.preCount, dttit);
            //
            // I create the XML document as a string ...
            var sb = "";
            sb += "<ss username=\"" + App.Utils.htmlEncode(this.username) + "\" ";
            sb += "password=\"" + App.Utils.htmlEncode(this.password) + "\" ";
            sb += "dom=\"" + App.Utils.htmlEncode(dom) + "\" ";
            sb += "lastdoc=\"" + App.Utils.htmlEncode(lastdoc) + "\" ";
            sb += "ts=\"" + ts + "\" ";
            sb += "from=\"" + from + "\" ";
            if (this.preCount)
              sb += "precount=\"1\" ";
            sb += "max=\"" + this.maxDocuments + "\">";
            //
            var afterClientVariations = function (result, error) {
              if (error) {
                doLoop(null, error);
                return;
              }
              //
              if (result)
                sb += result;
              //
              // Add the request of classes to resynchronize
              for (var i = 0; i < this.classToResync.length; i++)
                sb += "<rs cls=\"" + App.Utils.htmlEncode(this.classToResync[i]) + "\" />";
              //
              // Close the root tag
              sb += "</ss>";
              //
              // Send post to server
              var postdata = {};
              postdata.SYNCDATA = sb;
              postdata.WCI = "SYNC";
              postdata.FTR = this.responseFormat;
              //
              var options = {params: postdata};
              if (this.connectionTimeout > 0)
                options.timeOut = this.connectionTimeout * 1000;
              //
              var startHttpRequest = new Date();
              this.app.fs.url(this.serverUrl).post(options, function (response) {
                if (response.error)
                  return doLoop(null, response.error);
                //
                var endHttpRequest = new Date();
                //
                var dttit = this.app.dttobj.addItem("", App.DTT.ItemTypeMap.INFO, "SyncFoundation.synchronize");
                this.app.dttobj.addItemParam("Message", "Received " + response.body.length + " chars from server", dttit);
                if (response.body.length < 20000)
                  this.app.dttobj.addItemParam("Content", response.body, dttit);
                //
                // Let's see what I told the server
                var rn = null;
                try {
                  rn = JSON.parse(response.body).ss;
                }
                catch (e) {
                  dttit.type = App.DTT.ItemTypeMap.EXCEPTION;
                  this.app.dttobj.addItemParam("Error Message", "Parse JSON failed", dttit);
                  doLoop(null, e);
                  return;
                }
                //
                this.errList = [];
                //
                if (rn) {
                  if (rn.do && !(rn.do instanceof Array))
                    rn.do = [rn.do];
                  if (rn.rs && !(rn.rs instanceof Array))
                    rn.rs = [rn.rs];
                  //
                  this.updateElapdedTimes(rn, endHttpRequest.getTime() - startHttpRequest.getTime());
                  //
                  // Verify any errors in the response
                  if (!this.checkResponse(rn, "synchronize")) {
                    this.aborted = true;
                    allok = false;
                    doloop();
                    return;
                  }
                  //
                  var transOpened = false;
                  var doInTransaction = function (result, error) {
                    // If the server has finished send the data or if a full synchronization is required
                    // I can clear the list of classes resynchronize
                    if (!this.preCount && (!rn.lastdoc || rn.resync === "yes"))
                      this.classToResync = [];
                    //
                    // Handle the preCount
                    if (this.preCount) {
                      this.totalDocuments = rn.total;
                      this.preCount = false;
                    }
                    else if (this.totalDocuments === -1 && this.maxDocuments === 0) {
                      this.totalDocuments = 0;
                      if (rn.do instanceof Array) {
                        for (var i = 0; i < rn.do.length; i++) {
                          if (rn.do[i].ss_err !== 2)
                            this.totalDocuments++;
                        }
                      }
                    }
                    //
                    if (this.totalDocuments !== -1) {
                      var dttit = this.app.dttobj.addItem("", App.DTT.ItemTypeMap.INFO, "SyncFoundation.synchronize");
                      this.app.dttobj.addItemParam("Message", "Total documents to sync = " + this.totalDocuments, dttit);
                    }
                    //
                    this.dontAddVariations = true;
                    var afterProcesssAndSave = function (result, error) {
                      if (error) {
                        doLoop(null, error);
                        return;
                      }
                      //
                      rok = result;
                      //
                      this.dontAddVariations = false;
                      //
                      this.completed = (rn.completed !== "no");
                      //
                      allok = allok && this.errList.length === 0 && rok && !this.aborted;
                      //
                      // Given that everything is fine, set the maximum ID found
                      // and other data, then gate changes from the client
                      var sql = "update ZZ_SYNC set ";
                      sql += "DNA=" + this.dataStore.toSql(rn.maxid, App.Database.DataTypes.VARCHAR) + ", ";
                      sql += "DOCDOM=" + this.dataStore.toSql(rn.dom, App.Database.DataTypes.VARCHAR) + ", ";
                      sql += "TS=" + this.dataStore.toSql(this.completed && allok ? new Date() : tsDT, App.Database.DataTypes.DATETIME) + ", ";
                      sql += "CONTENT=" + this.dataStore.toSql(rn.lastdoc, App.Database.DataTypes.VARCHAR) + " ";
                      sql += "where ID=1";
                      this.dataStore.executeSQL(sql, false, function (result, error) {
                        if (error)
                          return doLoop(null, error);
                        //
                        this.dataStore.executeSQL("delete from ZZ_SYNC where STATUS NOT IN ('.','0','1','2')", false, function (result, error) {
                          if (error) {
                            doLoop(null, error);
                            return;
                          }
                          //
                          var dttit = this.app.dttobj.addItem("", App.DTT.ItemTypeMap.INFO, "SyncFoundation.synchronize");
                          this.app.dttobj.addItemParam("Message", "End synchronization. Server MaxID=" + rn.maxid, dttit);
                          //
                          if (!this.completed) {
                            dttit = this.app.dttobj.addItem("", App.DTT.ItemTypeMap.INFO, "SyncFoundation.synchronize");
                            this.app.dttobj.addItemParam("Message", "Synchronization is not yet completed. Running again.", dttit);
                            this.app.dttobj.addItemParam("Aborted", this.aborted, dttit);
                          }
                          //
                          // Close the transaction
                          if (transOpened)
                            this.dataStore.commitTransaction(function (result, error) {
                              // Even if the commit was unsuccessful does not throw error
                              doLoop();
                            });
                          else
                            doLoop();
                        }.bind(this));
                      }.bind(this));
                    }.bind(this);
                    //
                    // All received documents must be processed
                    if (this.testSessionFlag(App.SyncFoundation.FL_SYNCME))
                      this.processAndSave(rn, "", afterProcesssAndSave);
                    else
                      afterProcesssAndSave(rok);
                  }.bind(this);
                  //
                  if (!this.dataStore.isInTransaction()) {
                    transOpened = true;
                    this.dataStore.beginTransaction(doInTransaction);
                  }
                  else
                    doInTransaction();
                }
              }.bind(this));
            }.bind(this);
            //
            if (this.testSessionFlag(App.SyncFoundation.FL_SYNCPARTNER))
              this.getClientVariations(afterClientVariations);
            else
              afterClientVariations();
          }.bind(this));
        }.bind(this);
        //
        doLoop();
      }
      else {
        /*
         this.serverErrList = null;
         var args =
         {
         DBNAME: this.dataStore.DefaultConnStr(),
         SERVERURL: this.serverURL,
         USERNAME: this.username,
         PASSWORD: this.password,
         CONTIM: this.connectionTimeout,
         MAXDOCS: this.maxDocuments,
         PRECOUNT: this.preCount ? "1" : "0",
         SESSFLAGS: this.sessionFlags,
         CLASSTORESYNC: this.classToResync.join(",")
         };
         this.app.ShellObject.SendCmd("SYNC_START", args);
         //
         this.app.DTTObj.AddMsg(DTTEngine.DTTMSG_INFO, "", 603, "SyncToServer", "Native synchronization started (Async=" + async + "); for more detail see debug of shell");
         //
         // Se è stata richiesta una sincronizzazione asincrona esco subito
         if (async)
         return true;
         //
         this.app.StartPhase(0, this.app.RTCObj.GetConst("F163BC6B-AB8F-46bc-BCCD-74BBF92B0154", "Sincronizzazione in corso...", false), this.canAbort());
         var progress = 0; // 0-99: in corso, 100: terminata con successo, -1: terminata con errori
         while (progress >= 0 && progress < 1)
         {
         IDL.Sleep(500);
         //
         // Aggiorno la progressbar
         var Aborted = !this.app.TrackPhase(progress * 100, 100);
         //
         progress = this.getProgress(Aborted).dblValue();
         }
         //
         this.app.DTTObj.AddMsg(DTTEngine.DTTMSG_INFO, "", 603, "SyncToServer", "Native synchronization terminated (Success=" + (progress == 1) + ")");
         return (progress == 1);
         */
      }
    }.bind(this));
  }.bind(this));
};


/**
 * Gets the changes of client
 * @param {Function} callback - function to be called at the end
 */
App.SyncFoundation.prototype.getClientVariations = function (callback)
{
  var sql = "select ID, DNA, STATUS, CONTENT from ZZ_SYNC where STATUS NOT IN ('.','0','1','2') order by ID";
  this.dataStore.executeSQL(sql, true, function (rs, error) {
    if (error)
      return callback(null, error);
    //
    var sb = "";
    for (var r = 0; r < this.dataStore.getRowCount(rs); r++) {
      var row = this.dataStore.getRow(rs, r);
      var id = row.ID;
      var dna = row.DNA;
      var stt = row.STATUS;
      var cnt = row.CONTENT;
      //
      // If this node belongs to a class that I resynchronizing, skip it
      var tors = false;
      for (var i = 0; i < this.classToResync.length; i++) {
        var s = this.classToResync[i];
        var p = dna.indexOf("\"" + s + "\"");
        //
        // The DNA contained the document, as the last of the series, so it was really him
        if (p > -1 && dna.indexOf("}{", p) === -1) {
          tors = true;
          break;
        }
      }
      if (tors)
        continue;
      //
      // Settle the node that is to be cnt
      sb += "<do ss_id=\"" + id + "\"";
      sb += " ss_dna=\"" + App.Utils.htmlEncode(dna) + "\"";
      sb += " ss_stt=\"" + stt + "\"";
      //
      // Remove <do
      if (cnt.length > 3)
        sb += cnt.substring(3);
      else
        sb += "/>";
    }
    //
    callback(sb);
  }.bind(this));
};


/**
 * Processes the documents arrived and then saves them
 * @param {Object} rn - variation to process
 * @param {String} localDomain
 * @param {Function} callback - function to be called at the end
 */
App.SyncFoundation.prototype.processAndSave = function (rn, localDomain, callback)
{
  var docMap = {};
  var docList = [];
  var res = true;
  //
  // Handle classes to resync
  if (rn.rs) {
    for (var i = 0; i < rn.rs.length; i++) {
      var cls = rn.rs[i].cls;
      this.resyncClass(cls);
    }
  }
  //
  // Load documents and changes
  var v = 0;
  var nextVariation = function (ok, error) {
    if (error)
      return callback(null, error);
    //
    if (v > 0) {
      res = res && ok;
      //
      // Update the progressbar (every 100 ticks)
      this.updateProgressBar();
    }
    //
    if (rn.do && v < rn.do.length)
      this.processVariation(rn.do[v++], localDomain, docMap, docList, nextVariation);
    else
      afterProcessVariation();
  }.bind(this);
  //
  var afterProcessVariation = function (result, error) {
    if (error)
      return callback(null, error);
    //
    // All documents of doclist can be validated and then saved
    // Save in various waves, retry those who have not saved
    var yet = true;
    var nextTry = function (result, error) {
      if (error)
        return callback(null, error);
      //
      yet = false;
      this.errList = [];
      var i = 0;
      var nextDoc = function (result, error) {
        if (error)
          return callback(null, error);
        //
        if (i < docList.length) {
          var doc = docList[i++];
          var md = doc.schema;
          var noVal = md.fastResync && doc.inserted;
          var afterValidate = function (result, error) {
            if (error)
              return callback(null, error);
            //
            if (result) {
              doc.save({revalidate: false, dataStore: this.dataStore}, function (result, error) {
                if (result) {
                  yet = true;
                  this.actualDocument++;
                  //
                  // Update the progressbar (every 100 ticks)
                  this.updateProgressBar();
                }
                else
                  this.errList.push(doc);
                nextDoc();
              }.bind(this));
            }
            else {
              // If not valid now I add it to the list
              // I hope that you will validate after saving
              // other documents!
              this.errList.push(doc);
              nextDoc();
            }
          }.bind(this);
          if (!noVal) {
            var opts = {};
            opts.reason = App.Document.validationReasons.sync;
            opts.childLevel = 9999;
            doc.validate(opts, afterValidate);
          }
          else
            afterValidate(true);
        }
        else {
          // If there are errors, I finished
          if (this.errList.length === 0)
            yet = false;
          else {
            if (yet) {
              var dttit = this.app.dttobj.addItem("", App.DTT.ItemTypeMap.INFO, "SyncFoundation.processAndSave");
              this.app.dttobj.addItemParam("Message", this.errList.length + " documents cannot be saved. Trying again...", dttit);
              //
              // Try again documents with errors
              docList = this.errList.slice();
            }
            else {
              var dttit = this.app.dttobj.addItem("", App.DTT.ItemTypeMap.EXCEPTION, "SyncFoundation.processAndSave");
              this.app.dttobj.addItemParam("Message", this.errList.length + " documents cannot be saved.", dttit);
            }
          }
          //
          if (yet)
            nextTry();
          else {
            this.updateProgressBar(true);
            //
            // If there were errors, updating panels video
            if (!res)
              this.updatePanels();
            //
            callback(res);
          }
        }
      }.bind(this);
      nextDoc();
    }.bind(this);
    nextTry();
  }.bind(this);
  nextVariation();
};


/**
 * Updates the progressbar
 * @param {Boolean} force
 */
App.SyncFoundation.prototype.updateProgressBar = function (force)
{
  // Only every 100 ticks
  if ((this.processedDocs + this.actualDocument) % 100 > 0 && !force)
    return;
  //
  this.clientElapsedTime = (new Date()).getTime() - this.startSyncTS.getTime() - this.serverElapsedTime - this.networkElapsedTime;
  //
  // If it is a partial synchronization without precount I can not know how many records will be processed
  // Then esteem always the ones I have already tried and saved + other MaxDocuments
  var totalSteps = 0;
  if (this.totalDocuments === -1)
    totalSteps = this.processedDocs + this.actualDocument + (this.maxDocuments * 2);
  else
    totalSteps = this.totalDocuments * 2;
  //
  // Updates the progressbar
  if (totalSteps)
    this.progress = (this.processedDocs + this.actualDocument) / totalSteps;
  else
    this.progress = 1;
};


/**
 * Processes a data come from the client
 * @param {Object} xn
 * @param {String} localDomain
 * @param {Object} docMap
 * @param {Array} docList
 * @param {Function} callback - function to be called at the end
 */
App.SyncFoundation.prototype.processVariation = function (xn, localDomain, docMap, docList, callback) {
  var res = true;
  var dna = xn.ss_dna;
  var stt = xn.ss_stt;
  var sid = xn.ss_id;
  var serr = xn.ss_err;
  var isResync = (sid === 0);
  //
  // Documents in error to not resynchronizing (serr == 2) should not be saved
  if (serr !== "2")
    this.processedDocs++;
  else {
    docMap = {};
    docList = [];
  }
  //
  this.getFromCompleteDNA(dna, docMap, docList, true, false, function (doc, error) {
    if (error)
      return callback(null, error);
    //
    // The document was not found ... nothing to do
    if (!doc) {
      var dttit = this.app.dttobj.addItem("", App.DTT.ItemTypeMap.EXCEPTION, "SyncFoundation.processVariation");
      this.app.dttobj.addItemParam("Message", "Cannot get document from DNA " + dna, dttit);
      return callback(res);
    }
    //
    // If the document is in error ...
    var docErr = null;
    if (serr) {
      // and by resynchronizing will keep a copy with errors only
      if (serr === "1")
        docErr = new (doc.class)();
      else if (serr === "2")
        docErr = doc;
      //
      // and add it to the list of documents in error
      this.serverErrList.push(docErr);
    }
    //
    // Check if matches domain
    if (!this.verifyDocDomain(doc, localDomain))
      return callback(res);
    //
    // Retrieving objects corresponding to doc
    var docsUI = this.getUIDocs(doc);
    //
    // It supports lock?
    var lockopt = this.optLockList.indexOf(doc.schema.className) > -1;
    //
    // I apply the changes to the document, through a transaction document
    if (stt === "D") {
      doc.deleted = true;
      //
      // I do it for the objects on the screen, so you do not have to be read back from the DB
      for (var i = 0; i < docsUI.rows.length; i++) {
        var docui = docsUI.rows[i];
        docui.deleted = true;
        if (docui.collection)
          docui.collection.purgeDeleted();
      }
    }
    else {
      var md = doc.schema;
      //
      // Do not use transactions if a resync, if the document does not come from db
      // and db scheme coincides with that of the server
      var noTrans = isResync && !doc.loaded && md.fastResync;
      //
//      if (!noTrans)
//        doc.beginTransaction();
      //
      // If the object to be cleaned, I do it now
      if (isResync && doc.loaded) {
        for (var i = 0; i < md.properties.length; i++)
          doc[md.properties[i].name] = null;
      }
      //
      var attributes = Object.keys(xn);
      while (attributes.length > 0) {
        var ns = attributes.shift();
        var v = xn[ns];
        //
        // elements reserved by the synchronization service
        if (ns.startsWith("ss_"))
          continue;
        //
        // management nulls?
        if (v === App.SyncFoundation.NULL_Marker)
          v = null;
        else if (isResync && typeof v === "string")
          v = App.Utils.htmlDecode(v, true);
        //
        // It's an old property, now for the jump because I consider already under
        if (ns.endsWith("_old"))
          continue;
        //
        if (ns.endsWith("_err")) {
          // Is an error ...
          // Looking for the doc to video and the attack on the property, so you see the panel.
          var np = ns.substring(0, ns.length - 4);
          var errtxt = App.Utils.htmlDecode(v, true);
          for (var i = 0; i < docsUI.rows.length; i++)
            docsUI.rows[i].setError(errtxt, np);
          docErr.setError(errtxt, np);
          var dttit = this.app.dttobj.addItem("", App.DTT.ItemTypeMap.EXCEPTION, "SyncFoundation.processVariation");
          this.app.dttobj.addItemParam("Error Message", docErr.getDNA() + ": server error while saving document: property " + np + ": " + errtxt, dttit);
          res = false;
          //
          continue;
        }
        //
        // Find the property
        var propName = "";
        if (isResync && md.fastResync) {
          var idx = parseInt(ns.substring(1), 10);
          if (!isNaN(idx) && ("P" + idx) === ns && idx <= md.properties.length)
            propName = md.properties[idx - 1].name;
        }
        else
          propName = doc.findPropByName(ns);
        //
        if (propName) {
          // If it had not been ruled out, I can do ...
          if (!doc.do_dontSync || doc.do_dontSync.indexOf(propName) === -1) {
            var pd = md.columns[propName];
            if (v !== null)
              v = this.stringToValue(v, pd.datatype);
            //
            var oktoupdate = true;
            if (lockopt && pd.datatype !== App.Database.DataTypes.BLOB && this.testSessionFlag(App.SyncFoundation.FL_USEOPTLOCK) && !this.dontAddVariations) {
              var oldv = xn[pd.name + "_old"];
              if (oldv === App.SyncFoundation.NULL_Marker)
                oldv = null;
              else
                oldv = this.stringToValue(oldv, pd.datatype);
              //
              if (JSON.stringify(oldv) !== JSON.stringify(doc.getOriginalValue(propName)))
                oktoupdate = false;
            }
            //
            if (oktoupdate) {
              this.totalChanges++;
              if (noTrans)
                doc.setProp(propName, v);
              else {
                doc[propName] = v;
                //
                // I do it for the objects on the screen, so you do not have to be read back from the DB
                for (var i = 0; i < docsUI.rows.length; i++) {
                  var docui = docsUI.rows[i];
                  var oldupd = docui.Updated();
                  docui[propName] = v;
                  docui.setOriginalValue(propName, v);
                  docui.updated = oldupd;
                }
              }
            }
            else {
              // I must point error of conflict ...
              if (this.conflictList.indexOf(doc) === -1)
                this.conflictList.push(doc);
              this.conflictPropList.add(doc.getDNA() + "§§§" + propName);
            }
          }
        }
      }
      //
//      if (!noTrans)
//        doc.endTransaction();
      //
      // Attack in the document errors
      var ce = 1;
      while (true) {
        var et = xn["do_err" + ce];
        if (et) {
          for (var i = 0; i < docsUI.rows.length; i++)
            docsUI.rows[i].setError(et);
          docErr.setError(et);
          var dttit = this.app.dttobj.addItem("", App.DTT.ItemTypeMap.EXCEPTION, "SyncFoundation.processVariation");
          this.app.dttobj.addItemParam("Error Message", docErr.getDNA() + ": server error while saving document: " + et, dttit);
          res = false;
        }
        else
          break;
        ce++;
      }
    }
    //
    callback(res);
  }.bind(this));
};


/**
 * Prepare the document into memory
 * @param {String} dna
 * @param {Object} docMap
 * @param {Array} docList
 * @param {Boolean} create
 * @param {Boolean} skipLoad
 * @param {Function} callback - function to be called at the end
 */
App.SyncFoundation.prototype.getFromCompleteDNA = function (dna, docMap, docList, create, skipLoad, callback)
{
  if (skipLoad === undefined)
    skipLoad = false;
  //
  var res = null;
  //
  // Control first and last characters
  if (dna.length < 2)
    return callback(null);
  if (dna.charAt(0) !== "{" && dna.charAt(dna.length - 1) !== "}")
    return callback(null);
  //
  dna = dna.substr(1, dna.length - 2);
  //
  // Do you parse the interior }{
  var p = dna.indexOf("}{");
  var p1 = 0;
  var yet = true;
  var doWhile = function (result, error) {
    if (error)
      return callback(null, error);
    //
    if (!yet)
      return callback(res);
    //
    yet = false;
    var d = dna.substr(p1);
    if (p > 0) {
      yet = true;
      d = dna.substr(p1, p - p1);
      p1 = p + 2;
      p = dna.indexOf("}{", p + 1);
    }
    //
    // If not required loading is not needed to reconstruct the chain of fathers
    if (skipLoad && yet)
      doWhile();
    //
    // I have to take the first document
    if (!res) {
      res = docMap[d];
      //
      var afterFromDNA = function () {
        // If you are still in the map, I create/load time
        if (!res && create) {
          App.Document.loadByDNA(this.app, d, {skipLoad: skipLoad, dataStore: this.dataStore}, function (result, error) {
            res = result;
            if (res) {
              if (!res.loaded && !skipLoad)
                res.inserted = true;
              docMap[d] = res;
              docList.push(res);
              endWhile();
            }
            else // Back because I could not create the base object
              callback(null);
          });
        }
        else
          endWhile();
      }.bind(this);
      //
      // If the document is deleted then I have to consider
      // and I do not read from the DM
      if (res && res.deleted && !yet) {
        App.Document.loadByDNA(this.app, d, {skipLoad: true, dataStore: this.dataStore}, function (result, error) {
          if (error)
            return callback(null, error);
          //
          res = result;
          res.inserted = true;
          docMap[d] = res;
          docList.push(res);
          afterFromDNA();
        });
      }
      else
        afterFromDNA();
    }
    else {
      var afterLoad = function (result, error) {
        // I browse the collection to the document search
        var found = false;
        var md = res.schema;
        for (var i = 0; i < md.collections.length; i++) {
          var c = res[md.collections[i].name];
          for (var j = 0; j < c.rows.length; j++) {
            var chdoc = c.rows[j];
            if (chdoc.getDNA() === d) {
              // I found the document in the collection, however,
              // check whether the same document had already treated
              var chdoc1 = docMap[d];
              if (chdoc1 && chdoc !== chdoc1 && !chdoc1.deleted) {
                // In this case, remove the load from the DB and add to the map
                c.remove(chdoc);
                c.add(chdoc1);
                res.reconnectChildren(1);
                chdoc = chdoc1;
              }
              else
                docMap[d] = chdoc;
              //
              res = chdoc;
              found = true;
              break;
            }
          }
          if (found)
            break;
        }
        //
        if (!found && create) {
          // Let's see if I can create it. I take the first collection helpful
          var childtype = d.split(",")[0];
          if (!childtype)
            return callback(null);
          //
          childtype = childtype.substr(1, childtype.length - 2);
          var pp = childtype.indexOf("|");
          if (pp > -1)
            childtype = childtype.substr(pp + 1);
          pp = childtype.indexOf("$");
          if (pp > -1)
            childtype = childtype.replace(/\$/, ".");
          //
          for (var i = 0; i < md.collections.length; i++) {
            var mdc = md.collections[i];
            if (mdc.contentType === childtype) {
              // I can put here
              var c = res[mdc.name];
              //
              // Before creating the document verify if the same document had already treated
              var childdoc = docMap[d];
              if (!childdoc || childdoc.deleted) {
                // If I have not treated or was deleted I create new
                childdoc = new (App.Session.getClassByName(childtype))(this.app);
                childdoc.inserted = true;
                docMap[d] = childdoc;
              }
              //
              c.add(childdoc);
              res.reconnectFK(1);
              //
              res = childdoc;
              found = true;
              break;
            }
          }
        }
        //
        if (!found)
          callback(null, error);
        else
          endWhile();
      }.bind(this);
      //
      // If you are still in the map, check if I can create it
      if (res.loaded)
        res.load({childLevel: 1, dataStore: this.dataStore}, afterLoad); // If res was loaded, loading at least the collection of first level
      else
        afterLoad();
    }
  }.bind(this);
  //
  var endWhile = function () {
    // If I did not find the document here, I may as well quit
    if (!res)
      yet = false;
    doWhile();
  };
  //
  doWhile();
};


// ******************************************************************
// Ritorna tutti i documenti presenti nell'UI corrispondenti a quello passato
// ******************************************************************
App.SyncFoundation.prototype.getUIDocs = function (doc)
{
  var res = new App.Collection();
  //
  /*
   for (var i = 0; i < this.app.StackForm.Count(); i++)
   {
   var f = this.app.StackForm[i];
   for (var j = 0; j < f.Frames.length; j++)
   {
   var fr = f.Frames[j];
   if (fr == null)
   continue;
   //
   var frc = fr.Content;
   if (frc == null)
   continue;
   //
   if (frc instanceof IDPanelSrv)
   {
   if (frc.pCollection() != null) {
   var ii = frc.pCollection().GetLoopCollection();
   for (var k = 0; k < ii.Count(); k++)
   {
   var Doc2 = ii[k];
   this.searchForDocs(res, Doc2, doc);
   }
   }
   else if (frc.pDocument() != null) {
   this.searchForDocs(res, frc.pDocument(), doc);
   }
   }
   if (frc instanceof ATreeSrv)
   {
   for (var k = 0; k < frc.Nodes.Count(); k++)
   {
   var tn = frc.Nodes[k];
   if (tn.Doc != null && tn.Parent == null)
   this.searchForDocs(res, tn.Doc, doc);
   }
   }
   }
   }
   */
  return res;
};


// ******************************************************************
// Ritorna tutti i documenti presenti nell'UI corrispondenti a quello passato
// ******************************************************************
App.SyncFoundation.prototype.searchForDocs = function (c, docToSearch, docToFind)
{
  /*
   if (docToSearch.TypeName() == docToFind.TypeName() && docToSearch.GetDNA().stringValue() == docToFind.GetDNA().stringValue())
   {
   c.Add(docToSearch, null, true, false);
   return true;
   }
   //
   // Cerco nelle varie collection...
   var MD = docToSearch.schema;
   for (var i = 1; i <= MD.GetNumColl(); i++)
   {
   var mdc = MD.GetColl(i);
   if (mdc.ChildrenName == docToFind.TypeName())
   {
   var c2 = docToSearch.GetCollByIndex(i);
   var ii = c2.GetLoopCollection();
   for (var j = 0; j < ii.Count(); j++)
   {
   var chdoc = ii[j];
   if (this.searchForDocs(c, chdoc, docToFind))
   return true;
   }
   }
   }
   return false;
   */
};


// ******************************************************************
// Aggiorna lo stato di errore dei pannelli con documenti
// ******************************************************************
App.SyncFoundation.prototype.updatePanels = function ()
{
  /*
   for (var i = 0; i < this.app.StackForm.Count(); i++)
   {
   var f = this.app.StackForm[i];
   for (var j = 0; j < f.Frames.length; j++)
   {
   var fr = f.Frames[j];
   if (fr == null)
   continue;
   //
   var frc = fr.Content;
   if (frc == null)
   continue;
   //
   if (frc instanceof IDPanelSrv)
   {
   if (frc.pCollection() != null || frc.pDocument() != null) {
   frc.DumpDOErrors();
   }
   }
   }
   }
   */
};


/**
 * Send to the server a command to execute
 * @param {String} command
 * @param {Array} params
 * @param {Function} callback - function to be called at the end
 */
App.SyncFoundation.prototype.query = function (command, params, callback)
{
  this.checkSyncServer(function (result, error) {
    if (!result)
      return callback();
    //
    var rs = new App.DataMap();
    //
    // I create the XML document as a string ...
    var sb = "";
    sb += "<qry username=\"" + App.Utils.htmlEncode(this.username) + "\" ";
    sb += "password=\"" + App.Utils.htmlEncode(this.password) + "\" ";
    sb += "cmd=\"" + App.Utils.htmlEncode(command) + "\">";
    //
    // Add parameters
    if (params) {
      for (var i = 0; i < params.length; i++)
        sb += "<par val=\"" + App.Utils.htmlEncode(this.valueToString(params[i])) + "\"/>";
    }
    //
    // I close the root tag
    sb += "</qry>";
    //
    // Send post to server
    var postdata = {};
    postdata.SYNCDATA = sb;
    postdata.WCI = "SYNC";
    postdata.FTR = this.responseFormat;
    //
    var options = {params: postdata};
    if (this.connectionTimeout > 0)
      options.timeOut = this.connectionTimeout * 1000;
    //
    var startHttpRequest = new Date();
    this.app.fs.url(this.serverUrl).post(options, function (res) {
      if (res.error)
        return callback(null, res.error);
      //
      var endHttpRequest = new Date();
      //
      var dttit = this.app.dttobj.addItem("", App.DTT.ItemTypeMap.INFO, "SyncFoundation.query");
      this.app.dttobj.addItemParam("Message", "Received " + res.body.length + " chars from server", dttit);
      if (res.body.length < 20000)
        this.app.dttobj.addItemParam("Content", res.body, dttit);
      //
      var xn = null;
      try {
        xn = JSON.parse(res.body).qry;
      }
      catch (e) {
        dttit.type = App.DTT.ItemTypeMap.EXCEPTION;
        this.app.dttobj.addItemParam("Error Message", "Parse JSON failed", dttit);
        return callback(null, e);
      }
      //
      if (xn) {
        this.updateElapdedTimes(xn, endHttpRequest.getTime() - startHttpRequest.getTime());
        //
        // Verify any errors in the response
        if (!this.checkResponse(xn, "query"))
          return callback();
        //
        rs = this.loadDataMapFromJSON(xn);
        //
        this.app.dttobj.addItemParam("Total rows", rs.rows.length, dttit);
      }
      //
      callback(rs);
    }.bind(this));
  }.bind(this));
};


App.SyncFoundation.prototype.syncDocument = function (doc, numLevel)
{
  /*
   if (numLevel == undefined)
   numLevel = 9999;
   //
   var coll = new IDCollection();
   coll.Add(doc, null, true, false);
   return this.syncCollection(coll, numLevel);
   */
};


App.SyncFoundation.prototype.syncCollection = function (coll, numLevel)
{
  /*
   if (numLevel == undefined)
   numLevel = 9999;
   //
   // Il remoto non è raggiungibile, esco.
   if (!this.checkSyncServer())
   return false;
   //
   // Creo il documento xml come stringa...
   var sb = "";
   sb += "<sc username=\"" + App.Utils.htmlEncode(this.username) + "\" ";
   sb += "password=\"" + App.Utils.htmlEncode(this.password) + "\" ";
   sb += "numlev=\"" + numLevel + "\">";   //
   // Aggiungo i documenti da sincronizzare
   var lc = coll.GetLoopCollection();
   for (var i = 0; i < lc.Count(); i++)
   {
   var Doc = lc[i];
   sb += "<do dna=\"" + App.Utils.htmlEncode(Doc.GetDNA()) + "\"/>";
   }
   //
   // Chiudo il tag root
   sb += "</sc>";   //
   // Invio la post al remoto
   var postdata = new IDMap();
   postdata.set_Value(new IDVariant("SYNCDATA"), new IDVariant(sb));
   postdata.set_Value(new IDVariant("WCI"), new IDVariant("SYNC"));
   postdata.set_Value(new IDVariant("FTR"), new IDVariant(this.responseFormat));
   postdata.set_Value(new IDVariant("ID_RESPONSETYPE"), new IDVariant("TEXT"));
   if (this.connectionTimeout > 0)
   postdata.set_Value(new IDVariant("ID_TIMEOUT"), new IDVariant(this.connectionTimeout));
   //
   var startHttpRequest = new Date();
   var ris = IDL.HttpRequest(this.app, new IDVariant(this.serverUrl), postdata);
   var endHttpRequest = new Date();
   //
   this.app.dttobj.AddMsg(DTTEngine.DTTMSG_INFO, "", 604, "SyncCollection", "Received " + ris.stringValue().length + " bytes from server");
   if (ris.stringValue().length < 20000 && this.app.dttobj.MsgLevel >= DTTEngine.DTTMSG_VERB)
   this.app.dttobj.addItemParam("Content", ris.stringValue());
   //
   // Se la risposta inizia con un < mi hanno risposto con un XML
   if (this.responseFormat != XMLDoc.FT_XML && ris.stringValue().startsWith("<"))
   this.responseFormat = XMLDoc.FT_XML;
   //
   var xd = new XMLDoc();
   if (!xd.LoadFromString(ris.stringValue(), this.responseFormat))
   this.app.dttobj.AddMsg(DTTEngine.DTTMSG_ERROR, "", 628, "SyncCollection", "Parse XML failed");
   //
   var xn = xd.GetRootNode();
   if (xn != null && xn.Name() == "sc") {
   this.updateElapdedTimes(xn, endHttpRequest.getTime() - startHttpRequest.getTime());
   //
   // Verifico eventuali errori nella risposta
   if (!this.checkResponse(xn, "SyncCollection"))
   return false;
   //
   // Ogni attributo arrivato può essere rimpiazzato nella collection.
   while (xn.HasNextNode())
   {
   var xdo = xn.GetNextNode();
   if (xdo.Name() == "do") {
   var dna = xdo.GetAttribute("dna");
   var lc = coll.GetLoopCollection();
   for (var i = 0; i < lc.Count(); i++)
   {
   var Doc = lc[i];
   if (Doc.GetDNA().stringValue() == dna)
   {
   Doc.LoadFromXML(App.Utils.htmlDecode(xdo.GetAttribute("cnt"), true), false);
   break;
   }
   }
   }
   }
   }
   //
   return true;
   */
};


/**
 * Releases a lock
 * @param {Document} doc
 */
App.SyncFoundation.prototype.releaseLock = function (doc) {
  return this.getLock(doc, null, false);
};


/**
 * Get a lock
 * @param {Document} doc
 * @param {String} errMsg
 * @param {Boolean} flGet
 */
App.SyncFoundation.prototype.getLock = function (doc, errMsg, flGet)
{
  /*
   if (errMsg == undefined)
   errMsg = new IDVariant();
   if (flGet == undefined)
   flGet = true;
   // Il remoto non è raggiungibile, esco.
   if (!this.checkSyncServer())
   return false;
   //
   var ok = false;   //
   // Creo il documento xml come stringa...
   var sb = "";
   sb += "<gl username=\"" + App.Utils.htmlEncode(this.username) + "\" ";
   sb += "password=\"" + App.Utils.htmlEncode(this.password) + "\" ";
   sb += "get=\"" + (flGet ? "1" : "0") + "\" ";
   sb += "dna=\"" + App.Utils.htmlEncode(doc.GetDNA()) + "\">";   //
   // Chiudo il tag root
   sb += "</gl>";   //
   // Invio la post al remoto
   var postdata = new IDMap();
   postdata.set_Value(new IDVariant("SYNCDATA"), new IDVariant(sb));
   postdata.set_Value(new IDVariant("WCI"), new IDVariant("SYNC"));
   postdata.set_Value(new IDVariant("FTR"), new IDVariant(this.responseFormat));
   postdata.set_Value(new IDVariant("ID_RESPONSETYPE"), new IDVariant("TEXT"));
   if (this.connectionTimeout > 0)
   postdata.set_Value(new IDVariant("ID_TIMEOUT"), new IDVariant(this.connectionTimeout));
   //
   var startHttpRequest = new Date();
   var ris = IDL.HttpRequest(this.app, new IDVariant(this.serverUrl), postdata);
   var endHttpRequest = new Date();
   //
   this.app.dttobj.AddMsg(DTTEngine.DTTMSG_INFO, "", 604, "GetLock", "Received " + ris.stringValue().length + " bytes from server");
   if (ris.stringValue().length < 20000 && this.app.dttobj.MsgLevel >= DTTEngine.DTTMSG_VERB)
   this.app.dttobj.addItemParam("Content", ris.stringValue());
   //
   var xd = new XMLDoc();
   if (!xd.LoadFromString(ris.stringValue(), this.responseFormat))
   this.app.dttobj.AddMsg(DTTEngine.DTTMSG_ERROR, "", 628, "GetLock", "Parse XML failed");
   //
   var xn = xd.GetRootNode();
   if (xn != null && xn.Name() == "gl") {
   this.updateElapdedTimes(xn, endHttpRequest.getTime() - startHttpRequest.getTime());
   //
   // Verifico eventuali errori nella risposta
   if (!this.checkResponse(xn, "GetLock"))
   return false;
   //
   ok = xn.GetAttribute("ris") == "1";
   errMsg.set(new IDVariant(App.Utils.htmlDecode(xn.GetAttribute("err"), true)));
   }
   //
   return ok;
   */
};


/**
 * Check the domain of a document
 * @param {Document} doc
 * @param {String} clientDomain
 */
App.SyncFoundation.prototype.verifyDocDomain = function (doc, clientDomain)
{
  var d = doc;
  while (d && clientDomain) {
    var dd = this.getDomain(d);
    if (dd) {
      // The document was in another domain, I can not continue
      if (!dd.startsWith(clientDomain)) {
        var dttit = this.app.dttobj.addItem("", App.DTT.ItemTypeMap.EXCEPTION, "SyncFoundation.verifyDocDomain");
        this.app.dttobj.addItemParam("Error message", doc.getDNA() + ": client domain "
                + clientDomain + " cannot be match with document domain " + dd
                + ". The document won't be processed", dttit);
        return false;
      }
      break;
    }
    d = d.parent;
  }
  //
  return true;
};


/**
 * Convert value to string
 * @param {AnyType} v
 */
App.SyncFoundation.prototype.valueToString = function (v)
{
  if (v === null || v === undefined)
    return App.SyncFoundation.NULL_Marker;
  else if (v instanceof Date)
    return App.moment(v).format("YYYY-MM-DD HH:mm:ss");
  else if (typeof v === "boolean")
    return (v ? "-1" : "0");
  else
    return v.toString();
};


/**
 * Convert a string to value
 * @param {String} s
 * @param {String} dt
 */
App.SyncFoundation.prototype.stringToValue = function (s, dt)
{
  if (s === null || s === "" || s === App.SyncFoundation.NULL_Marker)
    return null;
  //
  // Adapt the datatype to the expected one
  if (App.Database.classDT(dt) === App.Database.DataTypeClasses.DATETIME)
    return App.moment(s, "YYYY-MM-DD HH:mm:ss", true).toDate();
  //
  if (App.Database.classDT(dt) === App.Database.DataTypeClasses.NUMBER && typeof s !== "number") {
    if (dt === App.Database.DataTypes.INT)
      return parseInt(s);
    else
      return parseFloat(s);
  }
  //
  if (App.Database.classDT(dt) === App.Database.DataTypeClasses.STRING && typeof s !== "string")
    return s + "";
  //
  if (dt === App.Database.DataTypes.BOOL)
    return (s === "-1" || s === -1);
  //
  return s;
};


/**
 * Check the synchronization server
 * @param {Function} callback - function to be called at the end
 */
App.SyncFoundation.prototype.checkSyncServer = function (callback)
{
  if (!this.serverUrl) {
    var dttit = this.app.dttobj.addItem("", App.DTT.ItemTypeMap.EXCEPTION, "SyncFoundation.checkSyncServer");
    this.app.dttobj.addItemParam("Error message", "No sync server specified", dttit);
    return callback(false);
  }
  //
  if ((!module && !this.app.device.isInsideShell()) || !this.checkServer)
    return callback(true);
  //
  var options = {timeOut: this.checkServer * 1000};
  this.app.fs.url(this.serverUrl).get(options, function (response) {
    if (response.error || response.status >= 400) {
      var dttit = this.app.dttobj.addItem("", App.DTT.ItemTypeMap.EXCEPTION, "SyncFoundation.checkSyncServer");
      this.app.dttobj.addItemParam("Error message", "Sync server " + this.serverUrl + " was not reached in " + this.checkServer + " seconds", dttit);
      callback(false);
    }
    else
      callback(true);
  }.bind(this));
};


/**
 * Checks for errors in the response
 * @param {Object} rootNode
 * @param {String} functName
 */
App.SyncFoundation.prototype.checkResponse = function (rootNode, functName)
{
  if (rootNode.canceled === "yes") {
    var dttit = this.app.dttobj.addItem("", App.DTT.ItemTypeMap.EXCEPTION, "SyncFoundation." + functName);
    this.app.dttobj.addItemParam("Error message", "Synchronization canceled by the server", dttit);
    return false;
  }
  //
  if (rootNode.error) {
    var dttit = this.app.dttobj.addItem("", App.DTT.ItemTypeMap.EXCEPTION, "SyncFoundation." + functName);
    this.app.dttobj.addItemParam("Error message", "Synchronization failed with error: " + App.Utils.htmlDecode(rootNode.error, true), dttit);
    return false;
  }
  //
  return true;
};


/**
 * Notify the end of the native synchronization
 * @param {Object} options
 */
App.SyncFoundation.prototype.endSynchronization = function (options)
{
  /*
   if (options.totalDocs == undefined)
   options.totalDocs = -1;
   if (options.totalChg == undefined)
   options.totalChg = -1;
   //
   this.totalDocuments = options.totalDocs;
   this.totalChanges = options.totalChg;
   //
   if (this.app.DTTObj.Level > 0)
   {     // Recupero gli errori dei documenti
   var serverErrors = new IDCollection();
   var clientErrors = new IDCollection();
   this.getAllDocsInError(serverErrors, clientErrors);
   //
   // Scrivo nel debug gli errori del server
   serverErrors.MoveFirst();
   while (!serverErrors.IsEof())
   {
   var d = serverErrors.GetAt();
   this.dumpDocErrors(d, true);
   serverErrors.MoveNext();
   }
   //
   // Scrivo nel debug gli errori del client
   clientErrors.MoveFirst();
   while (!clientErrors.IsEof())
   {
   var d = clientErrors.GetAt();
   this.dumpDocErrors(d, false);
   clientErrors.MoveNext();
   }
   }
   //
   // Notifico l'evento OnEndSynchronization
   var SkipRefreshUI = IDVariant.FALSE;
   this.app.DTTObj.AddMsg(DTTEngine.DTTMSG_INFO, null, 640, "SyncFoundation.EndSynchronization", "Native synchronization terminated: fire OnEndSynchronization event");
   this.app.OnEndSynchronization(new IDVariant(options.success), SkipRefreshUI, options.TablesChanged);
   //
   // Aggiorno i pannelli se richiesto
   if (SkipRefreshUI.isFalse() && options.TablesChanged.Size() > 0)
   this.refreshPanels(options.TablesChanged);
   */
};


/**
 * Update panels based on tables that have changed
 * @param {Array} tablesChanged
 */
App.SyncFoundation.prototype.refreshPanels = function (tablesChanged)
{
  /*
   for (var i = 0; i < this.app.StackForm.Count(); i++)
   {
   var f = this.app.StackForm[i];
   for (var j = 0; j < f.Frames.length; j++)
   {
   var fr = f.Frames[j];
   if (fr == null)
   continue;
   //
   var frc = fr.Content;
   if (!frc)
   continue;
   //
   if (frc instanceof IDPanelSrv)
   {
   // Salto i pannelli senza master query
   if (frc.bQueries(0) == null)
   continue;
   //
   // Salto i pannelli IMDB
   if (frc.IsIMDB())
   continue;
   //
   // Salto i pannelli in QBE o con dati modificati
   if (frc.Status() == Glb.PS_QBE || frc.IsContentModified())
   continue;
   //
   // Aggiorno i pannelli DB basati su tabelle che hanno subito modifiche
   if (!frc.IsDO() && tablesChanged.FindValue(new IDVariant(frc.GetMasterTable(0)), false).intValue() != -1)
   {
   frc.RefreshQuery(this.app);
   continue;
   }
   //
   // Ora controllo i pannelli DO
   if (frc.IsDO())
   {
   var doc = frc.bQueries(0).DocTempl;
   if (doc == null)
   doc = frc.bQueries(0).DocFilter;
   if (doc == null)
   doc = this.app.CreateDocument(frc.GetMasterTable(0));
   //
   // Salto se non conosco il tipo di documento di questo pannello
   if (doc == null)
   continue;
   //
   var ok = false;
   var md = doc.schema;
   while (md != null) {
   if (tablesChanged.FindValue(new IDVariant(md.DBCode), false).intValue() != -1)
   {
   ok = true;
   break;
   }
   md = md.BaseMD;
   }
   //
   // Se è basato su una tabella che è stata modificata ...
   if (ok)
   {
   // se è un master rieseguo la sua query
   // se è un detail ricarico la collection del documento master
   if (frc.bQueries(0).IsMasterDO)
   frc.RefreshQuery(this.app);
   else if (frc.MasterPanelIdx != 0 && frc.bQueries(0).IDColl != null) {
   frc.bQueries(0).IDColl.set_Loaded(false);
   frc.CheckMasterDocument(true);
   }
   }
   //
   continue;
   }
   }
   }
   }
   */
};


/**
 * Reactivation of sync for a document
 * @param {Document} doc
 */
App.SyncFoundation.prototype.resumeDocSync = function (doc)
{
  /*
   try
   {
   if (!this.checkTables())
   return true;
   //
   var dna = doc.GetDNA().stringValue();
   var sql = "insert into ZZ_SYNC(STATUS, DNA, DOCDOM, CONTENT) select ";
   if (Glb.ClassDB(this.dataStore.DBaseType()) == Glb.DBC_ACCESS)
   sql += "IIF(STATUS='0', 'I', IIF(STATUS='1', 'U', 'D'))";
   else
   sql += "CASE WHEN STATUS='0' THEN 'I' WHEN STATUS='1' THEN 'U' ELSE 'D' END";
   sql += ", DNA, DOCDOM, CONTENT from ZZ_SYNC where DNA LIKE ? and STATUS IN ('0','1','2')";
   //
   // Inserisco le variazioni del documento che erano in stato 0, 1, 2 con stato I, U, D
   this.app.DTTObj.AddQuery("SyncFoundation.ResumeDocSync", doc.GetName(1) + ": Resuming variations", "SyncFoundation.ResumeDocSync", DTTEngine.STMT_QRY_INSSEL, sql);
   this.app.DTTObj.AddParameter("", "", "DNA", dna);
   var idps = new IDPreparedStatement(true);
   idps.SetCommand(sql);
   idps.AddParameter(new IDVariant("%{" + dna + "}"));
   this.dataStore.DBO().ExecuteIDPS(idps, false);
   this.app.DTTObj.EndQuery("SyncFoundation.ResumeDocSync");
   //
   // Elimino le variazioni del documento che erano in stato 0, 1, 2
   sql = "delete from ZZ_SYNC where DNA LIKE ? and STATUS IN ('0','1','2')";
   this.app.DTTObj.AddQuery("SyncFoundation.ResumeDocSync", doc.GetName(1) + ": Deleting temporary variations", "SyncFoundation.ResumeDocSync", DTTEngine.STMT_QRY_DELETE, sql);
   this.app.DTTObj.AddParameter("", "", "DNA", dna);
   idps = new IDPreparedStatement(true);
   idps.SetCommand(sql);
   idps.AddParameter(new IDVariant("%{" + dna + "}"));
   this.dataStore.DBO().ExecuteIDPS(idps, false);
   this.app.DTTObj.EndQuery("SyncFoundation.ResumeDocSync");
   //
   return true;
   }
   catch (e)
   {
   this.app.DTTObj.AddMsg(DTTEngine.DTTMSG_ERROR, "", 601, "SyncFoundation.ResumeDocSync", "Unable to resume variations of " + doc.GetName(1) + ": " + e.Message);
   return false;
   }
   */
};


/**
 * Update elapsed times
 * @param {Object} rootNode
 * @param {Integer} httpRequestMS
 */
App.SyncFoundation.prototype.updateElapdedTimes = function (rootNode, httpRequestMS)
{
  if (rootNode.elapsedMS) {
    this.serverElapsedTime += rootNode.elapsedMS;
    this.networkElapsedTime += httpRequestMS - rootNode.elapsedMS;
  }
};


/**
 * Returns elapsed times
 */
App.SyncFoundation.prototype.getElapsedTimes = function ()
{
  var times = {};
  if (this.nativeSync && this.clientElapsedTime === 0) {
    /*
     var result = this.app.ShellObject.SendCmd("SYNC_ELAPSEDTIMES");
     var xd = new XMLDoc();
     if (result.length > 0 && xd.LoadFromString(result, XMLDoc.FT_JSON)) {
     var root = xd.GetRootNode();
     options.serverTime.set(new IDVariant(parseInt(root.GetAttribute("server"))));
     options.networkTime.set(new IDVariant(parseInt(root.GetAttribute("network"))));
     options.clientTime.set(new IDVariant(parseInt(root.GetAttribute("client"))));
     //
     // Se la sincronizzazione ha finito memorizzo nelle variabili locali
     if (root.GetAttribute("final") == "1") {
     this.serverElapsedTime = options.serverTime;
     this.networkElapsedTime = options.networkTime;
     this.clientElapsedTime = options.clientTime;
     }
     }
     */
  }
  else {
    times.serverTime = this.serverElapsedTime;
    times.networkTime = this.networkElapsedTime;
    times.clientTime = this.clientElapsedTime;
  }
  return times;
};


/**
 * Returns the number of documents to be synchronized
 */
App.SyncFoundation.prototype.getTotalDocuments = function ()
{
  if (this.totalDocuments !== -1)
    return this.totalDocuments;
  else
    return this.processedDocs;
};


/**
 * Returns documents in errors
 */
App.SyncFoundation.prototype.getAllDocsInError = function ()
{
  var serverErrors = new App.Collection(this.app);
  var clientErrors = new App.Collection(this.app);
  try {
    if (this.nativeSync && !this.serverErrList) {
      /*
       // Chiedo gli errori alla shell nativa
       this.serverErrList = new ArrayList();
       var result = this.app.ShellObject.SendCmd("SYNC_GETALLDOCSINERROR");
       var xd = new XMLDoc();
       if (result.length > 0 && xd.LoadFromString(result, XMLDoc.FT_XML)) {
       var root = xd.GetRootNode();
       while (root.HasNextNode()) {
       var xn = root.GetNextNode();
       while (xn.HasNextNode()) {
       var don = xn.GetNextNode();
       var doc = IDDocument.loadByDNA(new IDVariant(don.GetAttribute("ss_dna")), 0, this.app, this.app.getIMDB(), true);
       if (!doc)
       continue;
       //
       while (don.HasNextAttribute()) {
       var n = new IDVariant();
       var v = new IDVariant();
       don.GetNextAttribute(n, v);
       var ns = n.stringValue();
       //
       // Copio gli errori nel documento
       if (ns.endsWith("_err")) {
       var np = ns.substring(0, ns.length - 4);
       var idp = doc.GetPropertyIndex(np, false, false, false, true);
       if (idp > 0)
       doc.SetPropError(idp, v.stringValue());
       }
       else if (ns.startsWith("do_err"))
       doc.AddDocError(v.stringValue());
       }
       //
       // Aggiungo il documento alla lista giusta
       if (xn.Name() == "client")
       this.errList.Add(doc);
       if (xn.Name() == "server")
       this.serverErrList.Add(doc);
       }
       }
       }
       */
    }
    //
    if (this.serverErrList) {
      for (var i = 0; i < this.serverErrList.length; i++)
        serverErrors.add(this.serverErrList[i]);
      for (var i = 0; i < this.errList.length; i++)
        clientErrors.add(this.errList[i]);
    }
  }
  catch (e)
  {
  }
  return {serverErrors: serverErrors, clientErrors: clientErrors};
};


/**
 * Dump into debug errors in one document
 * @param {Document} doc
 * @param {Boolean} serverSide
 */
App.SyncFoundation.prototype.dumpDocErrors = function (doc, serverSide)
{
  /*
   var docErrors = new ArrayList();
   doc.GetDocErrors(docErrors);
   for (var i = 0; i < docErrors.Count(); i = i + 1)
   this.app.DTTObj.AddMsg(DTTEngine.DTTMSG_ERROR, "", 614, "dumpDocErrors", doc.GetName(1) + ": " + (serverSide ? "server" : "client") + " error while saving document: " + docErrors[i]);
   //
   var MD = doc.schema;
   for (var i = 1; i <= MD.GetNumProp(); i = i + 1) {
   var propErr = doc.GetPropError(i);
   if (propErr != null)
   this.app.DTTObj.AddMsg(DTTEngine.DTTMSG_ERROR, "", 614, "dumpDocErrors", doc.GetName(1) + ": " + (serverSide ? "server" : "client") + " error while saving document: property " + MD.GetProp(i).UIName + ": " + propErr);
   }
   */
};


/**
 * Load a dataMap from JSON
 * @param {Object} rsObj
 */
App.SyncFoundation.prototype.loadDataMapFromJSON = function (rsObj)
{
  var rs = new App.DataMap(this.app);
  //
  // Create metaschema
  rs.schema = new App.DataSchema(rs);
  //
  // If there is metaschema, I read it
  var mdObj = rsObj.MD;
  if (!mdObj)
    return rs;
  //
  if (!(mdObj.Column instanceof Array))
    mdObj.Column = [mdObj.Column];
  //
  for (var i = 0; i < mdObj.Column.length; i++) {
    var colObj = mdObj.Column[i];
    var col = new App.DataColumn(rs.schema);
    col.name = colObj.Name;
    col.index = i;
    switch (colObj.Type) {
      case 1:
        col.datatype = App.Database.DataTypes.INT;
        break;
      case 2:
        col.datatype = App.Database.DataTypes.FLOAT;
        break;
      case 3, 4:
        col.datatype = App.Database.DataTypes.DECIMAL;
        break;
      case 5:
        col.datatype = App.Database.DataTypes.VARCHAR;
        break;
      case 6:
        col.datatype = App.Database.DataTypes.DATE;
        break;
      case 7:
        col.datatype = App.Database.DataTypes.TIME;
        break;
      case 8:
        col.datatype = App.Database.DataTypes.DATETIME;
        break;
      case 10:
        col.datatype = App.Database.DataTypes.BLOB;
        break;
    }
    //
    rs.schema.columns[col.name] = col;
  }
  //
  // Load data
  var rowsObj = rsObj.Rows;
  for (var i = 0; i < rowsObj.Row.length; i++) {
    var rowObj = rowsObj.Row[i];
    var row = rs.createRow();
    for (var c in rs.schema.columns) {
      var dc = rs.schema.columns[c];
      var v = rowObj["c" + (dc.index + 1)];
      if (v && App.Database.classDT(dc.datatype) === App.Database.DataTypeClasses.DATETIME)
        v = App.moment(v, "DD-MM-YYYY HH:mm:ss", true).toDate();
      else
        v = this.stringToValue(v, dc.datatype);
      row.values[dc.name] = v;
    }
    rs.add(row);
  }
  return rs;
};


//export module for node
if (module)
  module.exports = App.SyncFoundation;
