/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */


/* global module, Proxy */

var App = App || {};


/**
 * @class FS
 * Represents a file system object, parent of files and directories classes
 * @param {Object} options
 */
App.FS = function (options)
{
  // Import modules
  if (module && !App.View && options !== true)
    App = require("../requireAll");
};


App.FS.internalType = {
  permanent: "permanent",
  temp: "temp",
  resource: "resource",
  blob: "blob",
  private: "private"
};


/**
 * Creates a file object with the appropriate driver and path
 * @param {string} path
 * @param {string} type
 * @param {string} publicUrl
 */
App.FS.prototype.file = function (path, type, publicUrl)
{
  return new App.File(this, type, path, publicUrl);
};


/**
 * Creates a directory object with the appropriate driver
 * @param {string} path
 * @param {string} type
 */
App.FS.prototype.directory = function (path, type)
{
  return new App.Directory(this, type, path);
};


/**
 * Creates a URL object with the appropriate driver
 * @param {string} url
 */
App.FS.prototype.url = function (url)
{
  return new App.Url(this, url);
};


/**
 * Create the appropriate file system
 * @param {Object} options
 */
App.FS.createFS = function (options)
{
  var fsType = "LocalDriver";
  if (module) {
    if (!App.View)
      App = require("../requireAll");
    fsType = "NodeDriver";
  }
  else if (options.app && options.app.device && options.app.device.isInsideShell())
    fsType = "ShellDriver";
  //
  return new App[fsType](options);
};


/**
 * Normalize a path
 * @param {String} path
 */
App.FS.normalizePath = function (path)
{
  // Check if path is out of root
  path = path.replace(/\\/g, "\/");
  //
  // Resources in master exception
  if (path.startsWith("../resources"))
    return path;
  //
  var parts = path.split("/");
  for (var i = 0; i < parts.length; i++) {
    switch (parts[i]) {
      case ".":
        parts.splice(i, 1);
        i--;
        break;
      case "..":
        if (i === 0)
          throw new Error("Invalid path " + path);
        parts.splice(i - 1, 2);
        i -= 2;
        break;
    }
  }
  //
  return path;
};


// export module for node
if (module)
  module.exports = App.FS;
