/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */


/* global module */

var App = App || {};


/**
 * @class Mailer
 * Represents an E-mail object
 */
App.Mailer = function ()
{
  // import modules
  if (module && !App.View) {
    App = require("../requireAll");
    //
    // create sendgrid object
    App.sendgrid = require("sendgrid")(App.Mailer.testKey);
    App.helper = require("sendgrid").mail;
  }
  //
  // create email object
  this.email = new App.helper.Mail();
  this.isTest = true;
  //
  this._personalization = App.helper.Personalization();
  this.email.addPersonalization(this._personalization);
};
App.Mailer.testKey = "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";
App.Mailer.isWaiting = false;


// class properties
Object.defineProperties(App.Mailer.prototype, {
  //
  // mail headers
  headers: {
    get: function () {
      return this.email.getHeaders();
    },
    set: function (obj) {
      this.email.headers = obj;
    }
  },
  //
  // delivery receipt option
  deliveryReceipt: {
    get: function () {
      if (this.email.headers) {
        if (this.email.headers["Return-Receipt-To"])
          return true;
        else
          return false;
      }
      else
        return false;
    },
    set: function (enable) {
      if (enable) {
        var from = this.email.getFrom();
        //
        // set only if there is a "from" address
        if (from) {
          if (!this.email.headers)
            this.email.headers = {};
          //
          // set the header
          this.email.addHeader(new App.helper.Header("Return-Receipt-To", from.email));
        }
      }
      else
        delete  this.email.headers["Return-Receipt-To"];
    }
  },
  //
  // read receipt option
  readReceipt: {
    get: function () {
      //
      if (this.email.headers) {
        //
        if (this.email.headers["Disposition-Notification-To"])
          return true;
        else
          return false;
      }
      else
        return false;
    },
    set: function (enable) {
      if (enable) {
        //
        // set only if there is a "from" address
        var from = this.email.getFrom();
        //
        // set only if there is a "from" address
        if (from) {
          if (!this.email.headers)
            this.email.headers = {};
          //
          // set the header
          this.email.addHeader(new App.helper.Header("Disposition-Notification-To", from.email));
        }
      }
      else
        delete  this.email.headers["Disposition-Notification-To"];
    }
  },
  //
  // date of email
  date: {
    get: function () {
      return this.email.date;
      this._personalization.getSendAt();
    },
    set: function (date) {
      this._personalization.setSendAt(Math.round(date.getTime() / 1000));
    }
  },
  //
  // mail priority
  priority: {
    get: function () {
      if (this.email.headers)
        return this.email.headers["X-Priority"];
      else
        return undefined;
    },
    set: function (priority) {
      if (!priority) {
        delete this.email.headers["X-Priority"];
        return;
      }
      if (typeof priority !== "string" && typeof priority !== "number")
        throw "Invalid value: it must be number or string";
      //
      // convert the number into string
      priority = priority.toString();
      if (this.email.headers)
        this.email.headers["X-Priority"] = priority;
      else
        this.email.headers = {
          "X-Priority": priority
        };
    }
  },
  //
  // mail subject
  subject: {
    get: function () {
      return this.email.getSubject();
    },
    set: function (text) {
      this.email.setSubject(text);
    }
  },
  //
  // mail body (normal text)
  body: {
    get: function () {
      var contents = this.email.getContents();
      //
      if (contents && contents[0].type === "text/plain")
        return contents[0].value;
      return null;
    },
    set: function (text) {
      this.email.contents = [];
      this.email.addContent(new App.helper.Content("text/plain", text));
    }
  },
  //
  // mail body (HTML)
  htmlBody: {
    get: function () {
      var contents = this.email.getContents();
      //
      if (contents && contents[0].type === "text/html")
        return contents[0].value;
      return null;
    },
    set: function (html) {
      this.email.contents = [];
      this.email.addContent(new App.helper.Content("text/html", html));
    }
  },
  //
  // "reply to" address
  replyTo: {
    get: function () {
      return this.email.getReplyTo();
    },
    set: function (address) {
      this.email.setReplyTo(new App.helper.Email(address));
    }
  },
  //
  // "from" address
  from: {
    get: function () {
      var from = this.email.getFrom();
      //
      if (from)
        return from.email;
      return null;
    },
    set: function (address) {
      var from = this.email.getFrom();
      //
      var name = null;
      //
      if (from)
        name = from.name;
      this.email.setFrom(new App.helper.Email(address, name));
    }
  },
  //
  // "from" name
  fromName: {
    get: function () {
      var from = this.email.getFrom();
      //
      if (from)
        return from.name;
      return null;
    },
    set: function (name) {
      var from = this.email.getFrom();
      //
      var address = null;
      //
      if (from)
        address = from.email;
      this.email.setFrom(new App.helper.Email(address, name));
    }
  }
});


/**
 * authentication service
 * @param {string} apiKey
 */
App.Mailer.prototype.setKeys = function (apiKey)
{
  if (apiKey) {
    App.sendgrid = require("sendgrid")(apiKey);
    if (apiKey !== App.Mailer.testKey)
      this.isTest = false;
  }
};


/**
 * Add TO address to email
 * @param {string} address
 * @param {string} name
 */
App.Mailer.prototype.addTO = function (address, name)
{
  this._personalization.addTo(new App.helper.Email(address, name));
};


/**
 * Add CC address to email
 * @param {string} address
 * @param {string} name
 */
App.Mailer.prototype.addCC = function (address, name)
{
  this._personalization.addCc(new App.helper.Email(address, name));
};


/**
 * Add BCC address to email
 * @param {string} address
 * @param {string} name
 */
App.Mailer.prototype.addBCC = function (address, name)
{
  this._personalization.addBcc(new App.helper.Email(address, name));
};


/**
 * Add attachment to email
 * @param {file/object} file
 * @param {boolean} inline
 * @param {string} type
 * @param {function} cb
 */
App.Mailer.prototype.addAttachment = function (file, inline, type, cb)
{
  try {
    var attachment = new App.helper.Attachment();
    var nodePath = require("path");
    //
    var isFile = file instanceof App.File;
    //
    if (!isFile && (!(file instanceof Object)))
      return cb(null, "Invalid param: it's must be a file or anobject");
    //
    // get name of attachment
    var fileName = file.fileName || nodePath.basename(isFile ? file.path : file.url);
    //
    attachment.setFilename(fileName);
    //
    // attach image inline
    if (inline) {
      attachment.setDisposition("inline");
      attachment.setContentId(fileName);
      attachment.setType(type || "image/" + (nodePath.extname(fileName)).substr(1));
    }
    else { // classic attach
      attachment.setType(type || "application/octet-stream");
      attachment.setDisposition("attachment");
    }
    //
    // check type of the arg
    if (isFile) {
      //
      // calculate the real path
      var path = file.fs.checkPath(false, file);
      //
      if (!path)
        return cb(null, "Invalid file path");
      //
      // read file content
      var fs = require('fs');
      var content = fs.readFileSync(path).toString("base64");
      //
      // set attachment content
      attachment.setContent(content);
      //
      // add attachment to mail
      this.email.addAttachment(attachment);
      cb();
    }
    else {
      // attachment from url
      if (file.url) {
        // Node module variable
        var request = require("request");
        //
        var buffers = [];
        var pthis = this;
        request.get(file.url)
                .on("response", function (response) {
                  // if the http code is different from 200, returns an error
                  if (response.statusCode !== 200) {
                    // abort request
                    this.abort();
                    return cb(null, "Error during request, status code:" + response.statusCode);
                  }
                })
                .on("error", function (err) {
                  return cb("Error during request:" + err);
                })
                .on("end", function (response) {
                  var buffer = Buffer.concat(buffers);
                  //
                  // set attachment content
                  attachment.setContent(buffer.toString("base64"));
                  //
                  // add attachment to mail
                  pthis.email.addAttachment(attachment);
                  cb();
                })
                .on("data", function (data) {
                  buffers.push(data);
                });
      }
      else
        return cb(null, "Missing URL Parameter");
    }
  }
  catch (ex) {
    return ex;
  }
};


/**
 * Send email
 * @param {function} cb
 */
App.Mailer.prototype.send = function (cb)
{
  var pthis = this;
  if (this.isTest && App.Mailer.isWaiting)
    return cb(null, "ERR: no sendgrid key was provided, waiting for 10 seconds.");
  //
  this.email.addPersonalization(this._personalization);
  //
  // creates request
  var request = App.sendgrid.emptyRequest({
    method: "POST",
    path: "/v3/mail/send",
    body: this.email.toJSON()
  });
  //
  // sends the email
  App.sendgrid.API(request, function (error, json) {
    if (error)
      cb(null, error);
    else {
      if (pthis.isTest) {
        App.Mailer.isWaiting = true;
        console.log("WARNING: no sendgrid key was provided, waiting for 10 seconds.");
        setTimeout(function () {
          App.Mailer.isWaiting = false;
          cb(json);
        }, 10000);
      }
      else {
        cb(json);
      }
    }
  });
};


//export module for node
if (module)
  module.exports = App.Mailer;
