/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

/* global module, process */

var App = App || {};

/**
 * @class Definition of HeaderList
 * @param {Session} app that owns this HeaderList instance
 * */
App.HeaderList = function (app) {
  if (module && !App.View)
    App = require("./requireAll");
  this.app = app;       // App that owns this device instance
  this.headers = [];
  //
  // Add default headers
  this.headers.push({headType: "meta", charset: "UTF-8"});
  this.headers.push({headType: "meta", name: "viewport", content: "width=device-width, initial-scale=1.0, minimum-scale=1, maximum-scale=1, user-scalable=no"});
  //
  this.id = "header-list-ui"; // used for calling setProp, onChange
  this.app.eleMap[this.id] = this;
};


/**
 * Add an object to the list
 * @param {Object} obj
 */
App.HeaderList.prototype.add = function (obj)
{
  this.headers.push(obj);
  //
  var e = [{obj: this.id, id: "add", cnt: obj}];
  this.app.sendCommands(e);
};


/**
 * Remove an object from the list
 * @param {Object} obj
 */
App.HeaderList.prototype.remove = function (obj)
{
  // Check if this is easy.. the user used the same objects
  if (this.headers.indexOf(obj) !== -1) {
    this.headers.splice(this.headers.indexOf(obj), 1);
  }
  else {
    // Search the local headers for a match and remove it
    var localK = Object.keys(obj);
    var removed = false;
    for (var i = 0; i < this.headers.length && !removed; i++) {
      var h = this.headers[i];
      var hk = Object.keys(h);
      if (localK.length === hk.length) {
        var matches = true;
        for (var i1 = 0; i1 < localK.length && matches; i1++) {
          if (obj[localK[i1]] !== h[localK[i1]])
            matches = false;
        }
        if (matches) {
          this.headers.splice(i, 1);
          removed = true;
        }
      }
    }
  }
  //
  // Send the request to the client
  var e = [{obj: this.id, id: "remove", cnt: obj}];
  this.app.sendCommands(e);
};


/**
 * Get the list of headers
 */
App.HeaderList.prototype.getList = function ()
{
  return this.headers;
};

//export module for node
if (module)
  module.exports = App.HeaderList;