/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

var App = App || {};

if (Object.observe) {
  console.warn("Using native observers");
}
else {
  console.warn("Using simulated observers");
  //
  // Observe map
  App.Observers = {};
  //
  // To avoid traceur overriding
  Object.__keys = Object.keys;
  //
  // Add observer
  Object.observe = function (obj, cb)
  {
    var ol = App.Observers[obj.id || "§"];
    //
    if (ol) {
      var o;
      // Search if already added
      for (var i = 0; i < ol.length; i++) {
        o = ol[i];
        if (o.obj === obj && o.cb === cb)
          return;
      }
    }
    else {
      ol = [];
      App.Observers[obj.id || "§"] = ol;
    }
    //
    o = {obj: obj, cb: cb, copy: Object.clone(obj)};
    ol.push(o);
  };
  //
  Object.unobserve = function (obj, cb)
  {
    var ol = App.Observers[obj.id || "§"];
    //
    if (ol) {
      for (var i = 0; i < ol.length; i++) {
        var o = ol[i];
        if (o.obj === obj && o.cb === cb) {
          ol.splice(i, 1);
          if (ol.length === 0)
            delete App.Observers[obj.id || "§"];
          return;
        }
      }
    }
  };
  //
  Object.getNotifier = function (obj)
  {
    var x = {};
    x.notify = function (change) {
      change.object = obj;
      var ol = App.Observers[obj.id || "§"];
      if (ol) {
        for (var i = 0; i < ol.length; i++) {
          var o = ol[i];
          if (o.obj === obj) {
            o.cb([change]);
          }
        }
      }
    };
    return x;
  };
  App.observeSkippedProps = {
    id: true,
    name: true,
    class: true,
    obs: true,
    pid: true,
    sendChg: true,
    view: true,
    app: true,
    animations: true,
    dataMap: true,
    elements: true,
    parent: true,
    rownum: true
  };
  Object.clone = function (obj)
  {
    var ris = {};
    //
    var k = Object.__keys(obj);
    for (var i = 0; i < k.length; i++) {
      // Skip known static properties
      if (App.observeSkippedProps[k[i]])
        continue;
      //
      var v = obj[k[i]];
      //
      // Skip functions
      if (typeof v === "function")
        continue;
      //
      // Skip child object properties
      if (v && (v instanceof App.RemElem || v instanceof App.DataMap || v instanceof App.DataRow))
        continue;
      //
      ris[k[i]] = v;
    }
    return ris;
  };
  App.observeObservers = function () {
    if (!App)
      return;
    var oll = Object.__keys(App.Observers);
    for (var n = 0; n < oll.length; n++) {
      var ol = App.Observers[oll[n]];
      for (var i = 0; i < ol.length; i++) {
        var o = ol[i];
        var k = Object.__keys(o.obj);
        //
        // It is important to change the value to undefined because
        // we need to reset the changes array for each observed object
        // JSHINT does not understand it.
        var changes = undefined; // jshint ignore:line
        //
        for (var j = 0; j < k.length; j++) {
          var pn = k[j];
          //
          // Skip known static properties
          if (App.observeSkippedProps[pn])
            continue;
          //
          var v = o.obj[pn];
          //
          // Skip functions
          if (typeof v === "function")
            continue;
          //
          // Skip child object properties
          if (v && (v instanceof App.RemElem || v instanceof App.DataMap || v instanceof App.DataRow))
            continue;
          //
          if (o.copy[pn] !== v) {
            if (!changes)
              changes = [];
            changes.push({name: k[j], object: o.obj, oldValue: o.copy[pn], type: "update"});
            o.copy[pn] = o.obj[pn];
          }
        }
        if (changes)
          o.cb(changes);
      }
    }
    delete App.observersTimerId;
  };
  //
  App.markObservers = function () {
    if (!App.observersTimerId) {
      App.observersTimerId = _setTimeout(App.observeObservers, 0);  // jshint ignore:line
    }
  };
  //
  App.deleteObservers = function (obj) {
    if (obj.id)
      delete App.Observers[obj.id];
    else {
      var ol = App.Observers["§"];
      if (ol) {
        for (var i = 0; i < ol.length; i++) {
          var o = ol[i];
          if (o.obj === obj) {
            ol.splice(i, 1);
            i--;
          }
        }
      }
    }
  };
  //
  // Redefine setTimeout e setImmediate for call markObservers
  window._setTimeout = window.setTimeout;
  window.setTimeout = function (funct, ms) {
    var args;
    if (arguments.length > 2)
      args = Array.prototype.slice.call(arguments, 2);
    return window._setTimeout(function () {
      if (App)
        App.markObservers();
      if (typeof funct === "string")
        eval(funct);    // jshint ignore:line
      else
        funct.apply(this, args);
    }, ms);
  };
  //
  if (window.setImmediate) {
    window._setImmediate = window.setImmediate;
    window.setImmediate = function (funct) {
      var args;
      if (arguments.length > 2)
        args = Array.prototype.slice.call(arguments, 2);
      return window._setImmediate(function () {
        if (App)
          App.markObservers();
        if (typeof funct === "string")
          eval(funct);    // jshint ignore:line
        else
          funct.apply(this, args);
      });
    };
  }
}
