/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */
/* global module, require */

var App = App || {};

if (module)
  App.Database = require("./database");


/**
 * @class A Oracle Database Object
 * @param {Session} app that owns this instance
 * @extends App.Database
 * */
App.Oracle = function (app)
{
  if (module && !App.View)
    App = require("./requireAll");
  //
  App.Database.call(this, app);
  //
  this.typemap = App.Oracle.DataTypeMap; // Map of data types
  this.keywords = App.Oracle.Keywords;   // List of reserved keywords
  this.defaultSchema = "SYSTEM";
  //
  // Definition of functions and operators
  this.functions = Object.assign({}, App.Database.Functions, App.Oracle.Functions);
};


// Make App.Oracle extend App.Database
App.Oracle.prototype = new App.Database(true);


App.Oracle.DataTypeMap = {
  i: ["number"],
  f: ["number"],
  d: ["number"],
  vc: ["nvarchar2", "varchar2"],
  c: ["nchar", "char"],
  bl: ["blob"],
  cl: ["nclob", "clob"],
  b: ["number"],
  da: ["date"],
  ti: ["date"],
  dt: ["date"],
  id: ["raw(16)"],
  ai: ["number"],
  j: ["nclob", "clob"]
};


App.Oracle.Keywords = [
  "ACCESS",
  "ACCOUNT",
  "ACTIVATE",
  "ADD",
  "ADMIN",
  "ADVISE",
  "AFTER",
  "ALL",
  "ALL_ROWS",
  "ALLOCATE",
  "ALTER",
  "ANALYZE",
  "AND",
  "ANY",
  "ARCHIVE",
  "ARCHIVELOG",
  "ARRAY",
  "AS",
  "ASC",
  "AT",
  "AUDIT",
  "AUTHENTICATED",
  "AUTHORIZATION",
  "AUTOEXTEND",
  "AUTOMATIC",
  "BACKUP",
  "BECOME",
  "BEFORE",
  "BEGIN",
  "BETWEEN",
  "BFILE",
  "BITMAP",
  "BLOB",
  "BLOCK",
  "BODY",
  "BY",
  "CACHE",
  "CACHE_INSTANCES",
  "CANCEL",
  "CASCADE",
  "CAST",
  "CFILE",
  "CHAINED",
  "CHANGE",
  "CHAR",
  "CHAR_CS",
  "CHARACTER",
  "CHECK",
  "CHECKPOINT",
  "CHOOSE",
  "CHUNK",
  "CLEAR",
  "CLOB",
  "CLONE",
  "CLOSE",
  "CLOSE_CACHED_OPEN_CURSORS",
  "CLUSTER",
  "COALESCE",
  "COLUMN",
  "COLUMNS",
  "COMMENT",
  "COMMIT",
  "COMMITTED",
  "COMPATIBILITY",
  "COMPILE",
  "COMPLETE",
  "COMPOSITE_LIMIT",
  "COMPRESS",
  "COMPUTE",
  "CONNECT",
  "CONNECT_TIME",
  "CONSTRAINT",
  "CONSTRAINTS",
  "CONTENTS",
  "CONTINUE",
  "CONTROLFILE",
  "CONVERT",
  "COST",
  "CPU_PER_CALL",
  "CPU_PER_SESSION",
  "CREATE",
  "CURRENT",
  "CURRENT_SCHEMA",
  "CURREN_USER",
  "CURSOR",
  "CYCLE",
  " ",
  "DANGLING",
  "DATABASE",
  "DATAFILE",
  "DATAFILES",
  "DATAOBJNO",
  "DATE",
  "DBA",
  "DBHIGH",
  "DBLOW",
  "DBMAC",
  "DEALLOCATE",
  "DEBUG",
  "DEC",
  "DECIMAL",
  "DECLARE",
  "DEFAULT",
  "DEFERRABLE",
  "DEFERRED",
  "DEGREE",
  "DELETE",
  "DEREF",
  "DESC",
  "DIRECTORY",
  "DISABLE",
  "DISCONNECT",
  "DISMOUNT",
  "DISTINCT",
  "DISTRIBUTED",
  "DML",
  "DOUBLE",
  "DROP",
  "DUMP",
  "EACH",
  "ELSE",
  "ENABLE",
  "END",
  "ENFORCE",
  "ENTRY",
  "ESCAPE",
  "EXCEPT",
  "EXCEPTIONS",
  "EXCHANGE",
  "EXCLUDING",
  "EXCLUSIVE",
  "EXECUTE",
  "EXISTS",
  "EXPIRE",
  "EXPLAIN",
  "EXTENT",
  "EXTENTS",
  "EXTERNALLY",
  "FAILED_LOGIN_ATTEMPTS",
  "FALSE",
  "FAST",
  "FILE",
  "FIRST_ROWS",
  "FLAGGER",
  "FLOAT",
  "FLOB",
  "FLUSH",
  "FOR",
  "FORCE",
  "FOREIGN",
  "FREELIST",
  "FREELISTS",
  "FROM",
  "FULL",
  "FUNCTION",
  "GLOBAL",
  "GLOBALLY",
  "GLOBAL_NAME",
  "GRANT",
  "GROUP",
  "GROUPS",
  "HASH",
  "HASHKEYS",
  "HAVING",
  "HEADER",
  "HEAP",
  "IDENTIFIED",
  "IDGENERATORS",
  "IDLE_TIME",
  "IF",
  "IMMEDIATE",
  "IN",
  "INCLUDING",
  "INCREMENT",
  "INDEX",
  "INDEXED",
  "INDEXES",
  "INDICATOR",
  "IND_PARTITION",
  "INITIAL",
  "INITIALLY",
  "INITRANS",
  "INSERT",
  "INSTANCE",
  "INSTANCES",
  "INSTEAD",
  "INT",
  "INTEGER",
  "INTERMEDIATE",
  "INTERSECT",
  "INTO",
  "IS",
  "ISOLATION",
  "ISOLATION_LEVEL",
  "KEEP",
  "KEY",
  "KILL",
  "LABEL",
  "LAYER",
  "LESS",
  "LEVEL",
  "LIBRARY",
  "LIKE",
  "LIMIT",
  "LINK",
  "LIST",
  "LOB",
  "LOCAL",
  "LOCK",
  "LOCKED",
  "LOG",
  "LOGFILE",
  "LOGGING",
  "LOGICAL_READS_PER_CALL",
  "LOGICAL_READS_PER_SESSION",
  "LONG",
  "MANAGE",
  "MASTER",
  "MAX",
  "MAXARCHLOGS",
  "MAXDATAFILES",
  "MAXEXTENTS",
  "MAXINSTANCES",
  "MAXLOGFILES",
  "MAXLOGHISTORY",
  "MAXLOGMEMBERS",
  "MAXSIZE",
  "MAXTRANS",
  "MAXVALUE",
  "MIN",
  "MEMBER",
  "MINIMUM",
  "MINEXTENTS",
  "MINUS",
  "MINVALUE",
  "MLSLABEL",
  "MLS_LABEL_FORMAT",
  "MODE",
  "MODIFY",
  "MOUNT",
  "MOVE",
  "MTS_DISPATCHERS",
  "MULTISET",
  "NATIONAL",
  "NCHAR",
  "NCHAR_CS",
  "NCLOB",
  "NEEDED",
  "NESTED",
  "NETWORK",
  "NEW",
  "NEXT",
  "NOARCHIVELOG",
  "NOAUDIT",
  "NOCACHE",
  "NOCOMPRESS",
  "NOCYCLE",
  "NOFORCE",
  "NOLOGGING",
  "NOMAXVALUE",
  "NOMINVALUE",
  "NONE",
  "NOORDER",
  "NOOVERRIDE",
  "NOPARALLEL",
  "NOPARALLEL",
  "NOREVERSE",
  "NORMAL",
  "NOSORT",
  "NOT",
  "NOTHING",
  "NOWAIT",
  "NULL",
  "NUMBER",
  "NUMERIC",
  "NVARCHAR2",
  "OBJECT",
  "OBJNO",
  "OBJNO_REUSE",
  "OF",
  "OFF",
  "OFFLINE",
  "OID",
  "OIDINDEX",
  "OLD",
  "ON",
  "ONLINE",
  "ONLY",
  "OPCODE",
  "OPEN",
  "OPTIMAL",
  "OPTIMIZER_GOAL",
  "OPTION",
  "OR",
  "ORDER",
  "ORGANIZATION",
  "OSLABEL",
  "OVERFLOW",
  "OWN",
  "PACKAGE",
  "PARALLEL",
  "PARTITION",
  "PASSWORD",
  "PASSWORD_GRACE_TIME",
  "PASSWORD_LIFE_TIME",
  "PASSWORD_LOCK_TIME",
  "PASSWORD_REUSE_MAX",
  "PASSWORD_REUSE_TIME",
  "PASSWORD_VERIFY_FUNCTION",
  "PCTFREE",
  "PCTINCREASE",
  "PCTTHRESHOLD",
  "PCTUSED",
  "PCTVERSION",
  "PERCENT",
  "PERMANENT",
  "PLAN",
  "PLSQL_DEBUG",
  "POST_TRANSACTION",
  "PRECISION",
  "PRESERVE",
  "PRIMARY",
  "PRIOR",
  "PRIVATE",
  "PRIVATE_SGA",
  "PRIVILEGE",
  "PRIVILEGES",
  "PROCEDURE",
  "PROFILE",
  "PUBLIC",
  "PURGE",
  "QUEUE",
  "QUOTA",
  "RANGE",
  "RAW",
  "RBA",
  "READ",
  "READUP",
  "REAL",
  "REBUILD",
  "RECOVER",
  "RECOVERABLE",
  "RECOVERY",
  "REF",
  "REFERENCES",
  "REFERENCING",
  "REFRESH",
  "RENAME",
  "REPLACE",
  "RESET",
  "RESETLOGS",
  "RESIZE",
  "RESOURCE",
  "RESTRICTED",
  "RETURN",
  "RETURNING",
  "REUSE",
  "REVERSE",
  "REVOKE",
  "ROLE",
  "ROLES",
  "ROLLBACK",
  "ROW",
  "ROWID",
  "ROWNUM",
  "ROWS",
  "RULE",
  "SAMPLE",
  "SAVEPOINT",
  "SB4",
  "SCAN_INSTANCES",
  "SCHEMA",
  "SCN",
  "SCOPE",
  "SD_ALL",
  "SD_INHIBIT",
  "SD_SHOW",
  "SEGMENT",
  "SEG_BLOCK",
  "SEG_FILE",
  "SELECT",
  "SEQUENCE",
  "SERIALIZABLE",
  "SESSION",
  "SESSION_CACHED_CURSORS",
  "SESSIONS_PER_USER",
  "SET",
  "SHARE",
  "SHARED",
  "SHARED_POOL",
  "SHRINK",
  "SIZE",
  "SKIP",
  "SKIP_UNUSABLE_INDEXES",
  "SMALLINT",
  "SNAPSHOT",
  "SOME",
  "SORT",
  "SPECIFICATION",
  "SPLIT",
  "SQL_TRACE",
  "STANDBY",
  "START",
  "STATEMENT_ID",
  "STATISTICS",
  "STOP",
  "STORAGE",
  "STORE",
  "STRUCTURE",
  "SUCCESSFUL",
  "SWITCH",
  "SYS_OP_ENFORCE_NOT_NULL$",
  "SYS_OP_NTCIMG$",
  "SYNONYM",
  "SYSDATE",
  "SYSDBA",
  "SYSOPER",
  "SYSTEM",
  "TABLE",
  "TABLES",
  "TABLESPACE",
  "TABLESPACE_NO",
  "TABNO",
  "TEMPORARY",
  "THAN",
  "THE",
  "THEN",
  "THREAD",
  "TIMESTAMP",
  "TIME",
  "TO",
  "TOPLEVEL",
  "TRACE",
  "TRACING",
  "TRANSACTION",
  "TRANSITIONAL",
  "TRIGGER",
  "TRIGGERS",
  "TRUE",
  "TRUNCATE",
  "TX",
  "TYPE",
  "UB2",
  "UBA",
  "UID",
  "UNARCHIVED",
  "UNDO",
  "UNION",
  "UNIQUE",
  "UNLIMITED",
  "UNLOCK",
  "UNRECOVERABLE",
  "UNTIL",
  "UNUSABLE",
  "UNUSED",
  "UPDATABLE",
  "UPDATE",
  "USAGE",
  "USE",
  "USER",
  "USING",
  "VALIDATE",
  "VALIDATION",
  "VALUE",
  "VALUES",
  "VARCHAR",
  "VARCHAR2",
  "VARYING",
  "VIEW",
  "WHEN",
  "WHENEVER",
  "WHERE",
  "WITH",
  "WITHOUT",
  "WORK",
  "WRITE",
  "WRITEDOWN",
  "WRITEUP",
  "XID",
  "YEAR",
  "ZONE"
];

/**
 * Definitions of functions and operators
 *   expr: implementation of the function/operator
 *   op: true if is an operator
 *   aggr: true if is an aggregated function
 * */
App.Oracle.Functions = {
  // Operators
  "like": {expr: "LOWER(@1) like LOWER(@2)", op: true},
  "not like": {expr: "LOWER(@1) not like LOWER(@2)", op: true},
  //
  // Date functions
  now: {expr: "SYSDATE"},
  today: {expr: "TRUNC(SYSDATE)"},
  time: {expr: "TO_DATE('1900-01-01 ' || TO_CHAR(SYSDATE, 'HH24:MI:SS'), 'YYYY-MM-DD HH24:MI:SS')"},
  toDate: {expr: "TO_DATE((@3)||'/'||(@2)||'/'||(@1),'dd/mm/yyyy')"},
  toTime: {expr: "TO_DATE('1899-12-30'||' '||(@1)||':'||(@2)||':'||(@3),'YYYY-MM-DD HH24:mi:ss')"},
  year: {expr: "TO_NUMBER(TO_CHAR(@1, 'yyyy'))"},
  month: {expr: "TO_NUMBER(TO_CHAR(@1, 'mm' ))"},
  day: {expr: "TO_NUMBER(TO_CHAR(@1, 'dd'))"},
  hour: {expr: "TO_NUMBER(TO_CHAR(@1, 'HH24'))"},
  minute: {expr: "TO_NUMBER(TO_CHAR(@1, 'mi'))"},
  second: {expr: "TO_NUMBER(TO_CHAR(@1, 'ss'))"},
  weekDay: {expr: "TO_NUMBER(TO_CHAR(@1, 'd'))"},
  yearDay: {expr: "TO_NUMBER(TO_CHAR(@1, 'ddd'))"},
  week: {expr: "TO_NUMBER( TO_CHAR ( @1, 'IW' ) )"},
  getDate: {expr: "TO_DATE(TO_NUMBER(TO_CHAR(@1, 'dd'))||'/'||TO_NUMBER(TO_CHAR(@1, 'mm' ))||'/'||TO_NUMBER(TO_CHAR(@1, 'yyyy')),'dd/mm/yyyy')"},
  //
  // Math functions
  int: {expr: "TRUNC(@1)"},
  ceil: {expr: "CEIL(@1)"},
  floor: {expr: "FLOOR(@1)"},
  round: {expr: "ROUND(@1, @2)"},
  trunc: {expr: "TRUNC(@1, @2)"},
  sqrt: {expr: "SQRT(@1)"},
  log: {expr: "LN(@1)"},
  log10: {expr: "LOG(10, @1)"},
  cot: {expr: "(1/TAN(@1))"},
  //
  // String functions
  left: {expr: "SUBSTR(@1, 1, @2)"},
  right: {expr: "SUBSTR(@1, -(@2))"},
  substr: {expr: "SUBSTR(@1, @2, @3)"},
  spaces: {expr: "LPAD(' ', @1)"},
  replicate: {expr: "SUBSTR(RPAD('.', (@2)+1, SUBSTR(@1, 1, 1)), 2)"},
  fillLeft: {expr: "LPAD(@1, @2, SUBSTR(@3, 1, 1))"},
  fillRight: {expr: "RPAD(@1, @2, SUBSTR(@3, 1, 1))"},
  //
  // Various functions
  nullValue: {expr: "NVL(@1, @2)"},
  blobSize: {expr: "dbms_lob.getlength(@1)"}
};


/*
 * Return the phisical name of database
 * @return {String}
 */
App.Oracle.prototype.getDBName = function ()
{
  return this.schema.name;
};


/*
 * Sets connection options for this database
 * @param {Object} opt - connect options
 */
App.Oracle.prototype.setOptions = function (opt)
{
  App.Database.prototype.setOptions.call(this, opt);
  this.defaultSchema = opt.user;
};


/**
 * Open the connection to the database
 * @param {Function} callback - function to be called at the end
 */
App.Oracle.prototype._openConnection = function (callback)
{
  if (!module)
    return callback(null, new Error("Oracle is supported only inside Node"));
  //
  // Import module (could be missing)
  try {
    if (!App.oracledb) {
      App.oracledb = require("oracledb");
      App.oracledb.fetchAsBuffer = [App.oracledb.BLOB];
      App.oracledb.fetchAsString = [App.oracledb.CLOB];
      App.oracledb.poolTimeout = 30;
    }
  }
  catch (ex) {
    return callback(null, new Error("Oracle driver not found.\nInstall \"oracledb\" module and try again"));
  }
  //
  this.getPool(function (pool, error) {
    if (error)
      return callback(null, error);
    //
    pool.getConnection(function (err, connection) {
      callback(connection, err);
    });
  });
};


/**
 * Init the application pool
 * @param {Function} callback - function to be called at the end
 */
App.Oracle.prototype.getPool = function (callback) {
  if (!App.Oracle.pools)
    App.Oracle.pools = {};
  //
  var poolName = JSON.stringify(this.options);
  if (App.Oracle.pools[poolName])
    return callback(App.Oracle.pools[poolName]);
  //
  App.oracledb.createPool(this.options, function (error, pool) {
    if (error)
      return callback(null, error);
    //
    App.Oracle.pools[poolName] = pool;
    callback(pool);
  });
};


/**
 * Close the connection to the database
 * @param {Function} callback - function to be called at the end
 */
App.Oracle.prototype._closeConnection = function (callback)
{
  var conn = this.connection;
  delete this.connection;
  conn.release(function (error) {
    callback(null, error);
  });
};


/**
 * Save the schema definition in z_schema table
 * @param {Array} sqls - list of sql statement where to add
 * @param {Object} news - new schema
 */
App.Oracle.prototype.saveZSchema = function (sqls, news)
{
  if (!this.zSchemaExists)
    sqls.push({sql: "create table z_schema (\"schema\" varchar2(4000))"});
  sqls.push({sql: "delete from z_schema"});
  sqls.push({sql: "insert into z_schema values (?)", params: [JSON.stringify(news)]});
};


/**
 * Execute an SQL query
 * @param {String} sql to execute
 * @param {Boolean} readOnly - true if is a SELECT
 * @param {Array} parameters
 * @param {Function} callback - function to be called at the end
 * @return {Object/Integer}
 */
App.Oracle.prototype._executeSQL = function (sql, readOnly, parameters, callback)
{
  // Set output parameter for read value of counter field
  var bindParams = {};
  if (!readOnly && sql.endsWith("into :counter"))
    bindParams.counter = {type: App.oracledb.NUMBER, dir: App.oracledb.BIND_OUT};
  //
  // Add input parameters
  for (var i = 0; i < parameters.length; i++)
    bindParams["P" + (i + 1)] = parameters[i];
  //
  // Run the query
  var opts = {outFormat: App.oracledb.OBJECT, autoCommit: !this.transaction};
  this.connection.execute(sql, bindParams, opts, function (error, result) {
    callback(result, error);
  });
};


/**
 * Execute an SQL query with offset paging
 * @param {Object} params
 * @param {Function} callback - function to be called at the end
 * @return {DataMap/Object}
 */
App.Oracle.prototype.queryOffsetPaging = function (params, callback)
{
  var sql = "select * from (select rownum rnum, a.* from (";
  sql += params.sql;
  sql += ") a where rownum <= " + (params.offset + params.limit) + ")";
  sql += " where rnum > " + params.offset;
  this.executeSQL(sql, true, params.parameters, callback);
};


/*
 * Read some info from result
 * @param {String} sql
 * @param {Object} result
 */
App.Oracle.prototype._readInfoFromResult = function (sql, result)
{
  if (result) {
    this.lastRowsAffected = result.rowsAffected;
    this.lastInsertId = (result.outBinds ? result.outBinds.counter : null);
  }
};


/**
 * Recreate the primary key of table
 * @param {Array} sqls list of sql statements
 * @param {Object} t table to recreate
 * @param {Object} ot old version
 */
App.Oracle.prototype.recreatePK = function (sqls, t, ot)
{
  this.dropExternalTableFKS(sqls, ot);
  //
  var sql = "DECLARE pkname VARCHAR2(100);\n";
  sql += "BEGIN\n";
  sql += " SELECT constraint_name INTO pkname FROM all_constraints";
  sql += " WHERE lower(table_name) = '" + ot.name.toLowerCase() + "'";
  sql += " and owner = '" + (ot.schema || this.defaultSchema) + "'";
  sql += " and constraint_type = 'P';\n";
  sql += " IF pkname is not null THEN";
  sql += "   EXECUTE IMMEDIATE 'ALTER TABLE " + this.quote(ot.name) + " DROP CONSTRAINT ' || pkname);\n";
  sql += " END IF;\n";
  var pks = "";
  for (var i = 0; i < t.fields.length; i++) {
    if (t.fields[i].pk)
      pks += (pks ? "," : "") + this.quote(t.fields[i].name);
  }
  if (pks)
    sql += " ALTER TABLE " + this.quote(ot.name) + " ADD PRIMARY KEY (" + pks + ");\n";
  sql += "END;";
  sqls.push({sql: sql});
};


/**
 * Create a table
 * @param {Array} sqls list of sql statements
 * @param {Object} t table to create
 */
App.Oracle.prototype.createTable = function (sqls, t)
{
  // Getting create table stmt
  var s = "create table " + this.quote(t.name) + " (";
  //
  // Getting fields definition
  for (var i = 0; i < t.fields.length; i++)
    s += (i === 0 ? "" : ",") + "\n  " + this.getFieldDef(t.fields[i]);
  s += "\n)";
  //
  sqls.push({sql: s});
  //
  // Now create the PK
  s = "";
  for (var i = 0; i < t.fields.length; i++) {
    if (!t.fields[i].pk)
      continue;
    //
    // If it's the first one, prepare the stmt
    if (s === "")
      s += " alter table " + this.quote(t.name) + " add constraint " + this.quote(t.name + "_PK") + " primary key (";
    //
    s += t.fields[i].name + ",";
  }
  //
  // If there is a PK
  if (s) {
    // Remove the last , and add the close bracket
    s = s.substring(0, s.length - 1) + ")";
    //
    sqls.push({sql: s});
  }
  //
  // Handle counter fields
  for (var i = 0; i < t.fields.length; i++) {
    if (t.fields[i].datatype === App.Database.DataTypes.AUTOINCR)
      sqls.push({sql: "create sequence " + this.quote(t.name + "_" + t.fields[i].name) + " start with 1 nocache"});
  }
  //
  // Return the table with childs
  var nt = {};
  nt.name = t.name;
  nt.fields = t.fields;
  return nt;
};


/**
 * Rename a table
 * @param {Array} sqls list of sql statements
 * @param {Object} t table to rename
 * @param {Object} ot old version
 */
App.Oracle.prototype.renameTable = function (sqls, t, ot)
{
  sqls.push({sql: "rename " + this.quote(ot.name) + " to " + this.quote(t.name)});
  this.modifyTableFKs(sqls, t, ot);
  return true;
};


/**
 * Modify a field
 * @param {Array} sqls list of sql statements
 * @param {Object} t table to modify
 * @param {Object} f field to modify
 * @param {Object} of old version
 */
App.Oracle.prototype.modifyField = function (sqls, t, f, of)
{
  if (f.name.toLowerCase() !== of.name.toLowerCase()) {
    sqls.push({sql: "alter table " + this.quote(t.name) + " rename column " + this.quote(of.name) + " to " + this.quote(f.name)});
    of.name = f.name;
  }
  if (this.getFieldMaxLength(f) !== this.getFieldMaxLength(of) || this.getFieldScale(f) !== this.getFieldScale(of))
    sqls.push({sql: "alter table " + this.quote(t.name) + " modify " + this.quote(f.name) + " " + this.getDataType(f)});
  if (this.getFieldDefVal(f) !== this.getFieldDefVal(of))
    sqls.push({sql: "alter table " + this.quote(t.name) + " modify " + this.quote(f.name) + " default " + (this.getFieldDefVal(f) || "null")});
  //
  // If field became not null without default update table with dummy value
  if (f.notn && !of.notn)
    this.updateRequiredField(sqls, t, f);
  //
  if (f.notn !== of.notn)
    sqls.push({sql: "alter table " + this.quote(t.name) + " modify " + this.quote(f.name) + (f.notn ? " not" : "") + " null"});
  return true;
};


/**
 * Modify a foreign key
 * @param {Array} sqls list of sql statements
 * @param {Object} t table to modify
 * @param {Object} ot table old version
 * @param {Object} fk foreign key to modify
 * @param {Object} ofk old version
 */
App.Oracle.prototype.modifyFK = function (sqls, t, ot, fk, ofk)
{
  var ok = true;
  if (Object.keys(fk.refs).length !== Object.keys(ofk.refs).length)
    ok = false;
  else {
    for (var ref in fk.refs) {
      if (!ofk.refs[ref])
        ok = false;
    }
  }
  //
  if (!ok) {
    this.dropFK(sqls, t, ofk);
    this.createFK(sqls, t, fk);
  }
  else if (fk.name.toLowerCase() !== ofk.name.toLowerCase() || t.name.toLowerCase() !== ot.name.toLowerCase())
    sqls.push({sql: "alter table " + this.quote(t.name) + " rename constraint " + this.quote(this.getFKName(ot, ofk)) + " to " + this.quote(this.getFKName(t, fk))});
  //
  return true;
};


/**
 * Modify an index
 * @param {Array} sqls list of sql statements
 * @param {Object} t table owner of index
 * @param {Object} idx index to modify
 * @param {Object} oidx old version
 */
App.Oracle.prototype.modifyIndex = function (sqls, t, idx, oidx)
{
  var ok = true;
  if (idx.un !== oidx.un)
    ok = false;
  else if (idx.fields.length !== oidx.fields.length)
    ok = false;
  else {
    for (var i = 0; i < idx.fields.length; i++) {
      if (idx.fields[i] !== oidx.fields[i])
        ok = false;
    }
  }
  //
  if (!ok) {
    this.dropIndex(sqls, t, oidx);
    this.createIndex(sqls, t, idx);
  }
  else if (idx.name.toLowerCase() !== oidx.name.toLowerCase())
    sqls.push({sql: "alter index " + this.quote(oidx.name) + " rename to " + this.quote(idx.name)});
  return true;
};


/**
 * Get the datatype for the f field
 * @param {Object} f
 */
App.Oracle.prototype.getDataType = function (f)
{
  var maxlen = this.getFieldMaxLength(f);
  var scale = this.getFieldScale(f);
  switch (f.datatype) {
    case App.Database.DataTypes.INT:
    case App.Database.DataTypes.AUTOINCR:
    case App.Database.DataTypes.FLOAT:
    case App.Database.DataTypes.DECIMAL:
      return "number(" + maxlen + "," + scale + ")";

    case App.Database.DataTypes.BOOL:
      return "number(1)";

    case App.Database.DataTypes.JSON:
    case App.Database.DataTypes.VARCHAR:
      if (maxlen < 4000)
        return "nvarchar2" + "(" + (maxlen || "4000") + ")";
      else
        return "clob";

    case App.Database.DataTypes.CHAR:
    case App.Database.DataTypes.UUID:
      return "char(" + maxlen + ")";

    default:
      return this.typemap[f.datatype][0];
  }
};


/**
 * Get the action for the fk foreign key
 * @param {Object} fk
 */
App.Oracle.prototype.getFKActions = function (fk)
{
  var s = "";
  for (var i = 0; i < 2; i++) {
    var r = (i === 0 ? fk.ur : fk.dr);
    var op = (i === 0 ? " on update " : " on delete ");
    if (r === App.Database.FKActions.CASCADE)
      s += op + "cascade";
    else if (r === App.Database.FKActions.SETDEFAULT)
      s += op + "set default";
    else if (r === App.Database.FKActions.NOACTION)
      s += op + "no action";
    else if (r === App.Database.FKActions.SETNULL)
      s += op + "set null";
  }
  return s;
};


/**
 * Compose insert statement of a query
 * @param {Object} query to compose
 */
App.Oracle.prototype.createInsertStatement = function (query)
{
  var sql = "";
  //
  // Add set with nextval of sequence for autoincrement field
  if (query.counter)
    query.select.push({field: query.counter, expr: this.quote(query.table + "_" + this.getObjDMCode(query.table, query.counter)) + ".nextval"});
  //
  sql = App.Database.prototype.createInsertStatement.call(this, query);
  //
  // Read sequence value
  if (query.counter)
    sql += " returning " + this.quote(this.getObjDMCode(query.table, query.counter)) + " into :counter";
  //
  return sql;
};


/**
 * Compose sql top clause
 * @param {Object} token top
 * @param {Boolean} afterSelect
 */
App.Oracle.prototype.topClause = function (token, afterSelect)
{
  if (afterSelect)
    return "";
  else
    return "rownum <= " + this.createSqlExpression(token);
};


/**
 * Returns the i-th row of a resultSet
 * @param {Object} rs - recordset
 * @param {int} pos - position
 * @param {DataSchema} schema
 */
App.Oracle.prototype.getRow = function (rs, pos, schema)
{
  var row = (this.remoteConnector ? App.RemoteDriver.getRow(rs, pos) : rs.rows[pos]);
  if (!schema)
    return row;
  //
  var newrow = {};
  for (var c in schema.columns) {
    var cupper = c.toUpperCase();
    var v = (row.hasOwnProperty(cupper) ? row[cupper] : row[c]);
    if (v === undefined)
      continue;
    newrow[c] = this.convertValue(v, schema.columns[c].datatype);
  }
  return newrow;
};


/**
 * Returns the field default value
 * @param {Object} f
 */
App.Oracle.prototype.getFieldDefVal = function (f)
{
  if (f.defval) {
    if (f.datatype === App.Database.DataTypes.BOOL)
      return (f.defval === "true" ? "1" : "0");
  }
  return App.Database.prototype.getFieldDefVal.call(this, f);
};


/**
 * Convert a native value into javascript value
 * @param {String} v
 * @param {String} dt
 */
App.Oracle.prototype.convertValue = function (v, dt)
{
  if (v === null || v === undefined)
    return null;
  //
  if (dt === App.Database.DataTypes.BOOL)
    return (v !== 0);
  //
  if (dt === App.Database.DataTypes.JSON)
    return JSON.parse(v);
  //
  return App.Database.prototype.convertValue.call(this, v, dt);
};


/*
 * Begin a transaction
 * @param {Function} callback - function to be called at the end
 */
App.Oracle.prototype._beginTransaction = function (callback)
{
  callback();
};


/*
 * Commit a transaction
 * @param {Function} callback - function to be called at the end
 */
App.Oracle.prototype._commitTransaction = function (callback)
{
  this.connection.commit(function (error) {
    callback(null, error);
  });
};


/*
 * Rollback a transaction
 * @param {Function} callback - function to be called at the end
 */
App.Oracle.prototype._rollbackTransaction = function (callback)
{
  this.connection.rollback(function (error) {
    callback(null, error);
  });
};


/**
 * Read list of tables
 * @param {Object} options
 * @param {Function} callback - function to be called at the end
 */
App.Oracle.prototype.listTables = function (options, callback)
{
  var sql = "select table_name";
  sql += " from all_tables";
  sql += " where owner = '" + options.schema + "'";
  sql += " order by table_name";
  this.executeSQL(sql, true, function (result, error) {
    if (error)
      return callback(null, error);
    //
    var tables = [];
    for (var i = 0; i < this.getRowCount(result); i++)
      tables.push(this.getRow(result, i).TABLE_NAME);
    callback(tables);
  }.bind(this));
};


/**
 * Read list of views
 * @param {Object} options
 * @param {Function} callback - function to be called at the end
 */
App.Oracle.prototype.listViews = function (options, callback)
{
  var sql = "select view_name";
  sql += " from all_views";
  sql += " where owner = '" + options.schema + "'";
  sql += " order by view_name";
  this.executeSQL(sql, true, function (result, error) {
    if (error)
      return callback(null, error);
    //
    var views = [];
    for (var i = 0; i < this.getRowCount(result); i++)
      views.push(this.getRow(result, i).VIEW_NAME);
    callback(views);
  }.bind(this));
};


/**
 * Import fiels of a table
 * @param {Object} options
 * @param {Function} callback - function to be called at the end
 */
App.Oracle.prototype.importTableFields = function (options, callback)
{
  var fields = [];
  //
  var sql = "select t.column_name, t.data_type,";
  sql += " data_length, data_precision, data_scale,";
  sql += " nullable, (select count(*)";
  sql += "  from all_constraints uc";
  sql += "  left join all_cons_columns cc on (uc.table_name = cc.table_name and uc.owner = cc.owner and uc.constraint_name = cc.constraint_name)";
  sql += "  where uc.table_name = t.table_name and uc.owner = t.owner and cc.column_name = t.column_name and uc.constraint_type = 'P') as pk";
  sql += " from all_tab_columns t";
  sql += " where t.table_name = '" + options.table + "' and t.owner = '" + options.schema + "'";
  sql += " order by t.column_id";
  this.executeSQL(sql, true, function (result, error) {
    if (error)
      return callback(null, error);
    //
    for (var i = 0; i < this.getRowCount(result); i++) {
      var row = this.getRow(result, i);
      var field = {};
      field.name = row.COLUMN_NAME;
      if (row.PK > 0)
        field.pk = true;
      if (row.NULLABLE === "N")
        field.notNull = true;
      field.dataType = this.convertDataType(row.DATA_TYPE);
      if (row.DATA_PRECISION) {
        field.maxLen = row.DATA_PRECISION;
        if (row.DATA_SCALE !== null)
          field.scale = row.DATA_SCALE;
      }
      else if (row.DATA_LENGTH)
        field.maxLen = row.DATA_LENGTH;
      fields.push(field);
    }
    //
    callback(fields);
  }.bind(this));
};


/**
 * Import fiels of a view
 * @param {Object} options
 * @param {Function} callback - function to be called at the end
 */
App.Oracle.prototype.importViewFields = function (options, callback)
{
  options.table = options.view;
  delete options.view;
  App.Oracle.prototype.importTableFields.call(this, options, callback);
};


/**
 * Import fks of a table
 * @param {Object} options
 * @param {Function} callback - function to be called at the end
 */
App.Oracle.prototype.importTableFKs = function (options, callback)
{
  var fks = [];
  //
  // Import fks
  var sql = "select";
  sql += " rc.constraint_name,";
  sql += " rc.delete_rule,";
  sql += " kcu1.column_name as fk_column_name,";
  sql += " kcu2.table_name as ref_table_name,";
  sql += " kcu2.column_name as ref_column_name ";
  sql += "from all_constraints rc";
  sql += " inner join all_constraints cc";
  sql += "   on (rc.r_constraint_name = cc.constraint_name and rc.owner = cc.owner)";
  sql += " inner join all_cons_columns kcu1 ";
  sql += "   on (kcu1.constraint_name = rc.constraint_name and kcu1.owner = rc.owner)";
  sql += " inner join all_cons_columns kcu2 ";
  sql += "   on (kcu2.constraint_name = rc.r_constraint_name and kcu2.owner = rc.owner";
  sql += "    and kcu2.position = kcu1.position) ";
  sql += "where rc.table_name = '" + options.table + "' and rc.owner = '" + options.schema + "' ";
  sql += "  and rc.constraint_type = 'R' ";
  sql += "order by rc.constraint_name, kcu1.position";
  this.executeSQL(sql, true, function (result, error) {
    if (error)
      return callback(null, error);
    //
    var fk = {};
    for (var i = 0; i < this.getRowCount(result); i++) {
      var row = this.getRow(result, i);
      if (row.CONSTRAINT_NAME !== fk.name) {
        fk = {};
        fk.name = row.CONSTRAINT_NAME;
        fk.derived = row.REF_TABLE_NAME;
        fk.refs = {};
        fk.updateRule = App.Database.FKActions.NOACTION;
        switch (row.delete_rule) {
          case "NO ACTION":
            fk.deleteRule = App.Database.FKActions.NOACTION;
            break;
          case "CASCADE":
            fk.deleteRule = App.Database.FKActions.CASCADE;
            break;
          case "SET NULL":
            fk.deleteRule = App.Database.FKActions.SETNULL;
            break;
        }
        fks.push(fk);
      }
      //
      fk.refs[row.FK_COLUMN_NAME] = row.REF_COLUMN_NAME;
    }
    //
    callback(fks);
  }.bind(this));
};


/**
 * Check if a name require case sensitivity
 * @param {String} name
 */
App.Oracle.prototype.requireCaseSensitivity = function (name)
{
  return (name.toUpperCase() !== name);
};


/*
 * Define a query parameter
 * @param {Object} token
 */
App.Oracle.prototype.defineQueryParameter = function (token)
{
  this.parameters.push(this.forSql(token.value, token.dt));
  return ":P" + this.parameters.length;
};


/*
 * Verify if z_sync table exists
 * @param {String} handler - handler of message
 * @param {Function} callback - function to be called at the end
 */
App.Oracle.prototype.syncExistsTable = function (handler, callback)
{
  var sql = "select table_name";
  sql += " from all_tables";
  sql += " where owner = '" + this.defaultSchema + "'";
  sql += " and table_name = 'z_sync" + handler + "'";
  this.executeSQL(sql, true, function (result) {
    callback(result && this.getRowCount(result) === 1);
  }.bind(this));
};


/*
 * Create z_sync table
 * @param {String} handler - handler of message
 * @param {Function} callback - function to be called at the end
 */
App.Oracle.prototype.syncCreateTable = function (handler, callback)
{
  var sql = "create table z_sync" + handler + "(id numeric not null, \"uid\" char(24), \
device varchar2(4000), sender varchar2(4000), topics varchar2(4000), body nclob, ts date default SYSDATE, expire date, \
constraint z_sync_PK primary key (id))";
  this.executeSQL(sql, false, function (result, error) {
    if (error)
      return callback(null, error);
    //
    this.executeSQL("create sequence z_sync" + handler + "_id start with 1 nocache", false, callback);
  }.bind(this));
};


//export module for node
if (module)
  module.exports = App.Oracle;
