/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */
/* global module, process */

var App = App || {};

if (module)
  App.Database = require("./database");

/**
 * @class A Postgres Database Object
 * @param {Session} app that owns this instance
 * @extends App.Database
 * */
App.Postgres = function (app)
{
  if (module && !App.View)
    App = require("./requireAll");
  //
  App.Database.call(this, app);
  //
  this.typemap = App.Postgres.DataTypeMap; // Map of data types
  this.keywords = App.Postgres.Keywords;   // List of reserved keywords
  this.defaultSchema = "public";
  //
  // Definition of functions and operators
  this.functions = Object.assign({}, App.Database.Functions, App.Postgres.Functions);
};


// Make App.Postgres extend App.Database
App.Postgres.prototype = new App.Database(true);


App.Postgres.DataTypeMap = {
  i: ["integer", "bit", "bigint", "smallint"],
  f: ["real", "double precision"],
  d: ["numeric"],
  vc: ["varchar", "character varying"],
  c: ["char", "character", "\"char\""],
  bl: ["bytea"],
  cl: ["text"],
  b: ["boolean"],
  da: ["date"],
  ti: ["time without time zone", "time with time zone"],
  dt: ["timestamp with time zone", "timestamp without time zone"],
  id: ["uuid"],
  ai: ["serial", "bigserial", "smallserial"],
  j: ["json"]
};

App.Postgres.msgTypeMap = {
  createDB: "cdb"
};

App.Postgres.Keywords = [
  "ABORT",
  "ABSOLUTE",
  "ACCESS",
  "ACTION",
  "ADD",
  "ADMIN",
  "AFTER",
  "AGGREGATE",
  "ALL",
  "ALSO",
  "ALTER",
  "ALWAYS",
  "ANALYSE",
  "ANALIZE",
  "AND",
  "ANY",
  "ARRAY",
  "AS",
  "ASC",
  "ASSERTION",
  "ASSIGNMENT",
  "ASYMMETRIC",
  "AT",
  "ATTRIBUTE",
  "AUTHORIZATION",
  "BACKWARD",
  "BEFORE",
  "BEGIN",
  "BETWEEN",
  "BIGINT",
  "BINARY",
  "BIT",
  "BOOLEAN",
  "BOTH",
  "BY",
  "CACHE",
  "CALLED",
  "CASCADE",
  "CASCADED",
  "CASE",
  "CAST",
  "CATALOG",
  "CHAIN",
  "CHAR",
  "CHARACTER",
  "CHARACTERISTICS",
  "CHECK",
  "CHECKPOINT",
  "CLASS",
  "CLOSE",
  "CLUSTER",
  "COALESCE",
  "COLLATE",
  "COLLATION",
  "COLUMN",
  "COMMENT",
  "COMMENTS",
  "COMMIT",
  "COMMITTED",
  "CONCURRENTLY",
  "CONFIGURATION",
  "CONNECTION",
  "CONSTRAINT",
  "CONTRAINTS",
  "CONTENT",
  "CONTINUE",
  "CONVERSION",
  "COPY",
  "COST",
  "CREATE",
  "CROSS",
  "CSV",
  "CURRENT",
  "CURRENT_CATALOG",
  "CURRENT_DATE",
  "CURRENT_ROLE",
  "CURRENT_SCHEMA",
  "CURRENT_TIME",
  "CURRENT_TIMESTAMP",
  "CURRENT_USER",
  "CURSOR",
  "CYCLE",
  "DATA",
  "DATABASE",
  "DAY",
  "DEALLOCALTE",
  "DEC",
  "DECIMAL",
  "DECLARE",
  "DEFAULT",
  "DEFAULTS",
  "DEFERRABLE",
  "DEFERRED",
  "DEFINER",
  "DELETE",
  "DELIMITER",
  "DELIMITERS",
  "DESC",
  "DICTIONARY",
  "DISABLE",
  "DISCARD",
  "DISTINCT",
  "DO",
  "DOCUMENT",
  "DOMAIN",
  "DOUBLE",
  "DROP",
  "EACH",
  "ELSE",
  "ENABLE",
  "ENCODING",
  "END",
  "ENUM",
  "ESCAPE",
  "EVENT",
  "EXCEPT",
  "EXCLUDE",
  "EXCLUDING",
  "EXCLUSIVE",
  "EXECUTE",
  "EXISTS",
  "EXPLAIN",
  "EXTENSION",
  "EXTERNAL",
  "EXTRACT",
  "FALSE",
  "FAMILY",
  "FETCH",
  "FILTER",
  "FIRST",
  "FLOAT",
  "FOLLOWING",
  "FOR",
  "FORCE",
  "FOREIGN",
  "FORWARD",
  "FREEZE",
  "FROM",
  "FULL",
  "FUNCTION",
  "FUNCTIONS",
  "GLOBAL",
  "GRANT",
  "GRANTED",
  "GREATEST",
  "GROUP",
  "HANDLER",
  "HAVING",
  "HEADER",
  "HOLD",
  "HOUR",
  "IDENTITY",
  "IF",
  "ILIKE",
  "IMMEDIATE",
  "IMMUTABLE",
  "IMPLICIT",
  "IN",
  "INCLUDING",
  "INCREMENT",
  "INDEX",
  "INDEXES",
  "INHERIT",
  "INHERITS",
  "INITIALLY",
  "INLINE",
  "INNER",
  "INOUT",
  "INPUT",
  "INSENSITIVE",
  "INSERT",
  "INSTEAD",
  "INT",
  "INTEGER",
  "INTERSECT",
  "INTERVAL",
  "INTO",
  "INVOKER",
  "IS",
  "ISNULL",
  "ISOLATION",
  "JOIN",
  "KEY",
  "LABEL",
  "LANGUAGE",
  "LARGE",
  "LAST",
  "LATERAL",
  "LC_COLLATE",
  "LC_CTYPE",
  "LEADING",
  "LEAKPROOF",
  "LEAST",
  "LEFT",
  "LEVEL",
  "LIKE",
  "LIMIT",
  "LISTEN",
  "LOAD",
  "LOCAL",
  "LOCALTIME",
  "LOCALTIMESTAMP",
  "LOCATION",
  "LOCK",
  "MAPPING",
  "MATCH",
  "MATERIALIZED",
  "MAXVALUE",
  "MINUTE",
  "MINVALUE",
  "MODE",
  "MONTH",
  "MOVE",
  "NAME",
  "NAMES",
  "NATIONAL",
  "NATURAL",
  "NCHAR",
  "NEXT",
  "NO",
  "NONE",
  "NOT",
  "NOTHING",
  "NOTIFY",
  "NOTNULL",
  "NOWAIT",
  "NULL",
  "NULLIF",
  "NULLS",
  "NUMERIC",
  "OBJECTS",
  "OF",
  "OFF",
  "OFFSET",
  "OIDS",
  "ON",
  "ONLY",
  "OPERATOR",
  "OPTION",
  "OPTIONS",
  "OR",
  "ORDER",
  "ORDINALITY",
  "OUT",
  "OUTER",
  "OVER",
  "OVERLAPS",
  "OVERLAY",
  "OWNED",
  "OWNER",
  "PARSER",
  "PARTIAL",
  "PARTITION",
  "PASSING",
  "PASSWORD",
  "PLACING",
  "PLANS",
  "POSITION",
  "PRECEDING",
  "PRECISION",
  "PREPARE",
  "PREPARED",
  "PRESERVE",
  "PRIMARY",
  "PRIOR",
  "PRIVILEGES",
  "PROCEDURAL",
  "PROCEDURE",
  "PROGRAM",
  "QUOTE",
  "RANGE",
  "READ",
  "REAL",
  "REASSIGN",
  "RECHECK",
  "RECURSIVE",
  "REF",
  "REFERENCES",
  "REFRESH",
  "REINDEX",
  "RELATIVE",
  "RELEASE",
  "RENAME",
  "REPEATABLE",
  "REPLACE",
  "REPLICA",
  "RESET",
  "RESTART",
  "RESTRICT",
  "RETURNING",
  "RETURNS",
  "REVOKE",
  "RIGHT",
  "ROLE",
  "ROLLBACK",
  "ROW",
  "ROWS",
  "RULE",
  "SAVEPOINT",
  "SCHEMA",
  "SCROLL",
  "SEARCH",
  "SECOND",
  "SECURITY",
  "SELECT",
  "SEQUENCE",
  "SEQUENCES",
  "SERIALIZABLE",
  "SERVER",
  "SESSION",
  "SESSION_USER",
  "SET",
  "SETOF",
  "SHARE",
  "SHOW",
  "SIMILAR",
  "SIMPLE",
  "SMALLINT",
  "SNAPSHOT",
  "SOME",
  "STABLE",
  "STANDALONE",
  "START",
  "STATEMENT",
  "SATISTICS",
  "STDIN",
  "STDOUT",
  "STORAGE",
  "STRICT",
  "STRIP",
  "SUBSTRING",
  "SYMMETRIC",
  "SYSID",
  "SYSTEM",
  "TABLE",
  "TABLES",
  "TABLESPACES",
  "TEMP",
  "TEMPLATE",
  "TEMPORARY",
  "TEXT",
  "THEN",
  "TIME",
  "TIMESTAMP",
  "TO",
  "TRAILING",
  "TRANSACTION",
  "TREAT",
  "TRIGGER",
  "TRIM",
  "TRUE",
  "TRUNCATE",
  "TRUSTED",
  "TYPE",
  "TYPES",
  "UNBOUNDED",
  "UNCOMMITTED",
  "UNENCRYPTED",
  "UNION",
  "UNIQUE",
  "UNKNOWN",
  "UNLISTEN",
  "UNLOGGED",
  "UNTIL",
  "UPDATE",
  "USER",
  "USING",
  "VACUUM",
  "VALID",
  "VALIDATE",
  "VALIDATOR",
  "VALUE",
  "VARCHAR",
  "VARIADIC",
  "VARYING",
  "VERBOSE",
  "VERSION",
  "VIEW",
  "VIEWS",
  "VOLATILE",
  "WHEN",
  "WHERE",
  "WHITESPACE",
  "WINDOW",
  "WITH",
  "WITHIN",
  "WITHOUT",
  "WORK",
  "WRAPPER",
  "WRITE",
  "XML",
  "XMLATTRIBUTES",
  "XMLCONCAT",
  "XMLELEMENT",
  "XMLEXISTS",
  "XMLFOREST",
  "XMLPARSE",
  "XMLPI",
  "XMLROOT",
  "XMLSERIALIZE",
  "YEAR",
  "YES",
  "ZONE"
];

/**
 * Definitions of functions and operators
 *   expr: implementation of the function/operator
 *   op: true if is an operator
 *   aggr: true if is an aggregated function
 * */
App.Postgres.Functions = {
  // Operators
  "%": {expr: "mod(@1, @2)", op: true},
  "like": {expr: "@1 ilike @2", op: true},
  //
  // Date functions
  now: {expr: "now()"},
  today: {expr: "current_date"},
  time: {expr: "current_time"},
  toDate: {expr: "DATE((@1)||'-'||(@2)||'-'||(@3))"},
  toTime: {expr: "to_timestamp('18991230@1:@2:@3', 'YYYYMMDDHH24:MI:SS')"},
  year: {expr: "EXTRACT(YEAR FROM (@1))"},
  month: {expr: "EXTRACT(MONTH FROM (@1))"},
  day: {expr: "EXTRACT(DAY FROM (@1))"},
  hour: {expr: "EXTRACT(HOUR FROM (@1))"},
  minute: {expr: "EXTRACT(MINUTE FROM (@1))"},
  second: {expr: "TRUNC(EXTRACT(SECOND FROM (@1)))"},
  weekDay: {expr: "EXTRACT(DOW FROM (@1))"},
  yearDay: {expr: "EXTRACT(DOY FROM (@1))"},
  week: {expr: "EXTRACT(DOY FROM (@1))"},
  getDate: {expr: "DATE(EXTRACT(YEAR FROM (@1))||'-'||EXTRACT(MONTH FROM (@1))||'-'||EXTRACT(DAY FROM (@1)))"},
  //
  // Math functions
  int: {expr: "TRUNC(@1)"},
  ceil: {expr: "CEILING(@1)"},
  floor: {expr: "FLOOR(@1)"},
  round: {expr: "ROUND(cast(@1 as numeric), @2)"},
  trunc: {expr: "TRUNC(CAST(@1 as numeric), @2)"},
  sqrt: {expr: "SQRT(@1)"},
  log: {expr: "LN(@1)"},
  log10: {expr: "LOG(@1)"},
  cot: {expr: "COT(@1)"},
  //
  // String functions
  left: {expr: "SUBSTRING((@1) from 1 for (@2))"},
  right: {expr: "SUBSTRING(@1 from (LENGTH(@1)-(@2)+1) for (@2))"},
  substr: {expr: "SUBSTRING(@1, @2, @3)"},
  spaces: {expr: "LPAD(' ', @1)"},
  replicate: {expr: "REPEAT(SUBSTRING((@1) from 1 for 1), @2)"},
  fillLeft: {expr: "LPAD(@1, @2, SUBSTR(@3, 1, 1))"},
  fillRight: {expr: "RPAD(@1, @2, SUBSTR(@3, 1, 1))"},
  //
  // Various functions
  nullValue: {expr: "COALESCE(@1, @2)"},
  blobSize: {expr: "length(@1)"}
};


/**
 * Open the connection to the database
 * @param {Function} callback - function to be called at the end
 */
App.Postgres.prototype._openConnection = function (callback)
{
  if (!module)
    return callback(null, new Error("Postgres is supported only inside Node"));
  //
  // Import module (could be missing)
  try {
    if (!App.pg)
      App.pg = require("pg");
  }
  catch (ex) {
    return callback(null, new Error("Postgres driver not found.\nInstall \"pg\" module and try again"));
  }
  //
  App.pg.connect(this.options, function (error, client, done) {
    if (error)
      done();
    else
      this.done = done;
    //
    callback(client, error);
  }.bind(this));
};


/**
 * Close the connection to the database
 * @param {Function} callback - function to be called at the end
 */
App.Postgres.prototype._closeConnection = function (callback)
{
  // Release connection to pool
  this.done();
  delete this.done;
  //
  callback();
};


/**
 * Create a new database
 * @param {Function} callback - function to be called at the end
 */
App.Postgres.prototype.createDb = function (callback)
{
  // If the database is "remote" no create is needed
  if (this.remoteConnector)
    return callback();
  //
  // Attach a callback that the parent process will call when DB creation is completed
  this.createDBcallback = function (err) {
    callback(undefined, err);
  };
  //
  // Send message to the parent process asking him to create the database (if needed)
  return process.send({type: App.Postgres.msgTypeMap.createDB, dbname: this.schema.name});
};


/**
 * Execute an SQL query
 * @param {String} sql to execute
 * @param {Boolean} readOnly - true if is a SELECT
 * @param {Array} parameters
 * @param {Function} callback - function to be called at the end
 * @return {Object/Integer}
 */
App.Postgres.prototype._executeSQL = function (sql, readOnly, parameters, callback)
{
  // Run the query
  this.connection.query(sql, parameters, function (error, result) {
    if (error) {
      callback(null, error);
      //
      if (App.log)
        App.log("WARN", "Error while executing query: " + error.toString(), "Postgres.executeSQL",
                {sql: sql, database: this.schema.name});
      return;
    }
    //
    callback(result);
  }.bind(this));
};


/**
 * Execute an SQL query with offset paging
 * @param {Object} params
 * @param {Function} callback - function to be called at the end
 * @return {DataMap/Object}
 */
App.Postgres.prototype.queryOffsetPaging = function (params, callback)
{
  var sql = "select * from (";
  sql += params.sql;
  sql += ") a limit " + params.limit;
  sql += " offset " + params.offset;
  this.executeSQL(sql, true, params.parameters, callback);
};


/**
 * Return column's definition from resultSet
 * @param {Object} result
 */
App.Postgres.prototype.getColumnsDefByResult = function (result)
{
  // Cloud connector don't send metaschema
  if (this.remoteConnector)
    return App.Database.prototype.getColumnsDefByResult.call(this, result);
  //
  var columns = [];
  for (var i = 0; i < result.fields.length; i++) {
    var c = {};
    c.name = result.fields[i].name;
    switch (result.fields[i].dataTypeID) {
      case 20:   // bigint
      case 21:   // smalltint
      case 23:   // integer
        c.datatype = App.Database.DataTypes.INT;
        break;
      case 701:  // float
        c.datatype = App.Database.DataTypes.FLOAT;
        break;
      case 790:  // money
      case 1700: // numeric
        c.datatype = App.Database.DataTypes.DECIMAL;
        break;
      case 1043: // varchar
        c.datatype = App.Database.DataTypes.VARCHAR;
        break;
      case 18:   // char
        c.datatype = App.Database.DataTypes.CHAR;
        break;
      case 17:   // bytea
        c.datatype = App.Database.DataTypes.BLOB;
        break;
      case 25:   // text
        c.datatype = App.Database.DataTypes.CLOB;
        break;
      case 16:   // bool
        c.datatype = App.Database.DataTypes.BOOL;
        break;
      case 1082: // date
        c.datatype = App.Database.DataTypes.DATE;
        break;
      case 1266: // time with timezone
        c.datatype = App.Database.DataTypes.TIME;
        break;
      case 1184: // timestamp with time zone
        c.datatype = App.Database.DataTypes.DATETIME;
        break;
      case 2950: // uuid
        c.datatype = App.Database.DataTypes.UUID;
        break;
    }
    columns.push(c);
  }
  return columns;
};


/*
 * Read some info from result
 * @param {String} sql
 * @param {Object} result
 */
App.Postgres.prototype._readInfoFromResult = function (sql, result)
{
  this.lastRowsAffected = result.rowCount;
  if (result.rows.length === 1 && result.rows[0].counter > 0)
    this.lastInsertId = result.rows[0].counter;
};


/**
 * Recreate the primary key of table
 * @param {Array} sqls list of sql statements
 * @param {Object} t table to recreate
 * @param {Object} ot old version
 */
App.Postgres.prototype.recreatePK = function (sqls, t, ot)
{
  this.dropExternalTableFKS(sqls, ot);
  //
  var sql = "DO $$\n";
  sql += "DECLARE pkname TEXT;\n";
  sql += "BEGIN\n";
  sql += " pkname := (SELECT constraint_name FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS ";
  sql += " WHERE table_name ilike '" + ot.name + "'";
  sql += " and table_schema = '" + (ot.schema || this.defaultSchema) + "'";
  sql += " and constraint_type = 'PRIMARY KEY');\n";
  sql += " IF pkname is not null THEN";
  sql += "   EXECUTE 'ALTER TABLE " + this.quote(ot.name) + " DROP CONSTRAINT ' || pkname;\n";
  sql += " END IF;\n";
  var pks = "";
  for (var i = 0; i < t.fields.length; i++) {
    if (t.fields[i].pk)
      pks += (pks ? "," : "") + this.quote(t.fields[i].name);
  }
  if (pks)
    sql += " ALTER TABLE " + this.quote(ot.name) + " ADD PRIMARY KEY (" + pks + ");\n";
  sql += "END $$";
  sqls.push({sql: sql});
};


/**
 * Create a table
 * @param {Array} sqls list of sql statements
 * @param {Object} t table to create
 */
App.Postgres.prototype.createTable = function (sqls, t)
{
  // Getting create table stmt
  var s = "create table " + this.quote(t.name) + " (";
  //
  // Getting fields definition
  for (var i = 0; i < t.fields.length; i++)
    s += (i === 0 ? "" : ",") + "\n  " + this.getFieldDef(t.fields[i]);
  //
  // Creating PK constraint inside table definition
  var pkn = "";
  for (var i = 0; i < t.fields.length; i++) {
    var f = t.fields[i];
    if (f.pk)
      pkn += (pkn === "" ? "" : ", ") + this.quote(f.name);
  }
  //
  if (pkn !== "") {
    s += ",\n  primary key (" + pkn + ")";
    //s += " using index tablespace " + pk.tablespace;
  }
  //
  s += "\n)";
  //
  sqls.push({sql: s});
  //
  // Set the ownership
  if (this.dbnamePrefix)
    sqls.push({sql: "alter table " + this.quote(t.name) + " owner to " + this.quote(this.dbnamePrefix)});
  //
  // Return the table with childs
  var nt = {};
  nt.name = t.name;
  nt.fields = t.fields;
  return nt;
};


/**
 * Modify a field
 * @param {Array} sqls list of sql statements
 * @param {Object} t table to modify
 * @param {Object} f field to modify
 * @param {Object} of old version
 */
App.Postgres.prototype.modifyField = function (sqls, t, f, of)
{
  if (f.name.toLowerCase() !== of.name.toLowerCase()) {
    sqls.push({sql: "alter table " + this.quote(t.name) + " rename column " + this.quote(of.name) + " to " + this.quote(f.name)});
    of.name = f.name;
  }
  //
  var prefix = "alter table " + this.quote(t.name) + " alter " + this.quote(f.name);
  //
  if (f.datatype !== of.datatype || this.getFieldMaxLength(f) !== this.getFieldMaxLength(of) || this.getFieldScale(f) !== this.getFieldScale(of)) {
    // If the field has changed to a SERIAL field, I need to do something more complicated
    if (f.datatype === App.Database.DataTypes.AUTOINCR && of.datatype !== App.Database.DataTypes.AUTOINCR) {
      // First if the field was not an integer field, I need to change it to integer
      if (of.datatype !== App.Database.DataTypes.INT) {
        var oldt = f.datatype;
        f.datatype = App.Database.DataTypes.INT;
        sqls.push({sql: prefix + " set data type " + this.getDataType(f)});
        f.datatype = oldt;
      }
      //
      // Then I create a new sequence, sets its initial value and give the sequence to the field
      var seq = this.quote(t.name) + "_" + this.quote(f.name) + "_seq";
      sqls.push({sql: "create sequence " + seq});
      sqls.push({sql: "select setval('" + seq + "', (select max(" + this.quote(f.name) + ") from " + this.quote(t.name) + "))"});
      sqls.push({sql: prefix + " set default nextval('" + seq + "')"});
    }
    // If field is changed between smallserial,serial and bigserial change int datatype
    else if (f.datatype === App.Database.DataTypes.AUTOINCR && of.datatype === App.Database.DataTypes.AUTOINCR && this.getDataType(f) !== this.getDataType(of))
      sqls.push({sql: prefix + " set data type " + this.getDataType({datatype: App.Database.DataTypes.INT, maxlen: f.maxlen})});
    else {
      // If is changed data type first drop default
      if (f.datatype !== of.datatype && this.getFieldDefVal(of) && this.getFieldDefVal(f) !== this.getFieldDefVal(of))
        sqls.push({sql: prefix + " drop default"});
      //
      var sql = prefix + " set data type " + this.getDataType(f);
      //
      // Add cast expression
      if (f.datatype !== of.datatype) {
        sql += " using cast(";
        if (of.datatype === App.Database.DataTypes.BOOL)
          sql += "case when " + this.quote(f.name) + " then '1' else '0' end";
        else
          sql += this.quote(f.name);
        sql += " as " + this.getDataType(f) + ")";
      }
      //
      sqls.push({sql: sql});
      //
      // If the field has changed from SERIAL to something else, I need to drop the sequence
      if (f.datatype !== App.Database.DataTypes.AUTOINCR && of.datatype === App.Database.DataTypes.AUTOINCR) {
        // First I need to "detach" the sequence from the field
        sqls.push({sql: prefix + " drop default"});
        //
        // Then I can remove it
        var seq = this.quote(t.name) + "_" + this.quote(f.name) + "_seq";
        sqls.push({sql: "drop sequence " + seq});
      }
    }
  }
  //
  if (this.getFieldDefVal(f) !== this.getFieldDefVal(of))
    sqls.push({sql: prefix + (this.getFieldDefVal(f) ? " set default " + this.getFieldDefVal(f) : " drop default")});
  //
  // If field became not null without default update table with dummy value
  if (f.notn && !of.notn)
    this.updateRequiredField(sqls, t, f);
  //
  if (f.notn !== of.notn)
    sqls.push({sql: prefix + (f.notn ? " set" : " drop") + " not null"});
  //
  return true;
};


/**
 * Modify a foreign key
 * @param {Array} sqls list of sql statements
 * @param {Object} t table to modify
 * @param {Object} ot table old version
 * @param {Object} fk foreign key to modify
 * @param {Object} ofk old version
 */
App.Postgres.prototype.modifyFK = function (sqls, t, ot, fk, ofk)
{
  var ok = true;
  if (Object.keys(fk.refs).length !== Object.keys(ofk.refs).length)
    ok = false;
  else if (fk.ur !== ofk.ur || fk.dr !== ofk.dr)
    ok = false;
  else {
    for (var ref in fk.refs) {
      if (!ofk.refs[ref])
        ok = false;
    }
  }
  //
  if (!ok) {
    this.dropFK(sqls, t, ofk);
    this.createFK(sqls, t, fk);
  }
  else if (fk.name.toLowerCase() !== ofk.name.toLowerCase() || t.name.toLowerCase() !== ot.name.toLowerCase())
    sqls.push({sql: "alter table " + this.quote(t.name) + " rename constraint " + this.quote(this.getFKName(ot, ofk)) + " to " + this.quote(this.getFKName(t, fk))});
  //
  return true;
};


/**
 * Modify an index
 * @param {Array} sqls list of sql statements
 * @param {Object} t table owner of index
 * @param {Object} idx index to modify
 * @param {Object} oidx old version
 */
App.Postgres.prototype.modifyIndex = function (sqls, t, idx, oidx)
{
  var ok = true;
  if (idx.un !== oidx.un)
    ok = false;
  else if (idx.fields.length !== oidx.fields.length)
    ok = false;
  else {
    for (var i = 0; i < idx.fields.length; i++) {
      if (idx.fields[i] !== oidx.fields[i])
        ok = false;
    }
  }
  //
  if (!ok) {
    this.dropIndex(sqls, t, oidx);
    this.createIndex(sqls, t, idx);
  }
  else if (idx.name.toLowerCase() !== oidx.name.toLowerCase())
    sqls.push({sql: "alter index " + this.quote(oidx.name) + " rename to " + this.quote(idx.name)});
  return true;
};


/**
 * Drop an index
 * @param {Array} sqls list of sql statements
 * @param {Object} t table owner of index
 * @param {Object} idx index to drop
 */
App.Postgres.prototype.dropIndex = function (sqls, t, idx)
{
  sqls.push({sql: "drop index if exists " + this.quote(idx.name)});
  return true;
};


/**
 * Get the datatype for the f field
 * @param {Object} f
 */
App.Postgres.prototype.getDataType = function (f)
{
  var maxlen = this.getFieldMaxLength(f);
  var scale = this.getFieldScale(f);
  switch (f.datatype) {
    case App.Database.DataTypes.INT:
    {
      if (maxlen < 5)
        return "smallint"; // -32768 to +32767
      else if (maxlen < 10)
        return "integer"; // -2147483648 to +2147483647
      else
        return "bigint"; // -9223372036854775808 to +9223372036854775807
    }

    case App.Database.DataTypes.AUTOINCR:
    {
      if (maxlen < 5)
        return "smallserial"; // 1 to 32767 (Postgres 9.2+)
      else if (maxlen < 10)
        return "serial"; // 1 to 2147483647
      else
        return "bigserial"; // 1 to 9223372036854775807
    }

    case App.Database.DataTypes.FLOAT:
    {
      if (maxlen < 8)
        return "real"; // 6 decimal digits precision
      else
        return "double precision"; // 15 decimal digits precision
    }

    case App.Database.DataTypes.DECIMAL:
      return "decimal(" + maxlen + "," + scale + ")";

    case App.Database.DataTypes.VARCHAR:
    {
      if (maxlen)
        return "varchar(" + maxlen + ")";
      else
        return "text";
    }

    case App.Database.DataTypes.CHAR:
      return "char(" + maxlen + ")";

    case App.Database.DataTypes.UUID:
    {
      if (maxlen === 36)
        return "uuid";
      else
        return "char(" + maxlen + ")";
    }

    default:
      return this.typemap[f.datatype][0];
  }
};


/**
 * Compose insert statement of a query
 * @param {Object} query to compose
 */
App.Postgres.prototype.createInsertStatement = function (query)
{
  var sql = App.Database.prototype.createInsertStatement.call(this, query);
  if (query.counter)
    sql += " returning " + this.quote(this.getObjDMCode(query.table, query.counter)) + " as counter";
  return sql;
};


/**
 * Returns the i-th row of a resultSet
 * @param {Object} rs - recordset
 * @param {int} pos - position
 * @param {DataSchema} schema
 */
App.Postgres.prototype.getRow = function (rs, pos, schema)
{
  var row = (this.remoteConnector ? App.RemoteDriver.getRow(rs, pos) : rs.rows[pos]);
  if (!schema)
    return row;
  //
  var newrow = {};
  for (var c in schema.columns) {
    var clower = c.toLowerCase();
    var v = (row.hasOwnProperty(clower) ? row[clower] : row[c]);
    if (v === undefined)
      continue;
    newrow[c] = this.convertValue(v, schema.columns[c].datatype);
  }
  return newrow;
};


/**
 * Prepare a value for SQL
 * @param {Object} v
 * @param {String} dt - dataType
 */
App.Postgres.prototype.forSql = function (v, dt)
{
  if (v === null || v === undefined)
    return null;
  //
  if (dt === App.Database.DataTypes.BOOL)
    return v ? true : false;
  //
  return App.Database.prototype.forSql.call(this, v, dt);
};


/*
 * Begin a transaction
 * @param {Function} callback - function to be called at the end
 */
App.Postgres.prototype._beginTransaction = function (callback)
{
  this._executeSQL("begin", false, [], callback);
};


/*
 * Commit a transaction
 * @param {Function} callback - function to be called at the end
 */
App.Postgres.prototype._commitTransaction = function (callback)
{
  this._executeSQL("commit", false, [], callback);
};


/*
 * Rollback a transaction
 * @param {Function} callback - function to be called at the end
 */
App.Postgres.prototype._rollbackTransaction = function (callback)
{
  this._executeSQL("rollback", false, [], callback);
};


/**
 * Import fiels of a table
 * @param {Object} options
 * @param {Function} callback - function to be called at the end
 */
App.Postgres.prototype.importTableFields = function (options, callback)
{
  App.Database.prototype.importTableFields.call(this, options, function (fields, error) {
    if (error)
      return callback(null, error);
    //
    for (var i = 0; i < fields.length; i++) {
      if (fields[i].dataType === App.Database.DataTypes.AUTOINCR
              || (fields[i].defaultValue && fields[i].defaultValue.toLowerCase().startsWith("nextval("))) {
        fields[i].dataType = App.Database.DataTypes.AUTOINCR;
        delete fields[i].defaultValue;
        break;
      }
    }
    //
    callback(fields);
  });
};


/**
 * Check if a name require case sensitivity
 * @param {String} name
 */
App.Postgres.prototype.requireCaseSensitivity = function (name)
{
  return (name.toLowerCase() !== name);
};


/*
 * Define a query parameter
 * @param {Object} token
 */
App.Postgres.prototype.defineQueryParameter = function (token)
{
  this.parameters.push(this.forSql(token.value, token.dt));
  return "$" + this.parameters.length;
};


/*
 * Verify if z_sync table exists
 * @param {String} handler - handler of message
 * @param {Function} callback - function to be called at the end
 */
App.Postgres.prototype.syncExistsTable = function (handler, callback)
{
  this.executeSQL("select * from INFORMATION_SCHEMA.TABLES where table_name ilike 'z_sync" + handler + "'", true, function (result) {
    callback(result && this.getRowCount(result) === 1);
  }.bind(this));
};


/*
 * Create z_sync table
 * @param {String} handler - handler of message
 * @param {Function} callback - function to be called at the end
 */
App.Postgres.prototype.syncCreateTable = function (handler, callback)
{
  var sql = "create table z_sync" + handler + "(id serial not null primary key, uid char(24) unique, device text, sender text, topics text, body text, ts timestamp with time zone default now(), expire timestamp with time zone)";
  this.executeSQL(sql, false, function (result, error) {
    if (error)
      return callback(null, error);
    //
    // Set the ownership
    if (this.dbnamePrefix)
      this.executeSQL("alter table z_sync" + handler + " owner to " + this.quote(this.dbnamePrefix), false, callback);
    else
      callback();
  });
};


//export module for node
if (module)
  module.exports = App.Postgres;
