/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

var App = App || {};


/**
 * @class
 * @param {view} view
 * @param {defprop} defprop
 */
App.Pubsub = function (view, defprop)
{
  if (module && !App.View) {
    App = require("./requireAll");
    App.redis = require("redis");
  }
  this.name = defprop.name;
  this.id = defprop.id;
  this.pubID = App.Utils.generateUID24();
  this.view = view;
  this.app = view.app;
  this.listenerON = false;
  //
  // dfault host and port
  this.ihost = "127.0.0.1";
  this.iport = 6379;
  this.ipassword = null;
  this.itopic = null;
  //
  var pthis = this;
  //
  Object.defineProperty(this, "host", {
    get: function () {
      return pthis.ihost;
    },
    set: function (newValue) {
      if (newValue)
        pthis.ihost = newValue;
    },
    enumerable: true,
    configurable: true
  });
  //
  Object.defineProperty(this, "port", {
    get: function () {
      return pthis.iport;
    },
    set: function (newValue) {
      if (newValue)
        pthis.iport = newValue;
    },
    enumerable: true,
    configurable: true
  });
  //
  Object.defineProperty(this, "password", {
    get: function () {
      return pthis.ipassword;
    },
    set: function (newValue) {
      if (newValue)
        pthis.ipassword = newValue;
    },
    enumerable: true,
    configurable: true
  });
  //
  // define the topic
  Object.defineProperty(this, "topic", {
    get: function () {
      return pthis.itopic;
    },
    set: function (newValue) {
      if (pthis.itopic && pthis.subscriber) {
        pthis.subscriber.unsubscribe(pthis.itopic);
      }
      if (newValue)
        pthis.itopic = newValue;
      if (pthis.subscriber) {
        pthis.subscriber.subscribe(pthis.itopic);
      }
    },
    enumerable: true,
    configurable: true
  });
  //
  this.topic = defprop.topic;
  this.port = defprop.port;
  this.host = defprop.host;
  this.password = defprop.password;
  //
  // Put the element in the map
  this.app.eleMap[this.id] = this;
};


/*
 * Listens for messages received on the subscribed topic
 */
App.Pubsub.prototype.listener = function ()
{
  var pthis = this;
  //
  if (pthis.listenerON)
    return;
  //
  this.listenerON = true;
  //
  var topic = this.topic;
  // listen for a message with a specified topic
  this.subscriber.on("message", function (topic, message) {
    message = JSON.parse(message);
    //
    // if the massage comes from the same pubsub object is not returned
    if (message.id === pthis.pubID)
      return;
    else {
      return pthis.onMessage(message.id, message.content);
    }
  });
};


/*
 * Cretes the Redis clients
 */
App.Pubsub.prototype.connect = function ()
{
  var pthis = this;
  //
  // creates a subscriber client for Redis
  if (this.topic) {
    this.subscriber = App.redis.createClient(this.port, this.host, {});
    //
    // if a password is the subscriber is authenticated
    if (pthis.password)
      this.subscriber.auth(pthis.password, function (err) {
        if (err)
          return false;
      });
    //
    // the subscriber subscribes to the topic
    pthis.subscriber.subscribe(pthis.topic);
    //
    // creates a publusher client for Redis
    pthis.publisher = App.redis.createClient(pthis.port, pthis.host, {});
    //
    // if a password is the publisher is authenticated
    if (pthis.password)
      pthis.publisher.auth(pthis.password, function (err) {
        if (err)
          return false;
      });
    //
    // activate the listener
    pthis.listener();
  }
  else
    return (false);
};


/*
 * Send a message to the people subscribed to the topic channel
 * @param {string} message
 */
App.Pubsub.prototype.sendMessage = function (message)
{
  // if the topic is set the message is sent
  if (this.topic)
    this.publisher.publish(this.topic, JSON.stringify({id: this.pubID, content: message}));
  else
    return false;
};

/*
 *
 * @param {string} id
 * @param {obj} message
 */
App.Pubsub.prototype.onMessage = function (id, message)
{
};


/**
 * Remove the pubsub from the element map
 */
App.Pubsub.prototype.close = function ()
{
  // end the subscriber
  this.subscriber.end();
  //
  // end the publisher
  this.publisher.end();
  //
  // delete the obj from the map
  delete this.app.eleMap[this.id];
};


/**
 * Change a property by the IDE
 * @param {Object} pv property to change
 */
App.Pubsub.prototype.chgProp = function (pv)
{
  this[pv.name] = pv.value;
};


//export module for node
if (module)
  module.exports = App.Pubsub;

