/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

var App = App || {};

/**
 * @class
 * @param {View} view
 * @param {Object} defprop - element properties
 * @param {RemElem} parent
 * */
App.RemElem = function (view, defprop, parent)
{
  if (module && !App.View)
    App = require("./requireClasses");
  this.view = view;
  this.app = view.app;
  this.visible = true;
  if (parent)
    this.parent = parent;
  else
    this.parent = this.view;
  this.style = {id: this.app.generateCounter()};
  this.animations = [];
  this.dataMap = null;
  //
  // Load default properties
  if (defprop) {
    for (var x in defprop) {
      // Skip added properties that aren't real element values
      if (x.substr(0, 1) === "_")
        continue;
      //
      // Skip children
      if (x !== "children") {
        //
        // Handling style object
        if (x === "style") {
          var val = defprop[x];
          if (typeof val === "string") {
            if (val.length === 0)
              val = "{}";
            try {
              this[x] = JSON.parse(val);
            }
            catch (ex) {
              // Cannot parse style? No problem as in the real DOM
              this[x] = {};
            }
          }
          else
            this[x] = val;
          //
          this.handleParameters(this[x]);
        }
        else {
          // Copy element property value
          // translatable property?
          if (x.substr(0, 1) === "&")
            this[x.substring(1)] = App.Utils.t({id: defprop[x].id + x, t: defprop[x].t, lc: this.app.langCode});
          else
            this[x] = defprop[x];
          //
          // translatable list?
          if (this[x] instanceof Array) {
            for (var i = 0; i < this[x].length; i++) {
              var ele = this[x][i];
              if (ele.id) {
                if (ele.n)
                  ele.n = App.Utils.t({id: ele.id + "&text", t: ele.n, lc: this.app.langCode});
                if (ele.d)
                  ele.d = App.Utils.t({id: ele.id + "&detail", t: ele.d, lc: this.app.langCode});
              }
            }
          }
        }
      }
    }
    //
    // Put the element in the map
    // Save id from IDE if set
    if (this.id)
      this.pid = this.id;
    this.id = this.app.generateCounter();
    this.app.eleMap[this.id] = this;
    //
    // Create children
    if (defprop.children) {
      this.elements = [];
      for (var i = 0; i < defprop.children.length; i++) {
        //
        // Some children could refer subviews
        if (defprop.children[i].child) {
          var x = this.app.createView(defprop.children[i]);
          this.appendChild(x);
        }
        else {
          // This one is a normal element
          var x = new App.RemElem(view, defprop.children[i], this);
          this.elements.push(x);
        }
        //
        this[x.name] = x;
      }
    }
    //
    // Every changes can be sent if the observed are attached
    this.sendChg = true;
  }
};


/**
 * List of the properties that will not projected to client
 */
App.RemElem.excludedProp = {app: true, elements: true, view: true, sendChg: true, require: true, parent: true, row: true, obs: true, dataMap: true, clid: true, group: true};


/**
 * Attach the observers to this remelem
 * @param {Boolean} flag
 * @param {Boolean} children
 */
App.RemElem.prototype.attachObservers = function (flag, children)
{
  //
  // Already done?
  if (this.obs === flag)
    return;
  //
  var pthis = this;
  //
  // Selecting operation: attach or unattach?
  var fn = (flag) ? "observe" : "unobserve";
  this.obs = flag;
  //
  // Observe changes on the object itself
  Object[fn](this, function (changes) {
    var f = {};
    var send = false;
    var visibilityChanged = false;
    for (var i = 0; i < changes.length; i++) {
      var change = changes[i];
      if (change.name === "style") {
        var newStyle = change.object.style;
        if (typeof newStyle === "string") {
          try {
            newStyle = JSON.parse(newStyle);
          } catch (ex) {
          }
        }
        if (typeof newStyle === "object") {
          pthis.handleParameters(newStyle);
          change.object.style = JSON.stringify(newStyle);
        }
      }
      if (change.name === "visible")
        visibilityChanged = true;
      //
      // Check if the changed property must be sent or not
      if (change.object.sendChg && !App.RemElem.excludedProp[change.name]) {
        // Compressing the name (only online)
        var pn = undefined;
        if (!App.isOffline())
          pn = App.transPropMap[change.name];
        //
        if (pn === undefined)
          pn = change.name;
        //
        var vv = change.object[change.name];
        if (change.name === "value")
          vv = pthis.sanitizeValue(vv);
        f[pn] = vv;
        //
        send = true;
      }
    }
    //
    if (send) {
      // Is changed my visibility: notify them to owner dataMap
      if (pthis.dataMap && visibilityChanged)
        pthis.dataMap.onChangeElementVisibility(pthis);
      //
      // Purge clid properties that weren't changed by a client
      // and purge clid values
      var clid;
      if (pthis.clid) {
        var kk = Object.keys(pthis.clid);
        for (var i = 0; i < kk.length; i++) {
          var k = kk[i];
          var eq = true;
          var v = pthis[k];
          if (v !== null && v !== undefined && typeof v === "object") {
            var pp = Object.keys(v);
            for (var j = 0; j < pp.length && eq; j++) {
              if (pthis.clid[k].v[pp[j]] !== v[pp[j]])
                eq = false;
            }
          }
          else
            eq = pthis.clid[k].v === v;
          //
          if (eq) {
            if (!clid)
              clid = {};
            clid[k] = pthis.clid[k].id;
          }
        }
        if (clid)
          f.clid = clid;
      }
      //
      // Store this command to complete it with style commands before it starts
      App.RemElem.lastr = {obj: pthis.id, id: "updateElement", cnt: f};
      //
      var r = [App.RemElem.lastr];
      pthis.view.app.sendCommands(r);
    }
    //
    if (pthis.clid)
      delete pthis.clid;
  });
  //
  // Observe changes to the style property
  Object[fn](this.style, function (changes) {
    var f = {};
    //
    // Translate only if online
    var sn = App.isOffline() ? "style" : "s";
    //
    f[sn] = {};
    var send = false;
    for (var i = 0; i < changes.length; i++) {
      var change = changes[i];
      var n = change.name;
      var obj = change.object;
      var v = obj[n];
      if (pthis.sendChg) {
        // If it's a style parameter replace it
        var name = change.name;
        if (pthis.app.style[n])
          name = App.Utils.replaceParameter(n, pthis.app.style[n], "$" + n, pthis.app.device);
        var value = obj[n];
        if (pthis.app.style[v])
          value = App.Utils.replaceParameter(v, pthis.app.style[v], "$" + v, pthis.app.device);
        //
        // If it's a theme parameter replace it
        if (pthis.app.theme[v])
          value = App.Utils.replaceParameter(v, pthis.app.theme[v], "$" + v, pthis.app.device);
        //
        // Compress the name of the style property (only offline)
        if (!App.isOffline())
          name = pthis.app.findStylePropValue(name, true);
        f[sn][name] = value;
        send = true;
      }
    }
    //
    // If we have a stored command, complete it instead of creating another one
    if (App.RemElem.lastr && App.RemElem.lastr.obj === pthis.id) {
      App.RemElem.lastr.cnt[sn] = f[sn];
      send = false;
    }
    //
    if (send) {
      var r = [{obj: pthis.id, id: "updateElement", cnt: f}];
      pthis.view.app.sendCommands(r);
    }
  });
  //
  // Attach observers also to children if required
  if (children && this.elements) {
    for (var i = 0; i < this.elements.length; i++) {
      if (this.elements[i].attachObservers)
        this.elements[i].attachObservers(flag, children);
    }
  }
};


/**
 * Replace style and theme parameters
 * @param {Object} style
 */
App.RemElem.prototype.handleParameters = function (style)
{
  // Replace the css parameters with their value
  if (style) {
    var keys = Object.keys(style);
    for (var i = 0; i < keys.length; i++) {
      var prop = keys[i];
      var value = style[prop];
      //
      this.replaceStyleParameters(style, prop, value);
      this.replaceThemeParameters(style, prop, value);
    }
  }
};


/**
 * Replace style parameters
 * @param {Object} style
 * @param {String} prop
 * @param {String} value
 */
App.RemElem.prototype.replaceStyleParameters = function (style, prop, value)
{
  // Replace the right part of the css rule
  if (this.app.style[value]) {
    var newVal = App.Utils.replaceParameter(value, this.app.style[value], "$" + value, this.app.device);
    style[prop] = newVal;
  }
  //
  // Replace the left part of the css rule
  var p = App.Utils.camelCase(prop);
  if (this.app.style[p] && this.app.style[p] !== prop) {
    var newVal = App.Utils.replaceParameter(prop, this.app.style[p], "$" + prop, this.app.device);
    if (newVal !== prop) {
      style[newVal] = style[prop];
      delete style[prop];
    }
  }
};


/**
 * Replace theme parameters
 * @param {Object} style
 * @param {String} prop
 * @param {String} value
 */
App.RemElem.prototype.replaceThemeParameters = function (style, prop, value)
{
  var idx = value.indexOf("$");
  if (idx !== 0)
    return;
  //
  var param = value.substring(1);
  //
  // Replace the right part of the css rule
  if (this.app.theme[param]) {
    var newVal = App.Utils.replaceParameter(param, this.app.theme[param], "$" + param, this.app.device);
    style[prop] = newVal;
  }
};


/**
 * Tells something about this object to the DTT module
 * @param {Integer} childLevel - level of child to log
 */
App.RemElem.prototype.getDebugInfo = function (childLevel)
{
  var info = {};
  info._class = this.class;
  info._name = this.name;
  if (childLevel > 0) {
    if (this.row !== undefined)
      info.row = this.row;
    if (this.elements)
      info.elements = App.DTT.convertValue(this.elements, undefined, undefined, childLevel - 1);
  }
  return info;
};


/**
 * Get the element properties and the properties of its children
 * @returns {Object}
 */
App.RemElem.prototype.getConfig = function () {
  //
  // Preparing values for the client side
  var r = {};
  //
  // Get only attributes I need
  for (var x in this) {
    if ((typeof this[x] !== "function") && !(this[x] instanceof App.RemElem) && !(this[x] instanceof App.DataMap) &&
            !(this[x] instanceof App.View) && (!App.RemElem.excludedProp[x])) {
      //
      if (x === "style") {
        // The style property needs a special handling
        var s = this[x];
        var ts = {};
        var send = false;
        for (var ps in s) {
          if (ps === "id")
            continue;
          var t = this.app.findStylePropValue(ps, true);
          ts[t] = s[ps];
          send = true;
        }
        if (send)
          r.s = ts;
      }
      else {
        // This one is a normal property. The name will be compressed
        var t = App.transPropMap[x];
        //
        var v = this[x];
        if (x === "value")
          v = this.sanitizeValue(v);
        //
        r[t ? t : x] = v;
      }
    }
  }
  //
  // Get children properties
  if (this.elements) {
    var el = [];
    for (var i = 0; i < this.elements.length; i++) {
      if (this.elements[i].getConfig)
        el.push(this.elements[i].getConfig());
    }
    if (el.length > 0)
      r.children = el;
  }
  //
  // Get only the events I need
  var ev = [];
  for (var p in this)
  {
    if (p.substring(0, 2) === "on" && typeof this[p] === "function" && !p.endsWith("_dtt"))
      ev.push(p);
  }
  //
  // Attaching event array if at least one was found
  if (ev.length > 0)
    r.events = ev;
  //
  // If there is a datamap load the data (the same of DataMap.loadElementData but first search the
  // element to use)
  if (this.dataMap)
    this.dataMap.initElement(this.id);
  //
  // Attach animations
  if (this.animations.length > 0)
    r.animations = this.animations;
  else
    delete r.animations;
  //
  return r;
};


/**
 * Called by a client when a property changes
 * @param {Object} pv property and value
 */
App.RemElem.prototype.chgProp = function (pv)
{
  this[pv.name] = pv.value;
  if (pv.clid) {
    if (!this.clid)
      this.clid = {};
    this.clid[pv.name] = {id: pv.clid, v: pv.value};
  }
  //
  // If this element is buond to a datarow, inform it
  if (this.row)
    this.row.chgProp(this, pv.name, pv.value);
};


/**
 * Change data of the datamap linked to the remelem
 * @param {Object} op operation to execute on the datamap
 */
App.RemElem.prototype.chgData = function (op)
{
  // It this element is bound to a datamap, inform it
  if (this.dataMap)
    this.dataMap.chgData(op);
};


/**
 * Remove the element and its children from the element map
 */
App.RemElem.prototype.close = function ()
{
  if (this.elements) {
    for (var i = 0; i < this.elements.length; i++)
      this.elements[i].close();
  }
  //
  delete this.app.eleMap[this.id];
  //
  // Delete any observer on me
  if (App.deleteObservers) {
    App.deleteObservers(this);
    App.deleteObservers(this.style);
  }
};


/**
 * Create a new formula bound to another object
 * @param {String} myp - my property to change
 * @param {Object} obj - object to observe
 * @param {String} prop - property to observe
 * @param {Function} fn - function to call when change
 */
App.RemElem.prototype.addFormula = function (myp, obj, prop, fn)
{
  var pthis = this;
  Object.observe(obj, function (changes) {
    changes.forEach(function (change) {
      if (change.name === prop) {
        pthis[myp] = fn();
      }
    });
  });
};


/**
 * Append a child to the element
 * @param {RemElem|View} child - the element to append
 */
App.RemElem.prototype.appendChild = function (child)
{
  this.insertBefore(child);
};


/**
 * Insert a child element before another element
 * @param {RemElem} child - the element to insert
 * @param {String} sibId - the id of the existing element. The new element is inserted before this element
 */
App.RemElem.prototype.insertBefore = function (child, sibId)
{
  // Put the child in the elements array at the right position
  if (this.elements === undefined)
    this.elements = [];
  //
  var sib, idx = -1;
  if (sibId) {
    sib = this.app.eleMap[sibId];
    if (sib)
      idx = this.elements.indexOf(sib);
  }
  if (idx > -1)
    this.elements.splice(idx, 0, child);
  else
    this.elements.push(child);
  //
  // After attaching it, changes should be observed
  child.setParent(this.view, this);
  child.sendChanges(true);
  child.attachObservers(true, true);
  //
  // Tell the client to append the child
  if (this.view.isOpen) {
    // Adding a subview to an already opened view: in this case we must set the opened status explicitly
    // (if the view is not open the status will be set by the show method, recursive for all subviews)
    if (child instanceof App.View)
      child.setIsOpen(true);
    else if (child instanceof App.RemElem) {
      var ris = [];
      this.getSubviews(ris);
      for (var i = 0; i < ris.length; i++)
        ris[i].setIsOpen(true);
    }
    //
    this.app.sendCommands([{obj: this.id, id: "insertBefore", cnt: {child: child.getConfig(), sib: sibId}}]);
  }
};


/**
 * Insert a child element before another element, called by the IDE when the application is running and must be updated (AApp.updateCode)
 * @param {Object} childDef - the object representing the element to create and insert
 */
App.RemElem.prototype.insertBeforeIDE = function (childDef)
{
  // Get the sibling object using the pid
  var ris = [];
  if (childDef.sib)
    this.getElementsByPid(childDef.sib, ris);
  var sibid;
  if (ris.length > 0)
    sibid = ris[0].id;
  //
  // Some children could refer subviews
  if (childDef.child.child) {
    var x = this.app.createView(childDef.child);
    this.insertBefore(x, sibid);
  }
  else {
    // This one is a normal element
    var x = new App.RemElem(this.view, childDef.child, this);
    this.insertBefore(x, sibid);
  }
};


/**
 * Remove an element child
 * @param {RemElem} child - the element to remove
 * @param {Bool} triggerAnimation - true to trigger an animation for removing of this child
 */
App.RemElem.prototype.removeChild = function (child, triggerAnimation)
{
  // If the element has no children, I can't remove anything
  if (this.elements === undefined)
    return;
  //
  // Search the child in the elements array and remove it
  var idx = this.elements.indexOf(child);
  if (idx > -1) {
    var el = this.elements[idx];
    el.close();
    this.elements.splice(idx, 1);
    //
    // Tell the client to remove the child
    if (this.view.isOpen)
      this.app.sendCommands([{obj: this.id, id: "removeChild", cnt: {id: child.id, triggerAnimation: triggerAnimation}}]);
  }
};


/**
 * Remove an element child, called by the IDE when the application is running and must be updated (AApp.updateCode)
 * @param {Object} child - the object representing the element to remove (must use the pid to find the real element)
 */
App.RemElem.prototype.removeChildIDE = function (child)
{
  // Use the PID to find all the objects related to the 'project object' and remove them
  var ris = [];
  this.getElementsByPid(child.id, ris);
  for (var i = 0; i < ris.length; i++)
    this.removeChild(ris[i]);
};


/**
 * Return a copy of the element
 * @param {Object} map
 */
App.RemElem.prototype.clone = function (map, view)
{
  var el = new App.RemElem(view || this.view);
  //
  // Add the clone to map of cloned objects
  if (map)
    map[this.id] = el;
  //
  // Cloning properties
  var keys = Object.keys(this);
  for (var k = 0; k < keys.length; k++) {
    var v = keys[k];
    //
    // Don't copy pointer to row of owner dataMap and parent.
    // Don't copy obs property either because observed should be attached to the cloned element
    if (["row", "parent", "obs", "view"].indexOf(v) !== -1)
      continue;
    //
    // Don't copy RemElem (excluded parent) because are cloned and reassigned when clone elements
    if (this[v] instanceof App.RemElem || (this[v] instanceof App.View && v !== "view"))
      continue;
    //
    // Copying children
    if (v === "elements") {
      if (el.elements === undefined)
        el.elements = [];
      //
      // For each child, create a clone of it
      for (var i = 0; i < this.elements.length; i++) {
        var ne = this.elements[i].clone(map, view);
        el.elements.push(ne);
        //
        // Relink original parent
        if (ne instanceof App.DataMap)
          ne.parentElement = el;
        else
          ne.parent = el;
        //
        // Reassign also the direct pointer
        if (ne instanceof App.RemElem || ne instanceof App.View || ne instanceof App.DataMap)
          el[ne.name] = ne;
      }
    }
    else if (v === "animations") {
      if (this.animations.length > 0)
        el.animations = this.animations.slice(0);
    }
    else if (v === "id") {
      // The id property is special and needs to be generated here
      el[v] = this.app.generateCounter();
      this.app.eleMap[el.id] = el;
    }
    else if (v === "style") {
      // Handling style object
      for (var vs in this.style)
        el.style[vs] = this.style[vs];
    }
    else {
      // Copying simple properties
      el[v] = this[v];
    }
  }
  //
  return el;
};


/**
 * Set the parent of the element
 * @param {View} view
 * @param {RemElem} parent
 */
App.RemElem.prototype.setParent = function (view, parent)
{
  this.view = view;
  this.parent = parent;
  if (this.elements) {
    for (var i = 0; i < this.elements.length; i++) {
      if (this.elements[i].setParent)
        this.elements[i].setParent(view, this);
    }
  }
};


/**
 * Check if is child of another element
 * @param {RemElem} parent
 */
App.RemElem.prototype.isChildOf = function (parent)
{
  if (!(this.parent instanceof App.RemElem))
    return false;
  //
  if (this.parent === parent)
    return true;
  //
  return this.parent.isChildOf(parent);
};


/**
 * Sets the sendChg flag on the remelem
 * @param {Boolean} flag
 */
App.RemElem.prototype.sendChanges = function (flag)
{
  this.sendChg = flag;
  if (this.elements) {
    for (var i = 0; i < this.elements.length; i++) {
      if (this.elements[i].sendChanges)
        this.elements[i].sendChanges(flag);
    }
  }
};


/**
 * Get the list of all child elements
 * @param {Array} list
 */
App.RemElem.prototype.allElements = function (list)
{
  list.push(this);
  if (this.elements) {
    for (var i = 0; i < this.elements.length; i++) {
      if (this.elements[i].allElements)
        this.elements[i].allElements(list);
    }
  }
};


/**
 * Get the list of all subview requirements
 * @param {Object} list
 */
App.RemElem.prototype.allRequirements = function (list)
{
  if (this.elements) {
    for (var i = 0; i < this.elements.length; i++) {
      if (this.elements[i].allRequirements)
        this.elements[i].allRequirements(list);
    }
  }
};


/**
 * Bind this element to a DataRow
 * @param {DataRow} row
 */
App.RemElem.prototype.setDataRow = function (row)
{
  this.row = row;
  if (this.elements) {
    for (var i = 0; i < this.elements.length; i++) {
      if (this.elements[i].setDataRow)
        this.elements[i].setDataRow(row);
    }
  }
};


/**
 * Bind this element to a DataGroup
 * @param {DataGroup} group
 */
App.RemElem.prototype.setDataGroup = function (group)
{
  this.group = group;
  if (this.elements) {
    for (var i = 0; i < this.elements.length; i++) {
      if (this.elements[i].setDataGroup)
        this.elements[i].setDataGroup(group);
    }
  }
};


/*
 * Returns all elements by pid
 * @param {String} pid
 * @param {Array} ris
 */
App.RemElem.prototype.getElementsByPid = function (pid, ris)
{
  if (this.pid === pid)
    ris.push(this);
  if (this.elements) {
    for (var i = 0; i < this.elements.length; i++) {
      if (this.elements[i].getElementsByPid)
        this.elements[i].getElementsByPid(pid, ris);
    }
  }
};


/*
 * Returns all subviews
 * @param {Array} ris
 */
App.RemElem.prototype.getSubviews = function (ris)
{
  if (this.elements) {
    for (var i = 0; i < this.elements.length; i++) {
      if (this.elements[i] instanceof App.View)
        ris.push(this.elements[i]);
      else if (this.elements[i].getSubviews)
        this.elements[i].getSubviews(ris);
    }
  }
};


/**
 * Apply default values after view loading
 * @param {Object} defprop
 */
App.RemElem.prototype.applyDefaults = function (defprop)
{
  // _xc is true if this element should be merged
  if (defprop._xc) {
    for (var x in defprop) {
      // Skip added properties that aren't real element values
      if (x.substr(0, 1) === "_" || x === "id")
        continue;
      //
      // Skip children
      if (x !== "children") {
        //
        // Handling style object
        if (x === "style") {
          var val = defprop[x];
          if (val.length === 0)
            val = "{}";
          var ss = JSON.parse(val);
          for (var xs in ss)
            this.style[xs] = ss[xs];
        }
        else {
          // Copy element property value
          if (x.substr(0, 1) === "&")
            this[x.substring(1)] = App.Utils.t({id: defprop[x].id + x, t: defprop[x].t, lc: this.app.langCode});
          else
            this[x] = defprop[x];
        }
      }
    }
  }
  //
  // Merging children
  if (defprop.children) {
    for (var i = 0; i < defprop.children.length; i++) {
      var cd = defprop.children[i];
      var b = false;
      //
      // Searching the child in my elements
      for (var j = 0; j < this.elements.length; j++) {
        var e = this.elements[j];
        if (e.pid === cd.id || e.pid === cd.oid) {
          //
          // Found, update it - don't update if it is a sub-subview
          if (e instanceof App.RemElem)
            e.applyDefaults(cd);
          b = true;
          break;
        }
      }
      //
      if (!b) {
        // Not found: appending it
        var x;
        if (cd.child)
          x = this.app.createView(cd);
        else
          x = new App.RemElem(this.view, cd, this);
        this.elements.push(x);
        this[x.name] = x;
      }
    }
  }
};


/**
 * Send "focus", "blur", "click" command
 */
App.RemElem.prototype.focus = function ()
{
  this.app.sendCommands([{obj: this.id, id: "focus"}], true);
};
App.RemElem.prototype.blur = function ()
{
  this.app.sendCommands([{obj: this.id, id: "blur"}], true);
};
App.RemElem.prototype.click = function ()
{
  this.app.sendCommands([{obj: this.id, id: "click"}], true);
};


/**
 * Play an animation on this element
 * @param {Animation} animation - animation to play
 */
App.RemElem.prototype.playAnimation = function (animation)
{
  if (!animation)
    return;
  //
  this.app.sendCommands([{obj: this.id, id: "playAnimation", cnt: JSON.stringify(animation)}], true);
};


/**
 * Pause an animation on this element
 */
App.RemElem.prototype.pauseAnimation = function ()
{
  this.app.sendCommands([{obj: this.id, id: "pauseAnimation"}], true);
};


/**
 * Stop an animation on this element
 */
App.RemElem.prototype.stopAnimation = function ()
{
  this.app.sendCommands([{obj: this.id, id: "stopAnimation"}], true);
};


/**
 * Resumes an animation on this element
 */
App.RemElem.prototype.resumeAnimation = function ()
{
  this.app.sendCommands([{obj: this.id, id: "resumeAnimation"}], true);
};


/**
 * Reset an animation, the element is reset on the initial status before the animation.
 * If the animation is ongoing it will be stopped
 */
App.RemElem.prototype.resetAnimation = function ()
{
  this.app.sendCommands([{obj: this.id, id: "resetAnimation"}], true);
};


/**
 * Plays the animation in the reverse order
 * @param {Animation} animation - animation to play in the reverse order
 *                                (optional, if the animation was already played on the Element this parameter can be skipped;
 *                                in that case the animation to be reverted will be the last animation played)
 */
App.RemElem.prototype.revertAnimation = function (animation)
{
  this.app.sendCommands([{obj: this.id, id: "revertAnimation", cnt: animation ? JSON.stringify(animation) : ""}], true);
};


/**
 * Add the animation definition to the animations of the object
 * @param {Object} animation -
 */
App.RemElem.prototype.addAnimation = function (animation)
{
  this.animations.push(typeof animation === "string" ? animation : JSON.stringify(animation));
};


/**
 * Send "getBoundingClientRect", "addTableRow", "addTableColumn", "setCellContent",
 * "removeCellContent", "setRowSpan", "setColSpan" commands
 */
App.RemElem.prototype.getBoundingClientRect = function (cb) {
  this.app.sendCommands([{obj: this.id, id: "getBoundingClientRect", cnt: {params: [this.gencb(cb)]}}], true);
};
App.RemElem.prototype.addTableRow = function (position) {
  this.app.sendCommands([{obj: this.id, id: "addTableRow", cnt: {params: [position]}}], true);
};
App.RemElem.prototype.addTableColumn = function (position) {
  this.app.sendCommands([{obj: this.id, id: "addTableColumn", cnt: {params: [position]}}], true);
};
App.RemElem.prototype.setCellContent = function (element, row, column) {
  this.app.sendCommands([{obj: this.id, id: "setCellContent", cnt: {params: [element, row, column]}}], true);
};
App.RemElem.prototype.removeCellContent = function (row, column) {
  this.app.sendCommands([{obj: this.id, id: "removeCellContent", cnt: {params: [row, column]}}], true);
};
App.RemElem.prototype.setRowSpan = function (row, column, rowspan) {
  this.app.sendCommands([{obj: this.id, id: "setRowSpan", cnt: {params: [row, column, rowspan]}}], true);
};
App.RemElem.prototype.setColSpan = function (row, column, colspan) {
  this.app.sendCommands([{obj: this.id, id: "setColSpan", cnt: {params: [row, column, colspan]}}], true);
};


/**
 * Returns true if the selected element has the class
 * @param {String} className
 * @returns {Boolean}
 */
App.RemElem.prototype.containsClass = function (className) {
  if (!this.className || this.className === "" || !className.trim)
    return false;
  //
  var classList = this.className.split(" ");
  for (var i = 0; i < classList.length; i++)
    if (classList[i] === className.trim())
      return true;
  //
  return false;
};


/**
 * Add a class to the element
 * @param {String} newClass
 */
App.RemElem.prototype.addClass = function (newClass) {
  if (!newClass || !newClass.trim || newClass.trim() === "" || this.containsClass(newClass))
    return;
  //
  if (!this.className)
    this.className = "";
  this.className += (this.className.length === 0 ? "" : " ") + newClass;
};


/**
 * Remove a class from the element
 * @param {String} delClass
 */
App.RemElem.prototype.removeClass = function (delClass) {
  if (!delClass || !delClass.trim || delClass.trim() === "" || !this.containsClass(delClass))
    return;
  //
  var classList = this.className.split(" ");
  for (var i = 0; i < classList.length; i++)
    if (classList[i] === delClass.trim()) {
      classList.splice(i, 1);
      break;
    }
  //
  this.className = (classList.length > 0 ? classList.join(" ") : "");
};


/**
 * Generate a new callback id
 * @param {Function} cb
 */
App.RemElem.prototype.gencb = function (cb)
{
  var id = App.Utils.generateUID24();
  if (!this.cbmap)
    this.cbmap = {};
  this.cbmap[id] = cb;
  return id;
};


/**
 * Called when a function returns
 * @param {String|Boolean} response
 */
App.RemElem.prototype.cb = function (response)
{
  var cb = this.cbmap[response.cbId];
  if (cb) {
    //
    // I can't just call the callback... the confirm is an asynchronous
    // method and uses Generators... Calling it directly gives an error (who knows why???)
    setImmediate(function () {
      cb(response.res);
    });
    //
    delete this.cbmap[response.cbId];
  }
  //
  return response;
};


/**
 * Convert the value for the client
 * @param {String} value
 */
App.RemElem.prototype.sanitizeValue = function (value) {
  if (value && this.type === "password") {
    var ml = this.maxLength || 8;
    ml = Math.min(ml, 8);
    return "§".repeat(ml);
  }
  else {
    return value;
  }
};


//export module for node
if (module)
  module.exports = App.RemElem;
