/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */
/* global module */

var App = App || {};

if (module)
  App.Database = require("./database");


/**
 * @class A SQLServer Database Object
 * @param {Session} app that owns this instance
 * @extends App.Database
 * */
App.SQLServer = function (app)
{
  if (module && !App.View)
    App = require("./requireAll");
  //
  App.Database.call(this, app);
  //
  this.typemap = App.SQLServer.DataTypeMap; // Map of data types
  this.keywords = App.SQLServer.Keywords;   // List of reserved keywords
  this.defaultSchema = "dbo";
  //
  // Definition of functions and operators
  this.functions = Object.assign({}, App.Database.Functions, App.SQLServer.Functions);
};


// Make App.SQLServer extend App.Database
App.SQLServer.prototype = new App.Database(true);


App.SQLServer.DataTypeMap = {
  i: ["int", "bigint", "smallint", "tinyint"],
  f: ["real", "float"],
  d: ["decimal", "money", "numeric", "smallmoney"],
  vc: ["nvarchar", "varchar"],
  c: ["nchar", "char"],
  bl: ["image", "binary", "varbinary"],
  cl: ["ntext", "text"],
  b: ["bit"],
  da: ["date"],
  ti: ["time"],
  dt: ["datetimeoffset", "datetime", "datetime2", "smalldatetime", "timestamp"],
  id: ["uniqueidentifier"],
  ai: [],
  j: ["nvarchar", "varchar"]
};


App.SQLServer.Keywords = [
  "ADD",
  "ALL",
  "ALTER",
  "AND",
  "ANY",
  "AS",
  "ASC",
  "AUTHORIZATION",
  "BACKUP",
  "BEGIN",
  "BETWEEN",
  "BREAK",
  "BROWSE",
  "BULK",
  "BY",
  "CASCADE",
  "CASE",
  "CHECK",
  "CHECKPOINT",
  "CLOSE",
  "CLUSTERED",
  "COALESCE",
  "COLLATE",
  "COLUMN",
  "COMMIT",
  "COMPUTE",
  "CONSTRAINT",
  "CONTAINS",
  "CONTAINSTABLE",
  "CONTINUE",
  "CONVERT",
  "CREATE",
  "CROSS",
  "CURRENT",
  "CURRENT_DATE",
  "CURRENT_TIME",
  "CURRENT_TIMESTAMP",
  "CURRENT_USER",
  "CURSOR",
  "DATABASE",
  "DBCC",
  "DEALLOCATE",
  "DECLARE",
  "DEFAULT",
  "DELETE",
  "DENY",
  "DESC",
  "DISK",
  "DISTINCT",
  "DISTRIBUTED",
  "DOUBLE",
  "DROP",
  "DUMP",
  "ELSE",
  "END",
  "ERRLVL",
  "ESCAPE",
  "EXCEPT",
  "EXEC",
  "EXECUTE",
  "EXISTS",
  "EXIT",
  "EXTERNAL",
  "FETCH",
  "FILE",
  "FILLFACTOR",
  "FOR",
  "FOREIGN",
  "FREETEXT",
  "FREETEXTTABLE",
  "FROM",
  "FULL",
  "FUNCTION",
  "GOTO",
  "GRANT",
  "GROUP",
  "HAVING",
  "HOLDLOCK",
  "IDENTITY",
  "IDENTITYCOL",
  "IDENTITY_INSERT",
  "IF",
  "IN",
  "INDEX",
  "INNER",
  "INSERT",
  "INTERSECT",
  "INTO",
  "IS",
  "JOIN",
  "KEY",
  "KILL",
  "LEFT",
  "LIKE",
  "LINENO",
  "LOAD",
  "MERGE",
  "NATIONAL",
  "NOCHECK",
  "NONCLUSTERED",
  "NOT",
  "NULL",
  "NULLIF",
  "OF",
  "OFF",
  "OFFSETS",
  "ON",
  "OPEN",
  "OPENDATASOURCE",
  "OPENQUERY",
  "OPENROWSET",
  "OPENXML",
  "OPTION",
  "OR",
  "ORDER",
  "OUTER",
  "OVER",
  "PERCENT",
  "PIVOT",
  "PLAN",
  "PRECISION",
  "PRIMARY",
  "PRINT",
  "PROC",
  "PROCEDURE",
  "PUBLIC",
  "RAISERROR",
  "READ",
  "READTEXT",
  "RECONFIGURE",
  "REFERENCES",
  "REPLICATION",
  "RESTORE",
  "RESTRICT",
  "RETURN",
  "REVERT",
  "REVOKE",
  "RIGHT",
  "ROLLBACK",
  "ROWCOUNT",
  "ROWGUIDCOL",
  "RULE",
  "SAVE",
  "SCHEMA",
  "SECURITYAUDIT",
  "SELECT",
  "SEMANTICKEYPHRASETABLE",
  "SEMANTICSIMILARITYDETAILSTABLE",
  "SEMANTICSIMILARITYTABLE",
  "SESSION_USER",
  "SET",
  "SETUSER",
  "SHUTDOWN",
  "SOME",
  "STATISTICS",
  "SYSTEM_USER",
  "TABLE",
  "TABLESAMPLE",
  "TEXTSIZE",
  "THEN",
  "TO",
  "TOP",
  "TRAN",
  "TRANSACTION",
  "TRIGGER",
  "TRUNCATE",
  "TRY_CONVERT",
  "TSEQUAL",
  "UNION",
  "UNIQUE",
  "UNPIVOT",
  "UPDATE",
  "UPDATETEXT",
  "USE",
  "USER",
  "VALUES",
  "VARYING",
  "VIEW",
  "WAITFOR",
  "WHEN",
  "WHERE",
  "WHILE",
  "WITH",
  "WITHIN GROUP",
  "WRITETEXT"
];

/**
 * Definitions of functions and operators
 *   expr: implementation of the function/operator
 *   op: true if is an operator
 *   aggr: true if is an aggregated function
 * */
App.SQLServer.Functions = {
  // Operators
  //
  // Date functions
  now: {expr: "GETDATE()"},
  today: {expr: "convert(datetime,round(convert(float,getdate()),0,1))"},
  time: {expr: "convert(datetime,convert(varchar,getdate(),8))"},
  toDate: {expr: "convert(datetime,cast((@3) as varchar)+'/'+cast((@2) as varchar)+'/'+cast((@1) as varchar),103)"},
  toTime: {expr: "convert(datetime, cast('1899-12-30' as varchar)+' '+cast((@1) as varchar)+':'+cast((@2) as varchar)+':'+cast((@3) as varchar),20)"},
  year: {expr: "YEAR(@1)"},
  month: {expr: "MONTH(@1)"},
  day: {expr: "DAY(@1)"},
  hour: {expr: "DATEPART(hour, @1)"},
  minute: {expr: "DATEPART(minute, @1)"},
  second: {expr: "DATEPART(second, @1)"},
  weekDay: {expr: "DATEPART(weekday, @1)"},
  yearDay: {expr: "DATEPART(dayofyear, @1)"},
  week: {expr: "DATEPART(week, @1)"},
  getDate: {expr: "convert(datetime,cast(DAY(@1) as varchar)+'/'+cast(MONTH(@1) as varchar)+'/'+cast(YEAR(@1) as varchar),103)"},
  //
  // Math functions
  int: {expr: "ROUND(@1, 0, 1)"},
  ceil: {expr: "CEILING(@1)"},
  floor: {expr: "FLOOR(@1)"},
  round: {expr: "ROUND(@1, @2, 0)"},
  trunc: {expr: "ROUND(@1, @2, 1)"},
  sqrt: {expr: "SQRT(@1)"},
  log: {expr: "LOG(@1)"},
  log10: {expr: "LOG10(@1)"},
  cot: {expr: "COT(@1)"},
  //
  // String functions
  left: {expr: "SUBSTRING(@1,1,@2)"},
  right: {expr: "RIGHT(@1,@2)"},
  substr: {expr: "SUBSTRING(@1, @2, @3)"},
  spaces: {expr: "SPACE(@1)"},
  replicate: {expr: "REPLICATE(LEFT(@1, 1), @2)"},
  fillLeft: {expr: "RIGHT(REPLICATE(SUBSTRING(@3, 1, 1), @2) + @1, @2)"},
  fillRight: {expr: "SUBSTRING(@1 + REPLICATE(SUBSTRING(@3, 1, 1), @2), 1, @2)"},
  //
  // Various functions
  nullValue: {expr: "ISNULL(@1, @2)"},
  blobSize: {expr: "datalength(@1)"}
};


/*
 * Return the phisical name of database
 * @return {String}
 */
App.SQLServer.prototype.getDBName = function ()
{
  return this.schema.name;
};


/**
 * Quote a name
 * @param {String} s name to quote
 */
App.SQLServer.prototype.quote = function (s)
{
  if (!s)
    s = "";
  if (this.keywords.indexOf(s.toUpperCase()) === -1)
    return s;
  else
    return "[" + s + "]";
};


/**
 * Open the connection to the database
 * @param {Function} callback - function to be called at the end
 */
App.SQLServer.prototype._openConnection = function (callback)
{
  if (!module)
    return callback(null, new Error("SQLServer is supported only inside Node"));
  //
  // Import module (could be missing)
  try {
    if (!App.mssql)
      App.mssql = require("mssql");
  }
  catch (ex) {
    return callback(null, new Error("SQLServer driver not found.\nInstall \"mssql\" module and try again"));
  }
  //
  try {
    this.getPool(callback);
  }
  catch (e) {
    callback(null, e);
  }
};


/**
 * Init the application pool
 * @param {Function} callback - function to be called at the end
 */
App.SQLServer.prototype.getPool = function (callback) {
  if (!App.SQLServer.pools)
    App.SQLServer.pools = {};
  //
  var poolName = JSON.stringify(this.options);
  if (App.SQLServer.pools[poolName])
    return callback(App.SQLServer.pools[poolName]);
  //
  var pool = new App.mssql.Connection(this.options);
  pool.connect(function (error) {
    if (error)
      return callback(null, error);
    //
    App.SQLServer.pools[poolName] = pool;
    callback(pool);
  });
};


/**
 * Close the connection to the database
 * @param {Function} callback - function to be called at the end
 */
App.SQLServer.prototype._closeConnection = function (callback)
{
  callback();
};


/**
 * Save the schema definition in z_schema table
 * @param {Array} sqls - list of sql statement where to add
 * @param {Object} news - new schema
 */
App.SQLServer.prototype.saveZSchema = function (sqls, news)
{
  sqls.push({sql: "if not exists (select * from sysobjects where id = object_id(N'z_schema')) create table z_schema ([schema] text)"});
  sqls.push({sql: "delete from z_schema"});
  sqls.push({sql: "insert into z_schema values (?)", params: [JSON.stringify(news)]});
};


/**
 * Execute an SQL query
 * @param {String} sql to execute
 * @param {Boolean} readOnly - true if is a SELECT
 * @param {Array} parameters
 * @param {Function} callback - function to be called at the end
 * @return {Object/Integer}
 */
App.SQLServer.prototype._executeSQL = function (sql, readOnly, parameters, callback)
{
  var req = new App.mssql.Request(this.transaction || this.connection);
  //
  // Detect type of command
  if (!readOnly) {
    var command = "";
    if (sql.toLowerCase().indexOf("update ") !== -1)
      command = "update";
    else if (sql.toLowerCase().indexOf("delete ") !== -1)
      command = "delete";
    else if (sql.toLowerCase().indexOf("insert into ") !== -1)
      command = "insert";
    //
    // For INSERT, UPDATE and DELETE append another statement for info
    if (command) {
      req.multiple = true;
      sql += "; select @@rowcount as RowsAffected";
      if (command === "insert")
        sql += "; select @@identity as Counter";
    }
  }
  //
  // Add input parameters
  for (var i = 0; i < parameters.length; i++)
    req.input("P" + (i + 1), parameters[i]);
  //
  // Run the query
  req.query(sql, function (error, result) {
    callback(result, error);
  });
};


/**
 * Execute an SQL query with offset paging
 * @param {Object} params
 * @param {Function} callback - function to be called at the end
 * @return {DataMap/Object}
 */
App.SQLServer.prototype.queryOffsetPaging = function (params, callback)
{
  var sql = params.sql;
  sql += " offset " + params.offset + " rows ";
  sql += " fetch next " + params.limit + " rows only";
  this.executeSQL(sql, true, params.parameters, callback);
};


/*
 * Read some info from result
 * @param {String} sql
 * @param {Object} result
 */
App.SQLServer.prototype._readInfoFromResult = function (sql, result)
{
  if (result) {
    for (var i = result.length - 1; i >= 0; i--) {
      if (!this.hasOwnProperty("lastRowsAffected") && result[i][0].hasOwnProperty("RowsAffected"))
        this.lastRowsAffected = result[i][0].RowsAffected;
      if (!this.hasOwnProperty("lastInsertId") && result[i][0].hasOwnProperty("Counter"))
        this.lastInsertId = result[i][0].Counter;
    }
  }
};


/**
 * Recreate the primary key of table
 * @param {Array} sqls list of sql statements
 * @param {Object} t table to recreate
 * @param {Object} ot old version
 */
App.SQLServer.prototype.recreatePK = function (sqls, t, ot)
{
  this.dropExternalTableFKS(sqls, ot);
  //
  var sql = "BEGIN\n";
  sql += "DECLARE @pkname VARCHAR(MAX);\n";
  sql += " SET @pkname = (SELECT constraint_name FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS ";
  sql += " WHERE table_name like '" + ot.name + "'";
  sql += " and table_schema = '" + (ot.schema || this.defaultSchema) + "'";
  sql += " and constraint_type = 'PRIMARY KEY');\n";
  sql += " IF @pkname is not null";
  sql += "   EXECUTE ('ALTER TABLE " + this.quote(ot.name) + " DROP CONSTRAINT ' + @pkname);\n";
  var pks = "";
  for (var i = 0; i < t.fields.length; i++) {
    if (t.fields[i].pk)
      pks += (pks ? "," : "") + this.quote(t.fields[i].name);
  }
  if (pks)
    sql += " ALTER TABLE " + this.quote(ot.name) + " ADD PRIMARY KEY (" + pks + ");\n";
  sql += "END";
  sqls.push({sql: sql});
};


/**
 * Create a table
 * @param {Array} sqls list of sql statements
 * @param {Object} t table to create
 */
App.SQLServer.prototype.createTable = function (sqls, t)
{
  // Getting create table stmt
  var s = "create table " + this.quote(t.name) + " (";
  //
  // Getting fields definition
  for (var i = 0; i < t.fields.length; i++)
    s += (i === 0 ? "" : ",") + "\n  " + this.getFieldDef(t.fields[i]);
  //
  // Creating PK constraint inside table definition
  var pkn = "";
  for (var i = 0; i < t.fields.length; i++) {
    var f = t.fields[i];
    if (f.pk)
      pkn += (pkn === "" ? "" : ", ") + this.quote(f.name);
  }
  if (pkn !== "")
    s += ",\n  primary key (" + pkn + ")";
  //
  s += "\n)";
  //
  sqls.push({sql: s});
  //
  // Return the table with childs
  var nt = {};
  nt.name = t.name;
  nt.fields = t.fields;
  return nt;
};


/**
 * Rename a table
 * @param {Array} sqls list of sql statements
 * @param {Object} t table to rename
 * @param {Object} ot old version
 */
App.SQLServer.prototype.renameTable = function (sqls, t, ot)
{
  sqls.push({sql: "exec sp_rename '" + this.quote(ot.name) + "', '" + this.quote(t.name) + "'"});
  this.modifyTableFKs(sqls, t, ot);
  return true;
};


/**
 * Modify a field
 * @param {Array} sqls list of sql statements
 * @param {Object} t table to modify
 * @param {Object} f field to modify
 * @param {Object} of old version
 */
App.SQLServer.prototype.modifyField = function (sqls, t, f, of)
{
  // If field became not null without default update table with dummy value
  if (f.notn && !of.notn)
    this.updateRequiredField(sqls, t, of);
  //
  if (f.name !== of.name) {
    sqls.push({sql: "exec sp_rename '" + this.quote(t.name) + "." + this.quote(of.name) + "', '" + this.quote(f.name) + "', 'COLUMN'"});
    of.name = f.name;
  }
  //
  if (f.datatype !== of.datatype || this.getFieldMaxLength(f) !== this.getFieldMaxLength(of) ||
          this.getFieldScale(f) !== this.getFieldScale(of) || this.getFieldDefVal(f) !== this.getFieldDefVal(of) || f.notn !== of.notn)
    sqls.push({sql: "alter table " + this.quote(t.name) + " alter column " + this.getFieldDef(f)});
  //
  return true;
};


/**
 * Modify a foreign key
 * @param {Array} sqls list of sql statements
 * @param {Object} t table to modify
 * @param {Object} ot table old version
 * @param {Object} fk foreign key to modify
 * @param {Object} ofk old version
 */
App.SQLServer.prototype.modifyFK = function (sqls, t, ot, fk, ofk)
{
  var ok = true;
  if (Object.keys(fk.refs).length !== Object.keys(ofk.refs).length)
    ok = false;
  else {
    for (var ref in fk.refs) {
      if (!ofk.refs[ref])
        ok = false;
    }
  }
  //
  if (!ok) {
    this.dropFK(sqls, t, ofk);
    this.createFK(sqls, t, fk);
  }
  else if (fk.name !== ofk.name || t.name !== ot.name)
    sqls.push({sql: "exec sp_rename '" + this.quote(this.getFKName(ot, ofk)) + "', '" + this.quote(this.getFKName(t, fk)) + "', 'OBJECT'"});
  //
  return true;
};


/**
 * Modify an index
 * @param {Array} sqls list of sql statements
 * @param {Object} t table owner of index
 * @param {Object} idx index to modify
 * @param {Object} oidx old version
 */
App.SQLServer.prototype.modifyIndex = function (sqls, t, idx, oidx)
{
  var ok = true;
  if (idx.un !== oidx.un)
    ok = false;
  else if (idx.fields.length !== oidx.fields.length)
    ok = false;
  else {
    for (var i = 0; i < idx.fields.length; i++) {
      if (idx.fields[i] !== oidx.fields[i])
        ok = false;
    }
  }
  //
  if (!ok) {
    this.dropIndex(sqls, t, oidx);
    this.createIndex(sqls, t, idx);
  }
  else if (idx.name !== oidx.name)
    sqls.push({sql: "exec sp_rename '" + this.quote(t.name) + "." + this.quote(oidx.name) + "', '" + this.quote(idx.name) + "', 'INDEX'"});
  return true;
};


/**
 * Drop an index
 * @param {Array} sqls list of sql statements
 * @param {Object} t table owner of index
 * @param {Object} idx index to drop
 */
App.SQLServer.prototype.dropIndex = function (sqls, t, idx)
{
  sqls.push({sql: "drop index " + this.quote(idx.name) + " on " + this.quote(t.name)});
  return true;
};


/**
 * Get the datatype for the f field
 * @param {Object} f
 */
App.SQLServer.prototype.getDataType = function (f)
{
  var maxlen = this.getFieldMaxLength(f);
  var scale = this.getFieldScale(f);
  switch (f.datatype) {
    case App.Database.DataTypes.INT:
    {
      if (maxlen < 5)
        return "smallint"; // -32768 to +32767
      else if (maxlen < 10)
        return "int"; // -2147483648 to +2147483647
      else
        return "bigint"; // -9223372036854775808 to +9223372036854775807
    }

    case App.Database.DataTypes.AUTOINCR:
    {
      if (maxlen < 5)
        return "smallint identity"; // 1 to +32767
      else if (maxlen < 10)
        return "int identity"; // 1 to +2147483647
      else
        return "bigint identity"; // 1 to +9223372036854775807
    }

    case App.Database.DataTypes.FLOAT:
    {
      if (maxlen < 8)
        return "real"; // 7 decimal digits precision
      else
        return "float"; // 15 decimal digits precision
    }

    case App.Database.DataTypes.DECIMAL:
      return "decimal(" + maxlen + "," + scale + ")";

    case App.Database.DataTypes.JSON:
    case App.Database.DataTypes.VARCHAR:
    {
      if (maxlen < 8000)
        return "nvarchar" + "(" + (maxlen || "max") + ")";
      else
        return "text";        // maxlen: 2147483647
    }

    case App.Database.DataTypes.CHAR:
      return "char(" + maxlen + ")";

    case App.Database.DataTypes.UUID:
    {
      if (maxlen === 36)
        return "uniqueidentifier";
      else
        return "char(" + maxlen + ")";
    }

    default:
      return this.typemap[f.datatype][0];
  }
};


/**
 * Get the action for the fk foreign key
 * @param {Object} fk
 */
App.SQLServer.prototype.getFKActions = function (fk)
{
  var s = "";
  for (var i = 0; i < 2; i++) {
    var r = (i === 0 ? fk.ur : fk.dr);
    var op = (i === 0 ? " on update " : " on delete ");
    if (r === App.Database.FKActions.CASCADE)
      s += op + "cascade";
    else if (r === App.Database.FKActions.SETDEFAULT)
      s += op + "set default";
    else if (r === App.Database.FKActions.NOACTION)
      s += op + "no action";
    else if (r === App.Database.FKActions.SETNULL)
      s += op + "set null";
  }
  return s;
};


/**
 * Compose sql limit clause
 * @param {Object} token top
 * @param {Boolean} afterSelect
 */
App.SQLServer.prototype.limitClause = function (token, afterSelect)
{
  if (afterSelect)
    return "top " + this.createSqlExpression(token);
  else
    return "";
};


/**
 * Returns the number of rows of a resultSet
 * @param {Object} rs
 */
App.SQLServer.prototype.getRowCount = function (rs)
{
  // If database is remote call base method
  if (this.remoteConnector)
    return App.Database.prototype.getRowCount.call(this, rs);
  //
  return rs.length;
};


/**
 * Returns the i-th row of a resultSet
 * @param {Object} rs - recordset
 * @param {int} pos - position
 * @param {DataSchema} schema
 */
App.SQLServer.prototype.getRow = function (rs, pos, schema)
{
  var row = (this.remoteConnector ? App.RemoteDriver.getRow(rs, pos) : rs[pos]);
  if (!schema)
    return row;
  //
  var newrow = {};
  for (var c in schema.columns) {
    var clower = c.toLowerCase();
    var v = (row.hasOwnProperty(clower) ? row[clower] : row[c]);
    if (v === undefined)
      continue;
    newrow[c] = this.convertValue(v, schema.columns[c].datatype);
  }
  return newrow;
};


/**
 * Create a new database
 * @param {Function} callback - function to be called at the end
 */
App.SQLServer.prototype.createDb = function (callback)
{
  // If database is remote don't create database
  if (this.remoteConnector)
    return callback();
  //
  // Switch to master and create the new database
  var opt = Object.assign({}, this.options);
  opt.database = "master";
  //
  // create a new client to communicate with the db
  var conn = new App.mssql.Connection(opt);
  conn.connect(function (error) {
    if (error) {
      if (callback)
        callback(null, error);
      conn.close();
      return;
    }
    //
    // run the query to create a new database
    var sql = "CREATE DATABASE [" + this.getDBName() + "] COLLATE Latin1_General_CS_AI" +
            ";ALTER DATABASE [" + this.getDBName() + "] SET AUTO_SHRINK ON" +
            ";ALTER DATABASE [" + this.getDBName() + "] SET RECURSIVE_TRIGGERS ON" +
            ";ALTER DATABASE [" + this.getDBName() + "] SET RECOVERY SIMPLE";
    var req = new App.mssql.Request(conn);
    req.multiple = true;
    req.query(sql, function (error, result) {
      conn.close();
      if (callback) {
        if (error && error.number !== 1801)
          callback(null, error);
        else
          callback();
      }
    });
  }.bind(this));
};


/**
 * Returns the field default value
 * @param {Object} f
 */
App.SQLServer.prototype.getFieldDefVal = function (f)
{
  if (f.defval) {
    if (f.datatype === App.Database.DataTypes.BOOL)
      return (f.defval === "true" ? "1" : "0");
  }
  return App.Database.prototype.getFieldDefVal.call(this, f);
};


/**
 * Convert a native value into javascript value
 * @param {String} v
 * @param {String} dt
 */
App.SQLServer.prototype.convertValue = function (v, dt)
{
  if (v === null || v === undefined)
    return null;
  //
  if (dt === App.Database.DataTypes.BOOL)
    return (v !== 0);
  //
  if (dt === App.Database.DataTypes.JSON)
    return JSON.parse(v);
  //
  return App.Database.prototype.convertValue.call(this, v, dt);
};


/*
 * Begin a transaction
 * @param {Function} callback - function to be called at the end
 */
App.SQLServer.prototype._beginTransaction = function (callback)
{
  var tr = new App.mssql.Transaction(this.connection);
  tr.begin(function (error) {
    callback(tr, error);
  });
};


/*
 * Commit a transaction
 * @param {Function} callback - function to be called at the end
 */
App.SQLServer.prototype._commitTransaction = function (callback)
{
  this.transaction.commit(function (error) {
    callback(null, error);
  });
};


/*
 * Rollback a transaction
 * @param {Function} callback - function to be called at the end
 */
App.SQLServer.prototype._rollbackTransaction = function (callback)
{
  this.transaction.rollback(function (error) {
    callback(null, error);
  });
};


/**
 * Import fiels of a table
 * @param {Object} options
 * @param {Function} callback - function to be called at the end
 */
App.SQLServer.prototype.importTableFields = function (options, callback)
{
  App.Database.prototype.importTableFields.call(this, options, function (fields, error) {
    if (error)
      return callback(null, error);
    //
    var sql = "select name from sys.columns where is_identity = 1";
    sql += " and object_id = object_id('" + options.table + "')";
    this.executeSQL(sql, true, function (result, error) {
      if (error)
        return callback(null, error);
      //
      for (var i = 0; i < fields.length; i++) {
        // Adjust default value because is for example ((0)), ('s')
        if (fields[i].defaultValue)
          fields[i].defaultValue = fields[i].defaultValue.replace(/\(/g, "").replace(/\)/g, "");
        //
        // Check if field is autoincrement
        for (var k = 0; k < this.getRowCount(result); k++) {
          var row = this.getRow(result, k);
          if (fields[i].name === row.name) {
            fields[i].dataType = App.Database.DataTypes.AUTOINCR;
            break;
          }
        }
      }
      //
      callback(fields);
    }.bind(this));
  }.bind(this));
};


/*
 * Define a query parameter
 * @param {Object} token
 */
App.SQLServer.prototype.defineQueryParameter = function (token)
{
  this.parameters.push(this.forSql(token.value, token.dt));
  return "@P" + this.parameters.length;
};


/*
 * Create z_sync table
 * @param {String} handler - handler of message
 * @param {Function} callback - function to be called at the end
 */
App.SQLServer.prototype.syncCreateTable = function (handler, callback)
{
  var sql = "create table z_sync" + handler + "(id int identity not null primary key, uid char(24) unique, \
device text, sender text, topics text, body text, ts datetimeoffset default SYSDATETIMEOFFSET(), expire datetimeoffset)";
  this.executeSQL(sql, false, callback);
};


//export module for node
if (module)
  module.exports = App.SQLServer;
