/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */
/* global module*/

var App = App || {};

/**
 * @class Definition of SyncPlugin object
 * @param {Sync} sync that owns this instance
 * */
App.SyncPlugin = function (sync)
{
  if (module && !App.View && sync !== true)
    App = require("./requireAll");
  //
  this.sync = sync;
  this.paused = false;
  this.logLevel = App.SyncPlugin.logLevels.disabled;
};


App.SyncPlugin.logLevels = {
  disabled: "disabled",
  complete: "complete",
  verbose: "verbose"
};


Object.defineProperty(App.SyncPlugin.prototype, "dataStore", {
  get: function () {
    if (!this._dataStore && this.sync.dataStore)
      this.dataStore = this.sync.dataStore.schema.name;
    return this._dataStore;
  },
  set: function (newValue) {
    if (typeof newValue === "string")
      this._dataStore = new App[newValue](this.app);
    else
      this._dataStore = newValue;
  }
});


/**
 * Fires the event onConnect
 * @param {Object} options
 * @param {Function} callback - function to be called at the end
 */
App.SyncPlugin.prototype.fireOnConnect = function (options, callback)
{
  this.log("fired onConnect event");
  this.onConnect(options, callback);
};


/**
 * Event fired when the connection has been established
 * @param {Object} options
 * @param {Function} callback - function to be called at the end
 */
App.SyncPlugin.prototype.onConnect = function (options, callback)
{
  if (callback)
    callback();
};


/**
 * Event fired when the connection is closed
 */
App.SyncPlugin.prototype.onDisconnect = function ()
{
};


/**
 * Fires the event onDisconnect
 */
App.SyncPlugin.prototype.fireOnDisconnect = function ()
{
  // Fire event to me
  this.log("fired onDisconnect event");
  this.onDisconnect();
};


/**
 * Event fired when a message arrived from proxy
 * @param {Object} message received
 */
App.SyncPlugin.prototype.onProxyMessage = function (message)
{
};


/**
 * Event fired when synchronization of messages is ended
 */
App.SyncPlugin.prototype.onSynchronized = function ()
{
};


/**
 * Handle a message received from another session
 * @param {Object} message received
 */
App.SyncPlugin.prototype.handleMessage = function (message)
{
};


/**
 * Fire onMessageFiltering event
 * @param {Object} message
 * @param {Object} options
 * @param {Function} callback - function to be called at the end
 */
App.SyncPlugin.prototype.fireOnMessageFiltering = function (message, options, callback)
{
  this.onMessageFiltering(message, options, callback);
};


/**
 * onMessageFiltering event
 * @param {Object} message
 * @param {Object} options
 * @param {Function} callback - function to be called at the end
 */
App.SyncPlugin.prototype.onMessageFiltering = function (message, options, callback)
{
  callback();
};


/**
 * Delete expired messages stored into datastore
 * @param {Function} callback - function to be called at the end
 */
App.SyncPlugin.prototype.deleteExpiredMessages = function (callback)
{
  this.log("delete expired messages");
  this.sync.deleteExpiredMessages(this.id, callback);
};


/**
 * Send messages stored into datastore
 * @param {Function} callback - function to be called at the end
 */
App.SyncPlugin.prototype.sendStoredMessages = function (callback)
{
  this.log("send stored messages");
  this.sync.sendStoredMessages(this.id, callback);
};


/**
 * Request messages stored into datastore
 * @param {Function} callback - function to be called at the end
 */
App.SyncPlugin.prototype.requestStoredMessages = function (callback)
{
  this.log("request stored messages");
  this.sync.requestStoredMessages(this.id, callback);
};


/**
 * Returns the number of message stored
 * @param {Function} callback - function to be called at the end
 */
App.SyncPlugin.prototype.storedMessageCount = function (callback)
{
  this.sync.storedMessageCount(this.id, callback);
};


/**
 * Add a log message
 * @param {String} message
 */
App.SyncPlugin.prototype.log = function (message)
{
  if (this.logLevel === App.SyncPlugin.logLevels.disabled)
    return;
  //
  var args = ["Sync" + this.id, message];
  //
  // If logLevel is verbose log also additional arguments
  if (this.logLevel === App.SyncPlugin.logLevels.verbose)
    for (var i = 1; i < arguments.length; i++)
      args.push(arguments[i]);
  //
  console.log.apply(console, args);
};


//export module for node
if (module)
  module.exports = App.SyncPlugin;
