/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2016
 * All rights reserved
 */

var App = App || {};

/**
 * @class
 * @param {View} view that owns this timer instance
 * @param {Object} defprop - timer properties
 * */
App.Timer = function (view, defprop)
{
  if (module && !App.View)
    App = require("./requireAll");
  this.view = view;
  this.app = view.app;
  this.name = defprop.name;
  this.pid = defprop.id;
  this.id = this.app.generateCounter();
  //
  this.interval = defprop.interval;
  this.enabled = defprop.enabled;
  //
  // Put the element in the map
  this.app.eleMap[this.id] = this;
};


// Getter/setter of some properties
Object.defineProperties(App.Timer.prototype, {
  enabled: {
    get: function () {
      return this.iEnabled;
    },
    set: function (newValue) {
      this.iEnabled = newValue;
      //
      // Setting timer if enabled
      var pthis = this;
      if (this.iEnabled && !this.timerId)
        this.timerId = setInterval(function () {
          pthis.app.dttobj.beginRequest();
          pthis.app.dttobj.setRequestName(pthis.name + ".onTick");
          //
          // Restart simulated observers (if any)
          if (App.markObservers)
            App.markObservers();
          //
          try {
            pthis.onTick();
          }
          catch (ex) {
          }
          pthis.app.dttobj.endRequest();
        }, this.interval);
      //
      // Resetting timer if not enabled
      if (!this.iEnabled && this.timerId) {
        clearInterval(this.timerId);
        this.timerId = 0;
      }
    },
    enumerable: true
  },
  // Interval in milliseconds
  interval: {
    get: function () {
      return this.iInterval;
    },
    set: function (newValue) {
      if (this.iInterval === newValue)
        return;
      //
      this.iInterval = newValue;
      //
      // if the timer was already running, update it
      if (this.timerId) {
        this.enabled = false;
        this.enabled = true;
      }
    },
    enumerable: true
  }
});


/**
 * Set the parent of the timer
 * @param {View} view
 * @param {RemElem} parent
 */
App.Timer.prototype.setParent = function (view, parent)
{
  this.view = view;
  this.parent = parent;
};


/**
 * Timer callback
 */
App.Timer.prototype.onTick = function ()
{
};


/**
 * Remove the timer from the element map
 */
App.Timer.prototype.close = function ()
{
  this.enabled = false;
  delete this.app.eleMap[this.id];
};


/**
 * Change a property by the IDE
 * @param {Object} pv - property to change
 */
App.Timer.prototype.chgProp = function (pv)
{
  this[pv.name] = pv.value;
};


/*
 * Returns all elements by pid
 * @param {String} pid
 * @param {Array} ris
 */
App.Timer.prototype.getElementsByPid = function (pid, ris)
{
  if (this.pid === pid)
    ris.push(this);
};


//export module for node
if (module)
  module.exports = App.Timer;
