/*
 * Instant Developer Next
 * Copyright Pro Gamma Spa 2000-2015
 * All rights reserved
 */


/* global cordova */

var Plugin = Plugin || {};

/*
 * Create plugin object
 */
Plugin.Sqlite = {};


/*
 * Init plugin
 */
Plugin.Sqlite.init = function ()
{
  this.dbs = {};
  this.SQLite = window.cordova.require('cordova-sqlite-plugin.SQLite');
};


/*
 */
Plugin.Sqlite.openDatabase = function (req)
{
  var name = req.params.name;
  var id = (Math.random() + "").substring(2);
  var conn = new this.SQLite(name);
  var dbobj = {id: id, name: name, conn: conn, app: req.app};
  conn.open(function (err) {
    if (err) {
      req.setError(err.message);
    }
    else {
      this.dbs[id] = dbobj;
      req.setResult(id);
    }
  }.bind(this));
};


/*
 */
Plugin.Sqlite.closeDatabase = function (req)
{
  var id = req.params.id;
  var db = this.dbs[id];
  if (!db) {
    req.setError("database not open");
    return;
  }
  //
  db.conn.close(function (err) {
    if (err) {
      req.setError(err.message);
    }
    else {
      req.setResult("ok");
      delete this.dbs[id];
    }
  }.bind(this));
};


/*
 */
Plugin.Sqlite.query = function (req)
{
  var sql = req.params.sql || "";
  //
  var id = req.params.id;
  var db = this.dbs[id];
  if (!db) {
    req.setError("database not open");
    return;
  }
  //
  var isCommit = sql.substring(0, 6) === "commit";
  var isRollback = sql.substring(0, 8) === "rollback";
  //
  // If an error occurred in this transaction, the commit method behaves as rollback
  // and an exception will be thrown
  if (db.error && isCommit)
    sql = "rollback";
  //
  var params = req.params.params || [];
  //
  db.conn.query(sql, params, function (err, res) {
    if (err) {
      if (req.cbId)
        req.setError(err.message);
      else {
        // cache error for later
        if (!db.error)
          db.error = err.message;
      }
    }
    else {
      if (req.cbId) {
        //
        // if we are closing a transaction and an error occurred, throw it to the client
        if (isCommit && db.error)
          req.setError(db.error);
        else
          req.setResult(res);
        //
        // if we are closing a transaction, reset cached error
        if (isCommit || isRollback)
          delete db.error;
      }
    }
  }.bind(this));
};


/*
 */
Plugin.Sqlite.deleteDatabase = function (req)
{
  var name = req.params.name;
  this.SQLite.deleteDatabase(name, function (err) {
    if (err) {
      req.setError(err.message);
    }
    else {
      req.setResult("ok");
    }
  }.bind(this));
};


/*
 * An app has stopped, close dbs
 */
Plugin.Sqlite.stopApp = function (app)
{
  for (var n in this.dbs) {
    var db = this.dbs[n];
    if (db.app === app) {
      db.conn.close(function () {
      });
      delete this.dbs[n];
    }
  }
};
